@@ -102,6 +102,7 @@
 ** pbsnodes -c node1 node2  clear OFF_LINE or DOWN from listed nodes
 */
 #include <pbs_config.h>   /* the master config generated by configure */
+#include "pbsnodes.h"
 
 #include <stdio.h>
 #include <unistd.h>
@@ -113,6 +114,8 @@
 #include "pbs_ifl.h"
 #include        "mcom.h"
 #include        "cmds.h"
+#include "libcmds.h" /* TShowAbout_exit */
+
 
 #define LIST 1
 #define CLEAR 2
@@ -152,6 +155,7 @@ static int set_note(
 
   struct attropl  new;
   int             rc;
+  int             local_errno = 0;
 
   new.name     = ATTR_NODE_note;
   new.resource = NULL;
@@ -159,13 +163,14 @@ static int set_note(
   new.op       = SET;
   new.next     = NULL;
 
-  rc = pbs_manager(
+  rc = pbs_manager_err(
          con,
          MGR_CMD_SET,
          MGR_OBJ_NODE,
          name,
          &new,
-         NULL);
+         NULL,
+         &local_errno);
 
   if (rc && !quiet)
     {
@@ -177,12 +182,6 @@ static int set_note(
       fprintf(stderr, "%s\n",
         errmsg);
       }
-    else
-      {
-      fprintf(stderr, "(error %d) %s\n",
-        pbs_errno,
-        pbs_strerror(pbs_errno));
-      }
     }
 
   return(rc);
@@ -277,6 +276,7 @@ static int marknode(
 
   struct attropl  new[2];
   int             rc;
+  int             local_errno = 0;
 
   new[0].name     = ATTR_NODE_state;
   new[0].resource = NULL;
@@ -297,14 +297,14 @@ static int marknode(
     new[1].op     = op2;
     }
 
-  rc = pbs_manager(
-
+  rc = pbs_manager_err(
          con,
          MGR_CMD_SET,
          MGR_OBJ_NODE,
          name,
          new,
-         NULL);
+         NULL,
+         &local_errno);
 
   if (rc && !quiet)
     {
@@ -314,10 +314,6 @@ static int marknode(
     if ((errmsg = pbs_geterrmsg(con)) != NULL)
       fprintf(stderr, "%s\n",
               errmsg);
-    else
-      fprintf(stderr, "error: %d (%s)\n",
-              pbs_errno,
-              pbs_strerror(pbs_errno));
     }
 
   return(rc);
@@ -328,19 +324,20 @@ static int marknode(
 
 struct batch_status *statnode(
 
-        int   con,
-        char *nodearg)
+  int   con,
+  char *nodearg)
 
   {
 
   struct batch_status *bstatus;
-  char               *errmsg;
+  char                *errmsg;
+  int                  local_errno = 0;
 
-  bstatus = pbs_statnode(con, nodearg, NULL, NULL);
+  bstatus = pbs_statnode_err(con, nodearg, NULL, NULL, &local_errno);
 
   if (bstatus == NULL)
     {
-    if (pbs_errno)
+    if (local_errno)
       {
       if (!quiet)
         {
@@ -354,8 +351,8 @@ struct batch_status *statnode(
           {
           fprintf(stderr, "%s: Error %d (%s)\n",
                   progname,
-                  pbs_errno,
-                  pbs_strerror(pbs_errno));
+                  local_errno,
+                  pbs_strerror(local_errno));
           }
         }
 
@@ -417,31 +414,21 @@ void addxmlnode(
   } /* END addxmlnode() */
 
 
-enum NStateEnum
-  {
-  tnsNONE = 0, /* default behavior - show down, offline, and unknown nodes */
-  tnsActive,   /* one or more jobs running on node */
-  tnsAll,      /* list all nodes */
-  tnsBusy,     /* node cannot accept additional workload */
-  tnsDown,     /* node is down */
-  tnsFree,     /* node is idle/free */
-  tnsOffline,  /* node is offline */
-  tnsUnknown,  /* node is unknown - no contact recieved */
-  tnsUp,       /* node is healthy */
-  tnsLAST
-  };
-
 const char *NState[] =
   {
-  "NONE",
-  "active",
-  "all",
-  "busy",
-  "down",
-  "free",
-  "offline",
-  "unknown",
-  "up",
+  "",
+  ND_free,
+  ND_offline,
+  ND_down,
+  ND_reserve,
+  ND_job_exclusive,
+  ND_job_sharing,
+  ND_busy,
+  ND_state_unknown,
+  ND_timeshared,
+  ND_active,
+  ND_all,
+  ND_up,
   NULL
   };
 
@@ -471,39 +458,6 @@ int filterbystate(
 
       break;
 
-    case tnsActive:   /* one or more jobs running on node */
-
-      if (strstr(S, ND_busy) || strstr(S, ND_job_exclusive) || strstr(S, ND_job_sharing))
-        {
-        Display = 1;
-        }
-
-      break;
-
-    case tnsAll:      /* list all nodes */
-
-      Display = 1;
-
-      break;
-
-    case tnsBusy:     /* node cannot accept additional workload */
-
-      if (strstr(S, ND_busy))
-        {
-        Display = 1;
-        }
-
-      break;
-
-    case tnsDown:     /* node is down or unknown */
-
-      if (strstr(S, ND_down) || strstr(S, ND_state_unknown))
-        {
-        Display = 1;
-        }
-
-      break;
-
     case tnsFree:     /* node is idle/free */
 
       if (strstr(S, ND_free))
@@ -522,6 +476,51 @@ int filterbystate(
 
       break;
 
+    case tnsDown:     /* node is down or unknown */
+
+      if (strstr(S, ND_down) || strstr(S, ND_state_unknown))
+        {
+        Display = 1;
+        }
+
+      break;
+
+    case tnsReserve:
+
+      if (strstr(S, ND_reserve))
+        {
+        Display = 1;
+        }
+
+      break;
+
+    case tnsJobExclusive:
+
+      if (strstr(S, ND_job_exclusive))
+        {
+        Display = 1;
+        }
+
+      break;
+
+    case tnsJobSharing:
+
+      if (strstr(S, ND_job_sharing))
+        {
+        Display = 1;
+        }
+
+      break;
+
+    case tnsBusy:     /* node cannot accept additional workload */
+
+      if (strstr(S, ND_busy))
+        {
+        Display = 1;
+        }
+
+      break;
+
     case tnsUnknown:  /* node is unknown - no contact recieved */
 
       if (strstr(S, ND_state_unknown))
@@ -531,6 +530,30 @@ int filterbystate(
 
       break;
 
+    case tnsTimeshared:
+
+      if (strstr(S, ND_timeshared))
+        {
+        Display = 1;
+        }
+
+      break;
+
+    case tnsActive:   /* one or more jobs running on node */
+
+      if (strstr(S, ND_busy) || strstr(S, ND_job_exclusive) || strstr(S, ND_job_sharing))
+        {
+        Display = 1;
+        }
+
+      break;
+
+    case tnsAll:      /* list all nodes */
+
+      Display = 1;
+
+      break;
+
     case tnsUp:       /* node is healthy */
 
       if (!strstr(S, ND_down) && !strstr(S, ND_offline) && !strstr(S, ND_state_unknown))
@@ -695,9 +718,7 @@ int main(
           }
         else if ((optarg != NULL) && !strcmp(optarg, "about"))
           {
-          TShowAbout();
-
-          exit(0);
+          TShowAbout_exit();
           }
 
         errflg = 1;
@@ -751,8 +772,8 @@ int main(
       fprintf(stderr, "%s: cannot connect to server %s, error=%d (%s)\n",
         progname,
         (specified_server) ? specified_server : pbs_default(),
-        pbs_errno,
-        pbs_strerror(pbs_errno));
+        con * -1,
+        pbs_strerror(con * -1));
       }
 
     exit(1);
@@ -793,7 +814,7 @@ int main(
         }
       else
         {
-        nodeargs = malloc(2 * sizeof(char **));
+        nodeargs = calloc(2, sizeof(char **));
         nodeargs[0] = strdup("");
         nodeargs[1] = '\0';
         }
