@@ -1,81 +1,4 @@
-/*
-*         OpenPBS (Portable Batch System) v2.3 Software License
-*
-* Copyright (c) 1999-2000 Veridian Information Solutions, Inc.
-* All rights reserved.
-*
-* ---------------------------------------------------------------------------
-* For a license to use or redistribute the OpenPBS software under conditions
-* other than those described below, or to purchase support for this software,
-* please contact Veridian Systems, PBS Products Department ("Licensor") at:
-*
-*    www.OpenPBS.org  +1 650 967-4675                  sales@OpenPBS.org
-*                        877 902-4PBS (US toll-free)
-* ---------------------------------------------------------------------------
-*
-* This license covers use of the OpenPBS v2.3 software (the "Software") at
-* your site or location, and, for certain users, redistribution of the
-* Software to other sites and locations.  Use and redistribution of
-* OpenPBS v2.3 in source and binary forms, with or without modification,
-* are permitted provided that all of the following conditions are met.
-* After December 31, 2001, only conditions 3-6 must be met:
-*
-* 1. Commercial and/or non-commercial use of the Software is permitted
-*    provided a current software registration is on file at www.OpenPBS.org.
-*    If use of this software contributes to a publication, product, or
-*    service, proper attribution must be given; see www.OpenPBS.org/credit.html
-*
-* 2. Redistribution in any form is only permitted for non-commercial,
-*    non-profit purposes.  There can be no charge for the Software or any
-*    software incorporating the Software.  Further, there can be no
-*    expectation of revenue generated as a consequence of redistributing
-*    the Software.
-*
-* 3. Any Redistribution of source code must retain the above copyright notice
-*    and the acknowledgment contained in paragraph 6, this list of conditions
-*    and the disclaimer contained in paragraph 7.
-*
-* 4. Any Redistribution in binary form must reproduce the above copyright
-*    notice and the acknowledgment contained in paragraph 6, this list of
-*    conditions and the disclaimer contained in paragraph 7 in the
-*    documentation and/or other materials provided with the distribution.
-*
-* 5. Redistributions in any form must be accompanied by information on how to
-*    obtain complete source code for the OpenPBS software and any
-*    modifications and/or additions to the OpenPBS software.  The source code
-*    must either be included in the distribution or be available for no more
-*    than the cost of distribution plus a nominal fee, and all modifications
-*    and additions to the Software must be freely redistributable by any party
-*    (including Licensor) without restriction.
-*
-* 6. All advertising materials mentioning features or use of the Software must
-*    display the following acknowledgment:
-*
-*     "This product includes software developed by NASA Ames Research Center,
-*     Lawrence Livermore National Laboratory, and Veridian Information
-*     Solutions, Inc.
-*     Visit www.OpenPBS.org for OpenPBS software support,
-*     products, and information."
-*
-* 7. DISCLAIMER OF WARRANTY
-*
-* THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND. ANY EXPRESS
-* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-* OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT
-* ARE EXPRESSLY DISCLAIMED.
-*
-* IN NO EVENT SHALL VERIDIAN CORPORATION, ITS AFFILIATED COMPANIES, OR THE
-* U.S. GOVERNMENT OR ANY OF ITS AGENCIES BE LIABLE FOR ANY DIRECT OR INDIRECT,
-* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
-* OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
-* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-*
-* This license will be governed by the laws of the Commonwealth of Virginia,
-* without reference to its choice of law rules.
-*/
+#include "license_pbs.h" /* See here for the software license */
 /*
  * qmgr
  *      The qmgr command provides an administrator interface to the batch
@@ -736,7 +659,7 @@ struct server *make_connection(
     {
     if (! zopt)
       fprintf(stderr, "qmgr: cannot connect to server %s (errno=%d) %s\n",
-              name, pbs_errno, pbs_strerror(pbs_errno));
+              name, connection * -1, pbs_strerror(connection * -1));
     }
 
   return(svr);
@@ -1400,6 +1323,7 @@ int execute(
   int len;   /* Used for length of an err msg*/
   int error;   /* Error value returned */
   int perr;   /* Value returned from pbs_manager */
+  int local_errno;
   char *errmsg;   /* Error message from pbs_errmsg */
   char errnomsg[256];  /* Error message with pbs_errno */
 
@@ -1492,15 +1416,15 @@ int execute(
           {
 
           case MGR_OBJ_SERVER:
-            ss = pbs_statserver(sp -> s_connect, sa, NULL);
+            ss = pbs_statserver_err(sp -> s_connect, sa, NULL, &local_errno);
             break;
 
           case MGR_OBJ_QUEUE:
-            ss = pbs_statque(sp -> s_connect, pname -> obj_name, sa, NULL);
+            ss = pbs_statque_err(sp -> s_connect, pname -> obj_name, sa, NULL, &local_errno);
             break;
 
           case MGR_OBJ_NODE:
-            ss = pbs_statnode(sp -> s_connect, pname -> obj_name, sa, NULL);
+            ss = pbs_statnode_err(sp -> s_connect, pname -> obj_name, sa, NULL, &local_errno);
             break;
             /* DIAGTODO: handle new diag type for list */
             /* DIAGTODO: create a pbs_statdiag() */
@@ -1526,22 +1450,26 @@ int execute(
 
             if (sa == NULL)
               {
-              ss = pbs_statque(sp -> s_connect, NULL, NULL, NULL);
+              ss = pbs_statque_err(sp -> s_connect, NULL, NULL, NULL, &local_errno);
 
               if (ss != NULL)
                 display(MGR_OBJ_QUEUE, NULL, ss, TRUE);
               }
 
-            ss = pbs_statserver(sp -> s_connect, sa, NULL);
+            ss = pbs_statserver_err(sp -> s_connect, sa, NULL, &local_errno);
 
             break;
 
           case MGR_OBJ_QUEUE:
-            ss = pbs_statque(sp->s_connect, pname->obj_name, sa, NULL);
+
+            ss = pbs_statque_err(sp->s_connect, pname->obj_name, sa, NULL, &local_errno);
+
             break;
 
           case MGR_OBJ_NODE:
-            ss = pbs_statnode(sp->s_connect, pname->obj_name, sa, NULL);
+
+            ss = pbs_statnode_err(sp->s_connect, pname->obj_name, sa, NULL, &local_errno);
+
             break;
             /* DIAGTODO: handle new diag type for print */
           }
@@ -1557,13 +1485,14 @@ int execute(
         }
       else
         {
-        perr = pbs_manager(
+        perr = pbs_manager_err(
                  sp->s_connect,
                  oper,
                  type,
                  pname->obj_name,
                  attribs,
-                 NULL);
+                 NULL,
+                 &local_errno);
         }
 
       if (perr)
@@ -1588,16 +1517,9 @@ int execute(
             pstderr_big(pname -> obj_name, Svrname(sp), errmsg);
             }
           }
-        else if (pbs_errno == PBSE_PROTOCOL)
-          {
-          if (! zopt) fprintf(stderr, "qmgr: Protocol error, server disconnected\n");
-
-          exit(1);
-          }
         else
-          if (! zopt) fprintf(stderr, "qmgr: Error (%d - %s) returned from server\n",
-                                pbs_errno,
-                                pbs_strerror(pbs_errno));
+          if (! zopt)
+            fprintf(stderr, "qmgr: Error from server\n");
 
         if (aopt)
           return perr;
@@ -2732,7 +2654,8 @@ int is_valid_object(
     NULL, ATTR_NODE_state, "", "", 0
     };
 
-  int valid = 1;
+  int   local_errno = 0;
+  int   valid = 1;
   char *errmsg;
 
   if ((obj != NULL) && (obj->svr != NULL))
@@ -2742,13 +2665,13 @@ int is_valid_object(
 
       case MGR_OBJ_QUEUE:
 
-        batch_obj = pbs_statque(obj->svr->s_connect, obj->obj_name, &attrq, NULL);
+        batch_obj = pbs_statque_err(obj->svr->s_connect, obj->obj_name, &attrq, NULL, &local_errno);
 
         break;
 
       case MGR_OBJ_NODE:
 
-        batch_obj = pbs_statnode(obj->svr->s_connect, obj->obj_name, &attrn, NULL);
+        batch_obj = pbs_statnode_err(obj->svr->s_connect, obj->obj_name, &attrn, NULL, &local_errno);
 
         break;
 
