@@ -70,8 +70,10 @@
 #include "pbs_ifl.h"
 #endif
 #ifndef lONG_H
-#include "Long.h"
+#include "pbs_constants.h" /* Long */
 #endif
+#include "list_link.h"
+
 /*
  * This header file contains the definitions for attributes
  *
@@ -101,11 +103,13 @@
  * are created with default values.
  */
 
+#include <sys/time.h>
+#include <time.h>
+
 #ifndef ATTRIBUTE_H
 #define ATTRIBUTE_H 1
 
 /* define the size of fields in the structures */
-#include "list_link.h"
 
 #define ATRFLAG 16
 #define ATRTYPE  6  /* sync w/ATR_TYPE_*  (see #defines below) */
@@ -179,6 +183,7 @@ union attr_val    /* the attribute value */
 
   struct  pbsnode      *at_jinfo; /* ptr to node's job info  */
   short        at_short; /* short int; node's state */
+  struct timeval	at_timeval; 
   };
 
 
@@ -205,9 +210,9 @@ typedef struct attribute attribute;
 struct attribute_def
   {
   char *at_name;
-  int (*at_decode)(attribute *patr, char *name, char *rn, char *val);
+  int (*at_decode)(attribute *patr, char *name, char *rn, char *val, int perm);
   int (*at_encode)(attribute *pattr, tlist_head *phead,
-                      char *aname, char *rsname, int mode);
+                      char *aname, char *rsname, int mode, int perm);
   int (*at_set)(attribute *pattr, attribute *new, enum batch_op);
   int (*at_comp)(attribute *pattr, attribute *with);
   void (*at_free)(attribute *pattr);
@@ -238,6 +243,7 @@ typedef struct attribute_def attribute_def;
 #define ATR_TYPE_LL      9 /* Long (64 bit) integer */
 #define ATR_TYPE_SHORT   10 /* short integer    */
 #define ATR_TYPE_JINFOP  13 /* struct jobinfo*  */
+#define ATR_TYPE_TV		 14 /* struct timeval */
 
 /* Defines for  Flag field in attribute_def         */
 
@@ -347,48 +353,52 @@ extern int  attr_atomic_node_set(svrattrl *plist, attribute *old,
                                        int unkn, int privil, int *badattr);
 extern void attr_atomic_kill(attribute *temp, attribute_def *pdef, int);
 
-extern int  decode_b(attribute *patr, char *name, char *rn, char *val);
-extern int  decode_c(attribute *patr, char *name, char *rn, char *val);
-extern int  decode_l(attribute *patr, char *name, char *rn, char *val);
-extern int  decode_ll(attribute *patr, char *name, char *rn, char *val);
-extern int  decode_size(attribute *patr, char *name, char *rn, char *val);
-extern int  decode_str(attribute *patr, char *name, char *rn, char *val);
-extern int  decode_time(attribute *patr, char *name, char *rn, char *val);
-extern int  decode_arst(attribute *patr, char *name, char *rn, char *val);
-extern int  decode_arst_direct(attribute *patr, char *val);
-extern int  decode_arst_merge(attribute *,char *,char *,char *);
-extern int  decode_resc(attribute *patr, char *name, char *rn, char *val);
-extern int  decode_depend(attribute *patr, char *name, char *rn, char *val);
-extern int  decode_hold(attribute *patr, char *name, char *rn, char *val);
-extern int  decode_uacl(attribute *patr, char *name, char *rn, char *val);
-extern int  decode_unkn(attribute *patr, char *name, char *rn, char *val);
+int  decode_b(attribute *patr, char *name, char *rn, char *val, int);
+int  decode_c(attribute *patr, char *name, char *rn, char *val, int);
+int  decode_l(attribute *patr, char *name, char *rn, char *val, int);
+int  decode_ll(attribute *patr, char *name, char *rn, char *val, int);
+int  decode_size(attribute *patr, char *name, char *rn, char *val, int);
+int  decode_str(attribute *patr, char *name, char *rn, char *val, int);
+int  decode_time(attribute *patr, char *name, char *rn, char *val, int);
+int  decode_arst(attribute *patr, char *name, char *rn, char *val, int);
+int  decode_arst_direct(attribute *patr, char *val);
+int  decode_arst_merge(attribute *,char *,char *,char *);
+int  decode_resc(attribute *patr, char *name, char *rn, char *val, int);
+int  decode_depend(attribute *patr, char *name, char *rn, char *val, int);
+int  decode_hold(attribute *patr, char *name, char *rn, char *val, int);
+int  decode_uacl(attribute *patr, char *name, char *rn, char *val, int);
+int  decode_unkn(attribute *patr, char *name, char *rn, char *val, int);
+int  decode_tv(attribute *patr, char *name,  char *rescn,	char *val, int);
+ 
+int  encode_b(attribute *attr, tlist_head *phead, char *atname,
+                           char *rsname, int mode, int perm);
+int  encode_c(attribute *attr, tlist_head *phead, char *atname,
+                           char *rsname, int mode, int perm);
+int encode_l(attribute *attr, tlist_head *phead, char *atname,
+                           char *rsname, int mode, int perm);
+int encode_ll(attribute *attr, tlist_head *phead, char *atname,
+                           char *rsname, int mode, int perm);
+int encode_size(attribute *attr, tlist_head *phead, char *atname,
+                              char *rsname, int mode, int perm);
+int encode_str(attribute *attr, tlist_head *phead, char *atname,
+                             char *rsname, int mode, int perm);
+int encode_time(attribute *attr, tlist_head *phead, char *atname,
+                             char *rsname, int mode, int perm);
+int encode_arst(attribute *attr, tlist_head *phead, char *atname,
+                             char *rsname, int mode, int perm);
+int encode_resc(attribute *attr, tlist_head *phead, char *atname,
+                             char *rsname, int mode, int perm);
+int encode_inter(attribute *attr, tlist_head *phead, char *atname,
+                              char *rsname, int mode, int perm);
+int encode_unkn(attribute *attr, tlist_head *phead, char *atname,
+                             char *rsname, int mode, int perm);
+int encode_depend(attribute *attr, tlist_head *phead, char *atname,
+                               char *rsname, int mode, int perm);
+int encode_hold(attribute *attr, tlist_head *phead, char *atname,
+                             char *rsname, int mode, int perm);
+int encode_tv(attribute *attr, tlist_head *phead, char *atname,
+							char *rsname, int mode, int perm);
 
-extern int  encode_b(attribute *attr, tlist_head *phead, char *atname,
-                           char *rsname, int mode);
-extern int  encode_c(attribute *attr, tlist_head *phead, char *atname,
-                           char *rsname, int mode);
-extern int encode_l(attribute *attr, tlist_head *phead, char *atname,
-                           char *rsname, int mode);
-extern int encode_ll(attribute *attr, tlist_head *phead, char *atname,
-                           char *rsname, int mode);
-extern int encode_size(attribute *attr, tlist_head *phead, char *atname,
-                              char *rsname, int mode);
-extern int encode_str(attribute *attr, tlist_head *phead, char *atname,
-                             char *rsname, int mode);
-extern int encode_time(attribute *attr, tlist_head *phead, char *atname,
-                             char *rsname, int mode);
-extern int encode_arst(attribute *attr, tlist_head *phead, char *atname,
-                             char *rsname, int mode);
-extern int encode_resc(attribute *attr, tlist_head *phead, char *atname,
-                             char *rsname, int mode);
-extern int encode_inter(attribute *attr, tlist_head *phead, char *atname,
-                              char *rsname, int mode);
-extern int encode_unkn(attribute *attr, tlist_head *phead, char *atname,
-                             char *rsname, int mode);
-extern int encode_depend(attribute *attr, tlist_head *phead, char *atname,
-                               char *rsname, int mode);
-extern int encode_hold(attribute *attr, tlist_head *phead, char *atname,
-                             char *rsname, int mode);
 
 extern int set_b(attribute *attr, attribute *new, enum batch_op);
 extern int set_c(attribute *attr, attribute *new, enum batch_op);
@@ -402,6 +412,9 @@ extern int set_hostacl(attribute *attr, attribute *new, enum batch_op);
 extern int set_uacl(attribute *attr, attribute *new, enum batch_op);
 extern int set_unkn(attribute *attr, attribute *new, enum batch_op);
 extern int set_depend(attribute *attr, attribute *new, enum batch_op);
+extern int set_tv(struct attribute *attr, struct attribute *new, enum batch_op op);
+
+enum compare_types { LESS, EQUAL, GREATER, NOT_COMPARED };
 
 extern int comp_b(attribute *, attribute *);
 extern int comp_c(attribute *, attribute *);
@@ -411,10 +424,11 @@ extern int comp_size(attribute *, attribute *);
 extern int comp_str(attribute *, attribute *);
 extern int comp_arst(attribute *, attribute *);
 extern int comp_resc(attribute *, attribute *);
-extern int comp_resc2(attribute *, attribute *, int, char *);
+extern int comp_resc2(attribute *, attribute *, int, char *, enum compare_types);
 extern int comp_unkn(attribute *, attribute *);
 extern int comp_depend(attribute *, attribute *);
 extern int comp_hold(attribute *, attribute *);
+int comp_tv(struct attribute *attr, struct attribute *with);
 
 extern int action_depend(attribute *, void *, int);
 
@@ -424,7 +438,7 @@ extern void free_resc(attribute *);
 extern void free_depend(attribute *);
 extern void free_unkn(attribute *);
 extern int   parse_equal_string(char *, char **, char **);
-extern char *parse_comma_string(char *);
+extern char *parse_comma_string(char *,char **);
 
 #define NULL_FUNC (int (*)())0
 
@@ -432,25 +446,26 @@ extern char *parse_comma_string(char *);
 struct dynamic_string;
 
 extern int   acl_check(attribute *, char *canidate, int type);
+int          acl_check_my_array_string(struct array_strings *, char *, int);
 extern char *arst_string(char *str, attribute *pattr);
 extern void  attrl_fixlink(tlist_head *svrattrl);
 extern void  recov_acl(attribute *, attribute_def *, char *, char *);
 extern int   save_acl(attribute *, attribute_def *,  char *, char *);
-extern int   save_attr(attribute_def *, attribute *, int);
-extern int   save_attr_xml(attribute_def *,attribute *,int,int);
+extern int   save_attr(attribute_def *, attribute *, int, int, char *, size_t *, size_t);
+extern int   save_attr_xml(attribute_def *, attribute *, int, int);
 extern int   write_buffer(char *,int,int);
 extern int   size_to_str(struct size_value,char *,int);
 extern int   attr_to_str(struct dynamic_string *ds, attribute_def *,struct attribute,int);
 extern int   str_to_attr(char *,char *,struct attribute *,struct attribute_def *);
 
-extern int      encode_state(attribute *, tlist_head *, char *, char *, int);
-extern int      encode_props(attribute*, tlist_head*, char*, char*, int);
-extern int      encode_jobs(attribute*, tlist_head*, char*, char*, int);
-extern int      encode_ntype(attribute*, tlist_head*, char*, char*, int);
-extern int      decode_state(attribute*, char*, char*, char*);
-extern int      decode_props(attribute*, char*, char*, char*);
-extern int      decode_ntype(attribute*, char*, char*, char*);
-extern int      decode_null(attribute*, char*, char*, char*);
+extern int      encode_state(attribute *, tlist_head *, char *, char *, int, int);
+extern int      encode_props(attribute*, tlist_head*, char*, char*, int, int);
+extern int      encode_jobs(attribute*, tlist_head*, char*, char*, int, int);
+extern int      encode_ntype(attribute*, tlist_head*, char*, char*, int, int);
+extern int      decode_state(attribute*, char*, char*, char*, int);
+extern int      decode_props(attribute*, char*, char*, char*, int);
+extern int      decode_ntype(attribute*, char*, char*, char*, int);
+extern int      decode_null(attribute*, char*, char*, char*, int);
 extern int      comp_null(attribute*, attribute*);
 extern int      count_substrings(char*, int*);
 extern int      set_node_state(attribute*, attribute*, enum batch_op);
@@ -474,15 +489,19 @@ extern int      node_alt_name(attribute*, void*, int);
 extern int      set_note_str(attribute *attr, attribute *new, enum batch_op);
 extern int      set_alt_name_str(attribute *attr, attribute *new, enum batch_op);
 extern void     replace_attr_string(attribute*, char*);
+extern int 		job_radix_action (attribute *new, void *pobj, int actmode);
 
 /* Token manipulation functions */
 
-extern int  decode_tokens(attribute *, char *, char *, char *);
+extern int  decode_tokens(attribute *, char *, char *, char *, int);
+
+int timeval_subtract(struct timeval *result, struct timeval *x, struct timeval *y);
 
 /* "type" to pass to acl_check() */
-#define ACL_Host  1
-#define ACL_User  2
-#define ACL_Group 3
-#define ACL_Gid   4
+#define ACL_Host      1
+#define ACL_User      2
+#define ACL_Group 	  3
+#define ACL_Gid   	  4
+#define ACL_User_Host 5
 
 #endif  /* ATTRIBUTE_H */
