@@ -1,3 +1,5 @@
+#ifndef _DIS_INIT_H
+#define _DIS_INIT_H
 /*
 *         OpenPBS (Portable Batch System) v2.3 Software License
 *
@@ -78,14 +80,15 @@
 */
 
 /*
- * @(#) $Id: dis_init.h 3230 2009-11-05 00:12:42Z ataufer $
+ * @(#) $Id: dis_init.h 4975 2011-08-12 15:19:01Z jrosenquist $
  */
 
-
+/*
 extern int (*dis_getc) (int stream);
 extern int (*dis_puts) (int stream, const char *string, size_t count);
 extern int (*dis_gets) (int stream, char *string, size_t count);
 extern int (*disr_skip) (int stream, size_t nskips);
 extern int (*disw_commit) (int stream, int commit);
 extern int (*disr_commit) (int stream, int commit);
-
+*/
+#endif /* _DIS_INIT_H */
