@@ -78,7 +78,7 @@
 */
 
 /*
- * @(#) $Id: libpbs.h 4990 2011-08-19 19:28:51Z knielson $
+ * @(#) $Id: libpbs.h 5712 2012-02-07 18:08:09Z ataufer $
  */
 
 /* libpbs.h
@@ -96,11 +96,12 @@
 #ifndef _MEMORY_H
 #include <memory.h>
 #endif /* _MEMORY_H */
+#include <pthread.h>
 
 #include "pbs_ifl.h"
 #include "list_link.h"
 #include "pbs_error.h"
-#include "server_limits.h" /* Pulls in PBS_NET_MAX_CONNECTIONS */
+#include "u_memmgr.h"
 
 #ifndef LIBPBS_H
 #define LIBPBS_H
@@ -109,7 +110,7 @@
 #define PBS_BATCH_PROT_VER 2
 /* #define PBS_REQUEST_MAGIC (56) */
 /* #define PBS_REPLY_MAGIC   (57) */
-#define SCRIPT_CHUNK_Z (4096)
+#define SCRIPT_CHUNK_Z (65536)
 #ifndef TRUE
 #define TRUE 1
 #define FALSE 0
@@ -132,13 +133,14 @@ enum job_file
   Checkpoint
   };
 
-#define MH(type) (type *)malloc(sizeof(type))
+#define MH(type) (type *)calloc(1, sizeof(type))
 #define M(var,type) if( (var = MH(type)) == (type *)NULL ) \
       { return PBSE_SYSTEM; }
 #define STRLEN(s) ((s==(char *)NULL)?0:strlen(s))
 #define Str2QB(s) ((s==(char *)NULL)?(struct qbuf *)NULL:str2qb(s,strlen(s),0))
 #define QB2Str(q) ((q==(struct qbuf *)NULL)?(char *)NULL:qb2str(q))
 
+extern int pbs_errno;
 
 extern char pbs_current_user[];
 extern char pbs_current_group[];
@@ -146,7 +148,7 @@ extern char pbs_current_group[];
 /* set ENABLE_TRUSTED_AUTH to "TRUE" to bypass using pbs_iff for authentication */
 #define ENABLE_TRUSTED_AUTH FALSE
 
-#define NCONNECTS PBS_NET_MAX_CONNECTIONS
+#define NCONNECTS 10
 
 struct connect_handle
   {
@@ -155,6 +157,7 @@ struct connect_handle
   void   *ch_stream;
   int ch_errno; /* last error on this connection */
   char *ch_errtxt; /* pointer to last server error text */
+  pthread_mutex_t *ch_mutex;
   };
 
 extern struct connect_handle connection[];
@@ -199,15 +202,15 @@ struct brp_rescq              /* reply to Resource Query Request */
  * the following is the basic Batch Reply structure
  */
 
-#define BATCH_REPLY_CHOICE_NULL  1 /* no reply choice, just code */
-#define BATCH_REPLY_CHOICE_Queue 2 /* Job ID, see brp_jid   */
-#define BATCH_REPLY_CHOICE_RdytoCom 3 /* select, see brp_jid    */
-#define BATCH_REPLY_CHOICE_Commit 4 /* commit, see brp_jid   */
-#define BATCH_REPLY_CHOICE_Select 5 /* select, see brp_select */
-#define BATCH_REPLY_CHOICE_Status 6 /* status, see brp_status */
-#define BATCH_REPLY_CHOICE_Text  7 /* text,   see brp_txt   */
-#define BATCH_REPLY_CHOICE_Locate 8 /* locate, see brp_locate */
-#define BATCH_REPLY_CHOICE_RescQuery 9 /* Resource Query         */
+#define BATCH_REPLY_CHOICE_NULL      1  /* no reply choice, just code */
+#define BATCH_REPLY_CHOICE_Queue     2  /* Job ID, see brp_jid   */
+#define BATCH_REPLY_CHOICE_RdytoCom  3  /* select, see brp_jid    */
+#define BATCH_REPLY_CHOICE_Commit    4  /* commit, see brp_jid   */
+#define BATCH_REPLY_CHOICE_Select    5  /* select, see brp_select */
+#define BATCH_REPLY_CHOICE_Status    6  /* status, see brp_status */
+#define BATCH_REPLY_CHOICE_Text      7  /* text,   see brp_txt   */
+#define BATCH_REPLY_CHOICE_Locate    8  /* locate, see brp_locate */
+#define BATCH_REPLY_CHOICE_RescQuery 9  /* Resource Query         */
 
 struct batch_reply
   {
@@ -256,6 +259,8 @@ enum PBatchReqTypeEnum
 #define PBS_credentialtype_none 0
 const char *reqtype_to_txt(int);
 
+void initialize_connections_table();
+
 void
 PBS_free_aopl (struct attropl * aoplp);
 
@@ -272,8 +277,7 @@ PBSD_jscript (int connect, char *script_file, char *jobid);
 int
 PBSD_mgr_put (int connect, int func, int cmd, int objtype, char *objname, struct attropl *al, char *extend);
 
-int
-PBSD_manager  (int connect, int func, int cmd, int objtype, char *objname, struct attropl *al, char *extend);
+int PBSD_manager(int connect, int func, int cmd, int objtype, char *objname, struct attropl *al, char *extend_err, int *);
 
 int
 PBSD_msg_put (int connect, char *jobid, int fileopt, char *msg, char *extend);
@@ -299,15 +303,16 @@ PBSD_jobfile (int connect, int req_type, char *path, char *jobid, enum job_file
 int
 PBSD_status_put (int c, int func, char *id, struct attrl *attrib, char *extend);
 
-struct batch_reply *PBSD_rdrpy (int connect);
+struct batch_reply *PBSD_rdrpy(int *local_errno, int connect);
 
 void PBSD_FreeReply (struct batch_reply *);
 
-struct batch_status *PBSD_status (int c, int function, char *id, struct attrl *attrib, char *extend);
+struct batch_status *PBSD_status(int c, int function, int *, char *id, struct attrl *attrib, char *extend);
 
-struct batch_status *PBSD_status_get (int c);
+struct batch_status *PBSD_status_get(int *local_errno, int c);
 
-char * PBSD_queuejob (int c, char *j, char *d, struct attropl *a, char *ex);
+char *PBSD_queuejob (int c, int *, char *j, char *d, struct attropl *a, char *ex);
+int PBSD_QueueJob_hash(int c, char *j, char *d, memmgr **mm, job_data *ja, job_data *ra, char *ex, char **job_id, char **msg);
 
 
 extern int decode_DIS_JobId (int socket, char *jobid);
@@ -321,6 +326,7 @@ extern int encode_DIS_Manage (int socket, int cmd, int objt, char *, struct attr
 extern int encode_DIS_MoveJob (int socket, char *jid, char *dest);
 extern int encode_DIS_MessageJob (int socket, char *jid, int fopt, char *m);
 extern int encode_DIS_QueueJob (int socket, char *jid, char *dest, struct attropl *);
+int encode_DIS_QueueJob_hash(int socket, char *jid, char *destin, memmgr **mm, job_data *job_attr, job_data *res_attr);
 extern int encode_DIS_ReqExtend (int socket, char *extend);
 extern int encode_DIS_ReqHdr (int socket, int reqt, char *user);
 extern int encode_DIS_Rescq (int socket, char **rlist, int num);
@@ -330,6 +336,7 @@ extern int encode_DIS_SignalJob (int socket, char *jid, char *sig);
 extern int encode_DIS_Status (int socket, char *objid, struct attrl *);
 extern int encode_DIS_attrl (int socket, struct attrl *);
 extern int encode_DIS_attropl (int socket, struct attropl *);
+int encode_DIS_attropl_hash(int socket, memmgr **mm, job_data *job_attr, job_data *res_attr);
 
 extern int DIS_reply_read (int socket, struct batch_reply *preply);
 #endif /* LIBPBS_H */
