@@ -1,3 +1,9 @@
+#ifndef _NET_CONNECT_H
+#define _NET_CONNECT_H
+#include <stdlib.h>
+#include <pthread.h> /* pthread_mutex_t */
+
+
 /*
 *         OpenPBS (Portable Batch System) v2.3 Software License
 *
@@ -100,16 +106,16 @@ typedef unsigned long pbs_net_t;        /* for holding host addresses */
 */
 
 #define RM_PROTOCOL 1 /* resource monitor protocol number */
-#define RM_PROTOCOL_VER 1 /* resmon protocol version number */
+#define RM_PROTOCOL_VER 2 /* resmon protocol version number */
 
-#define TM_PROTOCOL 2 /* task manager protocol number */
-#define TM_PROTOCOL_VER 1 /* task manager protocol version number */
+#define TM_PROTOCOL 5 /* task manager protocol number */
+#define TM_PROTOCOL_VER 2 /* task manager protocol version number */
 
 #define IM_PROTOCOL 3 /* inter-manager protocol number */
-#define IM_PROTOCOL_VER 1 /* inter-manager protocol version number */
+#define IM_PROTOCOL_VER 2 /* inter-manager protocol version number */
 
 #define IS_PROTOCOL 4 /* inter-server protocol number */
-#define IS_PROTOCOL_VER 1 /* inter-server protocol version number */
+#define IS_PROTOCOL_VER 3 /* inter-server protocol version number */
 
 
 /*
@@ -118,12 +124,17 @@ typedef unsigned long pbs_net_t;        /* for holding host addresses */
 
 /* sync w/PBSServerCmds[] in resmom/mom_main.c and PBSServerCmds2[] in server/node_manager.c */
 
-#define IS_NULL   			0
-#define IS_HELLO  			1
-#define IS_CLUSTER_ADDRS 	2
-#define IS_UPDATE  			3
-#define IS_STATUS           4
-#define IS_GPU_STATUS		5
+#define IS_NULL           0
+#define IS_HELLO          1
+#define IS_CLUSTER_ADDRS  2
+#define IS_UPDATE         3
+#define IS_STATUS         4
+#define IS_GPU_STATUS     5
+
+
+/* tell pbs_mom the direction of the hello */
+#define TO_SERVER         0
+#define FROM_SERVER       1
 
 
 /* return codes for client_to_svr() */
@@ -137,6 +148,9 @@ typedef unsigned long pbs_net_t;        /* for holding host addresses */
 #define TSOCK_PATH "/tmp/.torque-unix"
 #endif
 
+/* End of message delineator for IS protocol */
+#define IS_EOL_MESSAGE "END_OF_LINE"
+
 #ifndef SCM_CREDS
 #define SCM_CREDS SCM_CREDENTIALS
 #endif
@@ -177,19 +191,18 @@ enum conn_type
 
 /* functions available in libnet.a */
 
-void add_conn(int, enum conn_type, pbs_net_t, unsigned int, unsigned int, void (*func)(int));
-int  find_conn(pbs_net_t);
+int add_conn(int, enum conn_type, pbs_net_t, unsigned int, unsigned int, void *(*func)(void *));
 int  client_to_svr(pbs_net_t, unsigned int, int, char *);
-void close_conn(int);
-pbs_net_t get_connectaddr(int);
+void close_conn(int,int);
+pbs_net_t get_connectaddr(int,int);
 int  get_connecthost(int sock, char *, int);
-pbs_net_t get_hostaddr(char *);
+pbs_net_t get_hostaddr(int *, char *);
 int  get_fullhostname(char *, char *, int, char *);
 unsigned int  get_svrport(char *, char *, unsigned int);
-int  init_network(unsigned int, void (*readfunc)());
+int  init_network(unsigned int, void *(*readfunc)(void *));
 void net_close(int);
 int  wait_request(time_t waittime, long *);
-void net_add_close_func(int, void(*)());
+void net_add_close_func(int, void(*func)(int), int);
 int get_max_num_descriptors(void);
 int get_fdset_size(void);
 char * netaddr_pbs_net_t(pbs_net_t);
@@ -204,8 +217,9 @@ struct connection
   unsigned short cn_socktype; /* authentication flags */
   enum conn_type cn_active;     /* idle or type if active */
   time_t cn_lasttime;    /* time last active */
-  void (*cn_func)(int);  /* read function when data rdy */
+  void *(*cn_func)(void *);  /* read function when data rdy */
   void (*cn_oncl)(int);  /* func to call on close */
+  pthread_mutex_t *cn_mutex;
   };
 
 struct netcounter
@@ -225,3 +239,4 @@ typedef struct listener_connection
   int first_time;
   } listener_connection;
 
+#endif /* _NET_CONNECT_H */
