@@ -85,6 +85,8 @@
 
 #ifndef _PBS_IFL_DEF
 #define _PBS_IFL_DEF
+#include "u_hash_map_structs.h"
+#include "u_memmgr.h"
 
 #include <sys/socket.h>
 
@@ -120,6 +122,21 @@
 #define ATTR_jobtype  "jobtype"
 #define ATTR_submit_host "submit_host"
 #define ATTR_init_work_dir "init_work_dir"
+/* The following were part of the variable_list */
+#define ATTR_pbs_o_initdir          "PBS_O_INITDIR"
+#define ATTR_pbs_o_rootdir          "PBS_O_ROOTDIR"
+#define ATTR_pbs_o_workdir          "PBS_O_WORKDIR" /* same as init_work_dir */
+#define ATTR_pbs_o_host             "PBS_O_HOST" /* same as submit_host */
+#define ATTR_pbs_o_server           "PBS_O_SERVER"
+#define ATTR_pbs_o_home             "PBS_O_HOME"
+#define ATTR_pbs_o_logname          "PBS_O_LOGNAME"
+#define ATTR_pbs_o_path             "PBS_O_PATH"
+#define ATTR_pbs_o_mail             "PBS_O_MAIL"
+#define ATTR_pbs_o_shell            "PBS_O_SHELL"
+#define ATTR_pbs_o_tz               "PBS_O_TZ"
+#define ATTR_pbs_o_lang             "PBS_O_LANG"
+#define ATTR_pbs_o_uid              "PBS_O_UID"
+#define ATTR_pbs_o_submit_filter    "PBS_O_SUBMIT_FILTER"
 
 /* additional job and general attribute names */
 
@@ -165,7 +182,10 @@
 #define ATTR_restart_name	"restart_name"   /* saved name of checkpoint restart file */
 #define ATTR_comp_time   	"comp_time"
 #define ATTR_reported    	"reported"
-#define ATTR_intcmd      "inter_cmd"
+#define ATTR_intcmd      	"inter_cmd"
+#define ATTR_job_radix		"job_radix"
+#define ATTR_sister_list	"sister_list"
+#define ATTR_total_runtime  "total_runtime"
 #define ATTR_P           "proxy_user"
 #define ATTR_node_exclusive "node_exclusive"
 #define ATTR_exec_gpus   "exec_gpus"
@@ -299,24 +319,27 @@
 #ifdef MUNGE_AUTH
   #define ATTR_authusers       "authorized_users"
 #endif
+#define ATTR_minthreads        "min_threads"
+#define ATTR_maxthreads        "max_threads"
+#define ATTR_threadidleseconds "thread_idle_seconds"
 #define ATTR_moabarraycompatible "moab_array_compatible"
 #define ATTR_nomailforce       "no_mail_force"
 /* additional node "attributes" names */
 
-#define ATTR_NODE_state          "state"
-#define ATTR_NODE_np             "np"
-#define ATTR_NODE_properties     "properties"
-#define ATTR_NODE_ntype          "ntype"
-#define ATTR_NODE_jobs           "jobs"
-#define ATTR_NODE_status         "status"
-#define ATTR_NODE_note           "note"
-#define ATTR_NODE_mom_port	     "mom_service_port"
-#define ATTR_NODE_mom_rm_port    "mom_manager_port" 
-#define ATTR_NODE_num_numa_nodes "num_numa_nodes"
-#define ATTR_NODE_numa_str       "numa_node_str"
-#define ATTR_NODE_gpus           "gpus"
-#define ATTR_NODE_gpustatus      "gpu_status"
-#define ATTR_NODE_gpus_str       "numa_gpu_node_str"
+#define ATTR_NODE_state            "state"
+#define ATTR_NODE_np               "np"
+#define ATTR_NODE_properties       "properties"
+#define ATTR_NODE_ntype            "ntype"
+#define ATTR_NODE_jobs             "jobs"
+#define ATTR_NODE_status           "status"
+#define ATTR_NODE_note             "note"
+#define ATTR_NODE_mom_port	       "mom_service_port"
+#define ATTR_NODE_mom_rm_port      "mom_manager_port" 
+#define ATTR_NODE_num_node_boards  "num_node_boards"
+#define ATTR_NODE_numa_str         "numa_board_str"
+#define ATTR_NODE_gpus             "gpus"
+#define ATTR_NODE_gpustatus        "gpu_status"
+#define ATTR_NODE_gpus_str         "numa_gpu_node_str"
 
 /* notification email formating */
 #define ATTR_mailsubjectfmt "mail_subject_fmt"
@@ -330,15 +353,17 @@
 #define NO_JOIN "n"
 #define NO_KEEP "n"
 #define MAIL_AT_ABORT "a"
+#define DEFAULT_PRIORITY "0"
 
 
-#define ARRAY_RANGE "array_range=" /* see qdel.c */
-#define DELDELAY  "deldelay=" /* see qdel.c */
-#define DELPURGE  "delpurge="   /* see qdel.c */
-#define DELASYNC  "delasync"   /* see req_delete.c */
-#define PURGECOMP  "purgecomplete="   /* see req_delete.c */
+
+#define ARRAY_RANGE  "array_range=" /* see qdel.c */
+#define DELDELAY     "deldelay=" /* see qdel.c */
+#define DELPURGE     "delpurge="   /* see qdel.c */
+#define DELASYNC     "delasync"   /* see req_delete.c */
+#define PURGECOMP    "purgecomplete="   /* see req_delete.c */
 #define EXECQUEONLY  "exec_queue_only"   /* see req_stat.c */
-#define RERUNFORCE "force"
+#define RERUNFORCE   "force"
 
 #define USER_HOLD   "u"
 #define OTHER_HOLD  "o"
@@ -377,7 +402,7 @@
 #define Q_DT_job_array          "job_array"
 
 /*constant related to sum of string lengths for above strings*/
-#define MAX_ENCODE_BFR  100
+#define MAX_ENCODE_BFR  150
 
 #define MGR_CMD_CREATE 0
 #define MGR_CMD_DELETE 1
@@ -467,10 +492,6 @@
 
 #define PBS_BATCH_SERVICE_NAME_DIS "pbs_dis" /* new DIS port   */
 
-#ifndef PBS_BATCH_SERVICE_PORT_DIS
-#define PBS_BATCH_SERVICE_PORT_DIS 15001  /* new DIS port   */
-#endif /* PBS_BATCH_SERVICE_PORT_DIS */
-
 #define PBS_MOM_SERVICE_NAME  "pbs_mom"
 
 #ifndef PBS_MOM_SERVICE_PORT
@@ -489,6 +510,10 @@
 #define PBS_SCHEDULER_SERVICE_PORT 15004
 #endif /* PBS_SCHEDULER_SERVICE_PORT */
 
+#ifndef TRQ_AUTHD_SERVICE_PORT
+#define TRQ_AUTHD_SERVICE_PORT 15005
+#endif /* TRQ_AUTHD_SERVICE_PORT */
+
 /*
  * Other misc checkpoint defines
  */
@@ -544,15 +569,16 @@ typedef int resource_t;  /* resource reservation handle */
 #define RESOURCE_T_NULL  (resource_t)0
 #define RESOURCE_T_ALL  (resource_t)-1
 
-extern int pbs_errno;  /* error number */
-
 extern char *pbs_server; /* server attempted to connect | connected to */
 /* see pbs_connect(3B)         */
 
 extern char *avail(int connect, char *resc);
 extern int pbs_asyrunjob(int c, char *jobid, char *location, char *extend);
+extern int pbs_asyrunjob_err(int c, char *jobid, char *location, char *extend, int *);
 extern int pbs_alterjob_async(int connect, char *job_id, struct attrl *attrib, char *extend);
+extern int pbs_alterjob_async_err(int connect, char *job_id, struct attrl *attrib, char *extend, int *);
 extern int pbs_alterjob(int connect, char *job_id, struct attrl *attrib, char *extend);
+extern int pbs_alterjob_err(int connect, char *job_id, struct attrl *attrib, char *extend, int *);
 extern int pbs_connect(char *server);
 extern int pbs_query_max_connections();
 extern char *pbs_default(void);
@@ -560,25 +586,27 @@ extern char *pbs_fbserver(void);
 extern char *pbs_get_server_list(void);
 
 extern int pbs_deljob(int connect, char *job_id, char *extend);
+extern int pbs_deljob_err(int connect, char *job_id, char *extend, int *);
 extern int pbs_disconnect(int connect);
 extern char *pbs_geterrmsg(int connect);
 extern int pbs_holdjob(int connect, char *job_id, char *hold_type, char *extend);
+extern int pbs_holdjob_err(int connect, char *job_id, char *hold_type, char *extend, int *);
 extern int pbs_checkpointjob(int connect, char *job_id, char *extend);
+extern int pbs_checkpointjob_err(int connect, char *job_id, char *extend, int *);
 extern char *pbs_locjob(int connect, char *job_id, char *extend);
+extern char *pbs_locjob_err(int connect, char *job_id, char *extend, int *);
 
-extern int
-  pbs_manager(int connect, int command, int obj_type, char *obj_name,
-                  struct attropl *attrib, char *extend);
+int pbs_manager(int connect, int command, int obj_type, char *obj_name, struct attropl *attrib, char *extend);
+int pbs_manager_err(int connect, int command, int obj_type, char *obj_name, struct attropl *attrib, char *extend, int *);
 
-extern int
-  pbs_movejob(int connect, char *job_id, char *destination, char *extend);
+int pbs_movejob(int connect, char *job_id, char *destination, char *extend);
+int pbs_movejob_err(int connect, char *job_id, char *destination, char *extend, int *);
 
-extern int
-  pbs_msgjob(int connect, char *job_id, int file, char *message,
-                 char *extend);
+int pbs_msgjob(int connect, char *job_id, int file, char *message, char *extend);
+int pbs_msgjob_err(int connect, char *job_id, int file, char *message, char *extend, int *);
 
-extern int
-  pbs_orderjob(int connect, char *job1, char *job2, char *extend);
+int pbs_orderjob(int connect, char *job1, char *job2, char *extend);
+int pbs_orderjob_err(int connect, char *job1, char *job2, char *extend, int *);
 
 extern int
   pbs_rescquery(int connect, char **rlist, int nresc, int *avail,
@@ -590,23 +618,23 @@ extern int
 extern int
   pbs_rescrelease(int connect, resource_t rhandle);
 
-extern int
-  pbs_rerunjob(int connect, char *job_id, char *extend);
+int pbs_rerunjob(int connect, char *job_id, char *extend);
+int pbs_rerunjob_err(int connect, char *job_id, char *extend, int *);
 
-extern int
-  pbs_rlsjob(int connect, char *job_id, char *hold_type, char *extend);
+int pbs_rlsjob(int connect, char *job_id, char *hold_type, char *extend);
+int pbs_rlsjob_err(int connect, char *job_id, char *hold_type, char *extend, int *);
 
-extern int
-  pbs_runjob(int connect, char *jobid, char *loc, char *extend);
+int pbs_runjob(int connect, char *jobid, char *loc, char *extend);
+int pbs_runjob_err(int connect, char *jobid, char *loc, char *extend, int *);
 
-extern char **
-  pbs_selectjob(int connect, struct attropl *select_list, char *extend);
+char **pbs_selectjob(int connect, struct attropl *select_list, char *extend);
+char **pbs_selectjob_err(int connect, struct attropl *select_list, char *extend, int *);
 
-extern int
-  pbs_sigjob(int connect, char *job_id, char *signal, char *extend);
+int pbs_sigjob(int connect, char *job_id, char *signal, char *extend);
+int pbs_sigjob_err(int connect, char *job_id, char *signal, char *extend, int *);
 
-extern int
-  pbs_sigjobasync (int connect, char *job_id, char *signal, char *extend);
+int pbs_sigjobasync(int connect, char *job_id, char *signal, char *extend);
+int pbs_sigjobasync_err(int connect, char *job_id, char *signal, char *extend, int *);
 
 extern void
   pbs_statfree(struct batch_status *stat);
@@ -614,38 +642,47 @@ extern void
 extern struct batch_status *
         pbs_statdest(int connect, char *id, char *extend);
 
-extern struct batch_status *
-        pbs_statjob(int connect, char *id, struct attrl *attrib, char *extend);
+struct batch_status *pbs_statjob(int connect, char *id, struct attrl *attrib, char *extend);
+struct batch_status *pbs_statjob_err(int connect, char *id, struct attrl *attrib, char *extend, int *);
 
-extern struct batch_status *
-        pbs_selstat(int connect, struct attropl *select_list, char *extend);
+struct batch_status *pbs_selstat(int connect, struct attropl *select_list, char *extend);
+struct batch_status *pbs_selstat_err(int connect, struct attropl *select_list, char *extend, int *);
 
-extern struct batch_status *
-        pbs_statque(int connect, char *id, struct attrl *attrib, char *extend);
+struct batch_status *pbs_statque(int connect, char *id, struct attrl *attrib, char *extend);
+struct batch_status *pbs_statque_err(int connect, char *id, struct attrl *attrib, char *extend, int *);
 
-extern struct batch_status *
-        pbs_statserver(int connect, struct attrl *attrib, char *extend);
+struct batch_status *pbs_statserver(int connect, struct attrl *attrib, char *extend);
+struct batch_status *pbs_statserver_err(int connect, struct attrl *attrib, char *extend, int *);
 
-extern struct batch_status *
-        pbs_statnode(int connect, char *id, struct attrl *attrib, char *extend);
+struct batch_status *pbs_statnode(int connect, char *id, struct attrl *attrib, char *extend);
+struct batch_status *pbs_statnode_err(int connect, char *id, struct attrl *attrib, char *extend, int *);
 
-extern char *
-  pbs_submit(int connect, struct attropl *attrib, char *script,
-                 char *destination, char *extend);
+char *pbs_submit(int connect, struct attropl *attrib, char *script, char *destination, char *extend);
+char *pbs_submit_err(int connect, struct attropl *attrib, char *script, char *destination, char *extend, int *);
 
-extern int
-  pbs_terminate(int connect, int manner, char *extend);
+int pbs_submit_hash(int connect, memmgr **mm, job_data *job_attr, job_data *res_attr, char *script, char *destination, char *extend, char **job_id, char **msg);
 
-extern int
-  totpool(int connect, int update);
+int pbs_terminate(int connect, int manner, char *extend);
+int pbs_terminate_err(int connect, int manner, char *extend, int *);
 
-extern int
-  usepool(int connect, int update);
+int totpool(int connect, int update);
 
-extern int pbs_gpumode(int connect, char *node, char *gpuid, int gpumode);
+int usepool(int connect, int update);
 
-extern int trq_set_preferred_network_interface(char *if_name, struct sockaddr *preferred_addr);
-extern char *trq_get_if_name(void);
+int pbs_gpumode(int connect, char *node, char *gpuid, int gpumode);
+int pbs_gpumode_err(int connect, char *node, char *gpuid, int gpumode, int *);
+
+int trq_set_preferred_network_interface(char *if_name, struct sockaddr *preferred_addr);
+char *trq_get_if_name(void);
+
+/* used by Moab */
+int closerm(int);
+char *getreq(int);
+unsigned int get_svrport(char *, char *, unsigned int);
+int openrm(char *, unsigned int);
+int addreq(int, char *);
+int pbs_stagein(int, char *, char *, char *);
+int pbs_stageout(int, char *, char *, char *);
 
 #endif /* _PBS_IFL_DEF */
 
