@@ -81,16 +81,26 @@
  * job.h - structure definations for job objects
  *
  * Include Files Required:
- * <sys/types.h>
  * "list_link.h"
  * "attribute.h"
  * "server_limits.h"
  */
 
+
 #ifndef PBS_JOB_H
 #define PBS_JOB_H 1
 
+#include <limits.h>
 #include "server_limits.h"
+#include "list_link.h"
+#include "pbs_ifl.h"
+#include "attribute.h"
+#include "resizable_array.h"
+#include "hash_table.h"
+#include "mom_hierarchy.h"
+#include "dynamic_string.h"
+
+#define SAVEJOB_BUF_SIZE 8192
 
 #ifndef MAX_LINE
 #define MAX_LINE 1024
@@ -342,6 +352,9 @@ enum job_atr
   JOB_ATR_reported, /* tracks whether job has been reported to scheduler */
   JOB_ATR_jobtype,     /* opaque job type string */
   JOB_ATR_inter_cmd,      /* command for interactive job */
+  JOB_ATR_job_radix,    /* 64 */
+  JOB_ATR_total_runtime, /* the difference between JOB_ATR_comp_time and JOB_ATR_start_time */
+  JOB_ATR_sister_list,  /* For job_radix. List of sisters in a particular radix */
   JOB_ATR_proxy_user,
   JOB_ATR_node_exclusive,
   JOB_ATR_submit_host,  /* host which submitted the job */
@@ -349,10 +362,10 @@ enum job_atr
 #ifdef USEJOBCREATE
   JOB_ATR_pagg_id,
 #endif /* USEJOBCREATE */
-  JOB_ATR_job_id,
 #ifdef NVIDIA_GPUS
   JOB_ATR_gpu_flags,		/* gpu flags - mode and reset flags */
 #endif  /* NVIDIA_GPUS */
+  JOB_ATR_job_id,
   JOB_ATR_arguments,
 #include "site_job_attr_enum.h"
 
@@ -377,12 +390,13 @@ extern attribute_def job_attr_def[];
 
 typedef struct hnodent
   {
-  int  hn_node; /* host (node) identifier (index) */
-  char *hn_host; /* hostname of node */
-  int  hn_stream; /* stream to MOM on node */
-  int  hn_sister; /* save error for KILL_JOB event */
-  unsigned short hn_port; /*  resmom port default 15003 */
-  tlist_head hn_events; /* pointer to list of events */
+  int                 hn_node; /* host (node) identifier (index) */
+  char               *hn_host; /* hostname of node */
+  int                 hn_stream; /* stream to MOM on node */
+  int                 hn_sister; /* save error for KILL_JOB event */
+  unsigned short      hn_port; /*  resmom port default 15003 */
+  tlist_head          hn_events; /* pointer to list of events */
+  struct sockaddr_in  sock_addr;
   } hnodent;
 
 
@@ -497,11 +511,10 @@ struct job
   /* Note: these members, up to ji_qs, are not saved to disk
             (except for ji_stdout, ji_stderr) */
 
-  list_link       ji_alljobs; /* link to next job in server job list */
-  list_link       ji_jobque;  /* SVR: link to next job in queue list */
 #ifndef PBS_MOM
-  list_link       ji_jobs_array_sum; /*linked list of jobs with arrays summarized as a single "placeholder" job */
   list_link       ji_jobque_array_sum;
+#else
+  list_link       ji_jobque;  /* used for polling in mom */
 #endif
   /* MOM: links to polled jobs */
   time_t  ji_momstat; /* SVR: time of last status from MOM */
@@ -511,6 +524,7 @@ struct job
   int  ji_radix;    /* number of nodes in a job radix. used for qsub -W job_radix option  */
 #ifdef PBS_MOM    /* MOM ONLY */
 
+  list_link       ji_alljobs; /* link to next job in server job list */
   struct grpcache *ji_grpcache; /* cache of user's groups */
   time_t  ji_checkpoint_time; /* periodic checkpoint time */
   time_t  ji_checkpoint_next; /* next checkpoint time */
@@ -532,7 +546,7 @@ struct job
   hnodent        *ji_hosts; /* ptr to job host management stuff */
   hnodent        *ji_sisters; /* ptr to job host management stuff for intermediate moms */
   vnodent        *ji_vnods; /* ptr to job vnode management stuff */
-  vnodent        *ji_sister_vnods; /* ptr to job vnode management stuff */
+  vnodent        *ji_sister_vnods; /* ptr to job vnode management stuff for job_radix requests*/
   noderes        *ji_resources; /* ptr to array of node resources */
   tlist_head     ji_tasks; /* list of task structs */
   tm_node_id     ji_nodekill; /* set to nodeid requesting job die */
@@ -546,23 +560,27 @@ struct job
   int            ji_im_stderr;
   int            ji_im_portout; /* for job_radix intermediate mom demux port for ji_stdout */
   int            ji_im_porterr; /* for job_radix intermediate mom demux port for ji_stderr */
+  short          ji_job_is_being_rerun; /* special flag for jobs being rerun */
 #ifdef PENABLE_LINUX26_CPUSETS
   int            ji_mempressure_curr;  /* current memory_pressure value */
   int            ji_mempressure_cnt;   /* counts MOM cycles memory_pressure is over threshold */
 #endif
 
 #else     /* END MOM ONLY */
-  tlist_head ji_svrtask; /* links to svr work_task list */
 
+  int               ji_has_delete_nanny;
   struct pbs_queue *ji_qhdr; /* current queue header */
-  int  ji_lastdest; /* last destin tried by route */
-  int  ji_retryok; /* ok to retry, some reject was temp */
-  tlist_head ji_rejectdest; /* list of rejected destinations */
-  struct job_array      *ji_arraystruct; /* pointer to job_array for this array */
-  int  ji_is_array_template;    /* set to TRUE if this is a "template job" for a job array*/
-  int  ji_have_nodes_request; /* set to TRUE if node spec uses keyword nodes */
+  int               ji_lastdest; /* last destin tried by route */
+  int               ji_retryok; /* ok to retry, some reject was temp */
+  tlist_head        ji_rejectdest; /* list of rejected destinations */
+  struct job_array *ji_arraystruct; /* pointer to job_array for this array */
+  int               ji_is_array_template;    /* set to TRUE if this is a "template job" for a job array*/
+  int               ji_have_nodes_request; /* set to TRUE if node spec uses keyword nodes */
 #endif/* PBS_MOM */   /* END SERVER ONLY */
 
+  pthread_mutex_t *ji_mutex;
+  char    ji_being_recycled;
+
   /*
    * fixed size internal data - maintained via "quick save"
    * some of the items are copies of attributes, if so this
@@ -584,9 +602,9 @@ struct job
     int     ji_state;    /* internal copy of state */
     int     ji_substate; /* job sub-state */
     int     ji_svrflags; /* server flags */
-    int     ji_numattr;  /* number of attributes in list */
-    int     ji_ordering; /* special scheduling ordering */
-    int     ji_priority; /* internal priority */
+    int     ji_numattr;  /* number of attributes in list - never used, delete me */
+    int     ji_ordering; /* special scheduling ordering - also never used */
+    int     ji_priority; /* internal priority  - also never used */
     time_t  ji_stime;    /* time job started execution */
     char    ji_jobid[PBS_MAXSVRJOBID + 1];   /* job identifier */
     char    ji_fileprefix[PBS_JOBBASE + 1];  /* job file prefix */
@@ -602,9 +620,9 @@ struct job
       struct   /* if in execution queue .. */
         {
         pbs_net_t ji_momaddr;  /* host addr of Server */
-        int       ji_exitstat; /* job exit status from MOM */
         unsigned short ji_momport;  /* host port of Server default 15002 */
         unsigned short ji_mom_rmport; /* host mom manager port of Server default 15003 */
+        int       ji_exitstat; /* job exit status from MOM */
         } ji_exect;
 
       struct
@@ -644,6 +662,58 @@ struct job
 typedef struct job job;
 
 
+
+#ifndef PBS_MOM
+#define INITIAL_JOB_SIZE           5000
+#define JOB_NOT_FOUND             -1
+#define MAX_RECYCLE_JOBS           5000
+#define TOO_MANY_JOBS_IN_RECYCLER -1
+#define JOBS_TO_REMOVE             1000
+
+/* on the server this array will replace many of the doubly linked-lists */
+struct all_jobs
+  {
+  resizable_array *ra;
+  hash_table_t    *ht;
+
+  pthread_mutex_t *alljobs_mutex;
+  };
+
+void initialize_all_jobs_array(struct all_jobs *);
+int  insert_job(struct all_jobs *, job *);
+int  insert_job_after(struct all_jobs *,job *before,job *after);
+int  insert_job_after_index(struct all_jobs *, int index, job *after);
+int  insert_job_first(struct all_jobs *,job *);
+int  get_jobs_index(struct all_jobs *, job *);
+int  remove_job(struct all_jobs *,job *);
+int  has_job(struct all_jobs *,job *);
+int  swap_jobs(struct all_jobs *,job *,job *);
+struct pbs_queue *get_jobs_queue(job **);
+
+job *next_job(struct all_jobs *,int *);
+job *next_job_from_back(struct all_jobs *,int *);
+
+
+typedef struct job_recycler
+  {
+  unsigned int     rc_next_id;
+  struct all_jobs  rc_jobs;
+  int              rc_iter;
+  unsigned int     rc_max_id;
+  pthread_mutex_t *rc_mutex;
+  } job_recycler;
+
+
+int   insert_into_recycler(job *);
+job  *get_recycled_job();
+void  update_recycler_next_id();
+void  initialize_recycler();
+void  garbage_collect_recycling();
+
+#endif
+
+
+
 #ifdef PBS_MOM
 /*
 ** Tasks are sessions belonging to a job, running on one of the
@@ -656,7 +726,7 @@ typedef struct task
   list_link ti_jobtask; /* links to tasks for this job */
   int  ti_fd;  /* DIS file descriptor to task */
   int  ti_flags; /* task internal flags */
-  tm_event_t ti_register; /* event if task registers */
+  tm_event_t ti_register; /* event if task registers - never used*/
   tlist_head ti_obits; /* list of obit events */
   tlist_head ti_info; /* list of named info */
 
@@ -700,6 +770,7 @@ typedef struct eventent
   {
   int  ee_command; /* command event is for */
   tm_event_t ee_event; /* event number */
+  tm_event_t ee_parent_event; /* For job_radix calls. event of parent calling intermediate MOM */
   tm_task_id ee_taskid; /* which task id */
   fwdevent ee_forward; /* event to get notified */
   char  **ee_argv; /* save args for spawn */
@@ -733,8 +804,6 @@ typedef struct infoent
 
 
 
-
-
 #ifdef PBS_MOM
 typedef struct job_file_delete_info
   {
@@ -751,7 +820,6 @@ typedef struct job_file_delete_info
 
 
 
-
 #define TI_FLAGS_INIT           1  /* task has called tm_init */
 #define TI_FLAGS_CHECKPOINT     2  /* task has checkpointed */
 
@@ -783,20 +851,38 @@ typedef struct job_file_delete_info
 #define IM_GET_RESC        9
 #define IM_ABORT_JOB      10
 #define IM_GET_TID        11
-#define IM_ERROR 99
+#define IM_RADIX_ALL_OK   12
+#define IM_JOIN_JOB_RADIX 13
+#define IM_KILL_JOB_RADIX 14
+#define IM_MAX            15
 
-#define IM_MAX          12
+#define IM_ERROR          99
 
-eventent *event_alloc( int  command,
+eventent *event_alloc(int  command,
                            hnodent *pnode,
                            tm_event_t event,
                            tm_task_id taskid);
 
-task *pbs_task_create( job *pjob,
-                           tm_task_id taskid);
+task *pbs_task_create(job *pjob, tm_task_id taskid);
+
+task *task_find(job *pjob, tm_task_id taskid);
+#else
+
+#define LOCUTION_SUCCESS  0
+#define LOCUTION_FAIL    -1
+#define LOCUTION_REQUEUE -2
+
+#define LOCUTION_SIZE     20
+
+
+typedef struct send_job_request
+  {
+  char      *jobid;
+  int        move_type;
+  void      *data;
+  } send_job_request;
+
 
-task *task_find( job *pjob,
-                     tm_task_id taskid);
 
 #endif /* MOM */
 
@@ -818,6 +904,7 @@ task *task_find( job *pjob,
 #define JOB_SVFLG_HasNodes 0x1000 /* job has nodes allocated to it */
 #define JOB_SVFLG_RescAssn 0x2000 /* job resources accumulated in server/que */
 #define JOB_SVFLG_CHECKPOINT_COPIED 0x4000 /* job checkpoint file that has been copied */
+#define JOB_SVFLG_INTERMEDIATE_MOM  0x8000 /* This is for job_radix. I am an intermediate mom */
 
 /*
  * Related defines
@@ -896,12 +983,15 @@ task *task_find( job *pjob,
 #define JOB_SUBSTATE_SUSPEND 43 /* job suspended, CRAY only */
 
 #define JOB_SUBSTATE_EXITING 50 /* Start of job exiting processing */
-#define JOB_SUBSTATE_STAGEOUT 51 /* job staging out (other) files   */
-#define JOB_SUBSTATE_STAGEDEL 52 /* job deleteing staged out files  */
-#define JOB_SUBSTATE_EXITED 53 /* job exit processing completed   */
-#define JOB_SUBSTATE_ABORT      54 /* job is being aborted by server  */
-#define JOB_SUBSTATE_NOTERM_REQUE 55 /* (MOM) on mom initialization. Requeue job
-                                        but do not terminate any running process */
+#define JOB_SUBSTATE_EXIT_WAIT 51 /* Waiting for response from other moms
+                                     or from server */
+#define JOB_SUBSTATE_STAGEOUT 52 /* job staging out (other) files   */
+#define JOB_SUBSTATE_STAGEDEL 53 /* job deleteing staged out files  */
+#define JOB_SUBSTATE_EXITED 54 /* job exit processing completed   */
+#define JOB_SUBSTATE_ABORT      55 /* job is being aborted by server  */
+#define JOB_SUBSTATE_NOTERM_REQUE 56 /* (MOM) on mom initialization.
+                                        Requeue job but do not terminate
+                                        any running process */
 #define JOB_SUBSTATE_PREOBIT    57 /* (MOM) preobit jobstat sent */
 #define JOB_SUBSTATE_OBIT       58 /* (MOM) job obit notice sent */
 #define JOB_SUBSTATE_COMPLETE   59 /* job is complete */
@@ -938,16 +1028,17 @@ dir so that job can be restarted */
 /* Special Job Exit Values,  Set by the job starter (child of MOM)   */
 /* see server/req_jobobit() & mom/start_exec.c        */
 
-#define JOB_EXEC_OK    0 /* job exec successful */
-#define JOB_EXEC_FAIL1   -1 /* job exec failed, before files, no retry */
-#define JOB_EXEC_FAIL2   -2 /* job exec failed, after files, no retry  */
-#define JOB_EXEC_RETRY   -3 /* job execution failed, do retry    */
-#define JOB_EXEC_INITABT  -4 /* job aborted on MOM initialization */
-#define JOB_EXEC_INITRST  -5 /* job aborted on MOM init, checkpoint, no migrate */
-#define JOB_EXEC_INITRMG  -6 /* job aborted on MOM init, checkpoint, ok migrate */
-#define JOB_EXEC_BADRESRT -7 /* job restart failed */
-#define JOB_EXEC_CMDFAIL  -8 /* exec() of user command failed */
+#define JOB_EXEC_OK          0 /* job exec successful */
+#define JOB_EXEC_FAIL1      -1 /* job exec failed, before files, no retry */
+#define JOB_EXEC_FAIL2      -2 /* job exec failed, after files, no retry  */
+#define JOB_EXEC_RETRY      -3 /* job execution failed, do retry    */
+#define JOB_EXEC_INITABT    -4 /* job aborted on MOM initialization */
+#define JOB_EXEC_INITRST    -5 /* job aborted on MOM init, checkpoint, no migrate */
+#define JOB_EXEC_INITRMG    -6 /* job aborted on MOM init, checkpoint, ok migrate */
+#define JOB_EXEC_BADRESRT   -7 /* job restart failed */
+#define JOB_EXEC_CMDFAIL    -8 /* exec() of user command failed */
 #define JOB_EXEC_STDOUTFAIL -9 /* could not create/open stdout stderr files */
+#define JOB_EXEC_OVERLIMIT  -10
 
 extern void  add_dest(job *);
 extern void  depend_clrrdy(job *);
@@ -955,6 +1046,8 @@ extern int   depend_on_que(attribute *, void *, int);
 extern int   depend_on_exec(job *);
 extern int   depend_on_term(job *);
 extern job  *find_job(char *);
+job         *find_job_regular_jobs(char *);
+job         *find_job_array_jobs(char *);
 extern char *get_egroup(job *);
 extern char *get_variable(job *, char *);
 extern int   init_chkmom(job *);
@@ -970,17 +1063,17 @@ extern void  job_purge(job *);
 extern job  *job_recov(char *);
 extern int   job_save(job *, int, int);
 extern int   modify_job_attr(job *, svrattrl *, int, int *);
-extern char *prefix_std_file(job *, int);
-extern char *add_std_filename(job *, char *, int);
+extern char *prefix_std_file(job *, dynamic_string *, int);
+extern char *add_std_filename(job *, char *, int, dynamic_string *);
 extern int   set_jobexid(job *, attribute *, char *);
-extern int   site_check_user_map(job *, char *, char *);
-extern void  svr_dequejob(job *);
-extern int   svr_enquejob(job *);
+extern int   site_check_user_map(job *, char *, char *, int);
+void  svr_dequejob(job *, int);
+extern int   svr_enquejob(job *, int, int);
 extern void  svr_evaljobstate(job *, int *, int *, int);
 extern void  svr_mailowner(job *, int, int, char *);
-extern void  set_resc_deflt(job *, attribute *);
+extern void  set_resc_deflt(job *, attribute *, int);
 extern void  set_statechar(job *);
-extern int   svr_setjobstate(job *, int, int);
+extern int   svr_setjobstate(job *, int, int, int);
 
 #ifdef BATCH_REQUEST_H
 extern job  *chk_job_request(char *, struct batch_request *);
@@ -1002,9 +1095,7 @@ extern void  set_chkpt_deflt(job *, pbs_queue *);
 #endif /* QUEUE_H */
 
 #ifdef WORK_TASK_H
-extern void  job_clone_wt(struct work_task *);
-extern int   issue_signal(job *, char *, void(*)(struct work_task *), void *);
-extern void   on_job_exit(struct work_task *);
+int   issue_signal(job **, char *, void(*)(struct work_task *), void *);
 #endif /* WORK_TASK_H */
 
 #endif /* PBS_JOB_H */
