@@ -1,3 +1,5 @@
+#ifndef _PORTABILITY_H
+#define _PORTABILITY_H
 /*
 *         OpenPBS (Portable Batch System) v2.3 Software License
 *
@@ -78,7 +80,7 @@
 */
 
 /*
- * @(#) $Id: portability.h 3230 2009-11-05 00:12:42Z ataufer $
+ * @(#) $Id: portability.h 4975 2011-08-12 15:19:01Z jrosenquist $
  */
 
 /* The following macros are for HPUX */
@@ -90,4 +92,4 @@
 #  define setegid(e) (setresgid(-1,e,-1))
 # endif
 #endif
-
+#endif /* _PORTABILITY_H */
