@@ -171,5 +171,8 @@ ATTR_joblogkeepdays,
 #ifdef MUNGE_AUTH
 ATTR_authusers,
 #endif
+ATTR_minthreads,
+ATTR_maxthreads,
+ATTR_threadidleseconds,
 ATTR_moabarraycompatible,
 ATTR_nomailforce,
