@@ -82,6 +82,14 @@
 #ifndef QUEUE_H
 #define QUEUE_H
 
+#include "resizable_array.h"
+#include "hash_table.h" /* hash_table_t */
+#include "pbs_ifl.h" /* PBS_MAXQUEUENAME */
+#include "server_limits.h" /* PBS_NUMJOBSTATE */
+#include "attribute.h" /* attribute_def, attribute */
+
+#define INITIAL_QUEUE_SIZE 5
+
 #define QTYPE_Unset 0
 #define QTYPE_Execution 1
 #define QTYPE_RoutePush 2
@@ -92,6 +100,7 @@
 #define ROUTE_RETRY          1
 #define ROUTE_DEFERRED       2
 
+
 /**
  * Attributes, including the various resource-lists are maintained in an
  * array in a "decoded or parsed" form for quick access to the value.
@@ -165,11 +174,16 @@ extern attribute_def que_attr_def[];
 
 /* at last we come to the queue definition itself */
 
-struct pbs_queue
+typedef struct pbs_queue
   {
-  list_link  qu_link;  /* forward/backward links */
-  tlist_head qu_jobs;  /* jobs in this queue */
-  tlist_head qu_jobs_array_sum; /* jobs with job arrays summarized */
+  int              q_being_recycled;
+#ifndef PBS_MOM
+  struct all_jobs *qu_jobs;  /* jobs in this queue */
+  struct all_jobs *qu_jobs_array_sum; /* jobs with job arrays summarized */
+#else
+  tlist_head       qu_jobs;  /* jobs in this queue */
+  tlist_head       qu_jobs_array_sum; /* jobs with job arrays summarized */
+#endif
 
   struct queuefix
     {
@@ -180,24 +194,49 @@ struct pbs_queue
     char   qu_name[PBS_MAXQUEUENAME]; /* queue name */
     } qu_qs;
 
-  int  qu_numjobs;  /* current numb jobs in queue */
-  int  qu_numcompleted;  /* current numb completed jobs in queue */
-  int  qu_njstate[PBS_NUMJOBSTATE]; /* # of jobs per state */
-  char qu_jobstbuf[100];
+  pthread_mutex_t *qu_mutex; /* this queue's mutex */
+
+  int              qu_numjobs;  /* current numb jobs in queue */
+  int              qu_numcompleted;  /* current numb completed jobs in queue */
+  int              qu_njstate[PBS_NUMJOBSTATE]; /* # of jobs per state */
+  char             qu_jobstbuf[100];
 
   /* the queue attributes */
 
   attribute qu_attr[QA_ATR_LAST];
-  };
+  } pbs_queue;
 
-typedef struct pbs_queue pbs_queue;
 
-extern pbs_queue *find_queuebyname(char *);
-extern pbs_queue *get_dfltque();
-extern pbs_queue *que_alloc(char *name);
-extern void   que_free(pbs_queue *);
-extern pbs_queue *que_recov_xml(char *);
-extern pbs_queue *que_recov(char *);
-extern int    que_save(pbs_queue *);
+typedef struct all_queues
+  {
+  resizable_array *ra;
+  hash_table_t    *ht;
+
+  pthread_mutex_t *allques_mutex;
+  } all_queues;
+
+typedef struct queue_recycler
+  {
+  unsigned int     next_id;
+  pthread_mutex_t *mutex;
+  all_queues       queues;
+  int              iter;
+  unsigned int     max_id;
+  } queue_recycler;
+
+
+int insert_into_queue_recycler(pbs_queue *pq);
+pbs_queue *get_recycled_queue();
+void update_recycler_queue_next_id();
+void initialize_recycler_trash();
+void garbage_collect_recycling();
+
+
+int lock_queue(struct pbs_queue *the_queue, const char *method_name, char *msg, int logging);
+int unlock_queue(struct pbs_queue *the_queue, const char *method_name, char *msg, int logging);
+pbs_queue *next_queue(all_queues *,int *);
+int        insert_queue(all_queues *,pbs_queue *);
+int        remove_queue(all_queues *,pbs_queue *);
+void       initialize_allques_array(all_queues *);
 
 #endif /* QUEUE_H */
