@@ -96,14 +96,15 @@
 #define SERVER_H
 
 #include "server_limits.h"
-
-#ifndef ulong
-#define ulong unsigned long
-#endif
+#include "attribute.h" /* attribute_def */
 
 #define NO_BUFFER_SPACE -2
 #define NO_ATTR_DATA    1
 #define ATTR_NOT_FOUND  -2
+/* default multi threading options */
+#define DEFAULT_MAX_THREADS    10
+#define DEFAULT_MIN_THREADS    10
+#define DEFAULT_THREAD_IDLE    -1
 
 enum srv_atr
   {
@@ -193,6 +194,9 @@ enum srv_atr
 #ifdef MUNGE_AUTH
   SRV_ATR_authusers,
 #endif
+  SRV_ATR_minthreads,
+  SRV_ATR_maxthreads,
+  SRV_ATR_threadidleseconds,
   SRV_ATR_MoabArrayCompatible,
   SRV_ATR_NoMailForce,
 
@@ -215,16 +219,19 @@ struct server
     time_t sv_savetm; /* time of server db update         */
     } sv_qs;
 
-  time_t   sv_started;  /* time server started */
-  time_t   sv_hotcycle;  /* if RECOV_HOT,time of last restart */
-  time_t   sv_next_schedule; /* when to next run scheduler cycle */
-  int   sv_jobstates[PBS_NUMJOBSTATE];  /* # of jobs per state */
-  char   sv_jobstbuf[100];
+  pthread_mutex_t *sv_qs_mutex;
+  pthread_mutex_t *sv_attr_mutex;
+  pthread_mutex_t *sv_jobstates_mutex;
+  time_t           sv_started;  /* time server started */
+  time_t           sv_hotcycle;  /* if RECOV_HOT,time of last restart */
+  time_t           sv_next_schedule; /* when to next run scheduler cycle */
+  int              sv_jobstates[PBS_NUMJOBSTATE];  /* # of jobs per state */
+  char             sv_jobstbuf[100];
 
-  attribute sv_attr[SRV_ATR_LAST]; /* the server attributes      */
+  attribute        sv_attr[SRV_ATR_LAST]; /* the server attributes      */
 
-  int   sv_trackmodifed; /* 1 if tracking list modified     */
-  int   sv_tracksize;  /* total number of sv_track entries */
+  int              sv_trackmodifed; /* 1 if tracking list modified     */
+  int              sv_tracksize;  /* total number of sv_track entries */
 
   struct tracking *sv_track; /* array of track job records     */
 
@@ -275,10 +282,21 @@ extern int svr_save(struct server *, int);
 #endif /* END MAX */
 
 #ifndef JobStatRate
-#define JobStatRate MAX(PBS_JOBSTAT_MIN,server.sv_attr[(int)SRV_ATR_JobStatRate].at_val.at_long)
+#define JobStatRate MAX(PBS_JOBSTAT_MIN,server.sv_attr[SRV_ATR_JobStatRate].at_val.at_long)
 #endif
 
 
+typedef struct mail_info
+  {
+  char *mailto;
+  char *exec_host;
+  char *jobid;
+  char *jobname;
+  char *text;        /* additional optional text */
+  int   mail_point;
+  } mail_info;
+
+
 /* maintain a list of new nodes */
 
 typedef struct new_node
@@ -288,11 +306,7 @@ typedef struct new_node
   } new_node;
 
 /* Threaded-HA code */
-#ifdef USE_HA_THREADS
 typedef pthread_mutex_t mutex_t;
-#else
-typedef int mutex_t;
-#endif /* USE_HA_THREADS */
 
 /* END server.h */
 
