@@ -83,20 +83,14 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
-#include <pwd.h>
 #include <signal.h>
 #include <string.h>
 #include <ctype.h>
 #include <errno.h>
-#include <fcntl.h>
 #include <time.h>
 #include <limits.h>
-#include <netdb.h>
-#include <grp.h>
 #include <sys/types.h>
 #include <sys/param.h>
-#include <sys/times.h>
-#include <sys/stat.h>
 #include <sys/select.h>
 
 #include "portability.h"
@@ -112,12 +106,14 @@
 #include "log.h"
 #include "mcom.h"
 
-#ifndef MAXLINE 
 #define MAXLINE 1024
-#endif
 #ifndef NULL
 #define NULL 0
 #endif
+#ifndef TRUE
+#define TRUE 1
+#define FALSE 0
+#endif
 
 #define BUFFER_OVERFLOW -5
 #define LT_ESCAPED       "&lt;"
@@ -145,13 +141,18 @@ extern void *tdelete (const u_long, tree **);
 extern struct pbsnode *tfind (const u_long, tree **);
 extern int tlist (tree *, char *, int);
 extern void tfree (tree **);
-
-/* moab-like utility functions in u_mu.c */
-extern int MUSNPrintF (char **, int *, char *, ...);
-extern int MUStrNCat (char **, int *, char *);
-extern int MUSleep (long);
-extern int MUReadPipe (char *, char *, int);
 extern int is_whitespace (char);
+extern int write_buffer (char *,int,int);
+
+/* utility functions in u_mu.c */
+int   MUSNPrintF (char **, int *, char *, ...);
+int   MUStrNCat (char **, int *, char *);
+int   MUSleep (long);
+int   MUReadPipe (char *, char *, int);
+int   is_whitespace (char);
+char *trim(char *);
+char *threadsafe_tokenizer(char **str, char  *delims);
+int   safe_strncat(char *, char *, size_t);
 
 /* MXML functions from u_MXML.c */
 extern int MXMLGetChild (mxml_t *, char *, int *, mxml_t **);
@@ -159,7 +160,12 @@ extern int MXMLAddE (mxml_t *, mxml_t *);
 extern int MXMLGetAttrF (mxml_t *, char *, int *, void *, enum MDataFormatEnum, int);
 extern int MXMLGetAttr (mxml_t *, char *, int *, char *, int);
 extern int MXMLToString (mxml_t *, char *, int, char **, mbool_t);
-extern int MXMLFromString(mxml_t **EP, char *XMLString, char **Tail, char *EMsg, int emsg_size);
+int        MXMLFromString(mxml_t **EP, char *XMLString, char **Tail, char *EMsg, int emsg_size);
+
+/* functions from u_xml.c */
+int get_parent_and_child(char *,char **,char **,char **);
+int escape_xml(char *,char *,int);
+int unescape_xml(char *,char *,int);
 
 /* functions from u_xml.c */
 int get_parent_and_child(char *,char **,char **,char **);
