@@ -78,7 +78,7 @@
 */
 
 /*
- * @(#) $Id: work_task.h 2367 2008-09-03 17:17:34Z josh $
+ * @(#) $Id: work_task.h 5572 2012-01-03 17:16:18Z jrosenquist $
  */
 
 /*
@@ -95,6 +95,11 @@
 #ifndef WORK_TASK_H
 #define WORK_TASK_H 1
 
+#include <pthread.h>
+#include "resizable_array.h"
+
+#define INITIAL_ALL_TASKS_SIZE 4
+
 enum work_type
   {
   WORK_Immed,  /* immediate action: see state */
@@ -108,24 +113,63 @@ enum work_type
   /* a WORK_Deferred_Child is ready            */
   };
 
-struct work_task
-  {
-  list_link  wt_linkall; /* link to event type work list */
-  list_link  wt_linkobj; /* link to others of same object */
-  long   wt_event; /* event id: time, pid, socket, ... */
-  enum work_type  wt_type; /* type of event */
-  void (*wt_func) A_((struct work_task *));
-  /* function to perform task */
-  void  *wt_parm1; /* obj pointer for use by func */
-  void  *wt_parm2; /* optional pointer for use by func */
-  void (*wt_parmfunc) A_((struct work_task *));
-  /* used in reissue_to_svr to store wt_func */
-  int   wt_aux; /* optional info: e.g. child status */
-  };
 
-extern struct work_task *set_task A_((enum work_type, long event, void (*func)(), void *param));
-extern void clear_task A_((struct work_task *ptask));
-extern void dispatch_task A_((struct work_task *));
-extern void delete_task A_((struct work_task *ptask));
+
+
+typedef struct all_tasks
+  {
+  resizable_array *ra;
+
+  pthread_mutex_t *alltasks_mutex;
+  } all_tasks;
+
+
+
+
+typedef struct work_task
+  {
+  all_tasks           *wt_tasklist; 
+  pthread_mutex_t     *wt_mutex; 
+  int                  wt_being_recycled;
+  long                 wt_event; /* event id: time, pid, socket, ... */
+  enum work_type       wt_type; /* type of event */
+  void (*wt_func)      (struct work_task *);
+  /* function to perform task */
+  void                *wt_parm1; /* obj pointer for use by func */
+  void                *wt_parm2; /* optional pointer for use by func */
+  void (*wt_parmfunc)  (struct work_task *);
+  /* used in reissue_to_svr to store wt_func */
+  int                  wt_aux; /* optional info: e.g. child status */
+  } work_task;
+
+void       initialize_all_tasks_array(all_tasks *);
+int        insert_task(all_tasks *, work_task *);
+int        insert_task_before(all_tasks *,work_task *before,work_task *after);
+int        insert_task_first(all_tasks *,work_task *);
+int        remove_task(all_tasks *,work_task *);
+int        has_task(all_tasks *);
+work_task *next_task(all_tasks *,int *);
+
+
+
+extern struct work_task *set_task(enum work_type, long event, void (*func)(), void *param, int);
+extern void clear_task(struct work_task *ptask);
+extern void dispatch_task(struct work_task *);
+extern void delete_task(struct work_task *ptask);
+
+
+#define TASKS_TO_REMOVE       1000
+#define MAX_TASKS_IN_RECYCLER 5000
+
+
+typedef struct task_recycler
+  {
+  unsigned int     next_id;
+  all_tasks        tasks;
+  int              iter;
+  unsigned int     max_id;
+  pthread_mutex_t *mutex;
+  } task_recycler;
+
 
 #endif /* WORK_TASK_H */
