@@ -100,9 +100,6 @@
  * The prototypes are declared in "attr_func.h"
  */
 
-/* Global Variables */
-
-extern int resc_access_perm; /* see lib/Libattr/attr_fn_resc.c */
 
 /*
  * attr_atomic_set - atomically set a attribute array with values from
@@ -121,18 +118,17 @@ int attr_atomic_set(
   int  *badattr)
 
   {
-  int     acc;
-  int     index;
-  int     listidx;
-  resource *prc;
-  int     rc;
-  attribute temp;
+  int        acc;
+  int        index;
+  int        listidx;
+  resource  *prc;
+  int        rc;
+  attribute  temp;
+  int        resc_access_perm = privil; /* set privilege for decode_resc() */
 
   for (index = 0;index < limit;index++)
     clear_attr(new + index, pdef + index);
 
-  resc_access_perm = privil; /* set privilege for decode_resc() */
-
   listidx = 0;
 
   rc = PBSE_NONE;
@@ -182,7 +178,59 @@ int attr_atomic_set(
 
     clear_attr(&temp, pdef + index);
 
-    if ((rc = (pdef + index)->at_decode(&temp, plist->al_name, plist->al_resc, plist->al_value)) != 0)
+    /* 
+     * special gpu cases
+     * 1) if only ncpus is specified, delete gpus resource if any
+     * 2) if both ncpus and gpus specified, replace both
+     */
+    
+    if ((strcmp(plist->al_name,ATTR_l) == 0) &&
+      (strcmp(plist->al_resc,"ncpus") == 0))
+      {
+      char      *pc;
+      if ((pc = strstr(plist->al_value,":gpus=")) != NULL)
+        {
+        /* save off gpu resource list then add new resource_list entry for it */
+        char *gpuval;
+
+        gpuval = strdup(pc+6);
+
+        (*pc) = '\0';
+
+        if (gpuval != NULL)
+          {
+          rc = (pdef + index)->at_decode(&temp, plist->al_name, "gpus",
+              gpuval,ATR_DFLAG_ACCESS);
+
+          free(gpuval);
+          if (rc != 0)
+            {
+            if ((rc == PBSE_UNKRESC) && (unkn > 0))
+              rc = 0; /* ignore the "error" */
+            else
+              break;
+            }
+          }
+        }
+      else
+        {
+        /* delete old resource_list.gpus value if any.
+         * this can be done by setting it to zero
+         */
+        rc = (pdef + index)->at_decode(&temp, plist->al_name, "gpus",
+            0,ATR_DFLAG_ACCESS);
+        if (rc != 0)
+          {
+          if ((rc == PBSE_UNKRESC) && (unkn > 0))
+            rc = 0; /* ignore the "error" */
+          else
+            break;
+          }
+        }
+      }
+
+    rc = (pdef + index)->at_decode(&temp, plist->al_name, plist->al_resc, plist->al_value,resc_access_perm);
+    if (rc != 0)
       {
       if ((rc == PBSE_UNKRESC) && (unkn > 0))
         rc = 0; /* ignore the "error" */
@@ -195,7 +243,7 @@ int attr_atomic_set(
     if (((old + index)->at_flags & ATR_VFLAG_SET) &&
         !((new + index)->at_flags & ATR_VFLAG_SET))
       {
-      if ((rc = (pdef + index)->at_set(new + index, old + index, plist->al_atopl.op)) != 0)
+      if ((rc = (pdef + index)->at_set(new + index, old + index, SET)) != 0)
         break;
 
       /*
@@ -221,7 +269,7 @@ int attr_atomic_set(
     /* update new copy with temp, MODIFY is set on ones changed */
 
     if ((plist->al_op != INCR) && (plist->al_op != DECR) &&
-        (plist->al_op != SET))
+        (plist->al_op != SET) && (plist->al_op != INCR_OLD))
       {
       plist->al_op = SET;
       }
@@ -271,17 +319,16 @@ int attr_atomic_set(
  * values from an svrattrl
 */
 
-int
-attr_atomic_node_set(
-  struct svrattrl *plist,  /*list of attribute modif structs*/
-  attribute *old,  /*unused*/
-  attribute *new,  /*new attribute array begins here*/
-  attribute_def *pdef,  /*begin array  definition structs*/
-  int limit,  /*number elts in definition array*/
-  int unkn,  /*<0 unknown attrib not permitted*/
-  int privil, /*requester's access privileges  */
-  int *badattr /*return list position wher bad  */
-)
+int attr_atomic_node_set(
+
+  struct svrattrl *plist,  /* list of attribute modif structs */
+  attribute *old,          /* unused */
+  attribute *new,          /* new attribute array begins here */
+  attribute_def *pdef,     /* begin array  definition structs */
+  int limit,               /* number elts in definition array */
+  int unkn,                /* <0 unknown attrib not permitted */
+  int privil,              /* requester's access privileges   */
+  int *badattr)            /* return list position wher bad   */
 
   {
   int     acc;
@@ -338,7 +385,7 @@ attr_atomic_node_set(
     clear_attr(&temp, pdef + index);
 
     if ((rc = (pdef + index)->at_decode(&temp, plist->al_name,
-                                        plist->al_resc, plist->al_value) != 0))
+                                        plist->al_resc, plist->al_value,0) != 0))
       {
       if ((rc == PBSE_UNKRESC) && (unkn > 0))
         rc = 0;              /*ignore the "error"*/
@@ -351,7 +398,7 @@ attr_atomic_node_set(
     (new + index)->at_flags &= ~ATR_VFLAG_MODIFY;
 
     if ((plist->al_op != INCR) && (plist->al_op != DECR) &&
-        (plist->al_op != SET))
+        (plist->al_op != SET) && (plist->al_op != INCR_OLD))
       plist->al_op = SET;
 
 
