@@ -124,7 +124,7 @@ void clear_attr(
 
 #endif /* NDEBUG */
 
-  memset((char *)pattr, 0, sizeof(struct attribute));
+  memset(pattr, 0, sizeof(struct attribute));
 
   pattr->at_type = pdef->at_type;
 
@@ -228,7 +228,7 @@ long attr_ifelse_long(
 
 /*
  * free_null - A free routine for attributes which do not
- * have malloc-ed space ( boolean, char, long ).
+ * have calloc-ed space ( boolean, char, long ).
  */
 
 /*ARGSUSED*/
@@ -316,7 +316,7 @@ svrattrl *attrlist_alloc(
 
   tsize = sizeof(svrattrl) + szname + szresc + szval;
 
-  pal = (svrattrl *)calloc(1, tsize);
+  pal = calloc(1, tsize);
 
   if (pal == NULL)
     {
@@ -412,10 +412,11 @@ free_attrlist(tlist_head *pattrlisthead)
     }
   pal = (svrattrl *)GET_NEXT(*pattrlisthead);
 
-  while (pal != (svrattrl *)0)
+  while (pal != NULL)
     {
     nxpal = (struct svrattrl *)GET_NEXT(pal->al_link);
     delete_link(&pal->al_link);
+    memset(pal, 254, sizeof(svrattrl));
     (void)free(pal);
     pal = nxpal;
     }
@@ -565,16 +566,23 @@ parse_equal_string(char *start, char **name, char **value)
 
 char *parse_comma_string(
 
-  char *start)  /* I (optional) */
+  char  *start, /* I (optional) */
+  char **ptr)   /* I */
 
   {
-  static char *pc; /* if start is null, restart from here */
+  char *pc; /* if start is null, restart from here */
 
   char     *back;
   char     *rv;
 
+  if ((ptr == NULL) ||
+      ((*ptr == NULL) && (start == NULL)))
+    return(NULL);
+
   if (start != NULL)
     pc = start;
+  else
+    pc = *ptr;
 
   if (*pc == '\0')
     {
@@ -613,6 +621,8 @@ char *parse_comma_string(
   if (*pc != '\0')
     *pc++ = '\0'; /* if not end, terminate this and adv past */
 
+  *ptr = pc;
+
   return(rv);
   }  /* END parse_comma_string() */
 
@@ -679,10 +689,10 @@ count_substrings(
  * within a svrattrl list.
  */
 
-void
-attrl_fixlink(
-  tlist_head *phead /* pointer to head of svrattrl list */
-)
+void attrl_fixlink(
+
+  tlist_head *phead) /* pointer to head of svrattrl list */
+
   {
   svrattrl *pal;
   svrattrl *pnxt;
