@@ -144,8 +144,7 @@ int cnt2server(
 
   if ((SpecServer != NULL) && (SpecServer[0] != '\0'))
     {
-    strncpy(Server, SpecServer, sizeof(Server));
-    Server[sizeof(Server) - 1] = '\0';
+    snprintf(Server, sizeof(Server), "%s", SpecServer);
     }
 
   /* NOTE:  env vars PBS_DEFAULT and PBS_SERVER will be checked and applied w/in pbs_connect() */
@@ -156,9 +155,10 @@ start:
 
   if (connect <= 0)
     {
-    if (pbs_errno > PBSE_)
+    /* PBSE_ * -1 is returned if applicable */
+    if ((connect * -1) > PBSE_)
       {
-      switch (pbs_errno)
+      switch (connect * -1)
         {
         case PBSE_BADHOST:
 
@@ -218,6 +218,9 @@ start:
 
         case PBSE_PROTOCOL:
 
+          fprintf(stderr, "protocol failure.\n");
+          break;
+
         default:
 
           if (thistime == 0)
@@ -228,7 +231,7 @@ start:
 
           break;
         }
-      }    /* END if (pbs_errno > PBSE_) */
+      }    /* END if (a PBSE_ was reported) */
     else
       {
       if (thistime == 0)
@@ -243,9 +246,7 @@ start:
 
             if ((fbserver != NULL) && (fbserver[0] != '\0'))
               {
-              strncpy(Server, fbserver, sizeof(Server));
-
-              Server[sizeof(Server) - 1] = '\0';
+              snprintf(Server, sizeof(Server), "%s", fbserver);
 
               if (getenv("PBSDEBUG") != NULL)
                 {
@@ -256,7 +257,7 @@ start:
               goto start;
               }
 
-            fprintf(stderr, "Cannot connect to default server host '%s' - check pbs_server daemon.\n",
+            fprintf(stderr, "Cannot connect to default server host '%s' - check pbs_server daemon and/or trqauthd.\n",
               pbs_default());
             }
           else
