@@ -78,8 +78,11 @@
 */
 #include <pbs_config.h>   /* the master config generated by configure */
 
-char *
-discul_(char *cp, unsigned long value, unsigned *ndigs)
+char *discul_(
+    
+  char          *cp,
+  unsigned long  value,
+  unsigned      *ndigs)
   {
   char  *ocp;
 
