@@ -79,7 +79,7 @@
 #include <pbs_config.h>   /* the master config generated by configure */
 
 #include <math.h>
-#include "dis_.h"
+#include "lib_dis.h"
 
 /*    expon
  * Returns 10 as a double precision value.
