@@ -103,22 +103,29 @@
 #include <stdlib.h>
 
 #include "dis.h"
-#include "dis_.h"
+#include "lib_dis.h"
+#include "rpp.h"
+#include "tcp.h"
 
-char *disrcs(stream, nchars, retval)
-int   stream;
-size_t  *nchars;
-int   *retval;
+char *disrcs(
+    
+  int     stream, 
+  size_t *nchars,
+  int    *retval)
+  
   {
   int  locret;
   int  negate;
   unsigned count = 0;
   char  *value = NULL;
+  int (*disr_commit)(int stream, int commit);
+  int (*dis_gets)(int, char *, size_t);
 
   assert(nchars != NULL);
   assert(retval != NULL);
-  assert(dis_gets != NULL);
-  assert(disr_commit != NULL);
+
+  disr_commit = tcp_rcommit;
+  dis_gets = tcp_gets;
 
   locret = disrsi_(stream, &negate, &count, 1);
   locret = negate ? DIS_BADSIGN : locret;
@@ -129,7 +136,7 @@ int   *retval;
       locret = DIS_BADSIGN;
     else
       {
-      value = (char *)malloc((size_t)count + 1);
+      value = (char *)calloc(1, (size_t)count + 1);
 
       if (value == NULL)
         locret = DIS_NOMALLOC;
