@@ -102,12 +102,16 @@
 #include <stdio.h>
 
 #include "dis.h"
-#include "dis_.h"
+#include "lib_dis.h"
+#include "rpp.h"
+#include "tcp.h"
 #undef disrd
 
-double disrd(stream, retval)
-int   stream;
-int   *retval;
+double disrd(
+    
+  int  stream,
+  int *retval)
+  
   {
   int  expon;
   unsigned uexpon;
@@ -116,9 +120,11 @@ int   *retval;
   unsigned ndigs;
   unsigned nskips;
   dis_long_double_t ldval;
+  int (*disr_commit)(int stream, int commit);
 
   assert(retval != NULL);
-  assert(disr_commit != NULL);
+
+  disr_commit = tcp_rcommit;
 
   ldval = 0.0L;
   locret = disrl_(stream, &ldval, &ndigs, &nskips, DBL_DIG, 1);
