@@ -101,7 +101,9 @@
 #include <stddef.h>
 
 #include "dis.h"
-#include "dis_.h"
+#include "lib_dis.h"
+#include "rpp.h"
+#include "tcp.h"
 #undef disrf
 
 static unsigned ndigs;
@@ -118,10 +120,20 @@ static int disrd_(
   int  negate;
   unsigned int  unum;
   char  *cp;
+  char scratch[DIS_BUFSIZ+1];
+  int (*dis_getc)(int);
+  int (*disr_skip)(int,size_t);
+  int (*dis_gets)(int, char *, size_t);
 
   if (dis_umaxd == 0)
     disiui_();
 
+  memset(scratch, 0, DIS_BUFSIZ+1);
+
+  dis_getc = tcp_getc;
+  dis_gets = tcp_gets;
+  disr_skip = tcp_rskip;
+
   c = (*dis_getc)(stream);
 
   switch (c)
@@ -253,12 +265,12 @@ static int disrd_(
 
       if (count > 1)
         {
-        if ((*dis_gets)(stream, dis_buffer + 1, count - 1) != (int)count - 1)
+        if ((*dis_gets)(stream, scratch + 1, count - 1) != (int)count - 1)
           {
           return(DIS_EOD);
           }
 
-        cp = dis_buffer;
+        cp = scratch;
 
         if (count >= dis_umaxd)
           {
@@ -267,7 +279,7 @@ static int disrd_(
 
           *cp = c;
 
-          if (memcmp(dis_buffer, dis_umax, dis_umaxd) > 0)
+          if (memcmp(scratch, dis_umax, dis_umaxd) > 0)
             break;
           }
 
@@ -336,10 +348,6 @@ float disrf(
 
   assert(retval != NULL);
   assert(stream >= 0);
-  assert(dis_getc != NULL);
-  assert(dis_gets != NULL);
-  assert(disr_skip != NULL);
-  assert(disr_commit != NULL);
 
   dval = 0.0;
 
@@ -386,7 +394,7 @@ float disrf(
       }
     }
 
-  if ((*disr_commit)(stream, locret == DIS_SUCCESS) < 0)
+  if (tcp_rcommit(stream, locret == DIS_SUCCESS) < 0)
     locret = DIS_NOCOMMIT;
 
   *retval = locret;
