@@ -98,22 +98,29 @@
 #include <stdlib.h>
 
 #include "dis.h"
-#include "dis_.h"
+#include "lib_dis.h"
+#include "rpp.h"
+#include "tcp.h"
+
+int disrfcs(
+    
+  int     stream,
+  size_t *nchars,
+  size_t  achars,
+  char   *value)
 
-int disrfcs(stream, nchars, achars, value)
-int   stream;
-size_t  *nchars;
-size_t  achars;
-char  *value;
   {
   int  locret;
   int  negate;
   unsigned count = 0;
+  int (*disr_commit)(int stream, int commit);
+  int (*dis_gets)(int, char *, size_t);
 
   assert(nchars != NULL);
   assert(value != NULL);
-  assert(dis_gets != NULL);
-  assert(disr_commit != NULL);
+
+  disr_commit = tcp_rcommit;
+  dis_gets = tcp_gets;
 
   locret = disrsi_(stream, &negate, &count, 1);
 
@@ -128,7 +135,6 @@ char  *value;
     }
 
   locret = (*disr_commit)(stream, locret == DIS_SUCCESS) ?
-
            DIS_NOCOMMIT : locret;
 
   if (locret != DIS_SUCCESS)
