@@ -103,7 +103,9 @@
 #endif
 
 #include "dis.h"
-#include "dis_.h"
+#include "lib_dis.h"
+#include "rpp.h"
+#include "tcp.h"
 
 int disrfst(
 
@@ -117,10 +119,13 @@ int disrfst(
 
   int  negate;
   unsigned count;
+  int (*disr_commit)(int stream, int commit);
+  int (*dis_gets)(int, char *, size_t);
 
   assert(value != NULL);
-  assert(dis_gets != NULL);
-  assert(disr_commit != NULL);
+  
+  disr_commit = tcp_rcommit;
+  dis_gets = tcp_gets;
 
   locret = disrsi_(stream, &negate, &count, 1);
 
