@@ -83,7 +83,9 @@
 #include <stddef.h>
 
 #include "dis.h"
-#include "dis_.h"
+#include "lib_dis.h"
+#include "rpp.h"
+#include "tcp.h"
 
 int disrl_(
 
@@ -101,10 +103,15 @@ int disrl_(
   char  *cp;
   dis_long_double_t fpnum;
   char  scratch[DIS_BUFSIZ+1];
+  int (*dis_getc)(int);
+  int (*disr_skip)(int, size_t);
+  int (*dis_gets)(int, char *, size_t);
 
   assert(stream >= 0);
-  assert(dis_getc != NULL);
-  assert(disr_skip != NULL);
+
+  disr_skip = tcp_rskip;
+  dis_getc = tcp_getc;
+  dis_gets = tcp_gets;
 
   memset(scratch, 0, DIS_BUFSIZ+1);
   if (dis_umaxd == 0)
