@@ -106,7 +106,9 @@
 #include <stddef.h>
 
 #include "dis.h"
-#include "dis_.h"
+#include "lib_dis.h"
+#include "rpp.h"
+#include "tcp.h"
 #undef disrsi
 
 int disrsi(
@@ -117,13 +119,13 @@ int disrsi(
   {
   int  locret;
   int  negate;
-  int  value;
+  int  value = 0;
   unsigned uvalue;
+  int (*disr_commit)(int stream, int commit);
 
   assert(retval != NULL);
-  assert(disr_commit != NULL);
 
-  value = 0;
+  disr_commit = tcp_rcommit;
 
   switch (locret = disrsi_(stream, &negate, &uvalue, 1))
     {
@@ -133,22 +135,28 @@ int disrsi(
       if (negate ? uvalue <= (unsigned) - (INT_MIN + 1) + 1 : uvalue <= (unsigned)INT_MAX)
         {
         value = negate ? -uvalue : uvalue;
+        *retval = ((*disr_commit)(stream, locret == DIS_SUCCESS) < 0) ?  DIS_NOCOMMIT : locret;
 
         break;
         }
       else
         {
         locret = DIS_OVERFLOW;
+        *retval = locret;
         }
 
     case DIS_OVERFLOW:
 
       value = negate ? INT_MIN : INT_MAX;
+      *retval = locret;
+      break;
+
+    default:
+      *retval = locret;
+      break;
+
     }
 
-  *retval = ((*disr_commit)(stream, locret == DIS_SUCCESS) < 0) ?
-
-            DIS_NOCOMMIT : locret;
 
   return(value);
   }
