@@ -106,7 +106,9 @@
 #include <stddef.h>
 
 #include "dis.h"
-#include "dis_.h"
+#include "lib_dis.h"
+#include "rpp.h"
+#include "tcp.h"
 #undef disrsl
 
 long disrsl(
@@ -119,9 +121,11 @@ long disrsl(
   int  negate;
   long  value;
   unsigned long uvalue;
+  int (*disr_commit)(int stream, int commit);
 
   assert(retval != NULL);
-  assert(disr_commit != NULL);
+
+  disr_commit = tcp_rcommit;
 
   value = 0;
 
@@ -144,7 +148,6 @@ long disrsl(
     }
 
   *retval = ((*disr_commit)(stream, locret == DIS_SUCCESS) < 0) ?
-
             DIS_NOCOMMIT : locret;
 
   return (value);
