@@ -85,7 +85,9 @@
 #include <stdlib.h>
 
 #include "dis.h"
-#include "dis_.h"
+#include "lib_dis.h"
+#include "rpp.h"
+#include "tcp.h"
 
 static char *ulmax;
 static unsigned ulmaxdigs = 0;
@@ -103,13 +105,16 @@ int disrsl_(
   unsigned long ndigs;
   char  *cp;
   char  scratch[DIS_BUFSIZ+1];
+  int (*dis_getc)(int);
+  int (*dis_gets)(int, char *, size_t);
 
   assert(negate != NULL);
   assert(value != NULL);
   assert(count);
   assert(stream >= 0);
-  assert(dis_getc != NULL);
-  assert(dis_gets != NULL);
+
+  dis_getc = tcp_getc;
+  dis_gets = tcp_gets;
 
   memset(scratch, 0, DIS_BUFSIZ+1);
 
@@ -117,7 +122,7 @@ int disrsl_(
     {
     cp = discul_(scratch + DIS_BUFSIZ, ULONG_MAX, &ulmaxdigs);
 
-    ulmax = (char *)malloc(ulmaxdigs);
+    ulmax = (char *)calloc(1, ulmaxdigs);
 
     if (ulmax == NULL)
       {
