@@ -108,20 +108,26 @@
 #include <stddef.h>
 
 #include "dis.h"
-#include "dis_.h"
+#include "lib_dis.h"
+#include "rpp.h"
+#include "tcp.h"
 #undef disrss
 
-short disrss(stream, retval)
-int   stream;
-int   *retval;
+short disrss(
+    
+  int  stream,
+  int *retval)
+  
   {
   int  locret;
   int  negate;
   short  value;
   unsigned uvalue;
+  int (*disr_commit)(int stream, int commit);
 
   assert(retval != NULL);
-  assert(disr_commit != NULL);
+
+  disr_commit = tcp_rcommit;
 
   value = 0;
 
@@ -143,7 +149,7 @@ int   *retval;
     }
 
   *retval = ((*disr_commit)(stream, locret == DIS_SUCCESS) < 0) ?
-
             DIS_NOCOMMIT : locret;
+
   return (value);
   }
