@@ -102,7 +102,9 @@
 #endif
 
 #include "dis.h"
-#include "dis_.h"
+#include "lib_dis.h"
+#include "rpp.h"
+#include "tcp.h"
 
 char *disrst(
 
@@ -113,12 +115,15 @@ char *disrst(
   int        locret;
   int        negate;
   unsigned int count;
+  int (*disr_commit)(int stream, int commit);
+  int (*dis_gets)(int, char *, size_t);
 
   char *value = NULL;
 
   assert(retval != NULL);
-  assert(dis_gets != NULL);
-  assert(disr_commit != NULL);
+
+  disr_commit = tcp_rcommit;
+  dis_gets = tcp_gets;
 
   locret = disrsi_(stream, &negate, &count, 1);
 
@@ -130,7 +135,7 @@ char *disrst(
       }
     else
       {
-      value = (char *)malloc((size_t)count + 1);
+      value = (char *)calloc(1, (size_t)count + 1);
 
       if (value == NULL)
         {
@@ -150,12 +155,9 @@ char *disrst(
           value[count] = '\0';
         }
       }
+    locret = ((*disr_commit)(stream, locret == DIS_SUCCESS) < 0) ?  DIS_NOCOMMIT : locret;
     }
 
-  locret = ((*disr_commit)(stream, locret == DIS_SUCCESS) < 0) ?
-
-           DIS_NOCOMMIT : locret;
-
   *retval = locret;
 
   if ((locret != DIS_SUCCESS) && (value != NULL))
