@@ -107,7 +107,9 @@
 #include <stddef.h>
 
 #include "dis.h"
-#include "dis_.h"
+#include "lib_dis.h"
+#include "rpp.h"
+#include "tcp.h"
 
 unsigned long disrul(
 
@@ -118,8 +120,9 @@ unsigned long disrul(
   int  locret;
   int  negate;
   unsigned long value;
+  int (*disr_commit)(int stream, int commit);
 
-  assert(disr_commit != NULL);
+  disr_commit = tcp_rcommit;
 
   locret = disrsl_(stream, &negate, &value, 1);
 
