@@ -110,19 +110,25 @@
 #include <stddef.h>
 
 #include "dis.h"
-#include "dis_.h"
+#include "lib_dis.h"
+#include "rpp.h"
+#include "tcp.h"
 #undef disrus
 
-unsigned short disrus(stream, retval)
-int   stream;
-int   *retval;
+unsigned short disrus(
+    
+  int  stream,
+  int *retval)
+  
   {
   int  locret;
   int  negate;
   unsigned value;
+  int (*disr_commit)(int stream, int commit);
 
   assert(retval != NULL);
-  assert(disr_commit != NULL);
+
+  disr_commit = tcp_rcommit;
 
   locret = disrsi_(stream, &negate, &value, 1);
 
