@@ -96,7 +96,9 @@
 #include <stdio.h>
 
 #include "dis.h"
-#include "dis_.h"
+#include "lib_dis.h"
+#include "rpp.h"
+#include "tcp.h"
 
 int diswcs(
 
@@ -106,9 +108,11 @@ int diswcs(
 
   {
   int retval;
+  int (*dis_puts)(int stream, const char *string, size_t count);
+  int (*disw_commit)(int stream, int commit);
 
-  assert(disw_commit != NULL);
-  assert(dis_puts != NULL);
+  dis_puts = tcp_puts;
+  disw_commit = tcp_wcommit;
 
   retval = diswui_(stream, (unsigned)nchars);
 
