@@ -100,7 +100,8 @@
 #include <stdio.h>
 
 #include "dis.h"
-#include "dis_.h"
+#include "lib_dis.h"
+#include "tcp.h"
 
 /* to work around a problem in a compiler */
 #if SIZEOF_LONG_DOUBLE == SIZEOF_DOUBLE
@@ -130,8 +131,12 @@
 #endif
 
 
-int
-diswl_(int stream, dis_long_double_t value, unsigned ndigs)
+int diswl_(
+    
+  int               stream,
+  dis_long_double_t value,
+  unsigned          ndigs)
+
   {
   int  c;
   int  expon;
@@ -142,11 +147,14 @@ diswl_(int stream, dis_long_double_t value, unsigned ndigs)
   char  *ocp;
   dis_long_double_t ldval;
   char  scratch[DIS_BUFSIZ+1];
+  int (*dis_puts)(int stream, const char *string, size_t count);
+  int (*disw_commit)(int stream, int commit);
 
   assert(ndigs > 0 && ndigs <= LDBL_DIG);
   assert(stream >= 0);
-  assert(dis_puts != NULL);
-  assert(disw_commit != NULL);
+  
+  dis_puts = tcp_puts;
+  disw_commit = tcp_wcommit;
 
   memset(scratch, 0, DIS_BUFSIZ+1);
   /* Make zero a special case.  If we don't it will blow exponent  */
