@@ -106,7 +106,8 @@
 #include <stdio.h>
 
 #include "dis.h"
-#include "dis_.h"
+#include "lib_dis.h"
+#include "tcp.h"
 #undef diswsi
 
 int diswsi(
@@ -123,10 +124,13 @@ int diswsi(
 
   int       rc;
   char		scratch[DIS_BUFSIZ+1];
+  int (*dis_puts)(int stream, const char *string, size_t count);
+  int (*disw_commit)(int stream, int commit);
 
   assert(stream >= 0);
-  assert(dis_puts != NULL);
-  assert(disw_commit != NULL);
+  
+  dis_puts = tcp_puts;
+  disw_commit = tcp_wcommit;
 
   memset(scratch, 0, DIS_BUFSIZ+1);
 
@@ -155,8 +159,6 @@ int diswsi(
              cp,
              strlen(cp)) < 0 ?  DIS_PROTO : DIS_SUCCESS;
 
-  /* disw_commit -> rpp_wcommit */
-
   rc = ((*disw_commit)(stream, retval == DIS_SUCCESS) < 0) ?
        DIS_NOCOMMIT : retval;
 
