@@ -106,7 +106,8 @@
 #include <stdio.h>
 
 #include "dis.h"
-#include "dis_.h"
+#include "lib_dis.h"
+#include "tcp.h"
 
 int diswsl(
 
@@ -120,10 +121,13 @@ int diswsl(
   char  c;
   char  *cp;
   char  scratch[DIS_BUFSIZ+1];
+  int (*dis_puts)(int stream, const char *, size_t);
+  int (*disw_commit)(int stream, int);
 
   assert(stream >= 0);
-  assert(dis_puts != NULL);
-  assert(disw_commit != NULL);
+
+  dis_puts = tcp_puts;
+  disw_commit = tcp_wcommit;
 
   memset(scratch, 0, DIS_BUFSIZ+1);
 
