@@ -106,16 +106,21 @@
 #include <stddef.h>
 
 #include "dis.h"
-#include "dis_.h"
+#include "lib_dis.h"
+#include "tcp.h"
 #undef diswui
 
-int diswui(stream, value)
-int   stream;
-unsigned  value;
+int diswui(
+    
+  int      stream,
+  unsigned value)
+
   {
   int  retval;
 
-  assert(disw_commit != NULL);
+  int (*disw_commit)(int stream, int commit);
+
+  disw_commit = tcp_wcommit;
 
   retval = diswui_(stream, value);
   return (((*disw_commit)(stream, retval == DIS_SUCCESS) < 0) ?
