@@ -107,7 +107,8 @@
 #include <stdio.h>
 
 #include "dis.h"
-#include "dis_.h"
+#include "lib_dis.h"
+#include "tcp.h"
 #include "log.h"
 
 
@@ -126,10 +127,13 @@ int diswul(
 
   int           rc;
   char  scratch[DIS_BUFSIZ+1];
+  int (*dis_puts)(int stream, const char *string, size_t count);
+  int (*disw_commit)(int stream, int commit);
 
   assert(stream >= 0);
-  assert(dis_puts != NULL);
-  assert(disw_commit != NULL);
+
+  dis_puts = tcp_puts;
+  disw_commit = tcp_wcommit;
 
   memset(scratch, 0, DIS_BUFSIZ+1);
   cp = discul_(&scratch[DIS_BUFSIZ], value, &ndigs);
