@@ -102,6 +102,8 @@ int PBSD_mgr_put(
   int rc;
   int sock;
 
+  pthread_mutex_lock(connection[c].ch_mutex);
+
   sock = connection[c].ch_socket;
 
   DIS_tcp_setup(sock);
@@ -112,18 +114,20 @@ int PBSD_mgr_put(
     {
     connection[c].ch_errtxt = strdup(dis_emsg[rc]);
 
-    pbs_errno = PBSE_PROTOCOL;
+    pthread_mutex_unlock(connection[c].ch_mutex);
 
-    return(pbs_errno);
+    return(PBSE_PROTOCOL);
     }
 
   if (DIS_tcp_wflush(sock))
     {
-    pbs_errno = PBSE_PROTOCOL;
+    pthread_mutex_unlock(connection[c].ch_mutex);
 
-    return(pbs_errno);
+    return(PBSE_PROTOCOL);
     }
 
+  pthread_mutex_unlock(connection[c].ch_mutex);
+
   return(0);
   }  /* END PBSD_mgr_put() */
 
