@@ -94,12 +94,20 @@
  * Send the MessageJob request, does not read the reply.
  */
 
-int
-PBSD_msg_put(int c, char *jobid, int fileopt, char *msg, char *extend)
+int PBSD_msg_put(
+    
+  int   c,
+  char *jobid,
+  int   fileopt,
+  char *msg,
+  char *extend)
+
   {
   int rc = 0;
   int sock;
 
+  pthread_mutex_lock(connection[c].ch_mutex);
+
   sock = connection[c].ch_socket;
   DIS_tcp_setup(sock);
 
@@ -108,14 +116,19 @@ PBSD_msg_put(int c, char *jobid, int fileopt, char *msg, char *extend)
       (rc = encode_DIS_ReqExtend(sock, extend)))
     {
     connection[c].ch_errtxt = strdup(dis_emsg[rc]);
-    return (pbs_errno = PBSE_PROTOCOL);
+
+    pthread_mutex_unlock(connection[c].ch_mutex);
+
+    return (PBSE_PROTOCOL);
     }
 
+  pthread_mutex_unlock(connection[c].ch_mutex);
+
   if (DIS_tcp_wflush(sock))
     {
-    pbs_errno = PBSE_PROTOCOL;
-    rc   = pbs_errno;
+    rc = PBSE_PROTOCOL;
     }
 
-  return rc;
-  }
+  return(rc);
+  } /* END PBSD_msg_put() */
+
