@@ -99,14 +99,17 @@
 
 struct batch_reply *PBSD_rdrpy(
 
-        int c)  /* I */
+  int *local_errno, /* O */
+  int  c)           /* I */
 
   {
   int      rc;
 
   struct batch_reply *reply;
   int      sock;
+  const char    *the_msg = NULL;
 
+  sock = connection[c].ch_socket;
   /* clear any prior error message */
 
   if (connection[c].ch_errtxt != NULL)
@@ -119,13 +122,12 @@ struct batch_reply *PBSD_rdrpy(
   if ((reply = (struct batch_reply *)calloc(1, sizeof(struct batch_reply))) == NULL)
     {
     connection[c].ch_errno = PBSE_SYSTEM;
-    pbs_errno = PBSE_SYSTEM;
+
+    *local_errno = PBSE_SYSTEM;
 
     return(NULL);
     }
 
-  sock = connection[c].ch_socket;
-
   DIS_tcp_setup(sock);
 
   if ((rc = decode_DIS_replyCmd(sock, reply)))
@@ -134,16 +136,18 @@ struct batch_reply *PBSD_rdrpy(
 
     if (DIS_tcp_istimeout(sock) == TRUE)
       {
-      pbs_errno = PBSE_TIMEOUT;
+      *local_errno = PBSE_TIMEOUT;
       }
     else
       {
-      pbs_errno = PBSE_PROTOCOL;
+      *local_errno = PBSE_PROTOCOL;
       }
 
-    connection[c].ch_errno = pbs_errno;
-
-    connection[c].ch_errtxt = strdup(dis_emsg[rc]);
+    connection[c].ch_errno = *local_errno;
+    if ((the_msg = dis_emsg[rc]) != NULL)
+      {
+      connection[c].ch_errtxt = strdup(the_msg);
+      }
 
     return(NULL);
     }
@@ -152,11 +156,14 @@ struct batch_reply *PBSD_rdrpy(
 
   connection[c].ch_errno = reply->brp_code;
 
-  pbs_errno = reply->brp_code;
+  *local_errno = reply->brp_code;
 
   if (reply->brp_choice == BATCH_REPLY_CHOICE_Text)
     {
-    connection[c].ch_errtxt = strdup(reply->brp_un.brp_txt.brp_str);
+    if ((the_msg = reply->brp_un.brp_txt.brp_str) != NULL)
+      {
+      connection[c].ch_errtxt = strdup(the_msg);
+      }
     }
 
   return(reply);
