@@ -91,15 +91,19 @@
  * Send the Signal Job Batch Request
 */
 
-int PBSD_sig_put(c, jobid, signal, extend)
-int c;
-char *jobid;
-char *signal;
-char *extend;
+int PBSD_sig_put(
+    
+  int   c,
+  char *jobid,
+  char *signal, 
+  char *extend)
+  
   {
   int sock;
   int rc = 0;
 
+  pthread_mutex_lock(connection[c].ch_mutex);
+
   sock = connection[c].ch_socket;
   DIS_tcp_setup(sock);
 
@@ -108,28 +112,36 @@ char *extend;
       (rc = encode_DIS_ReqExtend(sock, extend)))
     {
     connection[c].ch_errtxt = strdup(dis_emsg[rc]);
-    return (pbs_errno = PBSE_PROTOCOL);
+
+    pthread_mutex_unlock(connection[c].ch_mutex);
+
+    return (PBSE_PROTOCOL);
     }
 
   if (DIS_tcp_wflush(sock))
     {
-    pbs_errno = PBSE_PROTOCOL;
-    rc = pbs_errno;
+    rc = PBSE_PROTOCOL;
     }
 
+  pthread_mutex_unlock(connection[c].ch_mutex);
+
   return rc;
-  }
+  } /* END PBSD_sig_put() */
 
 
-int PBSD_async_sig_put(c,jobid,signal,extend)
-int c;
-char *jobid;
-char *signal;
-char *extend;
+int PBSD_async_sig_put(
+    
+  int   c,
+  char *jobid,
+  char *signal,
+  char *extend)
+  
   {
   int sock;
   int rc = 0;
 
+  pthread_mutex_lock(connection[c].ch_mutex);
+
   sock = connection[c].ch_socket;
   DIS_tcp_setup(sock);
 
@@ -138,15 +150,19 @@ char *extend;
       (rc = encode_DIS_ReqExtend(sock,extend)))
     {
     connection[c].ch_errtxt = strdup(dis_emsg[rc]);
-    return (pbs_errno = PBSE_PROTOCOL);
+
+    pthread_mutex_unlock(connection[c].ch_mutex);
+
+    return (PBSE_PROTOCOL);
     }
+
+  pthread_mutex_unlock(connection[c].ch_mutex);
     
   if (DIS_tcp_wflush(sock))
     {
-    pbs_errno = PBSE_PROTOCOL;
-    rc = pbs_errno;
+    rc = PBSE_PROTOCOL;
     }
 
   return(rc);
-  }
+  } /* END PBSD_async_sig_put() */
 
