@@ -85,6 +85,8 @@
 #include "portability.h"
 #include "libpbs.h"
 #include "dis.h"
+#include "u_hash_map_structs.h"
+#include "u_memmgr.h"
 
 /* PBSD_submit.c
 
@@ -104,10 +106,12 @@ int PBSD_rdytocmt(
   char *jobid)
 
   {
-  int rc;
+  int                 rc;
 
   struct batch_reply *reply;
-  int sock;
+  int                 sock;
+
+  pthread_mutex_lock(connection[connect].ch_mutex);
 
   sock = connection[connect].ch_socket;
   DIS_tcp_setup(sock);
@@ -118,30 +122,96 @@ int PBSD_rdytocmt(
     {
     connection[connect].ch_errtxt = strdup(dis_emsg[rc]);
 
-    pbs_errno = PBSE_PROTOCOL;
+    pthread_mutex_unlock(connection[connect].ch_mutex);
 
-    return(pbs_errno);
+    return(PBSE_PROTOCOL);
     }
 
+  pthread_mutex_unlock(connection[connect].ch_mutex);
   if (DIS_tcp_wflush(sock))
     {
-    pbs_errno = PBSE_PROTOCOL;
-
-    return(pbs_errno);
+  
+    return(PBSE_PROTOCOL);
     }
 
   /* read reply */
 
-  reply = PBSD_rdrpy(connect);
+  reply = PBSD_rdrpy(&rc, connect);
 
   PBSD_FreeReply(reply);
 
-  return(connection[connect].ch_errno);
+  return(rc);
   }
 
 
 
 
+int PBSD_commit_get_sid(
+
+  int   connect, /* I */
+  long *sid,     /* O */
+  char *jobid)   /* I */
+
+  {
+  struct batch_reply *reply;
+  int                 rc;
+  int                 sock;
+  int                 local_errno = 0;
+
+  pthread_mutex_lock(connection[connect].ch_mutex);
+
+  sock = connection[connect].ch_socket;
+
+  DIS_tcp_setup(sock);
+
+  if ((rc = encode_DIS_ReqHdr(sock, PBS_BATCH_Commit, pbs_current_user)) ||
+      (rc = encode_DIS_JobId(sock, jobid)) ||
+      (rc = encode_DIS_ReqExtend(sock, NULL)))
+    {
+    connection[connect].ch_errtxt = strdup(dis_emsg[rc]);
+
+    pthread_mutex_unlock(connection[connect].ch_mutex);
+
+    return(PBSE_PROTOCOL);
+    }
+
+  if (DIS_tcp_wflush(sock))
+    {
+    pthread_mutex_unlock(connection[connect].ch_mutex);
+  
+    return(PBSE_PROTOCOL);
+    }
+
+  /* PBSD_rdrpy sets connection[connect].ch_errno */
+  reply = PBSD_rdrpy(&local_errno, connect);
+
+  rc = connection[connect].ch_errno;
+ 
+  /* read the sid if given and no error */
+  if (rc == PBSE_NONE)
+    {
+    if (reply->brp_choice == BATCH_REPLY_CHOICE_Text)
+      {
+      if (sid != NULL)
+        *sid = atol(reply->brp_un.brp_txt.brp_str);
+      }
+    else
+      /* (reply->brp_choice == BATCH_REPLY_CHOICE_NULL) */
+      {
+      *sid = reply->brp_code;
+      }
+    }
+
+  PBSD_FreeReply(reply);
+
+  pthread_mutex_unlock(connection[connect].ch_mutex);
+
+  return(rc);
+  } /* END PBSD_commit_get_sid() */
+
+
+
+
 
 /* PBS_commit.c
 
@@ -157,8 +227,10 @@ int PBSD_commit(
   {
 
   struct batch_reply *reply;
-  int rc;
-  int sock;
+  int                 rc;
+  int                 sock;
+
+  pthread_mutex_lock(connection[connect].ch_mutex);
 
   sock = connection[connect].ch_socket;
 
@@ -170,25 +242,26 @@ int PBSD_commit(
     {
     connection[connect].ch_errtxt = strdup(dis_emsg[rc]);
 
-    pbs_errno = PBSE_PROTOCOL;
+    pthread_mutex_unlock(connection[connect].ch_mutex);
 
-    return(pbs_errno);
+    return(PBSE_PROTOCOL);
     }
 
   if (DIS_tcp_wflush(sock))
     {
-    pbs_errno = PBSE_PROTOCOL;
-
-    return(pbs_errno);
+    pthread_mutex_unlock(connection[connect].ch_mutex);
+  
+    return(PBSE_PROTOCOL);
     }
 
   /* PBSD_rdrpy sets connection[connect].ch_errno */
-
-  reply = PBSD_rdrpy(connect);
+  reply = PBSD_rdrpy(&rc, connect);
 
   PBSD_FreeReply(reply);
 
-  return(connection[connect].ch_errno);
+  pthread_mutex_unlock(connection[connect].ch_mutex);
+
+  return(rc);
   }  /* END PBSD_commit() */
 
 
@@ -215,8 +288,11 @@ static int PBSD_scbuf(
 
   struct batch_reply *reply;
 
-  int rc;
-  int sock;
+  int                 rc;
+  int                 sock;
+  int                 local_errno = 0;
+
+  pthread_mutex_lock(connection[c].ch_mutex);
 
   sock = connection[c].ch_socket;
 
@@ -231,25 +307,29 @@ static int PBSD_scbuf(
     {
     connection[c].ch_errtxt = strdup(dis_emsg[rc]);
 
-    pbs_errno = PBSE_PROTOCOL;
+    pthread_mutex_unlock(connection[c].ch_mutex);
 
-    return(pbs_errno);
+    return(PBSE_PROTOCOL);
     }
 
   if (DIS_tcp_wflush(sock))
     {
-    pbs_errno = PBSE_PROTOCOL;
+    pthread_mutex_unlock(connection[c].ch_mutex);
 
-    return(pbs_errno);
+    return(PBSE_PROTOCOL);
     }
 
   /* read reply */
 
-  reply = PBSD_rdrpy(c);
+  reply = PBSD_rdrpy(&local_errno, c);
 
   PBSD_FreeReply(reply);
 
-  return(connection[c].ch_errno);
+  rc = connection[c].ch_errno;
+
+  pthread_mutex_unlock(connection[c].ch_mutex);
+
+  return(rc);
   }
 
 
@@ -263,11 +343,17 @@ static int PBSD_scbuf(
  * transfer chunks of the script to the server.
 */
 
-int PBSD_jscript(int c, char *script_file, char *jobid)
+int PBSD_jscript(
+    
+  int   c,
+  char *script_file,
+  char *jobid)
+
   {
-  int i;
-  int fd;
-  int cc;
+  int  i;
+  int  fd;
+  int  cc;
+  int  rc;
   char s_buf[SCRIPT_CHUNK_Z];
 
   if ((fd = open(script_file, O_RDONLY, 0)) < 0)
@@ -290,7 +376,13 @@ int PBSD_jscript(int c, char *script_file, char *jobid)
   if (cc < 0) /* read failed */
     return (-1);
 
-  return connection[c].ch_errno;
+  pthread_mutex_lock(connection[c].ch_mutex);
+
+  rc = connection[c].ch_errno;
+
+  pthread_mutex_unlock(connection[c].ch_mutex);
+
+  return(rc);
   }
 
 
@@ -316,9 +408,13 @@ int PBSD_jobfile(
   {
   int   i;
   int   cc;
+  int   rc;
   int   fd;
   char  s_buf[SCRIPT_CHUNK_Z];
 
+  if (path[0] == '\0')
+    return(PBSE_NONE);
+
   if ((fd = open(path, O_RDONLY, 0)) < 0)
     {
     return(-1);
@@ -343,7 +439,13 @@ int PBSD_jobfile(
     return(-1);
     }
 
-  return(connection[c].ch_errno);
+  pthread_mutex_lock(connection[c].ch_mutex);
+
+  rc = connection[c].ch_errno;
+
+  pthread_mutex_unlock(connection[c].ch_mutex);
+
+  return(rc);
   }  /* END PBSD_jobfile() */
 
 
@@ -356,8 +458,9 @@ int PBSD_jobfile(
 
 char *PBSD_queuejob(
 
-  int             connect,  /* I */
-  char           *jobid,    /* I */
+  int             connect,     /* I */
+  int            *local_errno, /* O */
+  char           *jobid,       /* I */
   char           *destin,
   struct attropl *attrib,
   char           *extend)
@@ -369,7 +472,10 @@ char *PBSD_queuejob(
   int    rc;
   int    sock;
 
+  pthread_mutex_lock(connection[connect].ch_mutex);
+
   sock = connection[connect].ch_socket;
+  connection[connect].ch_errno = 0;
 
   DIS_tcp_setup(sock);
 
@@ -381,38 +487,42 @@ char *PBSD_queuejob(
     {
     connection[connect].ch_errtxt = strdup(dis_emsg[rc]);
 
-    pbs_errno = PBSE_PROTOCOL;
+    pthread_mutex_unlock(connection[connect].ch_mutex);
+
+    *local_errno = PBSE_PROTOCOL;
 
     return(return_jobid);
     }
 
   if (DIS_tcp_wflush(sock))
     {
-    pbs_errno = PBSE_PROTOCOL;
+    pthread_mutex_unlock(connection[connect].ch_mutex);
+
+    *local_errno = PBSE_PROTOCOL;
 
     return(return_jobid);
     }
 
   /* read reply from stream into presentation element */
 
-  reply = PBSD_rdrpy(connect);
+  reply = PBSD_rdrpy(local_errno, connect);
 
   if (reply == NULL)
     {
     if (PConnTimeout(sock) == 1)
       {
-      pbs_errno = PBSE_EXPIRED;
+      *local_errno = PBSE_EXPIRED;
       }
     else
       {
-      pbs_errno = PBSE_PROTOCOL;
+      *local_errno = PBSE_PROTOCOL;
       }
     }
   else if (reply->brp_choice &&
            reply->brp_choice != BATCH_REPLY_CHOICE_Text &&
            reply->brp_choice != BATCH_REPLY_CHOICE_Queue)
     {
-    pbs_errno = PBSE_PROTOCOL;
+    *local_errno = PBSE_PROTOCOL;
     }
   else if (connection[connect].ch_errno == 0)
     {
@@ -421,8 +531,111 @@ char *PBSD_queuejob(
 
   PBSD_FreeReply(reply);
 
+  pthread_mutex_unlock(connection[connect].ch_mutex);
+
   return(return_jobid);
   }  /* END PBSD_queuejob() */
 
-/* END PBSD_submit.c */
 
+
+
+int PBSD_QueueJob_hash(
+
+  int             connect,     /* I */
+  char           *jobid,       /* I */
+  char           *destin,
+  memmgr        **mm,
+  job_data       *job_attr,
+  job_data       *res_attr,
+  char           *extend,
+  char          **job_id,
+  char          **msg)
+
+  {
+  struct batch_reply *reply;
+  int                 rc = PBSE_NONE;
+  int                 sock;
+  int                 tmp_size = 0;
+
+  pthread_mutex_lock(connection[connect].ch_mutex);
+
+  sock = connection[connect].ch_socket;
+
+  DIS_tcp_setup(sock);
+
+  /* first, set up the body of the Queue Job request */
+
+  if ((rc = encode_DIS_ReqHdr(sock, PBS_BATCH_QueueJob, pbs_current_user)) ||
+      (rc = encode_DIS_QueueJob_hash(sock, jobid, destin, mm, job_attr, res_attr))||
+      (rc = encode_DIS_ReqExtend(sock, extend)))
+    {
+    if (connection[connect].ch_errtxt == NULL)
+      {
+      connection[connect].ch_errtxt = memmgr_strdup(mm, (char *)dis_emsg[rc], &tmp_size);
+      }
+    *msg = memmgr_strdup(mm, connection[connect].ch_errtxt, &tmp_size);
+
+    /* rc = PBSE_PROTOCOL; */
+
+    pthread_mutex_unlock(connection[connect].ch_mutex);
+
+    return rc;
+    }
+
+  if ((rc = DIS_tcp_wflush(sock)))
+    {
+    /* rc = PBSE_PROTOCOL; */
+    
+    pthread_mutex_unlock(connection[connect].ch_mutex);
+    if (connection[connect].ch_errtxt == NULL)
+      {
+      *msg = memmgr_strdup(mm, connection[connect].ch_errtxt, &tmp_size);
+      }
+    return rc;
+    }
+
+  /* read reply from stream into presentation element */
+
+  reply = PBSD_rdrpy(&rc, connect);
+
+  if (reply == NULL)
+    {
+    if (PConnTimeout(sock) == 1)
+      {
+      rc = PBSE_EXPIRED;
+      }
+    else
+      {
+      rc = PBSE_PROTOCOL;
+      }
+    }
+  else if (reply->brp_choice &&
+           reply->brp_choice != BATCH_REPLY_CHOICE_Text &&
+           reply->brp_choice != BATCH_REPLY_CHOICE_Queue)
+    {
+    rc = PBSE_PROTOCOL;
+    if (reply->brp_choice == BATCH_REPLY_CHOICE_Text)
+      {
+      *msg = memmgr_strdup(mm, reply->brp_un.brp_txt.brp_str, &tmp_size);
+      }
+    else
+      {
+      if (connection[connect].ch_errtxt == NULL)
+        {
+        *msg = memmgr_strdup(mm, connection[connect].ch_errtxt, &tmp_size);
+        }
+      }
+    }
+  else if (connection[connect].ch_errno == 0)
+    {
+    *job_id = memmgr_strdup(mm, reply->brp_un.brp_jid, &tmp_size);
+    }
+    
+  pthread_mutex_unlock(connection[connect].ch_mutex);
+
+  PBSD_FreeReply(reply);
+
+  return rc;
+  }  /* END PBSD_queuejob() */
+
+/* END PBSD_submit.c */
