@@ -98,12 +98,36 @@
 #include "batch_request.h"
 #include "dis.h"
 
-int
-decode_DIS_Authen(int sock, struct batch_request *preq)
+int decode_DIS_Authen(
+    
+  int                   sock,
+  struct batch_request *preq)
+
   {
   int rc;
 
   preq->rq_ind.rq_authen.rq_port = disrui(sock, &rc);
   return rc;
-  }
+  } /* END decode_DIS_Authen() */
+
+
+int decode_DIS_AltAuthen(
+    
+  int                   sock,
+  struct batch_request *preq)
+
+  {
+  int rc;
+
+  preq->rq_ind.rq_authen.rq_port = disrui(sock, &rc);
+  if (rc != 0)
+    {
+    return(rc);
+    }
+
+  rc = disrfst(sock, PBS_MAXCREDENTIAL_LEN, preq->rq_ind.rq_authen.rq_cred);
+
+  return rc;
+  } /* END decode_DIS_AltAuthen() */
+
 
