@@ -100,16 +100,21 @@
 #include "batch_request.h"
 #include "dis.h"
 
-int
-decode_DIS_MoveJob(int sock, struct batch_request *preq)
+int decode_DIS_MoveJob(
+    
+  int                   sock, 
+  struct batch_request *preq)
+
   {
   int rc;
 
-  rc = disrfst(sock, PBS_MAXSVRJOBID + 1, preq->rq_ind.rq_move.rq_jid);
+  rc = disrfst(sock, PBS_MAXSVRJOBID, preq->rq_ind.rq_move.rq_jid);
 
   if (rc) return rc;
 
-  rc = disrfst(sock, PBS_MAXDEST + 1, preq->rq_ind.rq_move.rq_destin);
+  rc = disrfst(sock,
+      (PBS_MAXSVRJOBID > PBS_MAXDEST ? PBS_MAXSVRJOBID : PBS_MAXDEST),
+      preq->rq_ind.rq_move.rq_destin);
 
   return rc;
   }
