@@ -99,14 +99,17 @@
 #include "batch_request.h"
 #include "dis.h"
 
-int
-decode_DIS_MessageJob(int sock, struct batch_request *preq)
+int decode_DIS_MessageJob(
+    
+  int                   sock,
+  struct batch_request *preq)
+
   {
   int rc;
 
   preq->rq_ind.rq_message.rq_text = 0;
 
-  rc = disrfst(sock, PBS_MAXSVRJOBID + 1, preq->rq_ind.rq_message.rq_jid);
+  rc = disrfst(sock, PBS_MAXSVRJOBID, preq->rq_ind.rq_message.rq_jid);
 
   if (rc) return rc;
 
