@@ -103,8 +103,11 @@
 #include "batch_request.h"
 #include "dis.h"
 
-int
-decode_DIS_Register(int sock, struct batch_request *preq)
+int decode_DIS_Register(
+    
+  int                   sock,
+  struct batch_request *preq)
+
   {
   int   rc;
 
