@@ -98,8 +98,11 @@
 #include "batch_request.h"
 #include "dis.h"
 
-int
-decode_DIS_ShutDown(int sock, struct batch_request *preq)
+int decode_DIS_ShutDown(
+    
+  int                   sock,
+  struct batch_request *preq)
+
   {
   int rc;
 
