@@ -99,16 +99,19 @@
 #include "batch_request.h"
 #include "dis.h"
 
-int
-decode_DIS_SignalJob(int sock, struct batch_request *preq)
+int decode_DIS_SignalJob(
+    
+  int                   sock,
+  struct batch_request *preq)
+
   {
   int rc;
 
-  rc = disrfst(sock, PBS_MAXSVRJOBID + 1, preq->rq_ind.rq_signal.rq_jid);
+  rc = disrfst(sock, PBS_MAXSVRJOBID, preq->rq_ind.rq_signal.rq_jid);
 
   if (rc) return rc;
 
-  rc = disrfst(sock, PBS_SIGNAMESZ + 1, preq->rq_ind.rq_signal.rq_signame);
+  rc = disrfst(sock, PBS_SIGNAMESZ, preq->rq_ind.rq_signal.rq_signame);
 
   return rc;
   }
