@@ -99,15 +99,18 @@
 #include "batch_request.h"
 #include "dis.h"
 
-int
-decode_DIS_Status(int sock, struct batch_request *preq)
+int decode_DIS_Status(
+    
+  int                   sock,
+  struct batch_request *preq)
+
   {
   int rc;
 
   CLEAR_HEAD(preq->rq_ind.rq_status.rq_attr);
   rc = disrfst(sock,
-               (PBS_MAXSVRJOBID > PBS_MAXDEST ? PBS_MAXSVRJOBID : PBS_MAXDEST) + 1,
-               preq->rq_ind.rq_status.rq_id);
+        (PBS_MAXSVRJOBID > PBS_MAXDEST ? PBS_MAXSVRJOBID : PBS_MAXDEST),
+        preq->rq_ind.rq_status.rq_id);
 
   if (rc) return rc;
 
