@@ -101,12 +101,15 @@
 #include "batch_request.h"
 #include "dis.h"
 
-int
-decode_DIS_TrackJob(int sock, struct batch_request *preq)
+int decode_DIS_TrackJob(
+    
+  int                   sock, 
+  struct batch_request *preq)
+
   {
   int rc;
 
-  rc = disrfst(sock, PBS_MAXSVRJOBID + 1, preq->rq_ind.rq_track.rq_jid);
+  rc = disrfst(sock, PBS_MAXSVRJOBID, preq->rq_ind.rq_track.rq_jid);
 
   if (rc) return rc;
 
@@ -114,7 +117,7 @@ decode_DIS_TrackJob(int sock, struct batch_request *preq)
 
   if (rc) return rc;
 
-  rc = disrfst(sock, PBS_MAXDEST + 1, preq->rq_ind.rq_track.rq_location);
+  rc = disrfst(sock, PBS_MAXDEST, preq->rq_ind.rq_track.rq_location);
 
   if (rc) return rc;
 
