@@ -108,12 +108,14 @@
 #include "libpbs.h"
 #include "dis.h"
 
-int
-decode_DIS_attrl(int sock, struct attrl **ppatt)
+int decode_DIS_attrl(
+    
+  int            sock,
+  struct attrl **ppatt)
+
   {
   int   hasresc;
   unsigned int  i;
-  unsigned int  name_len;
   unsigned int  numpat;
 
   struct attrl  *pat = NULL;
@@ -128,14 +130,13 @@ decode_DIS_attrl(int sock, struct attrl **ppatt)
 
   for (i = 0; i < numpat; ++i)
     {
-
-    name_len = disrui(sock, &rc); /* name_len is unusued here */
+    disrui(sock, &rc); /* name_len is unusued here */
 
     if (rc) break;
 
-    pat = malloc(sizeof(struct attrl));
+    pat = calloc(1, sizeof(struct attrl));
 
-    if (pat == 0)
+    if (pat == NULL)
       return DIS_NOMALLOC;
 
     pat->next     = (struct attrl *)0;
