@@ -181,7 +181,7 @@ int decode_DIS_replyCmd(
 
     case BATCH_REPLY_CHOICE_Commit:
 
-      if ((rc = disrfst(sock, PBS_MAXSVRJOBID + 1, reply->brp_un.brp_jid)))
+      if ((rc = disrfst(sock, PBS_MAXSVRJOBID, reply->brp_un.brp_jid)))
         {
         return(rc);
         }
@@ -205,7 +205,7 @@ int decode_DIS_replyCmd(
 
       while (ct--)
         {
-        psel = (struct brp_select *)malloc(sizeof(struct brp_select));
+        psel = (struct brp_select *)calloc(1, sizeof(struct brp_select));
 
         if (psel == NULL)
           {
@@ -214,9 +214,7 @@ int decode_DIS_replyCmd(
 
         psel->brp_next = NULL;
 
-        psel->brp_jobid[0] = '\0';
-
-        rc = disrfst(sock, PBS_MAXSVRJOBID + 1, psel->brp_jobid);
+        rc = disrfst(sock, PBS_MAXSVRJOBID, psel->brp_jobid);
 
         if (rc)
           {
@@ -249,24 +247,20 @@ int decode_DIS_replyCmd(
 
       while (ct--)
         {
-        pstcmd = (struct brp_cmdstat *)malloc(sizeof(struct brp_cmdstat));
+        pstcmd = (struct brp_cmdstat *)calloc(1, sizeof(struct brp_cmdstat));
 
         if (pstcmd == NULL)
           {
           return(DIS_NOMALLOC);
           }
 
-        pstcmd->brp_stlink = NULL;
-
-        pstcmd->brp_objname[0] = '\0';
-
-        pstcmd->brp_attrl = NULL;
-
         pstcmd->brp_objtype = disrui(sock, &rc);
 
         if (rc == 0)
           {
-          rc = disrfst(sock, PBS_MAXSVRJOBID + 1, pstcmd->brp_objname);
+          rc = disrfst(sock,
+              (PBS_MAXSVRJOBID > PBS_MAXDEST ? PBS_MAXSVRJOBID:PBS_MAXDEST),
+              pstcmd->brp_objname);
           }
 
         if (rc)
@@ -307,7 +301,7 @@ int decode_DIS_replyCmd(
 
       /* Locate Job Reply */
 
-      rc = disrfst(sock, PBS_MAXDEST + 1, reply->brp_un.brp_locate);
+      rc = disrfst(sock, PBS_MAXDEST, reply->brp_un.brp_locate);
 
       break;
 
@@ -327,13 +321,13 @@ int decode_DIS_replyCmd(
 
       reply->brp_un.brp_rescq.brq_number = ct;
 
-      reply->brp_un.brp_rescq.brq_avail  = (int *)malloc(ct * sizeof(int));
+      reply->brp_un.brp_rescq.brq_avail  = (int *)calloc(ct, sizeof(int));
 
-      reply->brp_un.brp_rescq.brq_alloc  = (int *)malloc(ct * sizeof(int));
+      reply->brp_un.brp_rescq.brq_alloc  = (int *)calloc(ct, sizeof(int));
 
-      reply->brp_un.brp_rescq.brq_resvd  = (int *)malloc(ct * sizeof(int));
+      reply->brp_un.brp_rescq.brq_resvd  = (int *)calloc(ct, sizeof(int));
 
-      reply->brp_un.brp_rescq.brq_down   = (int *)malloc(ct * sizeof(int));
+      reply->brp_un.brp_rescq.brq_down   = (int *)calloc(ct, sizeof(int));
 
       if ((reply->brp_un.brp_rescq.brq_avail == NULL) ||
           (reply->brp_un.brp_rescq.brq_alloc == NULL) ||
