@@ -111,8 +111,11 @@
 #include "attribute.h"
 #include "dis.h"
 
-int
-decode_DIS_svrattrl(int sock, tlist_head *phead)
+int decode_DIS_svrattrl(
+    
+  int         sock,
+  tlist_head *phead)
+
   {
   unsigned int i;
   unsigned int hasresc;
@@ -137,7 +140,7 @@ decode_DIS_svrattrl(int sock, tlist_head *phead)
 
     tsize = sizeof(svrattrl) + data_len;
 
-    if ((psvrat = (svrattrl *)malloc(tsize)) == 0)
+    if ((psvrat = (svrattrl *)calloc(1, tsize)) == 0)
       return DIS_NOMALLOC;
 
     CLEAR_LINK(psvrat->al_link);
