@@ -90,8 +90,13 @@
 #include "pbs_error.h"
 #include "dis.h"
 
-int
-encode_DIS_JobCred(int sock, int type, char *cred, int len)
+int encode_DIS_JobCred(
+    
+  int   sock,
+  int   type, 
+  char *cred,
+  int   len)
+
   {
   int   rc;
 
