@@ -93,8 +93,15 @@
 #include "pbs_error.h"
 #include "dis.h"
 
-int
-encode_DIS_JobFile(int sock, int seq, char *buf, int len, char *jobid, int which)
+int encode_DIS_JobFile(
+    
+  int   sock,
+  int   seq,
+  char *buf,
+  int   len,
+  char *jobid,
+  int   which)
+
   {
   int   rc;
 
