@@ -93,8 +93,11 @@
 #include "libpbs.h"
 #include "dis.h"
 
-int
-encode_DIS_JobId(int sock, char *jobid)
+int encode_DIS_JobId(
+    
+  int   sock,
+  char *jobid)
+
   {
   return (diswst(sock, jobid));
   }
