@@ -90,8 +90,14 @@
 #include "pbs_error.h"
 #include "dis.h"
 
-int
-encode_DIS_Manage(int sock, int command, int objtype, char *objname, struct attropl *aoplp)
+int encode_DIS_Manage(
+    
+  int             sock,
+  int             command,
+  int             objtype,
+  char           *objname,
+  struct attropl *aoplp)
+
   {
   int   rc;
 
