@@ -91,8 +91,13 @@
 #include "pbs_error.h"
 #include "dis.h"
 
-int
-encode_DIS_MessageJob(int sock, char *jobid, int fileopt, char *msg)
+int encode_DIS_MessageJob(
+    
+  int   sock,
+  char *jobid,
+  int   fileopt,
+  char *msg)
+
   {
   int   rc;
 
