@@ -102,8 +102,11 @@
 #include "batch_request.h"
 #include "dis.h"
 
-int
-encode_DIS_Register(int sock, struct batch_request *preq)
+int encode_DIS_Register(
+    
+  int                   sock,
+  struct batch_request *preq)
+
   {
   int   rc;
 
