@@ -91,8 +91,11 @@
 #include "dis.h"
 
 
-int
-encode_DIS_ReqHdr(int sock, int reqt, char *user)
+int encode_DIS_ReqHdr(
+    
+  int   sock,
+  int   reqt,
+  char *user)
   {
   int rc;
 
