@@ -91,8 +91,12 @@
 #include "pbs_error.h"
 #include "dis.h"
 
-int
-encode_DIS_RunJob(int sock, char *jobid, char *where, unsigned int resch)
+int encode_DIS_RunJob(
+    
+  int           sock,
+  char         *jobid,
+  char         *where,
+  unsigned int  resch)
   {
   int   rc;
 
