@@ -93,14 +93,15 @@
 
 int encode_DIS_Status(
 
-  int sock,
-  char *objid,
+  int           sock,
+  char         *objid,
   struct attrl *pattrl)
 
   {
   int rc;
 
-  if ((rc = diswst(sock, objid) != 0) || (rc = encode_DIS_attrl(sock, pattrl) != 0))
+  if ((rc = diswst(sock, objid) != 0) ||
+      (rc = encode_DIS_attrl(sock, pattrl) != 0))
     {
     return(rc);
     }
