@@ -101,8 +101,11 @@
 #include "dis.h"
 
 
-int
-encode_DIS_TrackJob(int sock, struct batch_request *preq)
+int encode_DIS_TrackJob(
+    
+  int                   sock,
+  struct batch_request *preq)
+
   {
   int   rc;
 
