@@ -107,7 +107,11 @@
 #include "dis.h"
 
 
-int encode_DIS_svrattrl(int sock, svrattrl *psattl)
+int encode_DIS_svrattrl(
+    
+  int       sock,
+  svrattrl *psattl)
+
   {
   unsigned int ct = 0;
   unsigned int name_len;
