@@ -88,18 +88,18 @@
 #include <stdlib.h>
 #include "libpbs.h"
 
-/* returns pbs_errno */
+/* returns pbs_error codes */
 
 int pbs_alterjob_asyncflag(
 
-  int           c,       /* I */
-  char         *jobid,   /* I */
-  struct attrl *attrib,  /* I */
-  char         *extend,  /* I */
-  int           asyncFlag)  /* I */
+  int           c,           /* I */
+  char         *jobid,       /* I */
+  struct attrl *attrib,      /* I */
+  char         *extend,      /* I */
+  int           asyncFlag,   /* I */
+  int          *local_errno) /* O */
 
   {
-
   struct attropl *ap = (struct attropl *)NULL;
 
   struct attropl *ap1 = (struct attropl *)NULL;
@@ -108,9 +108,7 @@ int pbs_alterjob_asyncflag(
 
   if ((c < 0) || (jobid == NULL) || (*jobid == '\0'))
     {
-    pbs_errno = PBSE_IVALREQ;
-
-    return(pbs_errno);
+    return(PBSE_IVALREQ);
     }
 
   /* copy the attrl to an attropl */
@@ -131,9 +129,7 @@ int pbs_alterjob_asyncflag(
       {
       /* FAILURE */
 
-      pbs_errno = PBSE_SYSTEM;
-
-      return(-1);
+      return(PBSE_SYSTEM);
       }
 
     ap->name = attrib->name;
@@ -141,8 +137,9 @@ int pbs_alterjob_asyncflag(
     ap->resource = attrib->resource;
     ap->value = attrib->value;
 
-    /* attributes now must have an op specified. Inside TORQUE this defaults
-     * to SET, ensuring the same behavior as before */
+    /* attributes now need an op specified. Within TORQUE this defaults
+     * to SET, ensuring the same functionality as before */
+
     ap->op = attrib->op;
     ap->next = (struct attropl *)NULL;
     attrib = attrib->next;
@@ -155,7 +152,8 @@ int pbs_alterjob_asyncflag(
         MGR_OBJ_JOB,
         jobid,
         ap1,
-        extend);
+        extend,
+        local_errno);
 
   /* free up the attropl we just created */
 
@@ -171,26 +169,65 @@ int pbs_alterjob_asyncflag(
   return(i);
   }  /* END pbs_alterjob_asyncflag() */
 
-int pbs_alterjob_async(
+int pbs_alterjob_async_err(
 
-  int           c,       /* I */
-  char         *jobid,   /* I */
-  struct attrl *attrib,  /* I */
-  char         *extend)  /* I */
+  int           c,           /* I */
+  char         *jobid,       /* I */
+  struct attrl *attrib,      /* I */
+  char         *extend,      /* I */
+  int          *local_errno) /* O */
 
   {
-  return(pbs_alterjob_asyncflag(c, jobid, attrib, extend, TRUE));
+  return(pbs_alterjob_asyncflag(c, jobid, attrib, extend, TRUE, local_errno));
+  }  /* END pbs_alterjob_async_err() */
+
+
+
+
+
+int pbs_alterjob_async(
+
+  int           c,           /* I */
+  char         *jobid,       /* I */
+  struct attrl *attrib,      /* I */
+  char         *extend)      /* I */
+
+  {
+  pbs_errno = 0;
+
+  return(pbs_alterjob_asyncflag(c, jobid, attrib, extend, TRUE, &pbs_errno));
   }  /* END pbs_alterjob_async() */
 
+
+
+
+
+int pbs_alterjob_err(
+
+  int           c,           /* I */
+  char         *jobid,       /* I */
+  struct attrl *attrib,      /* I */
+  char         *extend,      /* I */
+  int          *local_errno) /* O */
+
+  {
+  return(pbs_alterjob_asyncflag(c, jobid, attrib, extend, FALSE, local_errno));
+  }  /* END pbs_alterjob_err() */
+
+
+
+
 int pbs_alterjob(
 
-  int           c,       /* I */
-  char         *jobid,   /* I */
-  struct attrl *attrib,  /* I */
-  char         *extend)  /* I */
+  int           c,           /* I */
+  char         *jobid,       /* I */
+  struct attrl *attrib,      /* I */
+  char         *extend)      /* I */
 
   {
-  return(pbs_alterjob_asyncflag(c, jobid, attrib, extend, FALSE));
+  pbs_errno = 0;
+
+  return(pbs_alterjob_asyncflag(c, jobid, attrib, extend, FALSE, &pbs_errno));
   }  /* END pbs_alterjob() */
 
 /* END pbsD_alterjo.c */
