@@ -88,30 +88,31 @@
 #include "libpbs.h"
 #include "dis.h"
 
-int pbs_asyrunjob(
+int pbs_asyrunjob_err(
 
   int   c,
   char *jobid,    /* I */
   char *location,
-  char *extend)
+  char *extend,
+  int  *local_errno)
 
   {
   int                 rc;
 
   struct batch_reply *reply;
   unsigned int        resch = 0;
-  int               sock;
+  int                 sock;
 
   if ((c < 0) || (jobid == NULL) || (*jobid == '\0'))
     {
-    pbs_errno = PBSE_IVALREQ;
-
-    return(pbs_errno);
+    return(PBSE_IVALREQ);
     }
 
   if (location == NULL)
     location = "";
 
+  pthread_mutex_lock(connection[c].ch_mutex);
+
   sock = connection[c].ch_socket;
 
   /* setup DIS support routines for following DIS calls */
@@ -126,28 +127,49 @@ int pbs_asyrunjob(
     {
     connection[c].ch_errtxt = strdup(dis_emsg[rc]);
 
-    pbs_errno = PBSE_PROTOCOL;
+    pthread_mutex_unlock(connection[c].ch_mutex);
 
-    return(pbs_errno);
+    return(PBSE_PROTOCOL);
     }
 
   if (DIS_tcp_wflush(sock))
     {
-    pbs_errno = PBSE_PROTOCOL;
+    pthread_mutex_unlock(connection[c].ch_mutex);
 
-    return(pbs_errno);
+    return(PBSE_PROTOCOL);
     }
 
   /* get reply */
 
-  reply = PBSD_rdrpy(c);
+  reply = PBSD_rdrpy(local_errno, c);
 
   rc = connection[c].ch_errno;
 
+  pthread_mutex_unlock(connection[c].ch_mutex);
+
   PBSD_FreeReply(reply);
 
   return(rc);
-  }  /* END pbs_asyrunjob() */
+  }  /* END pbs_asyrunjob_err() */
+
+
+
+
+int pbs_asyrunjob(
+
+  int   c,
+  char *jobid,    /* I */
+  char *location,
+  char *extend)
+
+  {
+  pbs_errno = 0;
+
+  return(pbs_asyrunjob_err(c, jobid, location, extend, &pbs_errno));
+  } /* END pbs_asyrunjob() */
+
+
+
 
 /* END pbsD_asyrun.c */
 
