@@ -96,7 +96,6 @@
 #include <stdio.h>
 #include <pwd.h>
 #include <string.h>
-#include <ctype.h>
 #include <signal.h>
 #include <netdb.h>
 #include <unistd.h>
@@ -120,10 +119,13 @@
 #include "csv.h"
 #include "dis.h"
 #include "net_connect.h"
-#include "log.h"
-
-
+#include "log.h" /* log */
+#include "../Liblog/log_event.h" /* log_event */
+#include "../Libnet/lib_net.h" /* socket_* */
+#include "../Libifl/lib_ifl.h" /* AUTH_TYPE_IFF, DIS_* */
+#include "pbs_constants.h" /* LOCAL_IP */
 
+#define LOCAL_LOG_BUF 1024
 #define CNTRETRYDELAY 5
 #define MUNGE_SIZE 256 /* I do not know what the proper size of this should be. My 
                           testing with munge shows it creates a string of 128 bytes */
@@ -131,8 +133,6 @@
 
 /* NOTE:  globals, must not impose per connection constraints */
 
-static uid_t pbs_current_uid;               /* only one uid per requestor */
-
 extern time_t pbs_tcp_timeout;              /* source? */
 
 static unsigned int dflt_port = 0;
@@ -175,6 +175,7 @@ void empty_alarm_handler(int signo)
  */
 
 char *pbs_get_server_list(void)
+
   {
   FILE *fd;
   char *pn;
@@ -217,7 +218,7 @@ char *pbs_get_server_list(void)
         strcat(server_list, ",");
         strcat(server_list, tmp);
         len = strlen(server_list);
-        if(server_list[len-1] == '\n')
+        if (server_list[len-1] == '\n')
           {
           server_list[len-1] = '\0';
           }
@@ -227,19 +228,28 @@ char *pbs_get_server_list(void)
       }
     else
       {
-      strncpy(server_list, server, sizeof(server_list));
+      snprintf(server_list, sizeof(server_list), "%s", server);
       }
 
     got_dflt = TRUE;
     }  /* END if (got_dflt != TRUE) */
 
   return(server_list);
+  } /* END pbs_get_server_list() */
+
+
+
+void get_port_from_server_name_file(unsigned int *server_name_file_port)
+  {
+  char *the_server = pbs_default();
+
+  if (the_server != NULL)
+    {
+    PBS_get_server(the_server, server_name_file_port);
+    }
   }
 
 
-
-
-
 /**
  * The routine is called to get the name of the primary
  * server.  It can possibly trigger reading of the server name
@@ -252,11 +262,12 @@ char *pbs_get_server_list(void)
  */
 
 char *pbs_default(void)
+
   {
   char *cp;
 
   pbs_get_server_list();
-  server_name[0] = 0;
+  server_name[0] = '\0';
   cp = csv_nth(server_list, 0); /* get the first item from list */
 
   if (cp)
@@ -266,7 +277,7 @@ char *pbs_default(void)
     }
 
   return(server_name);
-  }
+  } /* END pbs_default() */
 
 
 
@@ -284,6 +295,7 @@ char *pbs_default(void)
  */
 
 char *pbs_fbserver(void)
+
   {
   char *cp;
 
@@ -298,11 +310,11 @@ char *pbs_fbserver(void)
     }
 
   return(server_name);
-  }
+  } /* END pbs_fbserver() */
 
 
 
-static char *PBS_get_server(
+char *PBS_get_server(
 
   char         *server,  /* I (NULL|'\0' for not set,modified) */
   unsigned int *port)    /* O */
@@ -323,7 +335,7 @@ static char *PBS_get_server(
     dflt_port = get_svrport(
                   PBS_BATCH_SERVICE_NAME,
                   "tcp",
-                  PBS_BATCH_SERVICE_PORT_DIS);
+                  PBS_BATCH_SERVICE_PORT);
     }
 
   /* first, get the "net.address[:port]" into 'server_name' */
@@ -337,7 +349,7 @@ static char *PBS_get_server(
     }
   else
     {
-    strncpy(server_name, server, PBS_MAXSERVERNAME);
+    snprintf(server_name, sizeof(server_name), "%s", server);
     }
 
   /* now parse out the parts from 'server_name' */
@@ -370,7 +382,6 @@ int PBSD_munge_authenticate(
   int handle) /* I */
 
   {
-  static char        *id = "PBSD_munge_authenticate";
   int                 rc;
 
   int                 fd;
@@ -380,7 +391,8 @@ int PBSD_munge_authenticate(
   char               *ptr; /* pointer to the current place to copy data into munge_buf */
   int                 bytes_read;
   int                 total_bytes_read = 0;
-
+  int                 local_errno = 0;
+  
   /* user id and name stuff */
   struct passwd      *pwent;
   uid_t               myrealuid;
@@ -388,13 +400,14 @@ int PBSD_munge_authenticate(
   unsigned short      user_port = 0;
   struct sockaddr_in  sockname;
   socklen_t           socknamelen = sizeof(sockname);
-  
-  snprintf(munge_command, sizeof(munge_command), "munge -n 2>/dev/null");
-  
-  memset(munge_buf, 0, sizeof(munge_buf));
-  ptr = munge_buf;
 
-  if ((munge_pipe = popen(munge_command, "r")) == NULL)
+  snprintf(munge_command,sizeof(munge_command),
+    "munge -n 2>/dev/null");
+
+  memset(munge_buf, 0, MUNGE_SIZE);
+  ptr = munge_buf; 
+
+  if ((munge_pipe = popen(munge_command,"r")) == NULL)
     {
     /* FAILURE */
     return(-1);
@@ -413,19 +426,19 @@ int PBSD_munge_authenticate(
   if (bytes_read == -1)
     {
     /* read failed */
-    log_err(errno, id, "error reading pipe in PBSD_munge_authenticate");
-
+    local_errno = errno;
+    log_err(local_errno, __func__, "error reading pipe in PBSD_munge_authenticate");
     return(-1);
     }
-
+  
   /* if we got no bytes back then Munge may not be installed etc. */
   if (total_bytes_read == 0)
     {
     return(PBSE_MUNGE_NOT_FOUND);
     }
-  
+
   /* We got the certificate. Now make the PBS_BATCH_AltAuthenUser request */
-  myrealuid = getuid();
+  myrealuid = getuid();  
   pwent = getpwuid(myrealuid);
   
   rc = getsockname(psock, (struct sockaddr *)&sockname, &socknamelen);
@@ -439,7 +452,7 @@ int PBSD_munge_authenticate(
   user_port = ntohs(sockname.sin_port);
   
   DIS_tcp_setup(psock);
-  
+
   if ((rc = encode_DIS_ReqHdr(psock, PBS_BATCH_AltAuthenUser, pwent->pw_name)) ||
       (rc = diswui(psock, user_port)) ||
       (rc = diswst(psock, munge_buf)) ||
@@ -452,14 +465,12 @@ int PBSD_munge_authenticate(
   else
     {
     /* read the reply */
-    if ((reply = PBSD_rdrpy(handle)) != NULL)
+    if ((reply = PBSD_rdrpy(&local_errno, handle)) != NULL)
       free(reply);
-
+  
     return(PBSE_NONE);
     }
-
-  return(0);  
-  }
+  } /* END PBSD_munge_authenticate() */
 #endif /* ifdef MUNGE_AUTH */
 
 
@@ -468,164 +479,133 @@ int PBSD_munge_authenticate(
  */
 
 #ifndef MUNGE_AUTH
-int PBSD_authenticate(
 
-  int psock)  /* I */
+int parse_daemon_response(long long code, long long len, char *buf)
+  {
+  int rc = PBSE_NONE;
+  if (code == 0)
+    {
+    /* Success */
+    }
+  else
+    {
+    fprintf(stderr, "Error code - %lld : message [%s]\n", code, buf);
+    rc = code;
+    }
+  return rc;
+  }
+
+int get_parent_client_socket(int psock, int *pcsock)
+  {
+  int rc = PBSE_NONE;
+  struct sockaddr_in sockname;
+  socklen_t socknamelen = sizeof(sockname);
+  if (getsockname(psock, (struct sockaddr *)&sockname, &socknamelen) < 0)
+    {
+    rc = PBSE_SOCKET_FAULT;
+    fprintf(stderr, "iff: cannot get sockname for socket %d, errno=%d (%s)\n", psock, errno, strerror(errno));
+    }
+  else
+    {
+    *pcsock = ntohs(sockname.sin_port);
+    }
+  return rc;
+  }
+
+int validate_socket(
+
+  int psock)
 
   {
-  char   cmd[PBS_MAXSERVERNAME + 80];
-  int    cred_type;
-  int    i;
-  int    j;
-  FILE *piff;
-  char  *ptr;
-  char  *path_copy;
-  char  *token;
+  int            rc = PBSE_NONE;
+  static char    id[] = "validate_socket";
+  char           tmp_buf[LOCAL_LOG_BUF];
+  char           write_buf[1024];
+  char          *read_buf = NULL;
+  long long      read_buf_len = 0;
+  uid_t          myrealuid;
+  int            local_socket = 0;
+  int            parent_client_socket = 0;
+  struct passwd *pwent;
+  char          *err_msg = NULL;
+  char          *l_server = NULL;
+  int            l_server_len = 0;
+  unsigned short af_family;
+  long long      code = -1;
+  int            write_buf_len = 0;
+  int            local_errno;
 
-  struct stat buf;
+  myrealuid = getuid();
 
-  static char iffpath[1024];
-
-  int    rc;
-
-  /* use pbs_iff to authenticate me */
-
-  if (iffpath[0] == '\0')
+  if ((pwent = getpwuid(myrealuid)) == NULL)
     {
-    if ((ptr = getenv("PBSBINDIR")) != NULL)
+    snprintf(tmp_buf, LOCAL_LOG_BUF, "cannot get account info: uid %d, errno %d (%s)\n", (int)myrealuid, errno, strerror(errno));
+    log_event(PBSEVENT_ADMIN,PBS_EVENTCLASS_SERVER,id,tmp_buf);
+    }
+  else if ((rc = get_hostaddr_hostent_af(&local_errno, AUTH_IP, &af_family, &l_server, &l_server_len)) != PBSE_NONE)
+    {
+    }
+  else if ((rc = get_parent_client_socket(psock, &parent_client_socket)) != PBSE_NONE)
+    {
+    }
+  else
+    {
+    /* format is:
+     * trq_system|trq_port|Validation_type|user|psock|
+     */
+    sprintf(write_buf, "%d|%s|%d|%d|%d|%s|%d|", (int)strlen(server_name), server_name, server_port, AUTH_TYPE_IFF, (int)strlen(pwent->pw_name), pwent->pw_name, parent_client_socket);
+    /*
+     * total_length|val
+     */
+    write_buf_len = strlen(write_buf);
+    if ((local_socket = socket_get_tcp()) <= 0)
       {
-      snprintf(iffpath, sizeof(iffpath), "%s/pbs_iff",
-               ptr);
+      fprintf(stderr, "socket_get_tcp error\n");
+      rc = PBSE_SOCKET_FAULT;
+      }
+    else if ((rc = socket_connect(&local_socket, l_server, l_server_len, AUTH_PORT, AF_INET, 0, &err_msg)) != PBSE_NONE)
+      {
+      fprintf(stderr, "socket_connect error (VERIFY THAT trqauthd IS RUNNING)\n");
+      }
+    else if ((rc = socket_write(local_socket, write_buf, write_buf_len)) != write_buf_len)
+      {
+      rc = PBSE_SOCKET_WRITE;
+      fprintf(stderr, "socket_write error\n");
+      }
+    else if ((rc = socket_read_num(local_socket, &code)) != PBSE_NONE)
+      {
+      fprintf(stderr, "socket_read_num error\n");
+      }
+    else if ((rc = socket_read_str(local_socket, &read_buf, &read_buf_len)) != PBSE_NONE)
+      {
+      fprintf(stderr, "socket_read_str error\n");
+      }
+    else if ((rc = parse_daemon_response(code, read_buf_len, read_buf)) != PBSE_NONE)
+      {
+      fprintf(stderr, "parse_daemon_response error\n");
       }
     else
       {
-      strcpy(iffpath, IFF_PATH);
-      }
-
-    rc = stat(iffpath, &buf);
-
-    if (rc == -1)
-      {
-      /* cannot locate iff in default location - search PATH */
-
-      if ((ptr = getenv("PATH")) != NULL)
+      if (getenv("PBSDEBUG"))
         {
-        ptr = strtok(ptr, ":");
-
-        if ((path_copy = strdup(ptr)) == NULL)
-          {
-          fprintf(stderr,"Cannot allocate memory, FAILURE");
-
-          return(-1);
-          }
-
-        token = strtok(path_copy, ":");
-
-        while (token != NULL)
-          {
-          snprintf(iffpath, sizeof(iffpath), "%s/pbs_iff", token);
-
-          rc = stat(iffpath, &buf);
-
-          if (rc != -1)
-            break;
-
-          token = strtok(NULL, ":");
-          }  /* END while (token != NULL) */
-        }    /* END if ((ptr = getenv("PATH")) != NULL) */
-
-      if (rc == -1)
-        {
-        /* FAILURE */
-
-        if (getenv("PBSDEBUG"))
-          {
-          fprintf(stderr, "ALERT:  cannot verify file '%s', errno=%d (%s)\n",
-                  cmd,
-                  errno,
-                  strerror(errno));
-          }
-
-        /* cannot locate iff in default location or  PATH */
-
-        iffpath[0] = '\0';
-
-        return(PBSE_IFF_NOT_FOUND);
+        fprintf(stderr, "%s : Connection authorized (server socket %d)\n", id, parent_client_socket);
         }
       }
-    }    /* END if (iffpath[0] == '\0') */
-
-  snprintf(cmd, sizeof(cmd), "%s %s %u %d",
-           iffpath,
-           server_name,
-           server_port,
-           psock);
-
-  piff = popen(cmd, "r");
-
-  if (piff == NULL)
-    {
-    /* FAILURE */
-
-    if (getenv("PBSDEBUG"))
-      {
-      fprintf(stderr, "ALERT:  cannot open pipe, errno=%d (%s)\n",
-              errno,
-              strerror(errno));
-      }
-
-    return(-1);
     }
-
-  i = read(fileno(piff), &cred_type, sizeof(int));
-
-  if ((i != sizeof(int)) || (cred_type != PBS_credentialtype_none))
+  if (rc != PBSE_NONE)
     {
-    /* FAILURE */
-
-    if (getenv("PBSDEBUG"))
+    if (err_msg != NULL)
       {
-      if (i != sizeof(int))
-        {
-        fprintf(stderr, "ALERT:  cannot read pipe, rc=%d, errno=%d (%s)\n",
-                i,
-                errno,
-                strerror(errno));
-        }
-      else
-        {
-        fprintf(stderr, "ALERT:  invalid cred type %d reported\n",
-                cred_type);
-        }
+      fprintf(stderr, "Error in connection to trqauthd (%d)-[%s]\n", rc, err_msg);
       }
-
-    pclose(piff);
-
-    return(-1);
-    }  /* END if ((i != sizeof(int)) || ...) */
-
-  j = pclose(piff);
-
-  if (j != 0)
-    {
-    /* FAILURE */
-
-    if (getenv("PBSDEBUG"))
-      {
-      fprintf(stderr, "ALERT:  cannot close pipe, errno=%d (%s)\n",
-              errno,
-              strerror(errno));
-      }
-
-    /* report failure but do not fail (CRI) */
-
-    /* return(-1); */
     }
+  if (err_msg != NULL)
+    free(err_msg);
+  if (read_buf != NULL)
+    free(read_buf);
+  return rc;
+  }
 
-  /* SUCCESS */
-
-  return(0);
-  }  /* END PBSD_authenticate() */
 #endif /* ifndef MUNGE_AUTH */
 
 
@@ -679,28 +659,29 @@ ssize_t    send_unix_creds(int sd)
  * Return 0 on success. Non-zero on failure
  */
 #define MAX_IFS 64
-        
-int trq_set_preferred_network_interface(char *if_name, struct sockaddr *preferred_addr)
+
+int trq_set_preferred_network_interface(
+    
+  char            *if_name,
+  struct sockaddr *preferred_addr)
+
   {
-	struct ifreq *ifr, *ifend;
-	struct ifreq ifreqx;
+	struct ifreq *ifr;
+  struct ifreq *ifend;
+	struct ifreq  ifreqx;
 	struct ifconf ifc;
-	struct ifreq ifs[MAX_IFS];
-	int sockfd;
+	struct ifreq  ifs[MAX_IFS];
+	int           sockfd;
 
 	/* make sure we have a valid name for the interface */
-  if(if_name == NULL)
+  if ((if_name == NULL) ||
+      (preferred_addr == NULL))
 		return(PBSE_IVALREQ);	
 
-  if(preferred_addr == NULL)
-    {
-    return(PBSE_IVALREQ);
-    }
-    
   memset(preferred_addr, 0, sizeof(struct sockaddr));
 
 	sockfd = socket(AF_INET, SOCK_DGRAM, 0);
-	if(sockfd < 0)
+	if (sockfd < 0)
     {
     /* We can't do this without a socket */
     return(PBSE_SYSTEM);
@@ -710,6 +691,7 @@ int trq_set_preferred_network_interface(char *if_name, struct sockaddr *preferre
 	ifc.ifc_req = ifs;
 	if (ioctl(sockfd, SIOCGIFCONF, &ifc) < 0)
     {
+    close(sockfd);
     return(PBSE_SYSTEM);
     }
 
@@ -719,22 +701,28 @@ int trq_set_preferred_network_interface(char *if_name, struct sockaddr *preferre
     {
     if (ifr->ifr_addr.sa_family == AF_INET)
       {
-      if(!strncmp(if_name, ifr->ifr_name, IF_NAMESIZE))
+      if (!strncmp(if_name, ifr->ifr_name, IF_NAMESIZE))
         {
-        strncpy(ifreqx.ifr_name, ifr->ifr_name,sizeof(ifreqx.ifr_name));
+        snprintf(ifreqx.ifr_name, sizeof(ifreqx.ifr_name), "%s", ifr->ifr_name);
 
-        if(ioctl(sockfd, SIOCGIFADDR, &ifreqx) < 0)
+        if (ioctl(sockfd, SIOCGIFADDR, &ifreqx) < 0)
         	{
         	close(sockfd);
         	return(PBSE_SYSTEM);
           }
+
         /* get the address */
         memcpy(preferred_addr, &ifreqx.ifr_ifru.ifru_addr, sizeof(struct sockaddr));
         }
       }
     }
-    return(PBSE_NONE);
-  }
+
+  close(sockfd);
+
+  return(PBSE_NONE);
+  } /* END trq_set_preferred_network_interface() */
+
+
 
 
 /* returns socket descriptor or negative value (-1) on failure */
@@ -752,17 +740,18 @@ int pbs_original_connect(
   {
   struct sockaddr_in server_addr;
   char *if_name;
-  struct hostent *hp;
+  struct addrinfo *addr_info;
   int out;
   int i;
-  int auth;
   int rc;
+  int local_errno;
 
   struct sockaddr preferred_addr; /* set if TRQ_IFNAME set in torque.cfg */
   struct passwd *pw;
   int use_unixsock = 0;
-#ifdef ENABLE_UNIX_SOCKETS
+  uid_t pbs_current_uid;
 
+#ifdef ENABLE_UNIX_SOCKETS
   struct sockaddr_un unserver_addr;
   char hnamebuf[256];
 #endif
@@ -775,32 +764,38 @@ int pbs_original_connect(
 
   for (i = 1;i < NCONNECTS;i++)
     {
-    if (connection[i].ch_inuse)
-      continue;
+    if (connection[i].ch_mutex == NULL)
+      {
+      connection[i].ch_mutex = calloc(1, sizeof(pthread_mutex_t));
+      pthread_mutex_init(connection[i].ch_mutex,NULL);
+      }
 
-    out = i;
+    pthread_mutex_lock(connection[i].ch_mutex);
 
-    connection[out].ch_inuse  = 1;
+    if (connection[i].ch_inuse == FALSE)
+      {
+      out = i;
+      
+      connection[out].ch_inuse  = TRUE;
+      connection[out].ch_errno  = 0;
+      connection[out].ch_socket = -1;
+      connection[out].ch_errtxt = NULL;
 
-    connection[out].ch_errno  = 0;
+      break;
+      }
 
-    connection[out].ch_socket = -1;
-
-    connection[out].ch_errtxt = NULL;
-
-    break;
+    pthread_mutex_unlock(connection[i].ch_mutex);
     }
 
   if (out < 0)
     {
-    pbs_errno = PBSE_NOCONNECTS;
-
     if (getenv("PBSDEBUG"))
       fprintf(stderr, "ALERT:  cannot locate free channel\n");
 
     /* FAILURE */
+    /* no need to unlock mutex here - in this case no connection was found */
 
-    return(-1);
+    return(PBSE_NOCONNECTS * -1);
     }
 
   /* get server host and port */
@@ -809,13 +804,14 @@ int pbs_original_connect(
 
   if (server == NULL)
     {
-    connection[out].ch_inuse = 0;
-    pbs_errno = PBSE_NOSERVER;
+    connection[out].ch_inuse = FALSE;
+
+    pthread_mutex_unlock(connection[out].ch_mutex);
 
     if (getenv("PBSDEBUG"))
       fprintf(stderr, "ALERT:  PBS_get_server() failed\n");
 
-    return(-1);
+    return(PBSE_NOSERVER * -1);
     }
 
   /* determine who we are */
@@ -824,15 +820,15 @@ int pbs_original_connect(
 
   if ((pw = getpwuid(pbs_current_uid)) == NULL)
     {
-    pbs_errno = PBSE_SYSTEM;
-
     if (getenv("PBSDEBUG"))
       {
       fprintf(stderr, "ALERT:  cannot get password info for uid %ld\n",
               (long)pbs_current_uid);
       }
 
-    return(-1);
+    pthread_mutex_unlock(connection[out].ch_mutex);
+
+    return(PBSE_SYSTEM * -1);
     }
 
   strcpy(pbs_current_user, pw->pw_name);
@@ -856,6 +852,7 @@ int pbs_original_connect(
   if (use_unixsock)
     {
     connection[out].ch_socket = socket(AF_UNIX, SOCK_STREAM, 0);
+    DIS_tcp_init(connection[out].ch_socket);
 
     if (connection[out].ch_socket < 0)
       {
@@ -866,9 +863,9 @@ int pbs_original_connect(
                 strerror(errno));
         }
 
-      connection[out].ch_inuse = 0;
+      connection[out].ch_inuse = FALSE;
 
-      pbs_errno = PBSE_PROTOCOL;
+      local_errno = PBSE_PROTOCOL;
 
       use_unixsock = 0;
       }
@@ -888,8 +885,8 @@ int pbs_original_connect(
       {
       close(connection[out].ch_socket);
 
-      connection[out].ch_inuse = 0;
-      pbs_errno = errno;
+      connection[out].ch_inuse = FALSE;
+      local_errno = errno;
 
       if (getenv("PBSDEBUG"))
         {
@@ -915,8 +912,8 @@ int pbs_original_connect(
 
       close(connection[out].ch_socket);
 
-      connection[out].ch_inuse = 0;
-      pbs_errno = PBSE_PROTOCOL;
+      connection[out].ch_inuse = FALSE;
+      local_errno = PBSE_PROTOCOL;
 
       use_unixsock = 0;  /* will try again with inet socket */
       }
@@ -931,6 +928,7 @@ int pbs_original_connect(
      * try */
 
     connection[out].ch_socket = socket(AF_INET, SOCK_STREAM, 0);
+    DIS_tcp_init(connection[out].ch_socket);
 
     if (connection[out].ch_socket < 0)
       {
@@ -942,40 +940,20 @@ int pbs_original_connect(
                 strerror(errno));
         }
 
-      connection[out].ch_inuse = 0;
+      connection[out].ch_inuse = FALSE;
 
-      pbs_errno = PBSE_PROTOCOL;
+      pthread_mutex_unlock(connection[out].ch_mutex);
 
-      return(-1);
-      }
-
-    server_addr.sin_family = AF_INET;
-
-    hp = NULL;
-    hp = gethostbyname(server);
-
-    if (hp == NULL)
-      {
-      close(connection[out].ch_socket);
-      connection[out].ch_inuse = 0;
-      pbs_errno = PBSE_BADHOST;
-
-      if (getenv("PBSDEBUG"))
-        {
-        fprintf(stderr, "ERROR:  cannot get servername (%s) errno=%d (%s)\n",
-                (server != NULL) ? server : "NULL",
-                errno,
-                strerror(errno));
-        }
-
-      return(PBSE_SYSTEM);
+      return(PBSE_PROTOCOL * -1);
       }
 
+    /* This is probably an IPv4 solution for the if_name and preferred_addr
+       We need to see what ioctl call we need for IPv6 */
     if_name = trq_get_if_name();
-    if(if_name)
+    if (if_name)
       {
       rc = trq_set_preferred_network_interface(if_name, &preferred_addr);
-      if(rc != PBSE_NONE)
+      if (rc != PBSE_NONE)
         {
         fprintf(stderr, "could not set preferred network interface (%s): %d\n",
                 if_name, rc);
@@ -985,11 +963,11 @@ int pbs_original_connect(
         }
 
       rc = bind(connection[out].ch_socket, &preferred_addr, sizeof(struct sockaddr));
-      if(rc < 0)
+      if (rc < 0)
         {
         fprintf(stderr, "ERROR: could not bind preferred network interface (%s): errno: %d",
                 if_name, errno);
-        if(if_name)
+        if (if_name)
           free(if_name);
         return(PBSE_SYSTEM);
         }
@@ -997,10 +975,31 @@ int pbs_original_connect(
 
     /* we are done with if_name at this point. trq_get_if_name allocated
        space for it. We need to free it */
-    if(if_name)
+    if (if_name)
       free(if_name);
 
-    memcpy((char *)&server_addr.sin_addr, hp->h_addr_list[0], hp->h_length);
+    server_addr.sin_family = AF_INET;
+
+    if (getaddrinfo(server, NULL, NULL, &addr_info) != 0)
+      {
+      close(connection[out].ch_socket);
+      connection[out].ch_inuse = FALSE;
+
+      if (getenv("PBSDEBUG"))
+        {
+        fprintf(stderr, "ERROR:  cannot get servername (%s) errno=%d (%s)\n",
+                (server != NULL) ? server : "NULL",
+                errno,
+                strerror(errno));
+        }
+
+      pthread_mutex_unlock(connection[out].ch_mutex);
+
+      return(PBSE_BADHOST * -1);
+      }
+
+    server_addr.sin_addr = ((struct sockaddr_in *)addr_info->ai_addr)->sin_addr;
+    freeaddrinfo(addr_info);
 
     server_addr.sin_port = htons(server_port);
 
@@ -1011,8 +1010,7 @@ int pbs_original_connect(
       {
       close(connection[out].ch_socket);
 
-      connection[out].ch_inuse = 0;
-      pbs_errno = errno;
+      connection[out].ch_inuse = FALSE;
 
       if (getenv("PBSDEBUG"))
         {
@@ -1021,7 +1019,9 @@ int pbs_original_connect(
                 strerror(errno));
         }
 
-      return(-1);
+      pthread_mutex_unlock(connection[out].ch_mutex);
+
+      return(errno * -1);
       }
 
     /* FIXME: is this necessary?  Contributed by one user that fixes a problem,
@@ -1035,70 +1035,61 @@ int pbs_original_connect(
 #endif
 
 #ifdef MUNGE_AUTH
-    auth = PBSD_munge_authenticate(connection[out].ch_socket, out);
-    if(auth != 0)
+    rc = PBSD_munge_authenticate(connection[out].ch_socket, out);
+
+    if (rc != 0)
       {
       close(connection[out].ch_socket);
 
-      connection[out].ch_inuse = 0;
+      connection[out].ch_inuse = FALSE;
 
-      if (auth == PBSE_MUNGE_NOT_FOUND)
-      {
-      pbs_errno = PBSE_MUNGE_NOT_FOUND;
-      
-      if (getenv("PBSDEBUG"))
+      if (rc == PBSE_MUNGE_NOT_FOUND)
         {
-        fprintf(stderr, "ERROR:  cannot find munge executable\n");
+        local_errno = PBSE_MUNGE_NOT_FOUND;
+        
+        if (getenv("PBSDEBUG"))
+          {
+          fprintf(stderr, "ERROR:  cannot find munge executable\n");
+          }
         }
-      }
       else
         {
-        pbs_errno = PBSE_PERM;
+        local_errno = PBSE_PERM;
         
         if (getenv("PBSDEBUG"))
           {
           fprintf(stderr, "ERROR:  cannot authenticate connection to server \"%s\", errno=%d (%s)\n",
-                  server,
-                  errno,
-                  strerror(errno));
+            server,
+            errno,
+            strerror(errno));
           }
         }
 
-      return(-1);
+      pthread_mutex_unlock(connection[out].ch_mutex);
+
+      return(-1 * local_errno);
       }
     
     
 #else  
-    /* Have pbs_iff authenticate connection */
-    if ((ENABLE_TRUSTED_AUTH == FALSE) && ((auth = PBSD_authenticate(connection[out].ch_socket)) != 0))
+    /* new version of iff using daemon */
+    if ((ENABLE_TRUSTED_AUTH == FALSE) && ((rc = validate_socket(connection[out].ch_socket)) != PBSE_NONE))
       {
       close(connection[out].ch_socket);
 
-      connection[out].ch_inuse = 0;
+      connection[out].ch_inuse = FALSE;
 
-      if (auth == PBSE_IFF_NOT_FOUND)
+      if (getenv("PBSDEBUG"))
         {
-        pbs_errno = PBSE_IFF_NOT_FOUND;
-        
-        if (getenv("PBSDEBUG"))
-          {
-          fprintf(stderr, "ERROR:  cannot find pbs_iif executable\n");
-          }
+        fprintf(stderr, "ERROR:  cannot authenticate connection to server \"%s\", errno=%d (%s)\n",
+                server, rc, pbs_strerror(rc));
         }
-      else
-        {
-        pbs_errno = PBSE_PERM;
+      
+      local_errno = PBSE_SOCKET_FAULT;
 
-        if (getenv("PBSDEBUG"))
-          {
-          fprintf(stderr, "ERROR:  cannot authenticate connection to server \"%s\", errno=%d (%s)\n",
-                  server,
-                  errno,
-                  strerror(errno));
-          }
-        }
+      pthread_mutex_unlock(connection[out].ch_mutex);
 
-      return(-1);
+      return(-1 * local_errno);
       }
 #endif /* ifdef MUNGE_AUTH */
 
@@ -1122,47 +1113,76 @@ int pbs_original_connect(
     pbs_tcp_timeout = 10800;      /* set for 3 hour time out */
     }
 
+  pthread_mutex_unlock(connection[out].ch_mutex);
+
   return(out);
   }  /* END pbs_original_connect() */
 
 
 
+int pbs_disconnect_socket(
+
+  int sock)  /* I (socket descriptor) */
+
+  {
+  char tmp_buf[THE_BUF_SIZE / 4];
+
+  DIS_tcp_setup(sock);
+
+  if ((encode_DIS_ReqHdr(sock, PBS_BATCH_Disconnect, pbs_current_user) == 0) &&
+      (DIS_tcp_wflush(sock) == 0))
+    {
+    int atime;
+    struct sigaction act;
+    struct sigaction oldact;
+
+    /* set alarm to break out of potentially infinite read */
+/*    act.sa_handler = SIG_IGN; */
+    act.sa_handler = empty_alarm_handler;  /* need SOME handler or blocking read never gets interrupted */
+
+    sigemptyset(&act.sa_mask);
+    act.sa_flags = 0;
+    sigaction(SIGALRM, &act, &oldact);
+    atime = alarm(5);
+
+    while (1)
+      {
+      /* wait for server to close connection */
+      /* NOTE:  if read of 'sock' is blocking, request below may hang forever
+         -- hence the signal handler above */
+      if (read(sock, &tmp_buf, sizeof(tmp_buf)) < 1)
+        break;
+      }
+
+    alarm(atime);
+    sigaction(SIGALRM, &oldact, NULL);
+    }
+
+  close(sock);
+  return(0);
+  }  /* END pbs_disconnect() */
 
 
 int pbs_disconnect(
 
-  int connect)  /* I (socket descriptor) */
+  int connect)  /* I (location in connection array) */
 
   {
   int  sock;
-  int  rc;
+
+  pthread_mutex_lock(connection[connect].ch_mutex);
 
   /* send close-connection message */
-
   sock = connection[connect].ch_socket;
 
-  DIS_tcp_setup(sock);
-
-  rc = encode_DIS_ReqHdr(sock, PBS_BATCH_Disconnect, pbs_current_user);
-  if(rc)
-    {
-    return(rc);
-    }
-
-   rc = DIS_tcp_wflush(sock);
-   if(rc)
-     {
-     return(rc);
-     }
-    
-  close(sock);
+  pbs_disconnect_socket(sock);
 
   if (connection[connect].ch_errtxt != (char *)NULL)
     free(connection[connect].ch_errtxt);
 
   connection[connect].ch_errno = 0;
-
-  connection[connect].ch_inuse = 0;
+  connection[connect].ch_inuse = FALSE;
+  pthread_mutex_unlock(connection[connect].ch_mutex);
 
   return(0);
   }  /* END pbs_disconnect() */
@@ -1181,7 +1201,10 @@ int pbs_disconnect(
  * @returns A file descriptor number.
  */
 
-int pbs_connect(char *server_name_ptr)    /* I (optional) */
+int pbs_connect(
+    
+  char *server_name_ptr)    /* I (optional) */
+
   {
   int connect = -1;
   int i, list_len;
@@ -1195,9 +1218,7 @@ int pbs_connect(char *server_name_ptr)    /* I (optional) */
 
   if (server_name_ptr && server_name_ptr[0])
     {
-    strncpy(server_name_list, server_name_ptr, sizeof(server_name_list) - 1);
-    strcat(server_name_list, ",");
-    strncat(server_name_list, pbs_get_server_list(), sizeof(server_name_list) -1 - strlen(server_name_ptr) - 1);
+    snprintf(server_name_list, sizeof(server_name_list), "%s", server_name_ptr);
 
     if (getenv("PBSDEBUG"))
       {
@@ -1207,7 +1228,7 @@ int pbs_connect(char *server_name_ptr)    /* I (optional) */
     }
   else
     {
-    strncpy(server_name_list, pbs_get_server_list(), sizeof(server_name_list) - 1);
+    snprintf(server_name_list, sizeof(server_name_list), "%s", pbs_get_server_list());
 
     if (getenv("PBSDEBUG"))
       {
@@ -1224,11 +1245,8 @@ int pbs_connect(char *server_name_ptr)    /* I (optional) */
 
     if (tp && tp[0])
       {
-      /* Trim any leading space */
-      while(isspace(*tp)) tp++;
-
       memset(current_name, 0, sizeof(current_name));
-      strncpy(current_name, tp, sizeof(current_name) - 1);
+      snprintf(current_name, sizeof(current_name), "%s", tp);
 
       if (getenv("PBSDEBUG"))
         {
@@ -1240,8 +1258,7 @@ int pbs_connect(char *server_name_ptr)    /* I (optional) */
         {
         if (getenv("PBSDEBUG"))
           {
-          fprintf(stderr, "pbs_connect: Successful connection to server \"%s\", fd = %d\n",
-                  current_name, connect);
+          fprintf(stderr, "pbs_connect: Successful connection to server \"%s\", fd = %d\n", current_name, connect);
           }
 
         return(connect);  /* Success, we have a connection, return it. */
@@ -1250,7 +1267,7 @@ int pbs_connect(char *server_name_ptr)    /* I (optional) */
     }
 
   return(connect);
-  }
+  } /* END pbs_connect() */
 
 
 /**
@@ -1261,7 +1278,11 @@ int pbs_connect(char *server_name_ptr)    /* I (optional) */
  * @param retry_seconds The period of time for which retrys should be attempted.
  * @returns A file descriptor number.
  */
-int pbs_connect_with_retry(char *server_name_ptr, int retry_seconds)
+int pbs_connect_with_retry(
+    
+  char *server_name_ptr, /* I */
+  int   retry_seconds)   /* I */
+
   {
   int n_times_to_try = retry_seconds / CNTRETRYDELAY;
   int connect = -1;
@@ -1279,13 +1300,30 @@ int pbs_connect_with_retry(char *server_name_ptr, int retry_seconds)
   }
 
 
-int
-pbs_query_max_connections(void)
+
+
+int pbs_query_max_connections(void)
 
   {
   return(NCONNECTS - 1);
   }
 
 
+
+
+void initialize_connections_table()
+
+  {
+  int i;
+
+  for (i = 1;i < PBS_NET_MAX_CONNECTIONS;i++)
+    {
+    connection[i].ch_mutex = calloc(1, sizeof(pthread_mutex_t));
+    pthread_mutex_init(connection[i].ch_mutex, NULL);
+    }
+  } /* END initialize_connections_table() */
+
+
+
 /* END pbsD_connect.c */
 
