@@ -87,17 +87,20 @@
 #include <stdio.h>
 #include "libpbs.h"
 
-int pbs_holdjob(c, jobid, holdtype, extend)
-int c;
-char *jobid;
-char *holdtype;
-char *extend;
+int pbs_holdjob_err(
+
+  int c,
+  char *jobid,
+  char *holdtype,
+  char *extend,
+  int  *local_errno)
+
   {
 
   struct attropl aopl;
 
   if ((jobid == (char *)0) || (*jobid == '\0'))
-    return (pbs_errno = PBSE_IVALREQ);
+    return(PBSE_IVALREQ);
 
   aopl.name = ATTR_h;
 
@@ -117,5 +120,28 @@ char *extend;
                       MGR_OBJ_JOB,
                       jobid,
                       &aopl,
-                      extend);
-  }
+                      extend,
+                      local_errno);
+  } /* END pbs_holdjob_err() */
+
+
+
+
+
+
+int pbs_holdjob(
+
+  int c,
+  char *jobid,
+  char *holdtype,
+  char *extend)
+
+  {
+  pbs_errno = 0;
+
+  return(pbs_holdjob_err(c, jobid, holdtype, extend, &pbs_errno));
+  } /* END pbs_holdjob() */
+
+
+
+
