@@ -89,8 +89,13 @@
 #include "dis.h"
 #include "log.h"
 
-char *
-pbs_locjob(int c, char *jobid, char *extend)
+char *pbs_locjob_err(
+    
+  int   c,
+  char *jobid,
+  char *extend,
+  int  *local_errno)
+
   {
   int rc;
 
@@ -101,10 +106,12 @@ pbs_locjob(int c, char *jobid, char *extend)
 
   if ((jobid == (char *)0) || (*jobid == '\0'))
     {
-    pbs_errno = PBSE_IVALREQ;
-    return (ploc);
+    *local_errno = PBSE_IVALREQ;
+    return(NULL);
     }
 
+  pthread_mutex_lock(connection[c].ch_mutex);
+
   sock = connection[c].ch_socket;
 
   /* setup DIS support routines for following DIS calls */
@@ -116,25 +123,30 @@ pbs_locjob(int c, char *jobid, char *extend)
       (rc = encode_DIS_ReqExtend(sock, extend)))
     {
     connection[c].ch_errtxt = strdup(dis_emsg[rc]);
-    pbs_errno = PBSE_PROTOCOL;
-    return (char *)NULL;
+
+    pthread_mutex_unlock(connection[c].ch_mutex);
+
+    *local_errno = PBSE_PROTOCOL;
+    return(NULL);
     }
 
   /* write data over tcp stream */
 
   if (DIS_tcp_wflush(sock))
     {
-    pbs_errno = PBSE_PROTOCOL;
-    return (char *)NULL;
+    pthread_mutex_unlock(connection[c].ch_mutex);
+
+    *local_errno = PBSE_PROTOCOL;
+    return(NULL);
     }
 
   /* read reply from stream */
 
-  reply = PBSD_rdrpy(c);
+  reply = PBSD_rdrpy(local_errno,c);
 
   if (reply == NULL)
     {
-    pbs_errno = PBSE_PROTOCOL;
+    *local_errno = PBSE_PROTOCOL;
     }
   else if (reply->brp_choice != BATCH_REPLY_CHOICE_NULL &&
            reply->brp_choice != BATCH_REPLY_CHOICE_Text &&
@@ -143,7 +155,7 @@ pbs_locjob(int c, char *jobid, char *extend)
 #ifndef NDEBUG
     fprintf(stderr, "advise: pbs_locjob\tUnexpected reply choice\n\n");
 #endif /* NDEBUG */
-    pbs_errno = PBSE_PROTOCOL;
+    *local_errno = PBSE_PROTOCOL;
     }
   else if (connection[c].ch_errno == 0)
     {
@@ -152,5 +164,26 @@ pbs_locjob(int c, char *jobid, char *extend)
 
   PBSD_FreeReply(reply);
 
-  return ploc;
-  }
+  pthread_mutex_unlock(connection[c].ch_mutex);
+
+  return(ploc);
+  } /* END pbs_locjob_err() */
+
+
+
+
+char *pbs_locjob(
+    
+  int   c,
+  char *jobid,
+  char *extend)
+
+  {
+  pbs_errno = 0;
+
+  return(pbs_locjob_err(c, jobid, extend, &pbs_errno));
+  } /* END pbs_locjob() */
+
+
+
+
