@@ -85,21 +85,28 @@
 #include "libpbs.h"
 #include "dis.h"
 
-int
-pbs_movejob(int c, char *jobid, char *destin, char *extend)
+int pbs_movejob_err(
+    
+  int   c,
+  char *jobid,
+  char *destin,
+  char *extend,
+  int  *local_errno)
+
   {
-  int      rc;
+  int                 rc;
 
   struct batch_reply *reply;
-  int      sock;
-
+  int                 sock;
 
   if ((jobid == (char *)0) || (*jobid == '\0'))
-    return (pbs_errno = PBSE_IVALREQ);
+    return (PBSE_IVALREQ);
 
   if (destin == (char *)0)
     destin = "";
 
+  pthread_mutex_lock(connection[c].ch_mutex);
+
   sock = connection[c].ch_socket;
 
   /* setup DIS support routines for following DIS calls */
@@ -111,19 +118,49 @@ pbs_movejob(int c, char *jobid, char *destin, char *extend)
       (rc = encode_DIS_ReqExtend(sock, extend)))
     {
     connection[c].ch_errtxt = strdup(dis_emsg[rc]);
-    return (pbs_errno = PBSE_PROTOCOL);
+
+    pthread_mutex_unlock(connection[c].ch_mutex);
+
+    return(PBSE_PROTOCOL);
     }
 
   if (DIS_tcp_wflush(sock))
     {
-    return (pbs_errno = PBSE_PROTOCOL);
+    pthread_mutex_unlock(connection[c].ch_mutex);
+
+    return(PBSE_PROTOCOL);
     }
 
   /* read reply */
 
-  reply = PBSD_rdrpy(c);
+  reply = PBSD_rdrpy(local_errno, c);
 
   PBSD_FreeReply(reply);
 
-  return connection[c].ch_errno;
-  }
+  rc = connection[c].ch_errno;
+
+  pthread_mutex_unlock(connection[c].ch_mutex);
+
+  return(rc);
+  } /* END pbs_movejob_err() */
+
+
+
+
+
+int pbs_movejob(
+    
+  int   c,
+  char *jobid,
+  char *destin,
+  char *extend)
+
+  {
+  pbs_errno = 0;
+
+  return(pbs_movejob_err(c, jobid, destin, extend, &pbs_errno));
+  } /* END pbs_movejob() */
+
+
+
+
