@@ -87,18 +87,25 @@
 #include "libpbs.h"
 #include "dis.h"
 
-int
-pbs_orderjob(int c, char *job1, char *job2, char *extend)
-  {
+int pbs_orderjob_err(
+    
+  int   c,
+  char *job1,
+  char *job2,
+  char *extend,
+  int  *local_errno)
 
+  {
   struct batch_reply *reply;
-  int rc;
-  int sock;
+  int                 rc;
+  int                 sock;
 
 
   if ((job1 == (char *)0) || (*job1 == '\0') ||
       (job2 == (char *)0) || (*job2 == '\0'))
-    return (pbs_errno = PBSE_IVALREQ);
+    return (PBSE_IVALREQ);
+
+  pthread_mutex_lock(connection[c].ch_mutex);
 
   sock = connection[c].ch_socket;
 
@@ -111,19 +118,49 @@ pbs_orderjob(int c, char *job1, char *job2, char *extend)
       (rc = encode_DIS_ReqExtend(sock, extend)))
     {
     connection[c].ch_errtxt = strdup(dis_emsg[rc]);
-    return (pbs_errno = PBSE_PROTOCOL);
+
+    pthread_mutex_unlock(connection[c].ch_mutex);
+
+    return (PBSE_PROTOCOL);
     }
 
   if (DIS_tcp_wflush(sock))
     {
-    return (pbs_errno = PBSE_PROTOCOL);
+    pthread_mutex_unlock(connection[c].ch_mutex);
+
+    return (PBSE_PROTOCOL);
     }
 
   /* read reply */
 
-  reply = PBSD_rdrpy(c);
+  reply = PBSD_rdrpy(local_errno, c);
 
   PBSD_FreeReply(reply);
 
-  return connection[c].ch_errno;
-  }
+  rc = connection[c].ch_errno;
+
+  pthread_mutex_unlock(connection[c].ch_mutex);
+
+  return(rc);
+  } /* END pbs_orderjob_err() */
+
+
+
+
+
+int pbs_orderjob(
+    
+  int   c,
+  char *job1,
+  char *job2,
+  char *extend)
+
+  {
+  pbs_errno = 0;
+
+  return(pbs_orderjob_err(c, job1, job2, extend, &pbs_errno));
+  } /* END pbs_orderjob() */
+
+
+
+
