@@ -88,16 +88,23 @@
 #include "libpbs.h"
 #include "dis.h"
 
-int
-pbs_rerunjob(int c, char *jobid, char *extend)
+int pbs_rerunjob_err(
+    
+  int   c,
+  char *jobid,
+  char *extend,
+  int  *local_errno)
+
   {
-  int rc;
+  int                 rc;
 
   struct batch_reply *reply;
-  int sock;
+  int                 sock;
 
   if ((jobid == (char *)0) || (*jobid == '\0'))
-    return (pbs_errno = PBSE_IVALREQ);
+    return (PBSE_IVALREQ);
+
+  pthread_mutex_lock(connection[c].ch_mutex);
 
   sock = connection[c].ch_socket;
 
@@ -110,21 +117,50 @@ pbs_rerunjob(int c, char *jobid, char *extend)
       (rc = encode_DIS_ReqExtend(sock, extend)))
     {
     connection[c].ch_errtxt = strdup(dis_emsg[rc]);
-    return (pbs_errno = PBSE_PROTOCOL);
+
+    pthread_mutex_unlock(connection[c].ch_mutex);
+
+    return (PBSE_PROTOCOL);
     }
 
   /* write data */
 
   if (DIS_tcp_wflush(sock))
     {
-    return (pbs_errno = PBSE_PROTOCOL);
+    pthread_mutex_unlock(connection[c].ch_mutex);
+
+    return (PBSE_PROTOCOL);
     }
 
   /* read reply from stream into presentation element */
 
-  reply = PBSD_rdrpy(c);
+  reply = PBSD_rdrpy(local_errno, c);
 
   PBSD_FreeReply(reply);
 
-  return connection[c].ch_errno;
-  }
+  rc = connection[c].ch_errno;
+
+  pthread_mutex_unlock(connection[c].ch_mutex);
+
+  return(rc);
+  } /* END pbs_rerunjob_err() */
+
+
+
+
+
+int pbs_rerunjob(
+    
+  int   c,
+  char *jobid,
+  char *extend)
+
+  {
+  pbs_errno = 0;
+
+  return(pbs_rerunjob_err(c, jobid, extend, &pbs_errno));
+  } /* END pbs_rerunjob_err() */
+
+
+
+
