@@ -105,16 +105,18 @@ static char *resc_nodes = "nodes";
  *     string resource query
  */
 
-static int encode_DIS_Resc(sock, rlist, ct, rh)
-int       sock;
-char     **rlist;
-int       ct;
-resource_t    rh;
+static int encode_DIS_Resc(
+    
+  int          sock,
+  char       **rlist,
+  int          ct,
+  resource_t   rh)
+  
   {
   int    i;
   int    rc;
 
-  if ((rc = diswsi(sock, rh)) == 0)     /* resource reservation handle */
+  if ((rc = diswsi(sock,rh)) == 0)     /* resource reservation handle */
     {
 
     /* next send the number of resource strings */
@@ -135,12 +137,19 @@ resource_t    rh;
   return rc;
   }
 
-static int
-PBS_resc(int c, int reqtype, char **rescl, int ct, resource_t rh)
+static int PBS_resc(
+    
+  int          c,
+  int          reqtype, 
+  char       **rescl,
+  int          ct,
+  resource_t   rh)
   {
   int rc;
   int sock;
 
+  pthread_mutex_lock(connection[c].ch_mutex);
+
   sock = connection[c].ch_socket;
 
   /* setup DIS support routines for following DIS calls */
@@ -152,16 +161,21 @@ PBS_resc(int c, int reqtype, char **rescl, int ct, resource_t rh)
       (rc = encode_DIS_ReqExtend(sock, (char *)0)))
     {
     connection[c].ch_errtxt = strdup(dis_emsg[rc]);
-    return (pbs_errno = PBSE_PROTOCOL);
+
+    pthread_mutex_unlock(connection[c].ch_mutex);
+
+    return (PBSE_PROTOCOL);
     }
 
+  pthread_mutex_unlock(connection[c].ch_mutex);
+
   if (DIS_tcp_wflush(sock))
     {
-    return (pbs_errno = PBSE_PROTOCOL);
+    return (PBSE_PROTOCOL);
     }
 
   return (0);
-  }
+  } /* END PBS_resc() */
 
 
 
@@ -185,26 +199,31 @@ int pbs_rescquery(
 
   struct batch_reply *reply;
   int                 rc = 0;
+  int                 local_errno = 0;
+
+  pthread_mutex_lock(connection[c].ch_mutex);
 
   if (resclist == NULL)
     {
     connection[c].ch_errno = PBSE_RMNOPARAM;
 
-    pbs_errno = PBSE_RMNOPARAM;
+    pthread_mutex_unlock(connection[c].ch_mutex);
 
-    return(pbs_errno);
+    return(PBSE_RMNOPARAM);
     }
 
   /* send request */
 
   if ((rc = PBS_resc(c, PBS_BATCH_Rescq, resclist, num_resc, (resource_t)0)) != 0)
     {
+    pthread_mutex_unlock(connection[c].ch_mutex);
+
     return(rc);
     }
 
   /* read in reply */
 
-  reply = PBSD_rdrpy(c);
+  reply = PBSD_rdrpy(&local_errno, c);
 
   if (((rc = connection[c].ch_errno) == PBSE_NONE))
     {
@@ -221,6 +240,8 @@ int pbs_rescquery(
 
   PBSD_FreeReply(reply);
 
+  pthread_mutex_unlock(connection[c].ch_mutex);
+
   return(rc);
   }  /* END pbs_rescquery() */
 
@@ -229,41 +250,54 @@ int pbs_rescquery(
 
 /* pbs_reserve() - reserve resources */
 
-int
-pbs_rescreserve(
-  int c,  /* connection */
-  char **rl,  /* list of resources */
-  int num_resc, /* number of items in list */
-  resource_t *prh /* ptr to resource reservation handle */
-)
+int pbs_rescreserve(
+  
+  int          c,        /* connection */
+  char       **rl,       /* list of resources */
+  int          num_resc, /* number of items in list */
+  resource_t  *prh)      /* ptr to resource reservation handle */
+
   {
-  int rc;
+  int                 rc;
+  int                 local_errno = 0;
 
   struct batch_reply *reply;
 
+  pthread_mutex_lock(connection[c].ch_mutex);
+
   if (rl == NULL)
     {
     connection[c].ch_errno = PBSE_RMNOPARAM;
-    return (pbs_errno = PBSE_RMNOPARAM);
+
+    pthread_mutex_unlock(connection[c].ch_mutex);
+
+    return (PBSE_RMNOPARAM);
     }
 
   if (prh == NULL)
     {
     connection[c].ch_errno = PBSE_RMBADPARAM;
-    return (pbs_errno = PBSE_RMBADPARAM);
+
+    pthread_mutex_unlock(connection[c].ch_mutex);
+
+    return (PBSE_RMBADPARAM);
     }
 
   /* send request */
 
   if ((rc = PBS_resc(c, PBS_BATCH_ReserveResc, rl, num_resc, *prh)) != 0)
+    {
+    pthread_mutex_unlock(connection[c].ch_mutex);
+
     return (rc);
+    }
 
   /*
    * now get reply, if reservation successful, the reservation handle,
    * resource_t, is in the  aux field
    */
 
-  reply = PBSD_rdrpy(c);
+  reply = PBSD_rdrpy(&local_errno, c);
 
   if (((rc = connection[c].ch_errno) == PBSE_NONE) ||
       (rc == PBSE_RMPART))
@@ -273,8 +307,10 @@ pbs_rescreserve(
 
   PBSD_FreeReply(reply);
 
+  pthread_mutex_unlock(connection[c].ch_mutex);
+
   return (rc);
-  }
+  } /* END pbs_rescreserve() */
 
 /*
  * pbs_release() - release a resource reservation
@@ -283,23 +319,32 @@ pbs_rescreserve(
  * list is empty.
  */
 
-int
-pbs_rescrelease(int c, resource_t rh)
+int pbs_rescrelease(
+    
+  int        c,
+  resource_t rh)
+
   {
 
   struct batch_reply *reply;
-  int rc;
+  int                 rc;
+  int                 local_errno = 0;
 
   if ((rc = PBS_resc(c, PBS_BATCH_ReleaseResc, (char **)0, 0, rh)) != 0)
     return (rc);
 
-  /* now get reply */
+  pthread_mutex_lock(connection[c].ch_mutex);
 
-  reply = PBSD_rdrpy(c);
+  /* now get reply */
+  reply = PBSD_rdrpy(&local_errno, c);
 
   PBSD_FreeReply(reply);
 
-  return (connection[c].ch_errno);
+  rc = connection[c].ch_errno;
+
+  pthread_mutex_unlock(connection[c].ch_mutex);
+
+  return(rc);
   }
 
 /*
@@ -328,7 +373,11 @@ int totpool(int con, int update)
 
 /* usepool() - return number of nodes in use, includes reserved and down */
 
-int usepool(int con, int update)
+int usepool(
+    
+  int con,
+  int update)
+
   {
   if (update)
     if (pbs_rescquery(con, &resc_nodes, 1, &nodes_avail,
@@ -348,7 +397,11 @@ int usepool(int con, int update)
  * "?" if error in request
  */
 
-char *avail(int con, char *resc)
+char *avail(
+    
+  int   con,
+  char *resc)
+
   {
   int av;
   int al;
