@@ -87,22 +87,8 @@
 #include "libpbs.h"
 #include "dis.h"
 
-
-int pbs_runjob(
-
-  int   c,
-  char *jobid,
-  char *location,
-  char *extend)
-
-  {
-  int rc;
-
-  struct batch_reply   *reply;
-  unsigned int resch = 0;
-  int sock;
-
-  /* NOTE:  routes over to req_runjob() on server side
+  
+/* NOTE:  routes over to req_runjob() on server side
        DIS_tcp_wflush  -----> wait_request()                       (net_server.c)
                                 process_request()                  (server/process_request.c)
                                   dis_request_read()               (server/dis_read.c)
@@ -129,7 +115,7 @@ int pbs_runjob(
       NOTE: stream=sock                           disrul(stream)      (Libdis/disrul.c)
                                                     disrsl_(stream)   (Libdis/disrsl_.c)
       NOTE: fd=stream                                 tcp_getc(fd)    (Libifl/tcp_dis.c)
-                                                        tcp_read(fd)  (Libifl/tcp_dis.c)
+                                                        tcp_read(fd,read_len,avail_len)  (Libifl/tcp_dis.c)
                                                           select(pbs_tcp_timeout)
                                                           read()
                                             svr_disconnect()
@@ -142,13 +128,27 @@ int pbs_runjob(
    *   PBSD_rdrpy      <-----
    */
 
+
+int pbs_runjob_err(
+
+  int   c,
+  char *jobid,
+  char *location,
+  char *extend,
+  int  *local_errno)
+
+  {
+  int                   rc;
+
+  struct batch_reply   *reply;
+  unsigned int          resch = 0;
+  int                   sock;
+
   /* NOTE:  set_task sets WORK_Deferred_Child : request remains until child terminates */
 
   if ((c < 0) || (jobid == NULL) || (*jobid == '\0'))
     {
-    pbs_errno = PBSE_IVALREQ;
-
-    return(pbs_errno);
+    return(PBSE_IVALREQ);
     }
 
   if (location == NULL)
@@ -156,6 +156,8 @@ int pbs_runjob(
     location = "";
     }
 
+  pthread_mutex_lock(connection[c].ch_mutex);
+
   sock = connection[c].ch_socket;
 
   /* setup DIS support routines for following DIS calls */
@@ -170,27 +172,49 @@ int pbs_runjob(
     {
     connection[c].ch_errtxt = strdup(dis_emsg[rc]);
 
-    pbs_errno = PBSE_PROTOCOL;
+    pthread_mutex_unlock(connection[c].ch_mutex);
 
-    return(pbs_errno);
+    return(PBSE_PROTOCOL);
     }
 
   if (DIS_tcp_wflush(sock))
     {
-    pbs_errno = PBSE_PROTOCOL;
+    pthread_mutex_unlock(connection[c].ch_mutex);
 
-    return(pbs_errno);
+    return(PBSE_PROTOCOL);
     }
 
   /* get reply */
 
-  reply = PBSD_rdrpy(c);
+  reply = PBSD_rdrpy(local_errno, c);
 
   rc = connection[c].ch_errno;
 
+  pthread_mutex_unlock(connection[c].ch_mutex);
+
   PBSD_FreeReply(reply);
 
   return(rc);
-  }  /* END pbs_runjob() */
+  }  /* END pbs_runjob_err() */
+
+
+
+
+
+int pbs_runjob(
+
+  int   c,
+  char *jobid,
+  char *location,
+  char *extend)
+
+  {
+  pbs_errno = 0;
+
+  return(pbs_runjob_err(c, jobid, location, extend, &pbs_errno));
+  } /* END pbs_runjob() */
+
+
+
 
 
