@@ -94,18 +94,41 @@
 #include "libpbs.h"
 #include "dis.h"
 
-static int PBSD_select_put A_((int, int, struct attropl *, char *));
-static char **PBSD_select_get A_((int));
+static int PBSD_select_put(int, int, struct attropl *, char *);
+static char **PBSD_select_get(int *, int);
+
+char **pbs_selectjob_err(
+    
+  int             c,
+  struct attropl *attrib,
+  char           *extend,
+  int            *local_errno)
 
-char **
-pbs_selectjob(int c, struct attropl *attrib, char *extend)
   {
-
   if (PBSD_select_put(c, PBS_BATCH_SelectJobs, attrib, extend) == 0)
-    return (PBSD_select_get(c));
+    return (PBSD_select_get(local_errno, c));
   else
     return ((char **)0);
-  }
+  } /* END pbs_selectjob_err() */
+
+
+
+
+char **pbs_selectjob(
+    
+  int             c,
+  struct attropl *attrib,
+  char           *extend)
+
+  {
+  pbs_errno = 0;
+
+  if (PBSD_select_put(c, PBS_BATCH_SelectJobs, attrib, extend) == 0)
+    return (PBSD_select_get(&pbs_errno, c));
+  else
+    return ((char **)0);
+  } /* END pbs_selectjob() */
+
 
 /*
  *  pbs_selstat() - Selectable status
@@ -113,24 +136,53 @@ pbs_selectjob(int c, struct attropl *attrib, char *extend)
  *  criteria.  This is a short-cut combination of pbs_selecljob()
  *  and repeated pbs_statjob().
  */
+struct batch_status * pbs_selstat_err(
+
+  int             c,
+  struct attropl *attrib,
+  char           *extend,
+  int            *local_errno)
 
-struct batch_status *
-      pbs_selstat(int c, struct attropl *attrib, char *extend)
   {
-
   if (PBSD_select_put(c, PBS_BATCH_SelStat, attrib, extend) == 0)
-    return (PBSD_status_get(c));
+    return (PBSD_status_get(local_errno, c));
   else
     return ((struct batch_status *)0);
-  }
+  } /* END pbs_selstat_err() */
 
 
-static int
-PBSD_select_put(int c, int type, struct attropl *attrib, char *extend)
+
+
+struct batch_status * pbs_selstat(
+
+  int             c,
+  struct attropl *attrib,
+  char           *extend)
+
+  {
+  pbs_errno = 0;
+
+  if (PBSD_select_put(c, PBS_BATCH_SelStat, attrib, extend) == 0)
+    return (PBSD_status_get(&pbs_errno, c));
+  else
+    return ((struct batch_status *)0);
+  } /* END pbs_selstat() */
+
+
+
+static int PBSD_select_put(
+
+  int             c,
+  int             type,
+  struct attropl *attrib,
+  char           *extend)
+
   {
   int rc;
   int sock;
 
+  pthread_mutex_lock(connection[c].ch_mutex);
+
   sock = connection[c].ch_socket;
 
   /* setup DIS support routines for following DIS calls */
@@ -142,47 +194,57 @@ PBSD_select_put(int c, int type, struct attropl *attrib, char *extend)
       (rc = encode_DIS_ReqExtend(sock, extend)))
     {
     connection[c].ch_errtxt = strdup(dis_emsg[rc]);
-    return (pbs_errno = PBSE_PROTOCOL);
+
+    pthread_mutex_unlock(connection[c].ch_mutex);
+
+    return (PBSE_PROTOCOL);
     }
 
+  pthread_mutex_unlock(connection[c].ch_mutex);
+
   /* write data */
 
   if (DIS_tcp_wflush(sock))
     {
-    return (pbs_errno = PBSE_PROTOCOL);
+    return(PBSE_PROTOCOL);
     }
 
-  return 0;
-  }
+  return(PBSE_NONE);
+  } /* END pbs_selstat() */
 
 
-static char **
-PBSD_select_get(int c)
+static char **PBSD_select_get(
+
+  int *local_errno,
+  int  c)
+
   {
-  int   i;
+  int                  i;
 
-  struct batch_reply *reply;
-  int   njobs;
-  char *sp;
-  int   stringtot;
-  size_t totsize;
+  struct batch_reply  *reply;
+  int                  njobs;
+  char                *sp;
+  int                  stringtot;
+  size_t               totsize;
 
-  struct brp_select *sr;
-  char **retval = (char **)NULL;
+  struct brp_select   *sr;
+  char               **retval = (char **)NULL;
+
+  pthread_mutex_lock(connection[c].ch_mutex);
 
   /* read reply from stream */
 
-  reply = PBSD_rdrpy(c);
+  reply = PBSD_rdrpy(local_errno, c);
 
   if (reply == NULL)
     {
-    pbs_errno = PBSE_PROTOCOL;
+    *local_errno = PBSE_PROTOCOL;
     }
   else if (reply->brp_choice != 0  &&
            reply->brp_choice != BATCH_REPLY_CHOICE_Text &&
            reply->brp_choice != BATCH_REPLY_CHOICE_Select)
     {
-    pbs_errno = PBSE_PROTOCOL;
+    *local_errno = PBSE_PROTOCOL;
     }
   else if (connection[c].ch_errno == 0)
     {
@@ -207,11 +269,14 @@ PBSD_select_get(int c)
 
     totsize = stringtot + (njobs + 1) * (sizeof(char *));
 
-    retval = (char **)malloc(totsize);
+    retval = (char **)calloc(1, totsize);
 
     if (retval == (char **)NULL)
       {
-      pbs_errno = PBSE_SYSTEM;
+      *local_errno = PBSE_SYSTEM;
+
+      pthread_mutex_unlock(connection[c].ch_mutex);
+
       return (char **)NULL;
       }
 
@@ -230,7 +295,10 @@ PBSD_select_get(int c)
     retval[i] = (char *)NULL;
     }
 
+  pthread_mutex_unlock(connection[c].ch_mutex);
+
   PBSD_FreeReply(reply);
 
-  return retval;
-  }
+  return(retval);
+  } /* END PBSD_select_get() */
+
