@@ -87,21 +87,28 @@
 #include "libpbs.h"
 #include "dis.h"
 
-int
-pbs_stagein(int c, char *jobid, char *location, char *extend)
+int pbs_stagein(
+    
+  int   c,
+  char *jobid,
+  char *location,
+  char *extend)
+
   {
-  int rc;
+  int                   rc;
+  int                   local_errno = 0;
 
   struct batch_reply   *reply;
-  int sock;
+  int                   sock;
 
 
   if ((jobid == (char *)0) || (*jobid == '\0'))
-    return (pbs_errno = PBSE_IVALREQ);
+    return (PBSE_IVALREQ);
 
   if (location == (char *)0)
     location = "";
 
+  pthread_mutex_lock(connection[c].ch_mutex);
 
   sock = connection[c].ch_socket;
 
@@ -116,21 +123,28 @@ pbs_stagein(int c, char *jobid, char *location, char *extend)
       (rc = encode_DIS_ReqExtend(sock, extend)))
     {
     connection[c].ch_errtxt = strdup(dis_emsg[rc]);
-    return (pbs_errno = PBSE_PROTOCOL);
+
+    pthread_mutex_unlock(connection[c].ch_mutex);
+
+    return (PBSE_PROTOCOL);
     }
 
   if (DIS_tcp_wflush(sock))
     {
-    return (pbs_errno = PBSE_PROTOCOL);
+    pthread_mutex_unlock(connection[c].ch_mutex);
+
+    return (PBSE_PROTOCOL);
     }
 
   /* get reply */
 
-  reply = PBSD_rdrpy(c);
+  reply = PBSD_rdrpy(&local_errno, c);
 
   rc = connection[c].ch_errno;
 
+  pthread_mutex_unlock(connection[c].ch_mutex);
+
   PBSD_FreeReply(reply);
 
-  return rc;
+  return(rc);
   }
