@@ -88,16 +88,16 @@
 #include <unistd.h>
 #include "libpbs.h"
 
-char *pbs_submit(
+char *pbs_submit_err(
 
   int             c,
   struct attropl *attrib,
   char           *script,
   char           *destination,
-  char           *extend)  /* (optional) */
+  char           *extend,       /* (optional) */
+  int            *local_errno)
 
   {
-
   struct attropl *pal;
   char * return_jobid = NULL;
 
@@ -107,7 +107,7 @@ char *pbs_submit(
     {
     if (access(script, R_OK) != 0)
       {
-      pbs_errno = PBSE_BADSCRIPT;
+      *local_errno = PBSE_BADSCRIPT;
 
       return(NULL);
       }
@@ -120,7 +120,7 @@ char *pbs_submit(
 
   /* Queue job with null string for job id */
 
-  return_jobid = PBSD_queuejob(c, "", destination, attrib, extend);
+  return_jobid = PBSD_queuejob(c, local_errno, "", destination, attrib, extend);
 
   if (return_jobid == NULL)
     {
@@ -133,7 +133,7 @@ char *pbs_submit(
     {
     if (PBSD_jscript(c, script, NULL) != 0)
       {
-      pbs_errno = PBSE_BADSCRIPT;
+      *local_errno = PBSE_BADSCRIPT;
 
       return(NULL);
       }
@@ -158,7 +158,26 @@ char *pbs_submit(
     }
 
   return(return_jobid);
-  }  /* END pbs_submit() */
+  }  /* END pbs_submit_err() */
+
+
+
+
+
+char *pbs_submit(
+
+  int             c,
+  struct attropl *attrib,
+  char           *script,
+  char           *destination,
+  char           *extend)       /* (optional) */
+
+  {
+  pbs_errno = 0;
+
+  return(pbs_submit_err(c, attrib, script, destination, extend, &pbs_errno));
+  } /* END pbs_submit() */
+
 
 
 /* END pbsD_submit.c */
