@@ -87,21 +87,23 @@
 #include "libpbs.h"
 #include "dis.h"
 
-int pbs_terminate(
+int pbs_terminate_err(
 
-  int   c,      /* I */
-  int   manner, /* I */
-  char *extend) /* I */
+  int   c,           /* I */
+  int   manner,      /* I */
+  char *extend,      /* I */
+  int  *local_errno) /* O */
 
   {
 
   struct batch_reply *reply;
 
-  int rc = 0;
-  int sock;
+  int                 rc = 0;
+  int                 sock;
+
+  pthread_mutex_lock(connection[c].ch_mutex);
 
   /* send request */
-
   sock = connection[c].ch_socket;
 
   /* setup DIS support routines for following DIS calls */
@@ -114,28 +116,45 @@ int pbs_terminate(
     {
     connection[c].ch_errtxt = strdup(dis_emsg[rc]);
 
-    pbs_errno = PBSE_PROTOCOL;
+    pthread_mutex_unlock(connection[c].ch_mutex);
 
-    return(pbs_errno);
+    return(PBSE_PROTOCOL);
     }
 
   if (DIS_tcp_wflush(sock))
     {
-    pbs_errno = PBSE_PROTOCOL;
+    pthread_mutex_unlock(connection[c].ch_mutex);
 
-    return(pbs_errno);
+    return(PBSE_PROTOCOL);
     }
 
   /* read in reply */
-
-  reply = PBSD_rdrpy(c);
+  reply = PBSD_rdrpy(local_errno, c);
 
   rc = connection[c].ch_errno;
 
+  pthread_mutex_unlock(connection[c].ch_mutex);
+
   PBSD_FreeReply(reply);
 
   return(rc);
-  }  /* END pbs_terminate() */
+  }  /* END pbs_terminate_err() */
+
+
+
+
+
+int pbs_terminate(
+
+  int   c,           /* I */
+  int   manner,      /* I */
+  char *extend)      /* I */
+
+  {
+  pbs_errno = 0;
+
+  return(pbs_terminate_err(c, manner, extend, &pbs_errno));
+  } /* END pbs_terminate() */
 
 
 
