@@ -107,12 +107,14 @@
 #include <sys/socket.h>
 #include <sys/time.h>
 #include <netinet/in.h>
+#include <pthread.h>
 #if defined(NTOHL_NEEDS_ARPA_INET_H) && defined(HAVE_ARPA_INET_H)
 #include <arpa/inet.h>
 #endif
 
 #include "rpp.h"
 #include "log.h"
+#include "../Liblog/pbs_log.h"
 
 #if !defined(H_ERRNO_DECLARED) && !defined(_AIX)
 extern int h_errno;
@@ -199,7 +201,6 @@ extern int h_errno;
 
 int RPPTimeOut = DEFAULT_RPP_TIMEOUT;
 int RPPRetry   = DEFAULT_RPP_RETRY;
-int wcommit_state = 0;
 
 char *server_alias = NULL;
 long rpp_throttle_sleeptime = 0;
@@ -209,6 +210,7 @@ long rpp_throttle_sleeptime = 0;
 void rpp_shutdown_on_exit(int, void *);
 
 /* END external prototypes */
+pthread_mutex_t *stream_mutexes = NULL;
 
 
 
@@ -418,6 +420,8 @@ struct stream
   int   recv_attempt; /* number bytes, from start */
   /* of current message, that */
   /* have been read */
+
+  int   being_attended; /* if a thread has been queued to handle this stream */
   };
 
 /*
@@ -446,6 +450,9 @@ static int  open_key = 0;  /* monotonicly increasing */
 static struct send_packet *top = NULL; /* ptrs to beginning and end */
 
 static struct send_packet *bottom = NULL; /* of master send list; */
+
+pthread_mutex_t *master_list_mutex = NULL;
+
 /* All sent date is linked */
 /* between top and bottom. */
 
@@ -466,6 +473,7 @@ int  rpp_fd = -1;
 ** entries.  The value of rpp_fd will be contained in this array.
 */
 int  *rpp_fd_array = NULL;
+/*pthread_mutex_t *rpp_fd_mutex = NULL;*/
 
 /*
 ** Number of elements in rpp_fd_array
@@ -835,8 +843,7 @@ next_seq(int *seq)
 ** Put a human readable representation of a network addres into
 ** a staticly allocated string.
 */
-char *
-netaddr(struct sockaddr_in *ap)
+char * netaddr(struct sockaddr_in *ap)
   {
   static char out[80];
   u_long  ipadd;
@@ -948,6 +955,14 @@ static void rpp_form_pkt(
   I8TOH(crc(pktp->data, (u_long)(len + RPP_PKT_CRC)),
         (char *)&pktp->data[len+RPP_PKT_CRC])
 
+  if (master_list_mutex == NULL)
+    {
+    master_list_mutex = malloc(sizeof(pthread_mutex_t));
+    pthread_mutex_init(master_list_mutex,NULL);
+    }
+
+  pthread_mutex_lock(master_list_mutex);
+
   if (bottom)
     bottom->down = pktp;
 
@@ -958,6 +973,8 @@ static void rpp_form_pkt(
 
   bottom = pktp;
 
+  pthread_mutex_unlock(master_list_mutex);
+
   return;
   }
 
@@ -1060,8 +1077,7 @@ bad:
 ** down to bottom.  Will not cause state change.
 */
 
-static void
-rpp_send_out(void)
+static void rpp_send_out(void)
 
   {
   struct send_packet *pp;
@@ -1074,6 +1090,14 @@ rpp_send_out(void)
 
   curr = time(NULL);
 
+  if (master_list_mutex == NULL)
+    {
+    master_list_mutex = malloc(sizeof(pthread_mutex_t));
+    pthread_mutex_init(master_list_mutex,NULL);
+    }
+
+  pthread_mutex_lock(master_list_mutex);
+
   for (pp = top;pp != NULL;pp = pp->down)
     {
     if ((curr - pp->time_sent) < RPPTimeOut)
@@ -1130,6 +1154,8 @@ rpp_send_out(void)
     pp->sent_out++;
     }  /* END for (pp) */
 
+  pthread_mutex_unlock(master_list_mutex);
+
   /* NOTE:  failure cannot be detected */
 
   return;
@@ -1142,8 +1168,7 @@ rpp_send_out(void)
 ** Create or reuse a position in stream_array.
 */
 
-static int
-rpp_create_sp(void)
+static int rpp_create_sp(void)
 
   {
   int i;
@@ -1162,6 +1187,14 @@ rpp_create_sp(void)
     memset(stream_array, '\0', sizeof(struct stream));
 
     stream_num = 1;
+
+    /* create the mutex */
+    stream_mutexes = (pthread_mutex_t *)malloc(sizeof(pthread_mutex_t));
+
+    if (stream_mutexes == NULL)
+      return(-1);
+    
+    pthread_mutex_init(stream_mutexes,NULL);
     }
 
   for (i = 0;i < stream_num;i++)
@@ -1203,10 +1236,33 @@ rpp_create_sp(void)
         }
 
       stream_num++;
+
+      stream_mutexes = (pthread_mutex_t *)realloc(
+        stream_mutexes,
+        (stream_num) * sizeof(pthread_mutex_t));
+
+      if (stream_mutexes == NULL)
+        return(-1);
+
+      pthread_mutex_init(stream_mutexes + stream_num - 1,NULL);
       }
     else
       {
+      int j;
+
       stream_num *= 2;
+
+      stream_mutexes = (pthread_mutex_t *)realloc(
+          stream_mutexes,
+          (stream_num) * sizeof(pthread_mutex_t));
+
+      if (stream_mutexes == NULL)
+        return(-1);
+
+      for (j = stream_num / 2; j < stream_num; j++)
+        {
+        pthread_mutex_init(stream_mutexes + j,NULL);
+        }
       }
 
     stream_array = sp;
@@ -1390,6 +1446,8 @@ static void dqueue(
   struct send_packet *pp)
 
   {
+  pthread_mutex_lock(master_list_mutex);
+
   if (pp->down == NULL)
     bottom = pp->up;
   else
@@ -1407,8 +1465,10 @@ static void dqueue(
 
   free(pp);
 
+  pthread_mutex_unlock(master_list_mutex);
+
   return;
-  }
+  } /* END dqueue() */
 
 
 
@@ -1454,12 +1514,16 @@ static void clear_send(
       struct send_packet *look; /* might not be */
       /* on send queue */
 
+      pthread_mutex_lock(master_list_mutex);
+
       for (look = top;look;look = look->down)
         {
         if (look == spp)
           break;
         }
 
+      pthread_mutex_unlock(master_list_mutex);
+
       if (look == NULL)
         {
         free(spp->data);
@@ -2105,6 +2169,14 @@ static int rpp_recv_pkt(
         I8TOH(crc(spp->data, (u_long)(len + RPP_PKT_CRC)),
               (char *)&spp->data[len+RPP_PKT_CRC])
 
+        if (master_list_mutex == NULL)
+          {
+          master_list_mutex = malloc(sizeof(pthread_mutex_t));
+          pthread_mutex_init(master_list_mutex,NULL);
+          }
+
+        pthread_mutex_lock(master_list_mutex);
+
         if (bottom)
           bottom->down = spp;
 
@@ -2116,6 +2188,8 @@ static int rpp_recv_pkt(
           top = spp;
 
         bottom = spp;
+
+        pthread_mutex_unlock(master_list_mutex);
         }
 
       break;
@@ -2844,8 +2918,7 @@ struct sockaddr_in *rpp_getaddr(
 ** Free all memory and close the socket.
 */
 
-void
-rpp_terminate(void)
+void rpp_terminate(void)
 
   {
 
@@ -2905,10 +2978,14 @@ rpp_terminate(void)
       }
     }
 
+  pthread_mutex_lock(master_list_mutex);
+
   top = NULL;
 
   bottom = NULL;
 
+  pthread_mutex_unlock(master_list_mutex);
+
   if (stream_array)
     free(stream_array);
 
@@ -3741,8 +3818,6 @@ int rpp_wcommit(
     }
 
   sp = &stream_array[index];
-  
-  wcommit_state = sp->state;
 
   switch (sp->state)
     {
@@ -3913,6 +3988,58 @@ int rpp_skip(
 
 
 
+/**
+ * Mark this stream as being serviced currently
+ */
+void started_servicing(
+
+  int index)
+
+  {
+  pthread_mutex_lock(stream_mutexes+index);
+  stream_array[index].being_attended = TRUE;
+  pthread_mutex_unlock(stream_mutexes+index);
+  } /* END started_servicing() */
+
+
+
+
+
+/**
+ * Mark this stream as no longer being serviced
+ */
+void done_servicing(
+
+  int index)
+
+  {
+  pthread_mutex_lock(stream_mutexes+index);
+  stream_array[index].being_attended = FALSE;
+  pthread_mutex_unlock(stream_mutexes+index);
+  } /* END done_servicing() */
+
+
+
+/**
+ * @return TRUE if this stream is being serviced, false otherwise 
+ */
+int being_serviced(
+
+  int index)
+
+  {
+  int rc;
+  
+  pthread_mutex_lock(stream_mutexes+index);
+  rc = stream_array[index].being_attended;
+  pthread_mutex_unlock(stream_mutexes+index);
+
+  return(rc);
+  }
+
+
+
+
 /*
 ** Check for any stream with a message waiting and
 ** return the stream number or a -1 if there are none.
@@ -3950,7 +4077,8 @@ int rpp_poll(void)
 
   for (i = 0;i < stream_num;i++)
     {
-    if (rpp_attention(i))
+    if ((rpp_attention(i)) &&
+        (!being_serviced(i)))
       break;
     }
 
