@@ -94,23 +94,41 @@
 #endif
 
 #include "dis.h"
-#include "dis_.h"
+#include "../Libdis/lib_dis.h"
 #include "dis_init.h"
 #include "log.h"
+#include "../Liblog/pbs_log.h"
+#include "../Libutils/u_lock_ctl.h"
+#include "../Libnet/lib_net.h" /* socket_* */
 
 #ifdef HAVE_SYS_POLL_H
 #include <sys/poll.h>
 #endif
 
-#define MAX_INT_LEN 256;
-
-static struct tcp_chan **tcparray = NULL;
-static int    tcparraymax = 0;
+#define MAX_SOCKETS 65536
+static struct tcp_chan *tcparray[MAX_SOCKETS] = {NULL};
 
 time_t pbs_tcp_timeout = 20;  /* reduced from 60 to 20 (CRI - Nov/03/2004) */
 
 
 
+void DIS_tcp_init(int sock)
+  {
+  int pos = 0;
+  if (sock > 0)
+    {
+    for (pos = 0; pos < MAX_SOCKETS; pos++)
+      {
+      tcparray[pos] = NULL;
+      }
+    }
+  else
+    {
+    tcparray[pos] = NULL;
+    }
+  }
+
+
 void DIS_tcp_settimeout(
 
   long timeout)  /* I */
@@ -123,19 +141,25 @@ void DIS_tcp_settimeout(
 
 
 
-
-
 int DIS_tcp_istimeout(
 
   int sock)
 
   {
+  int rc;
+
   if (tcparray == NULL)
     {
     return(0);
     }
 
-  return(tcparray[sock]->IsTimeout);
+  pthread_mutex_lock(&(tcparray[sock]->tcp_mutex));
+
+  rc = tcparray[sock]->IsTimeout;
+
+  pthread_mutex_unlock(&(tcparray[sock]->tcp_mutex));
+
+  return(rc);
   }  /* END DIS_tcp_istimeout() */
 
 
@@ -167,6 +191,7 @@ static void tcp_pack_buff(
       {
       *(tp->tdis_thebuf + i) = *(tp->tdis_thebuf + i + start);
       }
+    *(tp->tdis_thebuf + amt) = '\0';
 
     tp->tdis_leadp  -= start;
 
@@ -190,158 +215,125 @@ static void tcp_pack_buff(
  *  -2 if EOF (stream closed)
  */
 
-static int tcp_read(
-
-  int fd)  /* I */
-
+int tcp_read(
+  int fd,
+  long long *read_len,
+  long long *avail_len)
   {
-  long              i;
-  long              total = 0;
+  int               rc = PBSE_NONE;
   unsigned long     newsize;
-  void             *ptr;
-  int               done = 0;
-#ifdef HAVE_POLL
-
-  struct pollfd pollset;
-  int timeout;
-#else
-  fd_set            readset;
-
-  struct timeval    timeout;
-#endif
-
+  char             *ptr;
+  int               tdis_buf_len = 0;
+  int               max_read_len = 0;
+  char             *new_data = NULL;
   struct tcpdisbuf *tp;
+  int               tmp_leadp = 0;
+  int               tmp_trailp = 0;
+  int               tmp_eod = 0;
+  char              err_msg[1024];
 
+
+  pthread_mutex_lock(&(tcparray[fd]->tcp_mutex));
   tp = &tcparray[fd]->readbuf;
 
   /* must compact any uncommitted data into bottom of buffer */
-
   tcp_pack_buff(tp);
 
+  tcparray[fd]->IsTimeout = 0;
+  tcparray[fd]->SelectErrno = 0;
+  tcparray[fd]->ReadErrno = 0;
+  tdis_buf_len = tp->tdis_bufsize;
+  max_read_len = tp->tdis_bufsize - (tp->tdis_eod - tp->tdis_thebuf);
+  pthread_mutex_unlock(&(tcparray[fd]->tcp_mutex));
+
   /*
    * we don't want to be locked out by an attack on the port to
    * deny service, so we time out the read, the network had better
    * deliver promptly
    */
 
-  tcparray[fd]->IsTimeout = 0;
-  tcparray[fd]->SelectErrno = 0;
-  tcparray[fd]->ReadErrno = 0;
-
-  do
+  if ((rc = socket_read(fd, &new_data, read_len)) != PBSE_NONE)
     {
-#ifdef HAVE_POLL
-    /* poll()'s timeout is only a signed int, must be careful not to overflow */
-
-    if (INT_MAX / 1000 > pbs_tcp_timeout)
-      timeout = pbs_tcp_timeout * 1000;
-    else
-      timeout = INT_MAX;
-
-    pollset.fd = fd;
-
-    pollset.events = POLLIN | POLLHUP;
-
-    i = poll(&pollset, 1, timeout);
-
-#else
-    timeout.tv_sec = pbs_tcp_timeout;
-
-    timeout.tv_usec = 0;
-
-    FD_ZERO(&readset);
-
-    FD_SET(fd, &readset);
-
-    i = select(
-          fd + 1,
-          &readset,
-          NULL,
-          NULL,
-          &timeout);
-
-#endif
-    }
-  while ((i == -1) && (errno == EINTR));
-
-  if (i == 0)
-    {
-    /* timeout has occurred */
-
-    tcparray[fd]->IsTimeout = 1;
-
-    return(0);
-    }
-  else if (i < 0)
-    {
-    /* select failed */
-
-    tcparray[fd]->SelectErrno = errno;
-
-    return(-1);
-    }
-
-  while (1)
-    {
-    while ((i = read(
-                  fd,
-                  tp->tdis_eod,
-                  tp->tdis_thebuf + tp->tdis_bufsize - tp->tdis_eod)) == -1)
+    pthread_mutex_lock(&(tcparray[fd]->tcp_mutex));
+    switch (rc)
       {
-      if (errno != EINTR)
+      case PBSE_TIMEOUT:
+        tcparray[fd]->IsTimeout = 1;
+        /* This return 0. This isn't accurate on a timeout */
+/*        rc = 0; */
+        break;
+      default:
+        tcparray[fd]->SelectErrno = rc;
+        tcparray[fd]->ReadErrno = rc;
         break;
       }
-    
-    if (i < 0)
-      {
-      /* FAILURE - read failed */
-      
-      tcparray[fd]->ReadErrno = errno;
-      
-      return(-1);
-      }
-    else if (i == 0)
-      {
-      /* FAILURE - no data read */
-      
-      return(-2);
-      }
-    
-    /* SUCCESS */
-    if (i < tp->tdis_thebuf + tp->tdis_bufsize - tp->tdis_eod)
-      done = 1;
-    
-    tp->tdis_eod += i;
-    total += i;
-
-    /* done reading if we haven't read our max size */
-    if (done)
-      break;
-
-    /* otherwise alloc more space and read again, add 25% more space */
-    newsize = tp->tdis_bufsize * 1.25;
-    ptr = realloc(tp->tdis_thebuf,newsize);
-
-    if (ptr == NULL)
-      {
-      /* failure to allocate memory, return NULL? */
-      log_err(ENOMEM,"tcp_read","Could not allocate memory to read buffer");
-      return(-1);
-      }
-    else
-      {
-      /* adjust the new values */
-      tp->tdis_eod += ((char *)ptr) - tp->tdis_thebuf;
-      tp->tdis_trailp += ((char *)ptr) - tp->tdis_thebuf;
-      tp->tdis_leadp += ((char *)ptr) - tp->tdis_thebuf;
-      tp->tdis_thebuf = (char *)ptr;
-      tp->tdis_bufsize = newsize;
-
-      /* fall through to continue reading more */
-      }
+    pthread_mutex_unlock(&(tcparray[fd]->tcp_mutex));
+    return rc;
     }
+  /* data read is less than buffer size */
+  else if (max_read_len > *read_len)
+    {
+    pthread_mutex_lock(&(tcparray[fd]->tcp_mutex));
+/*This is a check to see if the ptr's have changed. Needed only in testing
+ * if (max_read_len != (int)(tp->tdis_bufsize - (tp->tdis_eod - tp->tdis_thebuf)))
+      {
+      snprintf(err_msg, sizeof(err_msg), "someone has been messing with the buffer!!!!");
+      log_err(PBSE_INTERNAL,__func__,err_msg);
+      exit(1);
+      }
+      */
+    memcpy(tp->tdis_eod, new_data, *read_len);
+    tp->tdis_eod += *read_len;
+    *avail_len = tp->tdis_eod - tp->tdis_leadp;
+    max_read_len = tp->tdis_eod - tp->tdis_thebuf;
+    pthread_mutex_unlock(&(tcparray[fd]->tcp_mutex));
+    if (max_read_len > tdis_buf_len)
+      {
+      snprintf(err_msg, sizeof(err_msg), "eod ptr BEYOND end of buffer!! (fit) Remaining buffer = %d, read_len = %lld", max_read_len, *read_len);
+      log_err(PBSE_INTERNAL,__func__,err_msg);
+      }
+    free(new_data);
+    }
+  /* data read is greater than buffer size */
+  else if (max_read_len < *read_len)
+    {
+    newsize = (tdis_buf_len + *read_len) * 2;
+    if ((ptr = (char *)calloc(1, newsize+1)) == NULL)
+      {
+      log_err(ENOMEM,__func__,"Could not allocate memory to read buffer");
+      rc = PBSE_MEM_MALLOC;
+      free(new_data);
+      return rc;
+      }
+    pthread_mutex_lock(&(tcparray[fd]->tcp_mutex));
 
-  return(total);
+    tmp_leadp = tp->tdis_leadp - tp->tdis_thebuf;
+    tmp_trailp = tp->tdis_trailp - tp->tdis_thebuf;
+    tmp_eod = tp->tdis_eod - tp->tdis_thebuf;
+
+    snprintf(ptr, newsize, "%s%s", tp->tdis_thebuf, new_data);
+    memset(tp->tdis_thebuf, '2', tp->tdis_bufsize);
+    free(tp->tdis_thebuf);
+    tp->tdis_thebuf = ptr;
+    tp->tdis_bufsize = newsize;
+    tp->tdis_eod = tp->tdis_thebuf + tmp_eod + *read_len;
+    tp->tdis_trailp = tp->tdis_thebuf + tmp_trailp;
+    tp->tdis_leadp = tp->tdis_thebuf + tmp_leadp;
+    *avail_len = tp->tdis_eod - tp->tdis_leadp;
+
+    max_read_len = tp->tdis_eod - tp->tdis_thebuf;
+    tdis_buf_len = newsize;
+    pthread_mutex_unlock(&(tcparray[fd]->tcp_mutex));
+
+    if (max_read_len > tdis_buf_len)
+      {
+      snprintf(err_msg, sizeof(err_msg), "eod ptr BEYOND end of buffer!!(expand) Remaining buffer = %d, read_len = %lld", max_read_len, *read_len);
+      log_err(PBSE_INTERNAL,__func__,err_msg);
+      }
+    free(new_data);
+
+    }
+  return rc;
   }  /* END tcp_read() */
 
 
@@ -366,15 +358,37 @@ int DIS_tcp_wflush(
   size_t ct;
   int  i;
   char *pb = NULL;
+  char *temp_pb = NULL;
+  char *orig_temp_pb = NULL;
+  char *pbs_debug = NULL;
+  int   rc = PBSE_NONE;
 
   struct tcpdisbuf *tp;
 
+  pbs_debug = getenv("PBSDEBUG");
+
+  pthread_mutex_lock(&(tcparray[fd]->tcp_mutex));
+
   tp = &tcparray[fd]->writebuf;
   pb = tp->tdis_thebuf;
-
   ct = tp->tdis_trailp - tp->tdis_thebuf;
 
-  while ((i = write(fd, pb, ct)) != (ssize_t)ct)
+  if ((orig_temp_pb = calloc(1, ct + 1)) == NULL)
+    {
+    if (pbs_debug != NULL)
+      fprintf(stderr, "DIS_tcp_wflush failed to on calloc of temp_pb: %d, (%s)\n", errno, strerror(errno));
+    rc = -1;
+    }
+  else
+    memcpy(orig_temp_pb, pb, ct);
+  temp_pb = orig_temp_pb;
+
+  pthread_mutex_unlock(&(tcparray[fd]->tcp_mutex));
+
+  if (rc != PBSE_NONE)
+    return(-1);
+ 
+  while ((i = write(fd, temp_pb, ct)) != (ssize_t)ct)
     {
     if (i == -1)
       {
@@ -385,29 +399,37 @@ int DIS_tcp_wflush(
 
       /* FAILURE */
 
-      if (getenv("PBSDEBUG") != NULL)
+      if (pbs_debug != NULL)
         {
         fprintf(stderr, "TCP write of %d bytes (%.32s) failed, errno=%d (%s)\n",
-                (int)ct,
-                pb,
-                errno,
-                strerror(errno));
+          (int)ct, temp_pb, errno, strerror(errno));
         }
-
+      free(orig_temp_pb);
       return(-1);
       }  /* END if (i == -1) */
-
-    ct -= i;
-
-    pb += i;
+    else
+      {
+      ct -= i;
+      temp_pb += i;
+      }
     }  /* END while (i) */
 
   /* SUCCESS */
 
+  free(orig_temp_pb);
+  /* make sure something did not clean up the connection while
+     we were writing */
+  if (tcparray[fd] == NULL)
+    return (-1);
+
+  pthread_mutex_lock(&(tcparray[fd]->tcp_mutex));
+
   tp->tdis_eod = tp->tdis_leadp;
 
   tcp_pack_buff(tp);
 
+  pthread_mutex_unlock(&(tcparray[fd]->tcp_mutex));
+
   return(0);
   }  /* END DIS_tcp_wflush() */
 
@@ -424,6 +446,7 @@ static void DIS_tcp_clear(
   struct tcpdisbuf *tp)
 
   {
+  memset(tp->tdis_thebuf, 0, tp->tdis_bufsize);
   tp->tdis_leadp  = tp->tdis_thebuf;
   tp->tdis_trailp = tp->tdis_thebuf;
   tp->tdis_eod    = tp->tdis_thebuf;
@@ -446,11 +469,15 @@ void DIS_tcp_reset(
 
   tcp = tcparray[fd];
 
+  pthread_mutex_lock(&tcp->tcp_mutex);
+
   if (i == 0)
     DIS_tcp_clear(&tcp->readbuf);
   else
     DIS_tcp_clear(&tcp->writebuf);
 
+  pthread_mutex_unlock(&tcp->tcp_mutex);
+
   return;
   }  /* END DIS_tcp_reset() */
 
@@ -464,7 +491,7 @@ void DIS_tcp_reset(
  * Returns: 0 on success, -1 on error
  */
 
-static int tcp_rskip(
+int tcp_rskip(
 
   int    fd,
   size_t ct)
@@ -473,104 +500,93 @@ static int tcp_rskip(
 
   struct tcpdisbuf *tp;
 
+  pthread_mutex_lock(&(tcparray[fd]->tcp_mutex));
+
   tp = &tcparray[fd]->readbuf;
 
   if (tp->tdis_leadp - tp->tdis_eod < (ssize_t)ct)
     {
     /* this isn't the best thing to do, but this isn't used, so */
+    pthread_mutex_unlock(&(tcparray[fd]->tcp_mutex));
 
     return(-1);
     }
 
   tp->tdis_leadp += ct;
 
+  pthread_mutex_unlock(&(tcparray[fd]->tcp_mutex));
+
   return(0);
   }
 
 
 
-
-
-/*
- * tcp_getc - tcp/dis support routine to get next character from read buffer
- *
- * Return: >0 number of characters read
- *  -1 if EOD or error
- *  -2 if EOF (stream closed)
- */
-
-static int tcp_getc(
-
-  int fd)
-
-  {
-  int x;
-
-  struct tcpdisbuf *tp;
-
-  tp = &tcparray[fd]->readbuf;
-
-  if (tp->tdis_leadp >= tp->tdis_eod)
-    {
-    /* not enough data, try to get more */
-
-    x = tcp_read(fd);
-
-    if (x <= 0)
-      {
-      return((x == -2) ? -2 : -1); /* Error or EOF */
-      }
-    }
-
-  return((int)*tp->tdis_leadp++);
-  }  /* END tcp_getc() */
-
-
-
-
-
 /*
  * tcp_gets - tcp/dis support routine to get a string from read buffer
  *
- * Return: >0 number of characters read
- *   0 if EOD (no data currently avalable)
+ * Return: number of characters read (>=1)
  *  -1 if error
- *  -2 if EOF (stream closed)
+ *  -2 if EOF/EOD (stream closed)
  */
 
-static int tcp_gets(
+int tcp_gets(
 
   int     fd,
   char   *str,
   size_t  ct)
 
   {
-  int             x;
-
+  int rc = 0;
   struct tcpdisbuf *tp;
+  long long data_read = 0;
+  long long data_avail = 0;
 
+  if (tcparray[fd] == NULL)
+    return(-2);
+
+  pthread_mutex_lock(&(tcparray[fd]->tcp_mutex));
   tp = &tcparray[fd]->readbuf;
+  /* length of usable data in current buffer */
+  data_avail = tp->tdis_eod - tp->tdis_leadp;
+  pthread_mutex_unlock(&(tcparray[fd]->tcp_mutex));
 
-  while (tp->tdis_eod - tp->tdis_leadp < (ssize_t)ct)
+  while ((size_t)data_avail < ct)
     {
     /* not enough data, try to get more */
-
-    x = tcp_read(fd);
-
-    if (x <= 0)
+    if ((rc = tcp_read(fd, &data_read, &data_avail)) != PBSE_NONE)
       {
-      return(x);  /* Error or EOF */
+      if (data_read == 0)
+        rc = -2;
+      else
+        rc = -1;
+      return(rc);  /* Error or EOF */
       }
     }
 
+  pthread_mutex_lock(&(tcparray[fd]->tcp_mutex));
   memcpy((char *)str, tp->tdis_leadp, ct);
-
   tp->tdis_leadp += ct;
+  pthread_mutex_unlock(&(tcparray[fd]->tcp_mutex));
 
   return((int)ct);
   }  /* END tcp_gets() */
 
 
+/*
+ * tcp_getc - see tcp_gets
+ */
+
+int tcp_getc(
+
+  int fd)
+
+  {
+  int rc = DIS_SUCCESS;
+  char ret_val;
+  if ((rc = tcp_gets(fd, &ret_val, 1)) < 0)
+    return rc;
+  return (int)ret_val;
+  }  /* END tcp_getc() */
 
 
 int PConnTimeout(
@@ -578,19 +594,20 @@ int PConnTimeout(
   int sock)  /* I */
 
   {
+  int rc; 
+
   if ((tcparray == NULL) || (tcparray[sock] == NULL))
     {
     return(0);
     }
 
-  if (tcparray[sock]->IsTimeout == 1)
-    {
-    /* timeout occurred, report 'TRUE' */
+  pthread_mutex_lock(&(tcparray[sock]->tcp_mutex));
 
-    return(1);
-    }
+  rc = (tcparray[sock]->IsTimeout == 1);
 
-  return(0);
+  pthread_mutex_unlock(&(tcparray[sock]->tcp_mutex));
+
+  return(rc);
   }  /* END PConnTimeout() */
 
 
@@ -601,12 +618,20 @@ int TConnGetReadErrno(
   int sock)  /* I */
 
   {
+  int rc;
+
   if ((tcparray == NULL) || (tcparray[sock] == NULL))
     {
     return(0);
     }
 
-  return(tcparray[sock]->ReadErrno);
+  pthread_mutex_lock(&(tcparray[sock]->tcp_mutex));
+
+  rc = tcparray[sock]->ReadErrno;
+
+  pthread_mutex_unlock(&(tcparray[sock]->tcp_mutex));
+
+  return(rc);
   }  /* END TConnGetReadErrno() */
 
 
@@ -618,12 +643,20 @@ int TConnGetSelectErrno(
   int sock)  /* I */
 
   {
+  int rc;
+
   if ((tcparray == NULL) || (tcparray[sock] == NULL))
     {
     return(0);
     }
 
-  return(tcparray[sock]->SelectErrno);
+  pthread_mutex_lock(&(tcparray[sock]->tcp_mutex));
+  
+  rc = tcparray[sock]->SelectErrno;
+
+  pthread_mutex_unlock(&(tcparray[sock]->tcp_mutex));
+
+  return(rc);
   }  /* END TConnGetSelectErrno() */
 
 
@@ -638,7 +671,7 @@ int TConnGetSelectErrno(
  *   -1 if error
  */
 
-static int tcp_puts(
+int tcp_puts(
 
   int         fd,  /* I */
   const char *str, /* I */
@@ -649,36 +682,58 @@ static int tcp_puts(
   char *id = "tcp_puts";
 #endif
 
-  struct tcp_chan *tcp = NULL;
+  struct tcp_chan  *tcp = NULL;
   struct tcpdisbuf *tp = NULL;
-  char *temp = NULL;
-  int leadpct, trailpct; 
-  size_t newbufsize;
+  char             *temp = NULL;
+  int               leadpct;
+  int               trailpct; 
+  size_t            newbufsize;
+  char              log_buf[LOCAL_LOG_BUF_SIZE];
 
   tcp = tcparray[fd];
 
+  if (tcp == NULL)
+    return(-2);
+
+  pthread_mutex_lock(&tcp->tcp_mutex);
+
+  /* NOTE:  currently, failures may occur if THE_BUF_SIZE is not large enough */
+  /*        this should be changed to allow proper operation with degraded    */
+  /*        performance (how?) */
+
   tp = &tcp->writebuf;
   if ((tp->tdis_thebuf + tp->tdis_bufsize - tp->tdis_leadp) < (ssize_t)ct)
     {
     /* not enough room, reallocate the buffer */
     leadpct = (int)(tp->tdis_thebuf - tp->tdis_leadp);
     trailpct = (int)(tp->tdis_thebuf - tp->tdis_trailp);
-    newbufsize = tp->tdis_bufsize + THE_BUF_SIZE + ct; /* grow by ct + THE_BUF_SIZE */
-    temp = (char *)malloc(newbufsize);
-    if(!temp)
+    newbufsize = tp->tdis_bufsize + THE_BUF_SIZE + ct*2;
+    temp = (char *)calloc(1, newbufsize+1);
+    if (!temp)
       {
       /* FAILURE */
+      snprintf(log_buf,sizeof(log_buf),
+        "out of space in buffer and cannot calloc message buffer (bufsize=%ld, buflen=%d, ct=%d)\n",
+        tp->tdis_bufsize,
+        (int)(tp->tdis_leadp - tp->tdis_thebuf),
+        (int)ct);
+      log_err(ENOMEM,id,log_buf);
 
-      DBPRT(("%s: error!  out of space in buffer and cannot realloc message buffer (bufsize=%ld, buflen=%d, ct=%d)\n",
-             id,
-             tp->tdis_bufsize,
-             (int)(tp->tdis_leadp - tp->tdis_thebuf),
-             (int)ct))
+      pthread_mutex_unlock(&tcp->tcp_mutex);
 
       return(-1);
       }
 
     memcpy(temp, tp->tdis_thebuf, tp->tdis_bufsize);
+    if (strlen(tp->tdis_thebuf) > tp->tdis_bufsize)
+      {
+      snprintf(log_buf, sizeof(log_buf),
+          "line #%d, The length of the string is GREATER than the size of buf",
+          __LINE__);
+      log_err(ENOMEM, __func__, log_buf);
+      }
+    memset(tp->tdis_thebuf, 170, tp->tdis_bufsize);
+
     free(tp->tdis_thebuf);
     tp->tdis_thebuf = temp;
     tp->tdis_bufsize = newbufsize;
@@ -691,6 +746,8 @@ static int tcp_puts(
 
   tp->tdis_leadp += ct;
 
+  pthread_mutex_unlock(&tcp->tcp_mutex);
+
   return(ct);
   }  /* END tcp_puts() */
 
@@ -701,7 +758,7 @@ static int tcp_puts(
  * tcp_rcommit - tcp/dis support routine to commit/uncommit read data
  */
 
-static int tcp_rcommit(
+int tcp_rcommit(
 
   int fd,
   int commit_flag)
@@ -710,6 +767,11 @@ static int tcp_rcommit(
 
   struct tcpdisbuf *tp;
 
+  if (tcparray[fd] == NULL)
+    return(-2);
+
+  pthread_mutex_lock(&(tcparray[fd]->tcp_mutex));
+
   tp = &tcparray[fd]->readbuf;
 
   if (commit_flag)
@@ -725,6 +787,8 @@ static int tcp_rcommit(
     tp->tdis_leadp = tp->tdis_trailp;
     }
 
+  pthread_mutex_unlock(&(tcparray[fd]->tcp_mutex));
+
   return(0);
   }  /* END tcp_rcommit() */
 
@@ -736,7 +800,7 @@ static int tcp_rcommit(
  * tcp_wcommit - tcp/dis support routine to commit/uncommit write data
  */
 
-static int tcp_wcommit(
+int tcp_wcommit(
 
   int fd,
   int commit_flag)
@@ -745,6 +809,11 @@ static int tcp_wcommit(
 
   struct tcpdisbuf *tp;
 
+  if (tcparray[fd] == NULL)
+    return(-2);
+
+  pthread_mutex_lock(&(tcparray[fd]->tcp_mutex));
+
   tp = &tcparray[fd]->writebuf;
 
   if (commit_flag)
@@ -760,63 +829,12 @@ static int tcp_wcommit(
     tp->tdis_leadp = tp->tdis_trailp;
     }
 
+  pthread_mutex_unlock(&(tcparray[fd]->tcp_mutex));
+
   return(0);
   }
 
 
-
-
-void
-DIS_tcp_funcs(void)
-
-  {
-  if (dis_getc == tcp_getc)
-    {
-    return;
-    }
-
-  dis_getc = tcp_getc;
-
-  dis_puts = tcp_puts;
-  dis_gets = tcp_gets;
-  disr_skip = tcp_rskip;
-  disr_commit = tcp_rcommit;
-  disw_commit = tcp_wcommit;
-
-  return;
-  }
-
-void DIS_tcp_close(
-
-  int fd)
-
-  {
-  struct tcp_chan *tcp;
-
-  /* On startup tcparray may not yet be initialized. check it */
-  if (tcparray == NULL)
-    return;
-
-  if (fd > tcparraymax)
-    return;
-
-  tcp = tcparray[fd];
-  if (tcp != NULL)
-    {
-    if (tcp->readbuf.tdis_thebuf != NULL)
-      free(tcp->readbuf.tdis_thebuf);
-    if (tcp->writebuf.tdis_thebuf != NULL)
-      free(tcp->writebuf.tdis_thebuf);
-
-    free(tcp);
-    tcparray[fd] = NULL;
-    }
-
-  return;
-  }
-
-
-
 /*
  * DIS_tcp_setup - setup supports routines for dis, "data is strings", to
  * use tcp stream I/O.  Also initializes an array of pointers to
@@ -832,8 +850,9 @@ void DIS_tcp_setup(
   int fd)
 
   {
-  struct tcp_chan *tcp = NULL;
+  struct tcp_chan  *tcp = NULL;
   struct tcpdisbuf *tp = NULL;
+  int tcp_setup = FALSE;
 
   /* check for bad file descriptor */
 
@@ -842,130 +861,90 @@ void DIS_tcp_setup(
     return;
     }
 
-  /* set DIS function pointers to tcp routines */
-
-  DIS_tcp_funcs();
-
-  if (fd >= tcparraymax)
+  lock_tcp_table();
+  if (tcparray[fd] == NULL)
     {
-    int hold = tcparraymax;
-    int flags;
-    
-    /*
-    ** Check if this is a valid file descriptor, if not log an error and don't
-    ** do any memory allocations
-    */
-
-    flags = fcntl(fd, F_GETFL);
-
-    if ((errno == EBADF) &&
-        (flags == -1))
+    if ((tcparray[fd] = (struct tcp_chan *)calloc(1, sizeof(struct tcp_chan))) == NULL)
       {
-      sprintf(log_buffer, "invalid file descriptor (%d) for socket",
-        fd);
-      log_err(errno, "DIS_tcp_setup", log_buffer);
-
-      return;
-      }
-
-    tcparraymax = fd + 10;
-
-    if (tcparray == NULL)
-      {
-      /* Remember calloc will initialize memory to 0 */
-      tcparray = (struct tcp_chan **)calloc(
-        tcparraymax,
-        sizeof(struct tcp_chan *));
-
-      if (tcparray == NULL)
-        {
-        /* FAILURE */
-
-        log_err(errno,"DIS_tcp_setup","calloc failure");
-
-        return;
-        }
+      log_err(ENOMEM, "DIS_tcp_setup", "calloc failure");
       }
     else
       {
-      struct tcp_chan **tmpTA;
-
-      tmpTA = (struct tcp_chan **)realloc(
-        tcparray,
-        tcparraymax * sizeof(struct tcp_chan *));
-
-      if (tmpTA == NULL)
-        {
-        /* FAILURE */
-
-        log_err(errno,"DIS_tcp_setup","realloc failure");
-
-        return;
-        }
-
-      tcparray = tmpTA;
-
-      memset(&tcparray[hold], '\0', (tcparraymax - hold) * sizeof(struct tcp_chan *));
+      tcp_setup = TRUE;
       }
-    }    /* END if (fd >= tcparraymax) */
+    }
+  unlock_tcp_table();
+  if (tcparray[fd] == NULL)
+    {
+    goto error;
+    }
 
   tcp = tcparray[fd];
-
-  if (tcp == NULL)
+  if (tcp_setup == TRUE)
     {
-    tcparray[fd] = (struct tcp_chan *)malloc(sizeof(struct tcp_chan));
 
-    memset(tcparray[fd], 0, sizeof(struct tcp_chan));
-
-    tcp = tcparray[fd];
-
-    if (tcp == NULL)
-      {
-      log_err(errno, "DIS_tcp_setup", "malloc failure");
-
-      return;
-      }
+    pthread_mutex_init(&tcp->tcp_mutex,NULL);
+    pthread_mutex_lock(&tcp->tcp_mutex);
 
     /* Setting up the read buffer */
     tp = &tcp->readbuf;
 
-    tp->tdis_thebuf = (char *)malloc(THE_BUF_SIZE);
-    if(tp->tdis_thebuf == NULL)
+    tp->tdis_thebuf = (char *)calloc(1, THE_BUF_SIZE+1);
+    if (tp->tdis_thebuf == NULL)
       {
-      log_err(errno,"DIS_tcp_setup","malloc failure");
+      pthread_mutex_unlock(&tcp->tcp_mutex);
 
-      return;
+      log_err(errno,"DIS_tcp_setup","calloc failure");
+
+      goto error;
       }
 
-    memset(tp->tdis_thebuf, 0, THE_BUF_SIZE);
-
     tp->tdis_bufsize = THE_BUF_SIZE;
 
     /* Setting up the write buffer */
     tp = &tcp->writebuf;
 
-    tp->tdis_thebuf = (char *)malloc(THE_BUF_SIZE);
-    if(tp->tdis_thebuf == NULL)
+    tp->tdis_thebuf = (char *)calloc(1, THE_BUF_SIZE+1);
+    if (tp->tdis_thebuf == NULL)
       {
-      log_err(errno,"DIS_tcp_setup","malloc failure");
+      pthread_mutex_unlock(&tcp->tcp_mutex);
 
-      return;
+      log_err(errno,"DIS_tcp_setup","calloc failure");
+
+      goto error;
       }
 
-    memset(tp->tdis_thebuf, 0, THE_BUF_SIZE);
-
     tp->tdis_bufsize = THE_BUF_SIZE;
 
     }
+  else
+    pthread_mutex_lock(&tcp->tcp_mutex);
 
   /* initialize read and write buffers */
-
   DIS_tcp_clear(&tcp->readbuf);
 
   DIS_tcp_clear(&tcp->writebuf);
 
+  pthread_mutex_unlock(&tcp->tcp_mutex);
+
+error:
+
   return;
   }  /* END DIS_tcp_setup() */
 
-
+void DIS_tcp_shutdown(int fd)
+  {
+  struct tcp_chan  *tcp = NULL;
+  struct tcpdisbuf *tp = NULL;
+  lock_tcp_table();
+  tcp = tcparray[fd];
+  tcparray[fd] = NULL;
+  unlock_tcp_table();
+  tp = &tcp->readbuf;
+  free(tp->tdis_thebuf);
+  tp = &tcp->writebuf;
+  free(tp->tdis_thebuf);
+  pthread_mutex_destroy(&tcp->tcp_mutex);
+  free(tcp);
+  }
 /* END tcp_dis.c */
