@@ -144,8 +144,6 @@ static int  local_conn = -1;
 static int  init_done = 0;
 int *tm_conn = &local_conn;
 
-void DIS_tcp_funcs();
-
 /*
 ** Events are the central focus of this library.  They are tracked
 ** in a hash table.  Many of the library calls return events.  They
@@ -282,7 +280,7 @@ add_event(tm_event_t event, tm_node_id node, int type, void *info)
   {
   event_info  *ep, **head;
 
-  ep = (event_info *)malloc(sizeof(event_info));
+  ep = (event_info *)calloc(1, sizeof(event_info));
   assert(ep != NULL);
 
   head = &event_hash[event % EVENT_HASH];
@@ -371,7 +369,7 @@ new_task(char *jobid, tm_node_id node, tm_task_id task)
     return task;
     }
 
-  if ((tp = (task_info *)malloc(sizeof(task_info))) == NULL)
+  if ((tp = (task_info *)calloc(1, sizeof(task_info))) == NULL)
     return TM_NULL_TASK;
 
   head = &task_hash[task % TASK_HASH];
@@ -434,19 +432,16 @@ static tm_node_id *node_table = NULL;
 #define PBS_NET_RC_FATAL -1
 #define PBS_NET_RC_RETRY -2
 
-static int
-localmom(void)
+static int localmom(void)
 
   {
-  static int            have_addr = 0;
+  static int             have_addr = 0;
+  static struct in_addr  hostaddr;
+  struct addrinfo       *addr_info;
 
-  static struct in_addr hostaddr;
-
-  struct hostent       *hp;
-  int          i;
-
-  struct sockaddr_in    remote;
-  int                   sock;
+  int                    i;
+  int                    sock;
+  struct sockaddr_in     remote;
 
   struct linger         ltime;
 
@@ -459,18 +454,18 @@ localmom(void)
     {
     /* lookup "localhost" and save address */
 
-    if ((hp = gethostbyname("localhost")) == NULL)
+    if (getaddrinfo("localhost", NULL, NULL, &addr_info) != 0)
       {
       DBPRT(("tm_init: localhost not found\n"))
 
       return(-1);
       }
 
-    assert((int)hp->h_length <= (int)sizeof(hostaddr));
-
-    memcpy(&hostaddr, hp->h_addr_list[0], hp->h_length);
+    hostaddr = ((struct sockaddr_in *)addr_info->ai_addr)->sin_addr;
 
     have_addr = 1;
+
+    freeaddrinfo(addr_info);
     }
 
   for (i = 0;i < 5;i++)
@@ -765,11 +760,11 @@ int tm_nodeinfo(
 
 int tm_spawn(
 
-  int          argc, /* in  */
-  char       **argv, /* in  */
-  char       **envp, /* in  */
+  int          argc,  /* in  */
+  char       **argv,  /* in  */
+  char       **envp,  /* in  */
   tm_node_id   where, /* in  */
-  tm_task_id  *tid, /* out */
+  tm_task_id  *tid,   /* out */
   tm_event_t  *event) /* out */
 
   {
@@ -857,12 +852,12 @@ int tm_spawn(
 ** Sends a <sig> signal to all the process groups in the task
 ** signified by the handle, <tid>.
 */
-int
-tm_kill(
-  tm_task_id tid,  /* in  */
-  int sig,  /* in  */
-  tm_event_t *event  /* out */
-)
+int tm_kill(
+
+  tm_task_id  tid,  /* in  */
+  int         sig,  /* in  */
+  tm_event_t *event)  /* out */
+
   {
   task_info *tp;
 
@@ -897,12 +892,11 @@ tm_kill(
 ** Returns an event that can be used to learn when a task
 ** dies.
 */
-int
-tm_obit(
-  tm_task_id tid,  /* in  */
-  int *obitval, /* out */
-  tm_event_t *event  /* out */
-)
+int tm_obit(
+
+  tm_task_id  tid,  /* in  */
+  int        *obitval, /* out */
+  tm_event_t *event)  /* out */
   {
   task_info *tp;
 
@@ -970,7 +964,7 @@ tm_taskinfo(
 
   DIS_tcp_wflush(local_conn);
 
-  thold = (struct taskhold *)malloc(sizeof(struct taskhold));
+  thold = (struct taskhold *)calloc(1, sizeof(struct taskhold));
 
   assert(thold != NULL);
 
@@ -1020,13 +1014,11 @@ struct reschold
 ** returns the event that the string specifying the resources on
 ** <node> is available.  It returns ERROR_EVENT otherwise.
 */
-int
-tm_rescinfo(
+int tm_rescinfo(
   tm_node_id node,  /* in  */
   char *resource, /* out */
   int len,  /* in  */
-  tm_event_t *event  /* out */
-)
+  tm_event_t *event)  /* out */
   {
 
   struct reschold *rhold;
@@ -1047,7 +1039,7 @@ tm_rescinfo(
 
   DIS_tcp_wflush(local_conn);
 
-  rhold = (struct reschold *)malloc(sizeof(struct reschold));
+  rhold = (struct reschold *)calloc(1, sizeof(struct reschold));
 
   assert(rhold != NULL);
 
@@ -1144,7 +1136,7 @@ tm_subscribe(
 
   DIS_tcp_wflush(local_conn);
 
-  ihold = (struct infohold *)malloc(sizeof(struct infohold));
+  ihold = (struct infohold *)calloc(1, sizeof(struct infohold));
 
   assert(ihold != NULL);
 
@@ -1167,8 +1159,7 @@ tm_subscribe(
 ** freed and any connection to the task manager (pbs_mom) is closed.
 ** This call is synchronous.
 */
-int
-tm_finalize(void)
+int tm_finalize(void)
   {
   event_info *e;
   int   i = 0;
@@ -1194,8 +1185,7 @@ tm_finalize(void)
 /*
 ** tm_notify() - set the signal to be sent on event arrival.
 */
-int
-tm_notify(int tm_signal)
+int tm_notify(int tm_signal)
   {
   if (!init_done)
     return TM_BADINIT;
@@ -1348,8 +1338,6 @@ int tm_poll(
   */
   pbs_tcp_timeout = wait ? FOREVER : 0;
 
-  DIS_tcp_funcs();
-
   prot = disrsi(local_conn, &ret);
 
   if (ret == DIS_EOD)
@@ -1600,7 +1588,7 @@ int tm_poll(
       if (ret != DIS_SUCCESS)
         break;
 
-      strncpy(rhold->resc, info, rhold->len);
+      snprintf(rhold->resc, rhold->len, "%s", info);
 
       free(info);
 
@@ -1739,7 +1727,8 @@ int tm_adopt(char *id, int adoptCmd, pid_t pid)
     return TM_ENOTCONNECTED;
 
   /* write the pid so the adopted process can be part of the cpuset if needed */
-  if (diswsi(local_conn, pid) != DIS_SUCCESS)
+
+  if (diswsi(local_conn,sid) != DIS_SUCCESS)
     return TM_ENOTCONNECTED;
 
   /* send job or alternative id */
