@@ -91,6 +91,7 @@
  */
 
 #include <pbs_config.h>   /* the master config generated by configure */
+#include "pbs_log.h"
 
 #include "portability.h"
 #include "pbs_error.h"
@@ -107,12 +108,15 @@
 #include <unistd.h>
 #include <errno.h>
 #include <dirent.h>
+#include <pthread.h>
 
 #include "log.h"
 #if SYSLOG
 #include <syslog.h>
 #endif
 
+#include <execinfo.h> /* backtrace information */
+
 void job_log_close(int msg);
 
 /* Global Data */
@@ -135,13 +139,16 @@ static volatile int  log_opened = 0;
 static int      syslogopen = 0;
 #endif /* SYSLOG */
 
+pthread_mutex_t *log_mutex;
+
 /* variables for job logging */
 static int      job_log_auto_switch = 0;
 static int      joblog_open_day;
 static FILE     *joblogfile;  /* open stream for log file */
 static char     *joblogpath = NULL;
-static int  job_log_opened = 0;
+static volatile int  job_log_opened = 0;
 
+pthread_mutex_t *job_log_mutex;
 /*
  * the order of these names MUST match the defintions of
  * PBS_EVENTCLASS_* in log.h
@@ -174,19 +181,18 @@ static char *mk_log_name(
   char *pbuf)     /* O (minsize=1024) */
 
   {
-
   struct tm *ptm;
   struct tm  tmpPtm;
-  time_t time_now;
+  time_t     time_now = time(NULL);
+  memset(&tmpPtm, 0, sizeof(struct tm));
 
-  time_now = time((time_t *)0);
   ptm = localtime_r(&time_now,&tmpPtm);
 
   if (log_suffix[0] != '\0')
     {
     if (!strcasecmp(log_suffix, "%h"))
       {
-      sprintf(pbuf, "%s/%04d%02d%02d.%s",
+      snprintf(pbuf, PATH_MAX, "%s/%04d%02d%02d.%s",
               log_directory,
               ptm->tm_year + 1900,
               ptm->tm_mon + 1,
@@ -195,7 +201,7 @@ static char *mk_log_name(
       }
     else
       {
-      sprintf(pbuf, "%s/%04d%02d%02d.%s",
+      snprintf(pbuf, PATH_MAX, "%s/%04d%02d%02d.%s",
               log_directory,
               ptm->tm_year + 1900,
               ptm->tm_mon + 1,
@@ -205,7 +211,7 @@ static char *mk_log_name(
     }
   else
     {
-    sprintf(pbuf, "%s/%04d%02d%02d",
+    snprintf(pbuf, PATH_MAX, "%s/%04d%02d%02d",
             log_directory,
             ptm->tm_year + 1900,
             ptm->tm_mon + 1,
@@ -226,22 +232,21 @@ static char *mk_log_name(
 
 static char *mk_job_log_name(
 
-  char *pbuf)     /* O (minsize=1024) */
+  char *pbuf,
+  int   buf_size)
 
   {
-
   struct tm *ptm;
   struct tm  tmpPtm;
-  time_t time_now;
+  time_t time_now = time(NULL);
 
-  time_now = time((time_t *)0);
   ptm = localtime_r(&time_now,&tmpPtm);
 
   if (log_suffix[0] != '\0')
     {
     if (!strcasecmp(log_suffix, "%h"))
       {
-      sprintf(pbuf, "%s/%04d%02d%02d.%s",
+      snprintf(pbuf, buf_size, "%s/%04d%02d%02d.%s",
               job_log_directory,
               ptm->tm_year + 1900,
               ptm->tm_mon + 1,
@@ -250,7 +255,7 @@ static char *mk_job_log_name(
       }
     else
       {
-      sprintf(pbuf, "%s/%04d%02d%02d.%s",
+      snprintf(pbuf, buf_size, "%s/%04d%02d%02d.%s",
               job_log_directory,
               ptm->tm_year + 1900,
               ptm->tm_mon + 1,
@@ -260,14 +265,16 @@ static char *mk_job_log_name(
     }
   else
     {
-    sprintf(pbuf, "%s/%04d%02d%02d",
+    snprintf(pbuf, buf_size, "%s/%04d%02d%02d",
             job_log_directory,
             ptm->tm_year + 1900,
             ptm->tm_mon + 1,
             ptm->tm_mday);
     }
 
+  pthread_mutex_lock(job_log_mutex);
   joblog_open_day = ptm->tm_yday; /* Julian date log opened */
+  pthread_mutex_unlock(job_log_mutex);
 
   return(pbuf);
   }  /* END mk_job_log_name() */
@@ -282,10 +289,16 @@ int log_init(
 
   {
   if (suffix != NULL)
-    strncpy(log_suffix, suffix, sizeof(log_suffix));
+    snprintf(log_suffix, sizeof(log_suffix), "%s", suffix);
 
   if (hostname != NULL)
-    strncpy(log_host, hostname, sizeof(log_host));
+    snprintf(log_host, sizeof(log_host), "%s", hostname);
+
+  log_mutex = calloc(1, sizeof(pthread_mutex_t));
+  pthread_mutex_init(log_mutex, NULL);
+
+  job_log_mutex = calloc(1, sizeof(pthread_mutex_t));
+  pthread_mutex_init(job_log_mutex, NULL);
 
   return(0);
   }  /* END log_init() */
@@ -307,7 +320,7 @@ int log_open(
   char *directory) /* normal log directory */
 
   {
-  char  buf[_POSIX_PATH_MAX];
+  char  buf[PATH_MAX];
   int   fds;
 
   if (log_opened > 0)
@@ -317,7 +330,7 @@ int log_open(
 
   if (log_directory != directory)  /* some calls pass in log_directory */
     {
-    strncpy(log_directory, directory, (_POSIX_PATH_MAX) / 2 - 1);
+    snprintf(log_directory, (_POSIX_PATH_MAX) / 2 - 1, "%s", directory);
     }
 
   if ((filename == NULL) || (*filename == '\0'))
@@ -367,12 +380,16 @@ int log_open(
 
   log_opened = 1;   /* note that file is open */
 
+  pthread_mutex_unlock(log_mutex);
+  
   log_record(
     PBSEVENT_SYSTEM,
     PBS_EVENTCLASS_SERVER,
     "Log",
     "Log opened");
 
+  pthread_mutex_lock(log_mutex);
+
   return(0);
   }  /* END log_open() */
 
@@ -399,18 +416,18 @@ int job_log_open(
 
   if (job_log_opened > 0)
     {
-    /*log_err(-1, id, "job log already open");*/
+    log_err(-1, id, "job log already open");
     return(1); /* already open */
     }
 
   if (job_log_directory != directory)  /* some calls pass in job_log_directory */
     {
-    strncpy(job_log_directory, directory, (_POSIX_PATH_MAX) / 2 - 1);
+    snprintf(job_log_directory, (_POSIX_PATH_MAX) / 2 - 1, "%s", directory);
     }
 
   if ((filename == NULL) || (*filename == '\0'))
     {
-    filename = mk_job_log_name(buf);
+    filename = mk_job_log_name(buf, _POSIX_PATH_MAX);
 
     job_log_auto_switch = 1;
     }
@@ -445,7 +462,7 @@ int job_log_open(
     fds = job_log_opened;
     }
 
-  /* save the path of the last opened logfile for log_roll */
+  /* save the path of the last opened joblogfile for log_roll */
   if (joblogpath != filename)
     {
     if (joblogpath != NULL)
@@ -475,9 +492,9 @@ int job_log_open(
 
 void log_err(
 
-  int   errnum,  /* I (errno or PBSErrno) */
-  char *routine, /* I */
-  char *text)    /* I */
+  int         errnum,  /* I (errno or PBSErrno) */
+  const char *routine, /* I */
+  char       *text)    /* I */
 
   {
   log_ext(errnum,routine,text,LOG_ERR);
@@ -503,10 +520,10 @@ void log_err(
 
 void log_ext(
 
-  int   errnum,   /* I (errno or PBSErrno) */
-  char *routine,  /* I */
-  char *text,     /* I */
-  int   severity) /* I */
+  int         errnum,   /* I (errno or PBSErrno) */
+  const char *routine,  /* I */
+  char       *text,     /* I */
+  int         severity) /* I */
 
   {
   char  buf[LOG_BUF_SIZE];
@@ -563,6 +580,8 @@ void log_ext(
 
   buf[LOG_BUF_SIZE - 1] = '\0';
 
+  pthread_mutex_lock(log_mutex);
+
   if (log_opened == 0)
     {
 #if !SYSLOG
@@ -576,15 +595,19 @@ void log_ext(
             msg_daemonname,
             buf);
     }
-
+    
   if (log_opened > 0)
     {
+    pthread_mutex_unlock(log_mutex);
+
     log_record(
       PBSEVENT_ERROR | PBSEVENT_FORCE,
       PBS_EVENTCLASS_SERVER,
       msg_daemonname,
       buf);
     }
+  else
+    pthread_mutex_unlock(log_mutex);
 
 #if SYSLOG
   if (syslogopen == 0)
@@ -668,6 +691,8 @@ int log_job_record(char *buf)
   now = time((time_t *)0);
   ptm = localtime_r(&now,&tmpPtm);
 
+  pthread_mutex_lock(job_log_mutex);
+
   /* do we need to switch the log to the new day? */
   if (job_log_auto_switch && (ptm->tm_yday != joblog_open_day))
     {
@@ -678,12 +703,14 @@ int log_job_record(char *buf)
     if (job_log_opened < 1)
       {
       log_err(-1, id, "job_log_opened < 1");
+      pthread_mutex_unlock(job_log_mutex);
       return(-1);
       }
     }
 
-
   fprintf(joblogfile, "%s\n", buf);
+  pthread_mutex_unlock(job_log_mutex);
+
   return(0);
   }
 
@@ -700,10 +727,10 @@ int log_job_record(char *buf)
 
 void log_record(
 
-  int   eventtype,  /* I */
-  int   objclass,   /* I */
-  char *objname,    /* I */
-  char *text)       /* I */
+  int         eventtype,  /* I */
+  int         objclass,   /* I */
+  const char *objname,    /* I */
+  char       *text)       /* I */
 
   {
   int tryagain = 2;
@@ -716,8 +743,11 @@ void log_record(
   char  *start = NULL, *end = NULL;
   size_t nchars;
 
+  pthread_mutex_lock(log_mutex);
+
   if (log_opened < 1)
     {
+    pthread_mutex_unlock(log_mutex);
     return;
     }
 
@@ -735,9 +765,11 @@ void log_record(
 
     if (log_opened < 1)
       {
+      pthread_mutex_unlock(log_mutex);
       return;
       }
     }
+      
 
   /*
    * Looking for the newline characters and splitting the output message
@@ -816,6 +848,8 @@ void log_record(
       }
     logfile = savlog;
     }
+  
+  pthread_mutex_unlock(log_mutex);
 
   return;
   }  /* END log_record() */
@@ -838,11 +872,13 @@ void log_close(
 
     if (msg)
       {
+      pthread_mutex_unlock(log_mutex);
       log_record(
         PBSEVENT_SYSTEM,
         PBS_EVENTCLASS_SERVER,
         "Log",
         "Log closed");
+      pthread_mutex_lock(log_mutex);
       }
 
     fclose(logfile);
@@ -1014,14 +1050,16 @@ void log_roll(
   int err = 0;
   char *source  = NULL;
   char *dest    = NULL;
+  
+  pthread_mutex_lock(log_mutex);
 
   if (!log_opened)
     {
+    pthread_mutex_unlock(log_mutex);
     return;
     }
 
   /* save value of log_auto_switch */
-
   as = log_auto_switch;
 
   log_close(1);
@@ -1035,9 +1073,9 @@ void log_roll(
 
   file_buf_len = sizeof(char) * (strlen(logpath) + suffix_size + 1);
 
-  source = (char*)malloc(file_buf_len);
+  source = (char*)calloc(1, file_buf_len);
 
-  dest   = (char*)malloc(file_buf_len);
+  dest   = (char*)calloc(1, file_buf_len);
 
   if ((source == NULL) || (dest == NULL))
     {
@@ -1098,6 +1136,8 @@ done_roll:
     log_open(logpath, log_directory);
     }
 
+  pthread_mutex_unlock(log_mutex);
+
   if (source != NULL)
     free(source);
 
@@ -1130,13 +1170,16 @@ void job_log_roll(
   int err = 0;
   char *source  = NULL;
   char *dest    = NULL;
+      
+  pthread_mutex_lock(job_log_mutex);
 
   if (!job_log_opened)
     {
+    pthread_mutex_unlock(job_log_mutex);
     return;
     }
 
-  /* save value of log_auto_switch */
+  /* save value of job_log_auto_switch */
 
   as = job_log_auto_switch;
 
@@ -1151,9 +1194,9 @@ void job_log_roll(
 
   file_buf_len = sizeof(char) * (strlen(joblogpath) + suffix_size + 1);
 
-  source = (char*)malloc(file_buf_len);
+  source = (char*)calloc(1, file_buf_len);
 
-  dest   = (char*)malloc(file_buf_len);
+  dest   = (char*)calloc(1, file_buf_len);
 
   if ((source == NULL) || (dest == NULL))
     {
@@ -1234,6 +1277,8 @@ done_job_roll:
       "Job Log",
       "Job Log Rolled");
     }
+    
+  pthread_mutex_unlock(job_log_mutex);
 
   return;
   } /* END job_log_roll() */
@@ -1255,6 +1300,8 @@ long log_size(void)
 
   struct stat file_stat;
 #endif
+  
+  pthread_mutex_lock(log_mutex);
 
 #if defined(HAVE_STRUCT_STAT64) && defined(HAVE_STAT64) && defined(LARGEFILE_WORKS)
 
@@ -1267,8 +1314,11 @@ long log_size(void)
 
     log_err(errno, "log_size", "PBS cannot fstat logfile");
 
+    pthread_mutex_unlock(log_mutex);
     return(0);
     }
+  
+  pthread_mutex_unlock(log_mutex);
 
   return(file_stat.st_size / 1024);
   }
@@ -1285,6 +1335,8 @@ long job_log_size(void)
 
   struct stat file_stat;
 #endif
+  
+  pthread_mutex_lock(job_log_mutex);
 
 #if defined(HAVE_STRUCT_STAT64) && defined(HAVE_STAT64) && defined(LARGEFILE_WORKS)
 
@@ -1295,13 +1347,35 @@ long job_log_size(void)
     {
     /* FAILURE */
 
-    log_err(errno, "log_size", "PBS cannot fstat logfile");
+    log_err(errno, "log_size", "PBS cannot fstat joblogfile");
+    pthread_mutex_unlock(job_log_mutex);
 
     return(0);
     }
+  
+  pthread_mutex_unlock(job_log_mutex);
 
   return(file_stat.st_size / 1024);
   }
 
 
+void print_trace(int socknum)
+  {
+  void *array[10];
+  int size;
+  char **meth_names;
+  int cntr;
+  char msg[120];
+  char meth_name[20];
+  size = backtrace(array, 10);
+  meth_names = backtrace_symbols(array, size);
+  snprintf(meth_name, sizeof(meth_name), "pt - pos %d", socknum);
+  snprintf(msg, sizeof(msg), "Obtained %d stack frames.\n", size);
+  log_record(PBSEVENT_SYSTEM, PBS_EVENTCLASS_SERVER, meth_name, msg); 
+  for (cntr = 0; cntr < size; cntr++)
+    {
+    log_record(PBSEVENT_SYSTEM, PBS_EVENTCLASS_SERVER, meth_name, meth_names[cntr]);
+    }
+  free(meth_names);
+  }
 /* END pbs_log.c */
