@@ -77,6 +77,7 @@
 * without reference to its choice of law rules.
 */
 #include <pbs_config.h>   /* the master config generated by configure */
+#include "lib_net.h"
 
 #include <ctype.h>
 #include <stdio.h>
@@ -93,6 +94,7 @@
 #include "net_connect.h"
 #include "pbs_error.h"
 #include "log.h"
+#include "../Liblog/log_event.h"
 
 #if !defined(H_ERRNO_DECLARED) && !defined(_AIX)
 extern int h_errno;
@@ -115,34 +117,43 @@ extern const char *hstrerror(int);
  */
 
 
-char *PAddrToString(
-
-  pbs_net_t *Addr)
-
-  {
-  static char tmpLine[1024];
-
-  sprintf(tmpLine, "%lu",
-          *(unsigned long *)Addr);
-
-  return(tmpLine);
-  }
-
-
-
-
-
 pbs_net_t get_hostaddr(
 
-  char *hostname) /* I */
+  int  *local_errno, /* O */    
+  char *hostname)    /* I */
 
   {
+  pbs_net_t       rval = 0;
+  char           *tmp_addr = NULL;
+  struct in_addr  hostaddr;
+  int             tmp_addr_len = 0;
+  unsigned short  af_family;
 
-  static struct in_addr  hostaddr;
+  if ((rval = get_hostaddr_hostent_af(local_errno, hostname, &af_family, &tmp_addr, &tmp_addr_len)) == PBSE_NONE)
+    {
+    memcpy((void *)&hostaddr, (void *)tmp_addr, tmp_addr_len);
+    rval = (pbs_net_t)ntohl(hostaddr.s_addr);
+    }
 
-  struct hostent        *hp;
+  if (tmp_addr != NULL)
+    free(tmp_addr);
 
-  extern int pbs_errno;
+  return(rval);
+  }
+
+int  get_hostaddr_hostent(
+
+  int   *local_errno,
+  char  *hostname,
+  char **host_addr,
+  int   *host_addr_len)
+
+  {
+  int                    rc = PBSE_NONE;
+  int                    addr_rc;
+  struct addrinfo       *addr_info;
+  char                   log_buf[LOCAL_LOG_BUF_SIZE];
+  char                  *tmp_ip = NULL;
 
 #ifdef NUMA_SUPPORT
   /* if this is a numa host, just get the parent node's address */
@@ -162,51 +173,146 @@ pbs_net_t get_hostaddr(
       *dash = '\0';
 
       /* check if this resolves to a hostname without the dash */
-      if (!(hp = gethostbyname(hostname)))
+      if ((addr_rc = getaddrinfo(hostname, NULL, NULL, &addr_info)) != 0)
         {
         /* not a numa-owned node, act normal */
         *dash = '-';
-        hp = gethostbyname(hostname);
+      
+        addr_rc = getaddrinfo(hostname, NULL, NULL, &addr_info);
         }
       }
     /* otherwise proceed with just the parent hostname so 
      * it can be resolved */
     else
-      hp = gethostbyname(hostname);
+      addr_rc = getaddrinfo(hostname, NULL, NULL, &addr_info);
     }
   else
-    hp = gethostbyname(hostname);
+    addr_rc = getaddrinfo(hostname, NULL, NULL, &addr_info);
 #else
-  hp = gethostbyname(hostname);
+  addr_rc = getaddrinfo(hostname, NULL, NULL, &addr_info);
 #endif /* NUMA_SUPPORT */
 
-
-  if (hp == NULL)
+  if (addr_rc != 0)
     {
-    snprintf(log_buffer, sizeof(log_buffer),
+    snprintf(log_buf, sizeof(log_buf),
       "cannot resolve IP address for host '%s' herror=%d: %s",
       hostname,
       h_errno,
       hstrerror(h_errno));
 
-    log_event(
-      PBSEVENT_SYSTEM,
-      PBS_EVENTCLASS_SERVER,
-      "get_hostaddr",
-      log_buffer);
+    log_event(PBSEVENT_SYSTEM,PBS_EVENTCLASS_SERVER,"get_hostaddr_hostent",log_buf);
 
     if (h_errno == TRY_AGAIN)
-      pbs_errno = PBS_NET_RC_RETRY;
+      *local_errno = rc = PBS_NET_RC_RETRY;
     else
-      pbs_errno = PBS_NET_RC_FATAL;
+      *local_errno = rc = PBS_NET_RC_FATAL;
 
-    return((pbs_net_t)0);
+    return(rc);
     }
 
-  memcpy((void *)&hostaddr, (void *)hp->h_addr_list[0], hp->h_length);
+  if ((tmp_ip = (char *)calloc(1, sizeof(struct in_addr) + 1)) == NULL)
+    {
+    *local_errno = rc = PBS_NET_RC_FATAL;
+    }
+  else
+    {
+    memcpy(tmp_ip, &((struct sockaddr_in *)addr_info->ai_addr)->sin_addr, sizeof(struct in_addr));
+    *host_addr = tmp_ip;
+    *host_addr_len = sizeof(struct in_addr);
+    }
+
+  freeaddrinfo(addr_info);
+
+  return(rc);
+  }  /* END get_hostaddr_hostent() */
+
+int  get_hostaddr_hostent_af(
+
+  int             *local_errno,
+  char            *hostname,
+  unsigned short  *af_family,
+  char           **host_addr,
+  int             *host_addr_len)
+
+  {
+  int                    rc = PBSE_NONE;
+  int                    addr_rc;
+  struct addrinfo       *addr_info;
+  char                   log_buf[LOCAL_LOG_BUF_SIZE];
+  char                  *tmp_ip = NULL;
+
+#ifdef NUMA_SUPPORT
+  /* if this is a numa host, just get the parent node's address */
+  char *dash;
+
+  if ((dash = strchr(hostname,'-')) != NULL)
+    {
+    char *tmp;
+
+    /* make sure to use the last dash */
+    while ((tmp = strchr(dash+1,'-')))
+      dash = tmp;
+
+    if (isdigit(*(dash+1)))
+      {
+      /* terminate string temporarily */
+      *dash = '\0';
+
+      /* check if this resolves to a hostname without the dash */
+      if ((addr_rc = getaddrinfo(hostname, NULL, NULL, &addr_info)) != 0)
+        {
+        /* not a numa-owned node, act normal */
+        *dash = '-';
+      
+        addr_rc = getaddrinfo(hostname, NULL, NULL, &addr_info);
+        }
+      }
+    /* otherwise proceed with just the parent hostname so 
+     * it can be resolved */
+    else
+      addr_rc = getaddrinfo(hostname, NULL, NULL, &addr_info);
+    }
+  else
+    addr_rc = getaddrinfo(hostname, NULL, NULL, &addr_info);
+#else
+  addr_rc = getaddrinfo(hostname, NULL, NULL, &addr_info);
+#endif /* NUMA_SUPPORT */
+
+  if (addr_rc != 0)
+    {
+    snprintf(log_buf, sizeof(log_buf),
+      "cannot resolve IP address for host '%s' herror=%d: %s",
+      hostname,
+      h_errno,
+      hstrerror(h_errno));
+
+    log_event(PBSEVENT_SYSTEM,PBS_EVENTCLASS_SERVER,"get_hostaddr_hostent",log_buf);
+
+    if (h_errno == TRY_AGAIN)
+      *local_errno = rc = PBS_NET_RC_RETRY;
+    else
+      *local_errno = rc = PBS_NET_RC_FATAL;
+
+    return(rc);
+    }
+
+  if ((tmp_ip = (char *)calloc(1, sizeof(struct in_addr) + 1)) == NULL)
+    {
+    *local_errno = rc = PBS_NET_RC_FATAL;
+    }
+  else
+    {
+    memcpy(tmp_ip, &((struct sockaddr_in *)addr_info->ai_addr)->sin_addr, sizeof(struct in_addr));
+    *host_addr = tmp_ip;
+    *host_addr_len = sizeof(struct in_addr);
+    *af_family = addr_info->ai_family;
+    }
+
+  freeaddrinfo(addr_info);
+
+  return(rc);
+  }  /* END get_hostaddr_hostent() */
 
-  return((pbs_net_t)ntohl(hostaddr.s_addr));
-  }  /* END get_hostaddr() */
 
 /* END get_hostaddr.c */
 
