@@ -80,6 +80,7 @@
 #define _DARWIN_UNLIMITED_SELECT
 
 #include <pbs_config.h>   /* the master config generated by configure */
+#include "lib_net.h"
 
 #include <netdb.h>
 #include <string.h>
@@ -99,6 +100,7 @@
 #include <sys/un.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#include <pthread.h> /* thread_func functions */
 
 #if defined(FD_SET_IN_SYS_SELECT_H)
 #include <sys/select.h>
@@ -106,7 +108,7 @@
 #if defined(NTOHL_NEEDS_ARPA_INET_H) && defined(HAVE_ARPA_INET_H)
 #include <arpa/inet.h>
 #endif
-
+#include <pthread.h>
 
 
 
@@ -114,12 +116,16 @@
 #include "server_limits.h"
 #include "net_connect.h"
 #include "log.h"
+#include "../Liblog/pbs_log.h" /* log_err */
+#include "../Liblog/log_event.h" /* log_event */
+#include "../Libifl/lib_ifl.h" /* DIS_* */
+#include "pbs_error.h" /* PBSE_NONE */
 
-extern int LOGLEVEL;
-
+extern int  LOGLEVEL;
 
 /* External Functions Called */
 
+void initialize_connections_table();
 extern void process_request(int);
 
 extern time_t time(time_t *);
@@ -137,22 +143,28 @@ struct connection svr_conn[PBS_NET_MAX_CONNECTIONS];
  * The following data is private to this set of network interface routines.
  */
 
-static int max_connection = PBS_NET_MAX_CONNECTIONS;
-static int num_connections = 0;
-static fd_set *GlobalSocketReadSet = NULL;
-static void (*read_func[2])(int);
+static int       max_connection = PBS_NET_MAX_CONNECTIONS;
+static int       num_connections = 0;
+pthread_mutex_t *num_connections_mutex = NULL;
+static fd_set   *GlobalSocketReadSet = NULL;
+pthread_mutex_t *global_sock_read_mutex = NULL;
 
-pbs_net_t pbs_server_addr;
+static void *(*read_func[2])(void *);
+
+pthread_mutex_t *netrates_mutex = NULL;
+pthread_mutex_t *nc_list_mutex  = NULL;
+
+pbs_net_t        pbs_server_addr;
 
 /* Private function within this file */
 
-static void accept_conn();
+void *accept_conn(void *);
 
 
 static struct netcounter nc_list[60];
 
-void
-netcounter_incr(void)
+void netcounter_incr(void)
+
   {
   time_t now, lastmin;
   int i;
@@ -160,6 +172,8 @@ netcounter_incr(void)
   now = time(NULL);
   lastmin = now - 60;
 
+  pthread_mutex_lock(nc_list_mutex);
+
   if (nc_list[0].time == now)
     {
     nc_list[0].counter++;
@@ -180,22 +194,39 @@ netcounter_incr(void)
         }
       }
     }
+
+  pthread_mutex_unlock(nc_list_mutex);
   }
 
 
 int get_num_connections()
   {
-  return(num_connections);
+  int ret_num_connections;
+
+  pthread_mutex_lock(num_connections_mutex);
+  ret_num_connections = num_connections;
+  pthread_mutex_unlock(num_connections_mutex);
+
+  return(ret_num_connections);
   }
 
 
-int *
-netcounter_get(void)
+int *netcounter_get(void)
+
   {
   static int netrates[3];
   int netsums[3] = {0, 0, 0};
   int i;
 
+  if (netrates_mutex == NULL)
+    {
+    netrates_mutex = calloc(1, sizeof(pthread_mutex_t));
+    pthread_mutex_init(netrates_mutex,NULL);
+    }
+
+  pthread_mutex_lock(netrates_mutex);
+  pthread_mutex_lock(nc_list_mutex);
+
   for (i = 0;i < 5;i++)
     {
     netsums[0] += nc_list[i].counter;
@@ -214,6 +245,8 @@ netcounter_get(void)
     netsums[2] += nc_list[i].counter;
     }
 
+  pthread_mutex_unlock(nc_list_mutex);
+
   if (netsums[0] > 0)
     {
     netrates[0] = netsums[0] / 5;
@@ -245,7 +278,7 @@ netcounter_get(void)
 int init_network(
 
   unsigned int  port,
-  void        (*readfunc)())
+  void        *(*readfunc)(void *))
 
   {
   int   i;
@@ -256,25 +289,46 @@ int init_network(
 
   struct sockaddr_in socname;
   enum conn_type   type;
+  pthread_mutexattr_t t_attr;
 #ifdef ENABLE_UNIX_SOCKETS
 
   struct sockaddr_un unsocname;
   int unixsocket;
   memset(&unsocname, 0, sizeof(unsocname));
 #endif
- 
+
+  pthread_mutexattr_init(&t_attr);
+  pthread_mutexattr_settype(&t_attr, PTHREAD_MUTEX_ERRORCHECK);
+
   MaxNumDescriptors = get_max_num_descriptors();
 
   memset(&socname, 0, sizeof(socname));
 
   if (initialized == 0)
     {
-    for (i = 0;i < PBS_NET_MAX_CONNECTIONS;i++)
-      svr_conn[i].cn_active = Idle;
+    DIS_tcp_init(-1);
+    initialize_connections_table();
 
+    for (i = 0;i < PBS_NET_MAX_CONNECTIONS;i++)
+      {
+      svr_conn[i].cn_mutex = (pthread_mutex_t *)calloc(1, sizeof(pthread_mutex_t));
+      pthread_mutex_init(svr_conn[i].cn_mutex,NULL);
+      pthread_mutex_lock(svr_conn[i].cn_mutex);
+      
+      svr_conn[i].cn_active = Idle;
+      
+      pthread_mutex_unlock(svr_conn[i].cn_mutex);
+      }
+    
     /* initialize global "read" socket FD bitmap */
     GlobalSocketReadSet = (fd_set *)calloc(1,sizeof(char) * get_fdset_size());
 
+    global_sock_read_mutex = calloc(1, sizeof(pthread_mutex_t));
+    pthread_mutex_init(global_sock_read_mutex,&t_attr);
+
+    num_connections_mutex = calloc(1, sizeof(pthread_mutex_t));
+    pthread_mutex_init(num_connections_mutex,&t_attr);
+    
     type = Primary;
     }
   else if (initialized == 1)
@@ -304,12 +358,12 @@ int init_network(
       return(-1);
       }
 
-  if (MaxNumDescriptors < PBS_NET_MAX_CONNECTIONS)
-    max_connection = MaxNumDescriptors;
+    if (MaxNumDescriptors < PBS_NET_MAX_CONNECTIONS)
+      max_connection = MaxNumDescriptors;
 
     i = 1;
 
-    setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (char *)&i, sizeof(i));
+    setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &i, sizeof(i));
 
     /* name that socket "in three notes" */
 
@@ -392,11 +446,21 @@ int init_network(
     {
     /* allocate a minute's worth of counter structs */
 
+    if (nc_list_mutex == NULL)
+      {
+      nc_list_mutex = calloc(1, sizeof(pthread_mutex_t));
+      pthread_mutex_init(nc_list_mutex,NULL);
+      }
+
+    pthread_mutex_lock(nc_list_mutex);
+
     for (i = 0;i < 60;i++)
       {
       nc_list[i].time = 0;
       nc_list[i].counter = 0;
       }
+
+    pthread_mutex_unlock(nc_list_mutex);
     }
 
   return(0);
@@ -419,9 +483,6 @@ int wait_request(
   long   *SState)     /* I (optional) */
 
   {
-  extern char *PAddrToString(pbs_net_t *);
-  void close_conn();
-
   int i;
   int n;
 
@@ -448,12 +509,15 @@ int wait_request(
 
   SelectSetSize = sizeof(char) * get_fdset_size();
   SelectSet = (fd_set *)calloc(1,SelectSetSize);
-
+  
+  pthread_mutex_lock(global_sock_read_mutex);
+  
   memcpy(SelectSet,GlobalSocketReadSet,SelectSetSize);
- 
+
   /* selset = readset;*/  /* readset is global */
   MaxNumDescriptors = get_max_num_descriptors();
 
+  pthread_mutex_unlock(global_sock_read_mutex);
   n = select(MaxNumDescriptors, SelectSet, (fd_set *)0, (fd_set *)0, &timeout);
 
   if (n == -1)
@@ -470,11 +534,11 @@ int wait_request(
 
       /* check all file descriptors to verify they are valid */
 
-      /* NOTE:  selset may be modified by failed select() */
+      /* NOTE: selset may be modified by failed select() */
 
-      for (i = 0;i < MaxNumDescriptors;i++)
+      for (i = 0; i < MaxNumDescriptors; i++)
         {
-        if (FD_ISSET(i, GlobalSocketReadSet) == 0)
+        if (FD_ISSET(i, SelectSet) == 0)
           continue;
 
         if (fstat(i, &fbuf) == 0)
@@ -482,55 +546,79 @@ int wait_request(
 
         /* clean up SdList and bad sd... */
 
+        pthread_mutex_lock(global_sock_read_mutex);
         FD_CLR(i, GlobalSocketReadSet);
-        }    /* END for (i) */
+        pthread_mutex_unlock(global_sock_read_mutex);
+        pthread_mutex_unlock(svr_conn[i].cn_mutex);
+
+        } /* END for each socket in global read set */
 
       free(SelectSet);
+
+      log_err(errno, id, "Unable to select sockets to read requests");
+
+
       return(-1);
       }  /* END else (errno == EINTR) */
     }    /* END if (n == -1) */
 
-  for (i = 0;(i < max_connection) && (n != 0);i++)
+  for (i = 0; (i < max_connection) && (n != 0); i++)
     {
+    pthread_mutex_lock(svr_conn[i].cn_mutex);
+
     if (FD_ISSET(i, SelectSet))
       {
       /* this socket has data */
-
       n--;
 
-      svr_conn[i].cn_lasttime = time((time_t *)0);
+      svr_conn[i].cn_lasttime = time(NULL);
 
       if (svr_conn[i].cn_active != Idle)
         {
+        void *(*func)(void *) = svr_conn[i].cn_func;
+
         netcounter_incr();
 
-        svr_conn[i].cn_func(i);
+        pthread_mutex_unlock(svr_conn[i].cn_mutex);
+
+        if (func != NULL)
+          func((void *)&i);
 
         /* NOTE:  breakout if state changed (probably received shutdown request) */
 
-        if ((SState != NULL) && (OrigState != *SState))
+        if ((SState != NULL) && 
+            (OrigState != *SState))
           break;
         }
       else
         {
+        pthread_mutex_lock(global_sock_read_mutex);
         FD_CLR(i, GlobalSocketReadSet);
-        close_conn(i);
-        sprintf(tmpLine,"closed connection to fd %d - num_connections=%d (select bad socket)",
+        pthread_mutex_unlock(global_sock_read_mutex);
+
+        close_conn(i, TRUE);
+
+        pthread_mutex_unlock(svr_conn[i].cn_mutex);
+        pthread_mutex_lock(num_connections_mutex);
+
+        sprintf(tmpLine, "closed connections to fd %d - num_connections=%d (select bad socket)",
           i,
           num_connections);
 
-        log_err(-1,id,tmpLine);
+        pthread_mutex_unlock(num_connections_mutex);
+        log_err(-1, id, tmpLine);
         }
       }
-    }    /* END for (i) */
+    else
+      pthread_mutex_unlock(svr_conn[i].cn_mutex);
+    } /* END for i */
+
+  free(SelectSet);
 
   /* NOTE:  break out if shutdown request received */
 
   if ((SState != NULL) && (OrigState != *SState))
-    {
-    free(SelectSet);
     return(0);
-    }
 
   /* have any connections timed out ?? */
 
@@ -538,37 +626,51 @@ int wait_request(
 
   for (i = 0;i < max_connection;i++)
     {
-
     struct connection *cp;
 
+    pthread_mutex_lock(svr_conn[i].cn_mutex);
+
     cp = &svr_conn[i];
 
     if (cp->cn_active != FromClientDIS)
+      {
+      pthread_mutex_unlock(svr_conn[i].cn_mutex);
+
       continue;
+      }
 
     if ((now - cp->cn_lasttime) <= PBS_NET_MAXCONNECTIDLE)
+      {
+      pthread_mutex_unlock(svr_conn[i].cn_mutex);
+  
       continue;
+      }
 
     if (cp->cn_authen & PBS_NET_CONN_NOTIMEOUT)
+      {
+      pthread_mutex_unlock(svr_conn[i].cn_mutex);
+  
       continue; /* do not time-out this connection */
+      }
 
     /* NOTE:  add info about node associated with connection - NYI */
 
-    snprintf(tmpLine, sizeof(tmpLine), "connection %d to host %s has timed out after %d seconds - closing stale connection\n",
-             i,
-             PAddrToString(&cp->cn_addr),
-             PBS_NET_MAXCONNECTIDLE);
-
+    snprintf(tmpLine, sizeof(tmpLine), "connection %d to host %lu has timed out after %d seconds - closing stale connection\n",
+      i,
+      cp->cn_addr,
+      PBS_NET_MAXCONNECTIDLE);
+    
     log_err(-1, "wait_request", tmpLine);
 
     /* locate node associated with interface, mark node as down until node responds */
 
     /* NYI */
 
-    close_conn(i);
+    close_conn(i, TRUE);
+
+    pthread_mutex_unlock(svr_conn[i].cn_mutex);
     }  /* END for (i) */
 
-  free(SelectSet);
   return(0);
   }  /* END wait_request() */
 
@@ -583,29 +685,41 @@ int wait_request(
  * the new socket is added to the select set and the connection
  * structure - the processing routine is set to the external
  * function: process_request(socket)
+ *
+ * NOTE: accept conn is called by functions that have a mutex on the socket already 
  */
 
-static void accept_conn(
+void *accept_conn(
 
-  int sd)  /* main socket with connection request pending */
+  void *new_conn)  /* main socket with connection request pending */
 
   {
   int newsock;
-
   struct sockaddr_in from;
-
   struct sockaddr_un unixfrom;
-
   torque_socklen_t fromsize;
+  int sd = *(int *)new_conn;
+  int sock_type = -1;
 
   from.sin_addr.s_addr = 0;
   from.sin_port = 0;
 
+  pthread_mutex_lock(num_connections_mutex);
+  if (num_connections >= max_connection)
+    {
+    pthread_mutex_unlock(num_connections_mutex);
+
+    return(NULL);
+    }
+  pthread_mutex_unlock(num_connections_mutex);
+
   /* update lasttime of main socket */
-
+  pthread_mutex_lock(svr_conn[sd].cn_mutex);
   svr_conn[sd].cn_lasttime = time((time_t *)0);
+  sock_type = svr_conn[sd].cn_socktype;
+  pthread_mutex_unlock(svr_conn[sd].cn_mutex);
 
-  if (svr_conn[sd].cn_socktype == PBS_SOCK_INET)
+  if (sock_type == PBS_SOCK_INET)
     {
     fromsize = sizeof(from);
     newsock = accept(sd, (struct sockaddr *) & from, &fromsize);
@@ -618,28 +732,25 @@ static void accept_conn(
 
   if (newsock == -1)
     {
-    return;
     }
-
-  if ((num_connections >= max_connection) ||
-      (newsock >= PBS_NET_MAX_CONNECTIONS))
+  else if (newsock >= PBS_NET_MAX_CONNECTIONS)
     {
     close(newsock);
-
-    return;  /* too many current connections */
+    }
+  else
+    {
+    /* add the new socket to the select set and connection structure */
+    add_conn(
+      newsock,
+      FromClientDIS,
+      (pbs_net_t)ntohl(from.sin_addr.s_addr),
+      (unsigned int)ntohs(from.sin_port),
+      svr_conn[sd].cn_socktype,
+      read_func[(int)svr_conn[sd].cn_active]);
     }
 
-  /* add the new socket to the select set and connection structure */
 
-  add_conn(
-    newsock,
-    FromClientDIS,
-    (pbs_net_t)ntohl(from.sin_addr.s_addr),
-    (unsigned int)ntohs(from.sin_port),
-    svr_conn[sd].cn_socktype,
-    read_func[(int)svr_conn[sd].cn_active]);
-
-  return;
+  return(NULL);
   }  /* END accept_conn() */
 
 
@@ -652,19 +763,31 @@ static void accept_conn(
  * NOTE:  This routine cannot fail.
  */
 
-void add_conn(
+int add_conn(
 
   int            sock,    /* socket associated with connection */
   enum conn_type type,    /* type of connection */
   pbs_net_t      addr,    /* IP address of connected host */
   unsigned int   port,    /* port number (host order) on connected host */
   unsigned int   socktype, /* inet or unix */
-  void (*func)(int))  /* function to invoke on data rdy to read */
+  void *(*func)(void *))  /* function to invoke on data rdy to read */
 
   {
+  if (num_connections_mutex == NULL)
+    {
+    usleep(100000);
+    /* To solve a race condition where there are jobs in the queue, but the network hasn't been initialized yet. */
+    return PBSE_SOCKET_FAULT;
+    }
+  pthread_mutex_lock(num_connections_mutex);
   num_connections++;
+  pthread_mutex_unlock(num_connections_mutex);
 
+  pthread_mutex_lock(global_sock_read_mutex);
   FD_SET(sock, GlobalSocketReadSet);
+  pthread_mutex_unlock(global_sock_read_mutex);
+
+  pthread_mutex_lock(svr_conn[sock].cn_mutex);
 
   svr_conn[sock].cn_active   = type;
   svr_conn[sock].cn_addr     = addr;
@@ -701,7 +824,9 @@ void add_conn(
 
 #endif /* !NOPRIVPORTS */
 
-  return;
+  pthread_mutex_unlock(svr_conn[sock].cn_mutex);
+
+  return PBSE_NONE;
   }  /* END add_conn() */
 
 
@@ -715,25 +840,36 @@ void add_conn(
 
 void close_conn(
 
-  int sd) /* I */
+  int sd,        /* I */
+  int has_mutex) /* I */
 
   {
+  char log_message[LOG_BUF_SIZE];
   if ((sd < 0) || (max_connection <= sd))
     {
     return;
     }
 
+  if (has_mutex == FALSE)
+    pthread_mutex_lock(svr_conn[sd].cn_mutex);
+
   if (svr_conn[sd].cn_active == Idle)
     {
+    if (has_mutex == FALSE)
+      pthread_mutex_unlock(svr_conn[sd].cn_mutex);
+
     return;
     }
 
-  close(sd);
-
   /* if there is a function to call on close, do it */
 
   if (svr_conn[sd].cn_oncl != 0)
+    {
+    snprintf(log_message, LOG_BUF_SIZE, "Connection %d - func %lx",
+        sd, (unsigned long)svr_conn[sd].cn_oncl);
+    log_event(PBSEVENT_SYSTEM,PBS_EVENTCLASS_NODE,"close_conn",log_message);
     svr_conn[sd].cn_oncl(sd);
+    }
 
   /* 
    * In the case of a -t cold start, this will be called prior to
@@ -741,9 +877,13 @@ void close_conn(
    */
 
   if (GlobalSocketReadSet != NULL)
-  {
+    {
+    pthread_mutex_lock(global_sock_read_mutex);
     FD_CLR(sd, GlobalSocketReadSet);
-  }
+    pthread_mutex_unlock(global_sock_read_mutex);
+    }
+
+  close(sd);
 
   svr_conn[sd].cn_addr = 0;
 
@@ -751,11 +891,16 @@ void close_conn(
 
   svr_conn[sd].cn_active = Idle;
 
-  svr_conn[sd].cn_func = (void (*)())0;
+  svr_conn[sd].cn_func = (void *(*)())0;
 
   svr_conn[sd].cn_authen = 0;
+    
+  if (has_mutex == FALSE)
+    pthread_mutex_unlock(svr_conn[sd].cn_mutex);
 
+  pthread_mutex_lock(num_connections_mutex);
   num_connections--;
+  pthread_mutex_unlock(num_connections_mutex);
 
   return;
   }  /* END close_conn() */
@@ -784,9 +929,13 @@ void net_close(
     {
     if (i != but)
       {
+      pthread_mutex_lock(svr_conn[i].cn_mutex);
+
       svr_conn[i].cn_oncl = 0;
 
-      close_conn(i);
+      close_conn(i,TRUE);
+
+      pthread_mutex_unlock(svr_conn[i].cn_mutex);
       }
     }    /* END for (i) */
 
@@ -799,48 +948,32 @@ void net_close(
 /*
  * get_connectaddr - return address of host connected via the socket
  * This is in host order.
+ *
+ * mutex is TRUE if the mutex should be obtained, false otherwise
  */
-
 pbs_net_t get_connectaddr(
 
-  int sock)  /* I */
+  int sock,   /* I */
+  int mutex)  /* I */
 
   {
-  return(svr_conn[sock].cn_addr);
+  pbs_net_t tmp;
+
+  if (mutex == TRUE)
+    pthread_mutex_lock(svr_conn[sock].cn_mutex);
+
+  tmp = svr_conn[sock].cn_addr;
+
+  if (mutex == TRUE)
+    pthread_mutex_unlock(svr_conn[sock].cn_mutex);
+
+  return(tmp);
   }
 
 
-
-
-
-int find_conn(
-
-  pbs_net_t addr)  /* I */
-
-  {
-  int index;
-
-  /* NOTE:  there may be multiple connections per addr (not handled) */
-
-  for (index = 0;index < PBS_NET_MAX_CONNECTIONS;index++)
-    {
-    if (addr == svr_conn[index].cn_addr)
-      {
-      return(index);
-      }
-    }    /* END for (index) */
-
-  return(-1);
-  }  /* END find_conn() */
-
-
-
-
-
 /*
  * get_connecthost - return name of host connected via the socket
  */
-
 int get_connecthost(
 
   int   sock,     /* I */
@@ -848,37 +981,42 @@ int get_connecthost(
   int   size)     /* I */
 
   {
+  struct in_addr          addr;
 
-  struct hostent *phe;
+  struct sockaddr        *addr_info_ptr;
+  struct sockaddr_in      addr_in;
+  static struct in_addr   serveraddr;
+  static char            *server_name = NULL;
+  static pthread_mutex_t *get_connecthost_mutex = NULL;
 
-  struct in_addr  addr;
-  int             namesize = 0;
+  if (get_connecthost_mutex == NULL)
+    {
+    get_connecthost_mutex = calloc(1, sizeof(pthread_mutex_t));
+    pthread_mutex_init(get_connecthost_mutex,NULL);
+    }
 
-  static struct in_addr  serveraddr;
-  static char           *server_name = NULL;
+  addr_in.sin_family = AF_INET;
+  addr_in.sin_port = 0;
+  pthread_mutex_lock(get_connecthost_mutex);
 
   if ((server_name == NULL) && (pbs_server_addr != 0))
     {
     /* cache local server addr info */
-
     serveraddr.s_addr = htonl(pbs_server_addr);
+    addr_in.sin_addr = serveraddr;
+    addr_info_ptr = (struct sockaddr *)&addr_in;
 
-    if ((phe = gethostbyaddr(
-                 (char *) & serveraddr,
-                 sizeof(struct in_addr),
-                 AF_INET)) == NULL)
-      {
-      server_name = strdup(inet_ntoa(serveraddr));
-      }
+    if (getnameinfo(addr_info_ptr, sizeof(addr_in), namebuf, size, NULL, 0, 0) == 0)
+      server_name = strdup(namebuf);
     else
-      {
-      server_name = strdup(phe->h_name);
-      }
+      server_name = strdup(inet_ntoa(serveraddr));
     }
 
   size--;
 
   addr.s_addr = htonl(svr_conn[sock].cn_addr);
+  addr_in.sin_addr = addr;
+  addr_info_ptr = (struct sockaddr *)&addr_in;
 
   if ((server_name != NULL) && (svr_conn[sock].cn_socktype & PBS_SOCK_UNIX))
     {
@@ -890,49 +1028,40 @@ int get_connecthost(
     {
     /* lookup request is for local server */
 
-    strcpy(namebuf, server_name);
+    snprintf(namebuf, size, "%s", server_name);
     }
-  else if ((phe = gethostbyaddr(
-                    (char *) & addr,
-                    sizeof(struct in_addr),
-                    AF_INET)) == NULL)
+  else if (getnameinfo(addr_info_ptr, sizeof(addr_in), namebuf, size, NULL, 0, 0) != 0)
     {
-    strcpy(namebuf, inet_ntoa(addr));
+    snprintf(namebuf, size, "%s", inet_ntoa(addr));
     }
   else
     {
-    namesize = strlen(phe->h_name);
-
-    strncpy(namebuf, phe->h_name, size);
-
-    *(namebuf + size) = '\0';
-    }
-
-  if (namesize > size)
-    {
-    /* FAILURE - buffer too small */
-
-    return(-1);
+    if (strcmp(namebuf, "localhost") == 0)
+      {
+      snprintf(namebuf, size, "%s", server_name);
+      }
+    else
+      {
+      /* already in namebuf, NO-OP */
+      }
     }
 
   /* SUCCESS */
+  pthread_mutex_unlock(get_connecthost_mutex);
 
   return(0);
   }  /* END get_connecthost() */
 
 
-
-
 /*
 ** Put a human readable representation of a network address into
 ** a staticly allocated string.
 */
 char *netaddr_pbs_net_t(
-
   pbs_net_t ipadd)
-
   {
-  static char out[80];
+  char  out[80];
+  char *return_value;
 
   if (ipadd == 0)
     return "unknown";
@@ -943,11 +1072,11 @@ char *netaddr_pbs_net_t(
           (ipadd & 0x0000ff00) >> 8,
           (ipadd & 0x000000ff));
 
-  return (out);
+  return_value = calloc(1, strlen(out) + 1);
+  strcpy(return_value,out);
+
+  return(return_value);
   }
 
-
-
-
 /* END net_server.c */
 
