@@ -105,13 +105,8 @@
 #include "dis.h"
 #include "dis_init.h"
 #include "rm.h"
-#if RPP
-#include "rpp.h"
-#else
-extern void DIS_tcp_funcs();
-#endif
-
 
+extern int pbs_errno;
 static int full = 1;
 
 /*
@@ -147,10 +142,9 @@ static int addrm(
 
   struct out *op, **head;
 
-  if ((op = (struct out *)malloc(sizeof(struct out))) == NULL)
+  if ((op = (struct out *)calloc(1, sizeof(struct out))) == NULL)
     {
-    pbs_errno = errno;
-    return -1;
+    return(errno);
     }
 
   head = &outs[stream % HASHOUT];
@@ -165,39 +159,17 @@ static int addrm(
 
 
 
-#if RPP
-static void
-funcs_dis(void)   /* The equivalent of DIS_tcp_funcs() */
-  {
-  if (dis_getc != rpp_getc)
-    {
-    dis_getc = (int (*)(int))rpp_getc;
-    dis_puts = (int (*)(int, const char *, size_t))rpp_write;
-    dis_gets = (int (*)(int, char *, size_t))rpp_read;
-    disr_commit = (int (*)(int, int))rpp_rcommit;
-    disw_commit = (int (*)(int, int))rpp_wcommit;
-    }
-  }
-
-#define setup_dis(x) funcs_dis() /* RPP doesn't need reset */
-#define close_dis(x) rpp_close(x)
-#define flush_dis(x) rpp_flush(x)
-
-#else
-
-#define funcs_dis() DIS_tcp_funcs()
-#define setup_dis(x) DIS_tcp_setup(x)
 #define close_dis(x) close(x)
 #define flush_dis(x) DIS_tcp_wflush(x)
 
-#endif
-
 char TRMEMsg[1024];  /* global rm error message */
 
 
 /*
 ** Connects to a resource monitor and returns a file descriptor to
 ** talk to it.  If port is zero, use default port.
+** Returns the stream handle (>i= 0) on success.
+** Returns value < 0 on error.
 */
 
 int openrm(
@@ -206,16 +178,11 @@ int openrm(
   unsigned int  port)  /* I (optional,0=DEFAULT) */
 
   {
-  int                   stream;
-
-#if RPP
-  static int  first = 1;
-#endif /* RPP */
+  int                 stream;
+  int                 rc;
 
   static unsigned int gotport = 0;
 
-  pbs_errno = 0;
-
   if (port == 0)
     {
     if (gotport == 0)
@@ -227,49 +194,18 @@ int openrm(
     port = gotport;
     }
 
-#if RPP
-
-  if (first)
-    {
-    int tryport = IPPORT_RESERVED;
-
-    first = 0;
-
-    while (--tryport > 0)
-      {
-      if (rpp_bind(tryport) != -1)
-        break;
-
-      if ((errno != EADDRINUSE) && (errno != EADDRNOTAVAIL))
-        break;
-      }
-    }
-
-  if ((stream = rpp_open(host, port, TRMEMsg)) == -1)
-    {
-    pbs_errno = errno;
-
-    return(-1);
-    }
-
-#else /* RPP */
-
   if ((stream = socket(AF_INET, SOCK_STREAM, 0)) != -1)
     {
     int tryport = IPPORT_RESERVED;
 
-    struct sockaddr_in addr;
+    struct sockaddr_in  addr;
+    struct addrinfo    *addr_info;
 
-    struct hostent  *hp;
-
-    if ((hp = gethostbyname(host)) == NULL)
+    if (getaddrinfo(host, NULL, NULL, &addr_info) != 0)
       {
-      DBPRT(("host %s not found\n",
-             host))
+      DBPRT(("host %s not found\n", host))
 
-      pbs_errno = ENOENT;
-
-      return(-1);
+      return(ENOENT * -1);
       }
 
     memset(&addr, '\0', sizeof(addr));
@@ -281,46 +217,49 @@ int openrm(
       {
       addr.sin_port = htons((u_short)tryport);
 
-      if (bind(stream, (struct sockaddr *)&addr, sizeof(addr)) != -1)
+      rc = bind(stream, (struct sockaddr *)&addr, sizeof(addr));
+      if (rc == 0)
         break;
 
-      if ((errno != EADDRINUSE) && (errno != EADDRNOTAVAIL))
-        break;
+      if ((errno == EADDRINUSE) || (errno == EADDRNOTAVAIL))
+        {
+        struct timespec rem;
+        /* We can't get the port we want. Wait a bit and try again */
+        rem.tv_sec = 0;
+        rem.tv_nsec = 3000000;
+        nanosleep(&rem, &rem);
+        continue;
+        }
+
+      return(-1 * errno);
       }
 
     memset(&addr, '\0', sizeof(addr));
 
-    addr.sin_family = hp->h_addrtype;
+    addr.sin_addr = ((struct sockaddr_in *)addr_info->ai_addr)->sin_addr;
+    addr.sin_family = AF_INET;
     addr.sin_port = htons((unsigned short)port);
 
-    memcpy(&addr.sin_addr, hp->h_addr, hp->h_length);
+    freeaddrinfo(addr_info);
 
     if (connect(stream, (struct sockaddr *)&addr, sizeof(addr)) == -1)
       {
-      pbs_errno = errno;
-
       close(stream);
 
-      return(-1);
+      return(-1 * errno);
       }
     }    /* END if ((stream = socket(AF_INET,SOCK_STREAM,0)) != -1) */
 
-#endif /* RPP */
-
-  pbs_errno = errno;
-
   if (stream < 0)
     {
-    return(-1);
+    return(-1 * errno);
     }
 
   if (addrm(stream) == -1)
     {
-    pbs_errno = errno;
-
     close_dis(stream);
 
-    return(-1);
+    return(-1 * errno);
     }
 
   return(stream);
@@ -379,7 +318,8 @@ static int delrm(
 
 static struct out *findout(
 
-        int stream)
+  int *local_errno,
+  int  stream)
 
   {
 
@@ -392,7 +332,7 @@ static struct out *findout(
     }
 
   if (op == NULL)
-    pbs_errno = ENOTTY;
+    *local_errno = ENOTTY;
 
   return(op);
   }
@@ -402,14 +342,13 @@ static struct out *findout(
 
 static int startcom(
 
-  int stream,  /* I */
-  int com)     /* I */
+  int  stream,      /* I */
+  int *local_errno, /* O */
+  int  com)         /* I */
 
   {
   int ret;
 
-  setup_dis(stream);
-
   ret = diswsi(stream, RM_PROTOCOL);
 
   if (ret == DIS_SUCCESS)
@@ -428,7 +367,7 @@ static int startcom(
 
     /* log_err(ret,"startcom - diswsi error",(char *)dis_emsg[ret]); */
 
-    pbs_errno = errno;
+    *local_errno = errno;
     }
 
   return(ret);
@@ -446,21 +385,21 @@ static int startcom(
 
 static int simplecom(
 
-  int stream,
-  int com)
+  int  stream,
+  int *local_errno,
+  int  com)
 
   {
-
   struct out *op;
 
-  if ((op = findout(stream)) == NULL)
+  if ((op = findout(local_errno, stream)) == NULL)
     {
     return(-1);
     }
 
   op->len = -1;
 
-  if (startcom(stream, com) != DIS_SUCCESS)
+  if (startcom(stream, local_errno, com) != DIS_SUCCESS)
     {
     close_dis(stream);
 
@@ -469,21 +408,16 @@ static int simplecom(
 
   if (flush_dis(stream) == -1)
     {
-    pbs_errno = errno;
+    *local_errno = errno;
 
     DBPRT(("simplecom: flush error %d (%s)\n",
-           pbs_errno, pbs_strerror(pbs_errno)))
+           *local_errno, pbs_strerror(*local_errno)))
 
     close_dis(stream);
 
     return(-1);
     }
 
-#if RPP
-  rpp_eom(stream);
-
-#endif
-
   return(0);
   }  /* END simplecom() */
 
@@ -497,7 +431,8 @@ static int simplecom(
 
 static int simpleget(
 
-  int stream)
+  int *local_errno,
+  int  stream)
 
   {
   int ret, num;
@@ -511,7 +446,7 @@ static int simpleget(
 
     /* log_err(ret,"simpleget",(char *)dis_emsg[ret]); */
 
-    pbs_errno = errno ? errno : EIO;
+    *local_errno = errno ? errno : EIO;
 
     close_dis(stream);
 
@@ -521,9 +456,9 @@ static int simpleget(
   if (num != RM_RSP_OK)
     {
 #ifdef ENOMSG
-    pbs_errno = ENOMSG;
+    *local_errno = ENOMSG;
 #else
-    pbs_errno = EINVAL;
+    *local_errno = EINVAL;
 #endif
 
     return(-1);
@@ -538,50 +473,56 @@ static int simpleget(
 
 /*
 ** Close connection to resource monitor.  Return result 0 if
-** all is ok or -1 if not (set pbs_errno).
+** all is ok or -1 if not (return errno).
 */
 
+int closerm_err(
+
+  int *local_errno,
+  int  stream)
+
+  {
+  simplecom(stream, local_errno, RM_CMD_CLOSE);
+
+  if (delrm(stream) == -1)
+    {
+    return(ENOTTY);
+    }
+
+  return(0);
+  }  /* END closerm_err() */
+
+
+
+
 int closerm(
 
   int stream)
 
   {
-  pbs_errno = 0;
-
-  simplecom(stream, RM_CMD_CLOSE);
-
-  if (delrm(stream) == -1)
-    {
-    pbs_errno = ENOTTY;
-
-    return(-1);
-    }
-
-  return(0);
-  }  /* END closerm() */
-
+  return(closerm_err(&pbs_errno, stream));
+  } /* END closerm() */
 
 
 
 
 /*
 ** Shutdown the resource monitor.  Return result 0 if
-** all is ok or -1 if not (set pbs_errno).
+** all is ok or -1 if not.
 */
 
 int downrm(
 
-  int stream)  /* I */
+  int *local_errno,
+  int  stream)  /* I */
 
   {
-  pbs_errno = 0;
-
-  if (simplecom(stream, RM_CMD_SHUTDOWN))
+  if (simplecom(stream, local_errno, RM_CMD_SHUTDOWN))
     {
     return(-1);
     }
 
-  if (simpleget(stream))
+  if (simpleget(local_errno, stream))
     {
     return(-1);
     }
@@ -596,13 +537,14 @@ int downrm(
 
 /*
 ** Cause the resource monitor to read the file named.
-** Return the result 0 if all is ok or -1 if not (set pbs_errno).
+** Return the result 0 if all is ok or -1 if not.
 */
 
 int configrm(
 
-  int   stream,  /* I */
-  char *file)    /* I */
+  int   stream,      /* I */
+  int  *local_errno, /* O */
+  char *file)        /* I */
 
   {
   int         ret;
@@ -610,9 +552,7 @@ int configrm(
 
   struct out *op;
 
-  pbs_errno = 0;
-
-  if ((op = findout(stream)) == NULL)
+  if ((op = findout(local_errno, stream)) == NULL)
     {
     return(-1);
     }
@@ -625,12 +565,10 @@ int configrm(
 
   if ((len = strlen(file)) > (size_t)65536)
     {
-    pbs_errno = EINVAL;
-
-    return(-1);
+    return(-1 * EINVAL);
     }
 
-  if (startcom(stream, RM_CMD_CONFIG) != DIS_SUCCESS)
+  if (startcom(stream, local_errno, RM_CMD_CONFIG) != DIS_SUCCESS)
     {
     return(-1);
     }
@@ -641,11 +579,11 @@ int configrm(
   if (ret != DIS_SUCCESS)
     {
 #if defined(ECOMM)
-    pbs_errno = ECOMM;
+    *local_errno = ECOMM;
 #elif defined(ENOCONNECT)
-    pbs_errno = ENOCONNECT;
+    *local_errno = ENOCONNECT;
 #else
-    pbs_errno = ETXTBSY;
+    *local_errno = ETXTBSY;
 #endif
 
     DBPRT(("configrm: diswcs %s\n",
@@ -656,15 +594,13 @@ int configrm(
 
   if (flush_dis(stream) == -1)
     {
-    pbs_errno = errno;
-
     DBPRT(("configrm: flush error %d (%s)\n",
-           pbs_errno, pbs_strerror(pbs_errno)))
+           errno, pbs_strerror(errno)))
 
-    return(-1);
+    return(-1 * errno);
     }
 
-  if (simpleget(stream))
+  if (simpleget(local_errno, stream))
     {
     return(-1);
     }
@@ -680,7 +616,7 @@ int configrm(
 ** Begin a new message to the resource monitor if necessary.
 ** Add a line to the body of an outstanding command to the resource
 ** monitor.
-** Return the result 0 if all is ok or -1 if not (set pbs_errno).
+** Return the result 0 if all is ok or -1 if not.
 */
 
 static int doreq(
@@ -690,14 +626,18 @@ static int doreq(
 
   {
   int ret;
+  int local_errno = 0;
 
   if (op->len == -1)
     {
     /* start new message */
 
-    if (startcom(op->stream, RM_CMD_REQUEST) != DIS_SUCCESS)
+    if (startcom(op->stream, &local_errno, RM_CMD_REQUEST) != DIS_SUCCESS)
       {
-      return(-1);
+      if (local_errno != 0)
+        return(-1 * local_errno);
+      else
+        return(-1);
       }
 
     op->len = 1;
@@ -709,17 +649,17 @@ static int doreq(
   if (ret != DIS_SUCCESS)
     {
 #if defined(ECOMM)
-    pbs_errno = ECOMM;
+    local_errno = ECOMM;
 #elif defined(ENOCONNECT)
-    pbs_errno = ENOCONNECT;
+    local_errno = ENOCONNECT;
 #else
-    pbs_errno = ETXTBSY;
+    local_errno = ETXTBSY;
 #endif
 
     DBPRT(("doreq: diswcs %s\n",
            dis_emsg[ret]))
 
-    return(-1);
+    return(-1 * local_errno);
     }
 
   return(0);
@@ -733,24 +673,20 @@ static int doreq(
 ** Add a request to a single stream.
 */
 
-int addreq(
+int addreq_err(
 
   int   stream,
+  int  *local_errno, 
   char *line)
 
   {
-
   struct out *op;
 
-  pbs_errno = 0;
-
-  if ((op = findout(stream)) == NULL)
+  if ((op = findout(local_errno, stream)) == NULL)
     {
     return(-1);
     }
 
-  funcs_dis();
-
   if (doreq(op, line) == -1)
     {
     delrm(stream);
@@ -759,7 +695,19 @@ int addreq(
     }
 
   return(0);
-  }  /* END addreq() */
+  }  /* END addreq_err() */
+
+
+
+
+int addreq(
+
+  int   stream,
+  char *line)
+
+  {
+  return(addreq_err(stream, &pbs_errno, line));
+  } /* END addreq() */
 
 
 
@@ -779,10 +727,6 @@ int allreq(
   struct out *op, *prev;
   int         i, num;
 
-  funcs_dis();
-
-  pbs_errno = 0;
-
   num = 0;
 
   for (i = 0; i < HASHOUT;i++)
@@ -828,12 +772,13 @@ int allreq(
 /*
 ** Finish (and send) any outstanding message to the resource monitor.
 ** Return a pointer to the next response line or a NULL if
-** there are no more or an error occured.  Set pbs_errno on error.
+** there are no more or an error occured. 
 */
 
-char *getreq(
+char *getreq_err(
 
-  int stream)  /* I */
+  int *local_errno,
+  int  stream)  /* I */
 
   {
   char *startline;
@@ -841,9 +786,7 @@ char *getreq(
   struct out *op;
   int ret;
 
-  pbs_errno = 0;
-
-  if ((op = findout(stream)) == NULL)
+  if ((op = findout(local_errno, stream)) == NULL)
     {
     return(NULL);
     }
@@ -854,10 +797,8 @@ char *getreq(
 
     if (flush_dis(stream) == -1)
       {
-      pbs_errno = errno;
-
       DBPRT(("getreq: flush error %d (%s)\n",
-             pbs_errno, pbs_strerror(pbs_errno)))
+             errno, pbs_strerror(errno)))
 
       delrm(stream);
 
@@ -866,16 +807,11 @@ char *getreq(
 
     op->len = -2;
 
-#if RPP
-    rpp_eom(stream);
-#endif
     }
 
-  funcs_dis();
-
   if (op->len == -2)
     {
-    if (simpleget(stream) == -1)
+    if (simpleget(local_errno, stream) == -1)
       {
       return(NULL);
       }
@@ -893,7 +829,8 @@ char *getreq(
 
   if (ret != DIS_SUCCESS)
     {
-    pbs_errno = errno ? errno : EIO;
+    if (!errno)
+      errno = EIO;
 
     DBPRT(("getreq: cannot read string %s\n",
            dis_emsg[ret]))
@@ -926,27 +863,34 @@ char *getreq(
     }    /* END if (!full) */
 
   return(startline);
-  }  /* END getreq() */
+  }  /* END getreq_err() */
 
 
 
 
+char *getreq(
+
+  int stream)
+
+  {
+  return(getreq_err(&pbs_errno, stream));
+  } /* END getreq() */
+
+
+
 
 /*
 ** Finish and send any outstanding messages to all resource monitors.
 ** Return the number of messages flushed.
 */
 
-int
-flushreq(void)
+int flushreq(void)
 
   {
 
   struct out *op, *prev;
   int         did, i;
 
-  pbs_errno = 0;
-
   did = 0;
 
   for (i = 0;i < HASHOUT;i++)
@@ -958,10 +902,8 @@ flushreq(void)
 
       if (flush_dis(op->stream) == -1)
         {
-        pbs_errno = errno;
-
         DBPRT(("flushreq: flush error %d (%s)\n",
-               pbs_errno, pbs_strerror(pbs_errno)))
+               errno, pbs_strerror(errno)))
 
         close_dis(op->stream);
 
@@ -972,12 +914,6 @@ flushreq(void)
 
       op->len = -2;
 
-#if RPP
-
-      rpp_eom(op->stream);
-
-#endif /* RPP */
-
       did++;
       }  /* END for (op) */
 
@@ -1037,16 +973,6 @@ int activereq(void)
   static char id[] = "activereq";
 #endif
 
-#if RPP
-
-  struct out *op;
-
-  int       try;
-
-  int         bucket;
-
-#endif /* RPP */
-
   int            i, num;
 
   struct timeval tv;
@@ -1055,80 +981,11 @@ int activereq(void)
 
   int MaxNumDescriptors = 0;
 
-  pbs_errno = 0;
-
   flushreq();
 
   MaxNumDescriptors = get_max_num_descriptors();
   FDSet = (fd_set *)calloc(1,sizeof(char) * get_fdset_size());
 
-#if RPP
-  for (try = 0;try < 3;)
-    {
-    if ((i = rpp_poll()) >= 0)
-      {
-      if ((op = findout(i)) != NULL)
-        {
-        free(FDSet);
-        return(i);
-        }
-
-      op = (struct out *)malloc(sizeof(struct out));
-
-      if (op == NULL)
-        {
-        pbs_errno = errno;
-
-        free(FDSet);
-        return(-1);
-        }
-
-      bucket = i % HASHOUT;
-
-      op->stream = i;
-      op->len = -2;
-      op->next = outs[bucket];
-
-      outs[bucket] = op;
-      }
-    else if (i == -1)
-      {
-      pbs_errno = errno;
-
-      free(FDSet);
-      return(-1);
-      }
-    else
-      {
-
-			FD_SET(rpp_fd, FDSet);
-      tv.tv_sec = 5;
-      tv.tv_usec = 0;
-			num = select(FD_SETSIZE, FDSet, NULL, NULL, &tv);
-
-      if (num == -1)
-        {
-        pbs_errno = errno;
-        DBPRT(("%s: select %d %s\n", id, pbs_errno, pbs_strerror(pbs_errno)))
-        free(FDSet);
-        return -1;
-        }
-
-      if (num == 0)
-        {
-        try++;
-
-        DBPRT(("%s: timeout %d\n", id, try))
-          }
-      }
-    }
-
-  free(FDSet);
-  return i;
-
-#else
-  pbs_errno = 0;
-
   for (i = 0; i < HASHOUT; i++)
     {
 
@@ -1151,8 +1008,7 @@ int activereq(void)
 
   if (num == -1)
     {
-    pbs_errno = errno;
-    DBPRT(("%s: select %d %s\n", id, pbs_errno, pbs_strerror(pbs_errno)))
+    DBPRT(("%s: select %d %s\n", id, errno, pbs_strerror(errno)))
     free(FDSet);
     return -1;
     }
@@ -1182,9 +1038,8 @@ int activereq(void)
     }
 
   free(FDSet);
-  return(-2);
 
-#endif
+  return(-2);
   }  /* END activereq() */
 
 
@@ -1197,17 +1052,11 @@ int activereq(void)
 ** This makes it possible to examine the entire line rather
 ** than just the answer following the equal sign.
 */
-void
-fullresp(int flag)
+void fullresp(
+    
+  int flag)
+
   {
-#if RPP
-
-  if (flag)
-    rpp_dbprt = 1 - rpp_dbprt; /* toggle RPP debug */
-
-#endif
-  pbs_errno = 0;
-
   full = flag;
 
   return;
