@@ -640,7 +640,7 @@ int MXMLToXString(
     {
     NewSize = MMAX_BUFFER;
 
-    if ((*Buf = (char *)malloc(NewSize)) == NULL)
+    if ((*Buf = (char *)calloc(1, NewSize)) == NULL)
       {
       /* cannot allocate buffer */
 
@@ -676,7 +676,7 @@ int MXMLToXString(
 
     NewSize = MIN(NewSize << 1, MaxBufSize);
 
-    tmpBuf = (char *)realloc(*Buf,NewSize);
+    tmpBuf = (char *)calloc(1, NewSize);
 
     if (tmpBuf == NULL)
       {
@@ -684,6 +684,8 @@ int MXMLToXString(
 
       return(FAILURE);
       }
+    strcat(tmpBuf, *Buf);
+    free(*Buf);
 
     *Buf = tmpBuf;
 
@@ -1084,14 +1086,12 @@ int MXMLGetAttr(
       {
       if ((AName != NULL) && (AName[0] == '\0'))
         {
-        strncpy(AName, E->AName[aindex], MMAX_NAME);
-        AName[MMAX_NAME - 1] = '\0';
+        snprintf(AName, MMAX_NAME, "%s", E->AName[aindex]);
         }
 
       if (AVal != NULL)
         {
-        strncpy(AVal, E->AVal[aindex], EVSize);
-        AVal[EVSize - 1] = '\0';
+        snprintf(AVal, EVSize, "%s", E->AVal[aindex]);
         }
 
       if (ATok != NULL)
