@@ -1,4 +1,10 @@
+
+
+#include <unistd.h>
+#include <sys/types.h>
+
 #include "utils.h"
+#include "../Liblog/pbs_log.h"
 
 
 
@@ -22,6 +28,40 @@ int is_whitespace(
 
 
 
+char *trim(
+
+  char *str) /* M */
+
+  {
+  char *front_ptr;
+  char *end_ptr;
+
+  if (str == NULL)
+    return(NULL);
+
+  front_ptr = str;
+  end_ptr = str + strlen(str) - 1;
+
+  while ((is_whitespace(*front_ptr) == TRUE) &&
+         (front_ptr <= end_ptr))
+    {
+    front_ptr++;
+    }
+
+  while ((is_whitespace(*end_ptr) == TRUE) &&
+         (end_ptr > front_ptr))
+    {
+    *end_ptr = '\0';
+    end_ptr--;
+    }
+
+  return(front_ptr);
+  } /* END trim() */
+
+
+
+
+
 /**
  * similar to MUSNPrintF in moab
 */
@@ -65,88 +105,83 @@ int MUSNPrintF(
 
 
 
+/*
+ * a threadsafe tokenizing function - this also alters the input string
+ * just like strtok does.
+ *
+ * @param str - a modified string pointer (advanced past the token)
+ * @param delims - the delimiters to look for
+ * @return - a pointer to the token
+ */
 
-char *MUStrTok(
+char *threadsafe_tokenizer(
 
-  char  *Line,  /* I (optional) */
-  char  *DList, /* I */
-  char **Ptr)   /* O point to termination character of returned token */
+  char **str,    /* M */
+  char  *delims) /* I */
 
   {
-  static char *Head = NULL;
-
-  int dindex;
-
-  mbool_t ignchar;
-
-  /* NOTE:  no handling if Line is NULL and Ptr is uninitialized */
-
-  if (Ptr == NULL)
-    {
-    /* tokPtr not specified */
+  char *current_char;
+  char *start;
 
+  if ((str == NULL) ||
+      (*str == NULL))
     return(NULL);
-    }
 
-  if (Line != NULL)
-    {
-    *Ptr = Line;
+  /* save start position */
+  start = *str;
 
-    if (Line[0] == '\0')
-      {
-      /* line is empty */
-
-      if (Ptr != NULL)
-        *Ptr = NULL;
-
-      return(NULL);
-      }
-    }
-  else if (*Ptr == NULL)
-    {
+  /* return NULL at the end of the string */
+  if (*start == '\0')
     return(NULL);
+
+  /* begin at the start */
+  current_char = start;
+
+  /* advance to the end of the string or until you find a delimiter */
+  while ((*current_char != '\0') &&
+         (!strchr(delims, *current_char)))
+    current_char++;
+
+  /* advance str */
+  if (*current_char != '\0')
+    {
+    /* not at the end of the string */
+    *str = current_char + 1;
+    *current_char = '\0';
+    }
+  else
+    {
+    /* at the end of the string */
+    *str = current_char;
     }
 
-  ignchar = FALSE;
+  return(start);
+  } /* END threadsafe_tokenizer() */
 
-  Head = NULL;
 
-  while (**Ptr != '\0')
-    {
-    for (dindex = 0;DList[dindex] != '\0';dindex++)
-      {
-      if (**Ptr == DList[dindex])
-        {
-        **Ptr = '\0';
 
-        (*Ptr)++;
+/*
+ * strcats if there is enough space, otherwise returns -1
+ */
 
-        if (Head != NULL)
-          {
-          return(Head);
-          }
-        else
-          {
-          ignchar = TRUE;
+int safe_strncat(
 
-          break;
-          }
-        }
-      }    /* END for (dindex) */
+  char   *str,
+  char   *to_append,
+  size_t  space_remaining)
 
-    if ((ignchar != TRUE) && (**Ptr != '\0'))
-      {
-      if (Head == NULL)
-        Head = *Ptr;
+  {
+  size_t len = strlen(to_append);
 
-      (*Ptr)++;
-      }
+  /* not enough space */
+  if (space_remaining < len)
+    return(-1);
+  else
+    strcat(str, to_append);
 
-    ignchar = FALSE;
-    }  /* END while (**Ptr != '\0') */
+  return(PBSE_NONE);
+  } /* END safe_strncat() */
 
-  return(Head);
-  }  /* END MUStrTok() */
 
 
 
@@ -286,4 +321,37 @@ int MUReadPipe(
 
 
 
+int write_buffer(
+
+  char *buf,
+  int   len,
+  int   fds)
+
+  {
+  char *id = "write_buffer";
+  int   written;
+
+  while ((written = write(fds,buf,len)) != len)
+    {
+    if ((errno == EINTR) &&
+        (written == -1))
+      continue;
+    else if (written > 0)
+      {
+      len -= written;
+      buf += written;
+      }
+    else
+      {
+      log_err(errno,id,"Unable to write to file or socket");
+      
+      return(-1);
+      }
+    }
+
+  return(PBSE_NONE);
+  } /* END write_buffer */
+
+
+
 
