@@ -82,6 +82,7 @@
 
 #include <string.h>
 #include "utils.h"
+#include "../Liblog/pbs_log.h"
 
 
 
@@ -93,9 +94,7 @@
  *
  * @param start - the start of the string to examine
  * @param parent - where to store the parent's name (without '<>')
- * @param parent_size - the max size of parent
  * @param child - where to store the child data, trims whitespace
- * @param child_size - the max size of child
  * @param end - pointer to the first character after the closing tag
  */
 int get_parent_and_child(
@@ -111,12 +110,11 @@ int get_parent_and_child(
   int   parent_len = 0;
   char *child_start;
   char *child_end;
-
-  char *id = "get_parent_and_child";
+  int   reached_space = FALSE;
 
   if (ptr == NULL)
     {
-    log_err(-1,id,"No parent tag found\n");
+    log_err(-1, __func__ ,"No parent tag found\n");
 
     return(-1);
     }
@@ -129,8 +127,19 @@ int get_parent_and_child(
   sprintf(closingTag,"</");
   while (*ptr != '>')
     {
-    closingTag[parent_len+2] = *ptr;
-    parent_len++;
+    if (*ptr == ' ')
+      {
+      reached_space = TRUE;
+      *ptr = '\0';
+      }
+    else if (reached_space == FALSE)
+      {
+      closingTag[parent_len+2] = *ptr;
+      parent_len++;
+      }
+    /* if attribute support becomes needed, start recording attribute strings 
+     * in an else here */
+    
     ptr++;
     }
 
@@ -149,7 +158,7 @@ int get_parent_and_child(
 
   if (child_end == NULL)
     {
-    log_err(-1,id,"Cannot find closing tag\n");
+    log_err(-1, __func__, "Cannot find closing tag\n");
 
     return(-1);
     }
@@ -206,12 +215,12 @@ int escape_xml(
   int final_len = 0;
   int len;
 
-  if(in == NULL)
+  if (in == NULL)
     {
     return(BUFFER_OVERFLOW);
     }
 
-  if(out == NULL)
+  if (out == NULL)
     {
     return(BUFFER_OVERFLOW);
     }
@@ -298,6 +307,31 @@ int escape_xml(
 
 
 
+char *find_next_tag(char *buffer, char **tag)
+  {
+  char *ptr;
+
+  ptr = buffer;
+
+  if (ptr == NULL || ptr == (char *)0)
+    {
+    return(NULL);
+    }
+  /* We want the next tag. If we are currently
+     pointing to a tag increment ptr*/
+  if (*ptr == '<')
+    {
+    ptr++;
+    }
+
+  while(*ptr != '<' && ptr[1] != '/' && ptr != (char *)0)
+    {
+    ptr++;
+    }
+
+  *tag = ptr;
+  return(*tag);
+  }
 
 
 
