@@ -18,6 +18,8 @@
 
 extern char *optarg;
 
+#include "net_connect.h"
+#include "dis.h"
 #include "mcom.h"
 
 #include "pbs_error.h"
@@ -25,6 +27,7 @@ extern char *optarg;
 #include "resmon.h"
 #include "rm.h"
 #include "log.h"
+#include "../lib/Liblog/chk_file_sec.h"
 
 #define MAX_QUERY  128
 
@@ -81,8 +84,10 @@ int main(
 
   char *HPtr;
 
-  int rc;
+
   int c;
+  int rc = PBSE_NONE;
+  int local_errno = 0;
 
   int HostCount;
   int FailCount;
@@ -94,7 +99,7 @@ int main(
 
   if (IamRoot() == 0)
     {
-	exit(EXIT_FAILURE);
+    exit(EXIT_FAILURE);
     }
 
   while ((c = getopt(ArgC, ArgV, OptString)) != EOF)
@@ -157,6 +162,7 @@ int main(
       case 'f':
 
         {
+        int   rc;
 
         FILE *fp;
 
@@ -255,6 +261,7 @@ int main(
 
           int   size;
 
+          int   rc;
 
           char *ptr;
           char *cptr;
@@ -373,7 +380,7 @@ int main(
 
       /* get a batch_status entry for each node in ":property" */
 
-      bstatus = pbs_statnode(con,HPtr,NULL,NULL);
+      bstatus = pbs_statnode_err(con,HPtr,NULL,NULL, &local_errno);
 
       if (bstatus != NULL)
         {
@@ -387,7 +394,7 @@ int main(
               {
               if (!strstr(nodeattrs->value, ND_down))
                 {
-                do_mom(pbstat->name, MOMPort, CmdIndex) >= 0 ? HostCount++ : FailCount++;
+                rc = do_mom(pbstat->name, MOMPort, CmdIndex) >= 0 ? HostCount++ : FailCount++;
                 }
               else
                 {
@@ -417,7 +424,7 @@ int main(
     else
       {
       rc = do_mom(HPtr, MOMPort, CmdIndex);
-      if(rc >= 0)
+      if (rc >= 0)
         HostCount++;
       else
         FailCount++;
@@ -434,6 +441,10 @@ int main(
             FailCount);
     }
 
+  /* test success of do_mom before returning success */
+  if (rc != PBSE_NONE)
+    exit(EXIT_FAILURE);
+    
   /* SUCCESS */
 
   exit(EXIT_SUCCESS);
@@ -442,6 +453,141 @@ int main(
 
 
 
+int send_command(
+
+  int stream,
+  int cmd)
+
+  {
+  int rc;
+
+  rc = diswsi(stream,cmd);
+
+  if (cmd == RM_CMD_CONFIG)
+    {
+    diswst(stream,ConfigBuf);
+    }
+
+  DIS_tcp_wflush(stream);
+
+  return(rc);
+  } /* END send_command() */
+
+
+
+
+
+/*
+ * send_command_str
+ *
+ * @param stream - the stream we're writing to
+ * @param cmd - the command we're writing
+ */
+
+int send_command_str(
+
+  int   stream,  /* I */
+  int   cmd,
+  char *query)     /* I */
+
+  {
+  int rc = send_command(stream,cmd);
+
+  if (rc != DIS_SUCCESS)
+    return(rc);
+
+  rc = diswcs(stream,query,strlen(query));
+
+  DIS_tcp_wflush(stream);
+
+  return(rc);
+  } /* END send_command_str() */
+
+
+
+
+
+int check_success(
+
+  int stream)
+
+  {
+  int rc;
+  int local_errno;
+  int status = disrsi(stream,&rc);
+
+  if (rc != DIS_SUCCESS)
+    return(rc);
+  else if (status != RM_RSP_OK)
+    {
+#ifdef ENOMSG
+    local_errno = ENOMSG;
+#else
+    local_errno = EINVAL;
+#endif
+
+    return(-1 * local_errno);
+    }
+  else
+    return(PBSE_NONE);
+  } /* END check_success() */
+
+
+
+
+char *read_mom_reply(
+
+  int *local_errno, /* O */
+  int  stream)      /* I */
+
+  {
+  int   rc;
+  char *value;
+
+  if (check_success(stream))
+    return(NULL);
+
+  value = disrst(stream,&rc);
+
+  if (rc != DIS_SUCCESS)
+    {
+    *local_errno = EIO;
+
+    return(NULL);
+    }
+
+  return(value);
+  } /* END read_mom_reply() */
+
+
+
+
+int start_dialogue(
+
+  int stream)
+
+  {
+  int rc;
+
+  DIS_tcp_setup(stream);
+
+  rc = diswsi(stream, RM_PROTOCOL);
+
+  if (rc != DIS_SUCCESS)
+    return(rc);
+
+  rc = diswsi(stream, RM_PROTOCOL_VER);
+
+  if (rc != DIS_SUCCESS)
+    return(rc);
+
+  rc = diswsi(stream, QueryI);
+
+  return(rc);
+  } /* END start_dialogue() */
+
+
+
 
 int do_mom(
 
@@ -451,6 +597,7 @@ int do_mom(
 
   {
   int sd;
+  int local_errno = 0;
 
   if ((sd = openrm(HPtr, MOMPort)) < 0)
     {
@@ -459,9 +606,9 @@ int do_mom(
     extern char TRMEMsg[];
 
     fprintf(stderr, "cannot connect to MOM on node '%s', errno=%d (%s)\n",
-            HPtr,
-            pbs_errno,
-            strerror(pbs_errno));
+      HPtr,
+      errno,
+      strerror(errno));
 
     if (TRMEMsg[0] != '\0')
       {
@@ -472,6 +619,22 @@ int do_mom(
     return(sd);
     }
 
+  /* send protocol and version, plus how many queries we're sending */
+  if (QueryI == 0)
+    QueryI = 1;
+
+  if (start_dialogue(sd) != DIS_SUCCESS)
+    {
+    fprintf(stderr,"ERROR:    Unable to write the number of queries to %s (errno=%d-%s)\n",
+        HPtr,
+        errno,
+        strerror(errno));
+    
+    send_command(sd,RM_CMD_CLOSE);
+    
+    return(-1);
+    }
+
   if (IsVerbose == TRUE)
     {
     fprintf(stderr, "INFO:     successfully connected to %s\n",
@@ -491,34 +654,32 @@ int do_mom(
       snprintf(tmpLine, 1024, "clearjob=%s",
                (JPtr != NULL) ? JPtr : "all");
 
-      if (addreq(sd, tmpLine) != 0)
+      if (send_command_str(sd, RM_CMD_REQUEST, tmpLine) != 0)
         {
         /* FAILURE */
 
-        fprintf(stderr,"ERROR:    cannot request job clear on %s (errno=%d-%s: %d-%s)\n",
+        fprintf(stderr,"ERROR:    cannot request job clear on %s (errno=%d-%s)\n",
           HPtr,
           errno,
-          pbs_strerror(errno),
-          pbs_errno,
-          pbs_strerror(pbs_errno));
+          strerror(errno));
 
-        closerm(sd);
+        send_command(sd,RM_CMD_CLOSE);
 
         return(-1);
         }
 
-      if ((Value = (char *)getreq(sd)) == NULL)
+      if ((Value = (char *)read_mom_reply(&local_errno, sd)) == NULL)
         {
         /* FAILURE */
 
-        fprintf(stderr,"ERROR:    job clear failed on %s (errno=%d-%s: %d-%s)\n",
+        fprintf(stderr,"ERROR:    job clear failed on %s (errno=%d - %s: %d - %s)\n",
           HPtr,
           errno,
           pbs_strerror(errno),
-          pbs_errno,
-          pbs_strerror(pbs_errno));
+          local_errno,
+          pbs_strerror(local_errno));
 
-        closerm(sd);
+        send_command(sd,RM_CMD_CLOSE);
 
         return(-1);
         }
@@ -534,22 +695,17 @@ int do_mom(
     case momShutdown:
 
       {
-      int rc;
-
-      rc = downrm(sd);
-
-      if (rc != 0)
+      if ((send_command(sd,RM_CMD_SHUTDOWN) != PBSE_NONE) ||
+          (check_success(sd) != PBSE_NONE))
         {
         /* FAILURE */
 
-        fprintf(stderr,"ERROR:    cannot shutdown mom daemon on %s (errno=%d-%s: %d-%s)\n",
+        fprintf(stderr,"ERROR:    cannot shutdown mom daemon on %s (errno=%d-%s)\n",
           HPtr,
           errno,
-          pbs_strerror(errno),
-          pbs_errno,
-          pbs_strerror(pbs_errno));
+          pbs_strerror(errno));
 
-        closerm(sd);
+        send_command(sd,RM_CMD_CLOSE);
 
         exit(EXIT_FAILURE);
         }
@@ -563,22 +719,17 @@ int do_mom(
     case momReconfig:
 
       {
-      int rc;
-
-      rc = configrm(sd, ConfigBuf);
-
-      if (rc != 0)
+      if ((send_command(sd,RM_CMD_CONFIG) != PBSE_NONE) ||
+          (check_success(sd) != PBSE_NONE))
         {
         /* FAILURE */
 
-        fprintf(stderr,"ERROR:    cannot reconfigure mom on %s (errno=%d-%s: %d-%s)\n",
+        fprintf(stderr,"ERROR:    cannot reconfigure mom on %s (errno=%d-%s)\n",
           HPtr,
           errno,
-          pbs_strerror(errno),
-          pbs_errno,
-          pbs_strerror(pbs_errno));
+          pbs_strerror(errno));
 
-        closerm(sd);
+        send_command(sd,RM_CMD_CLOSE);
 
         return(-1);
         }
@@ -600,18 +751,15 @@ int do_mom(
 
       char *Value;
 
-      for (rindex = 0;rindex < QueryI;rindex++)
+      for (rindex = 0; rindex < QueryI; rindex++)
         {
-        if (addreq(sd, Query[rindex]) != 0)
+        if (send_command_str(sd, RM_CMD_REQUEST, Query[rindex]) != 0)
           {
-          fprintf(stderr,"ERROR:    cannot add query for '%s' on %s (errno=%d-%s: %d-%s)\n",
+          fprintf(stderr,"ERROR:    cannot add query for '%s' on %s (errno=%d-%s)\n",
             Query[rindex],
             HPtr,
             errno,
-
-          pbs_strerror(errno),
-          pbs_errno,
-          pbs_strerror(pbs_errno));
+            pbs_strerror(errno));
           }
         }
 
@@ -622,16 +770,16 @@ int do_mom(
           *ptr = '\0';
           }
 
-        if ((Value = (char *)getreq(sd)) == NULL)
+        if ((Value = (char *)read_mom_reply(&local_errno, sd)) == NULL)
           {
-          fprintf(stderr, "ERROR:    query[%d] '%s' failed on %s (errno=%d-%s: %d-%s)\n",
+          fprintf(stderr, "ERROR:    query[%d] '%s' failed on %s (errno=%d - %s : %d - %s)\n",
             rindex,
             Query[rindex],
             HPtr,
             errno,
             pbs_strerror(errno),
-            pbs_errno,
-            pbs_strerror(pbs_errno));
+            local_errno,
+            pbs_strerror(local_errno));
           }
         else
           {
@@ -665,7 +813,7 @@ int do_mom(
     break;
     }  /* END switch(CmdIndex) */
 
-  closerm(sd);
+  send_command(sd,RM_CMD_CLOSE);
 
   return(0);
   } /* END do_mom() */
