@@ -104,10 +104,11 @@
 #include <st_client.h>
 #endif    /* IBM SP2 */
 
-static char ident[] = "@(#) $RCSfile$ $Revision: 3230 $";
+static char ident[] = "@(#) $RCSfile$ $Revision: 5026 $";
 
 /* Global Variables */
 
+extern int multi_mom;
 extern int  exiting_tasks;
 extern char  mom_host[];
 extern tlist_head svr_alljobs;
@@ -151,11 +152,9 @@ set_globid(job *pjob, struct startjob_rtn *sjr)
  * set_mach_vars - setup machine dependent environment variables
  */
 
-int
-set_mach_vars(
+int set_mach_vars(
   job *pjob,    /* pointer to the job */
-  struct var_table *vtab    /* pointer to variable table */
-)
+  struct var_table *vtab)    /* pointer to variable table */
   {
 #if IBM_SP2==1 || IBM_SP2==2 /* IBM SP2 with PSSP 2.x or 3.x*/
   char  buf[256];
@@ -189,8 +188,9 @@ set_mach_vars(
   return 0;
   }
 
-char *
-set_shell(job *pjob, struct passwd *pwdp)
+char * set_shell(
+    job *pjob,
+    struct passwd *pwdp)
   {
   char *cp;
   int   i;
@@ -244,6 +244,7 @@ scan_for_terminated(void)
   job  *pjob;
   task  *ptask;
   int  statloc;
+  unsigned int momport = 0;
 
   /* update the latest intelligence about the running jobs;         */
   /* must be done before we reap the zombies, else we lose the info */
@@ -320,7 +321,12 @@ scan_for_terminated(void)
 
       pjob->ji_momsubt = 0;
 
-      (void)job_save(pjob, SAVEJOB_QUICK);
+      if(multi_mom)
+        {
+        momport = pbs_rm_port;
+        }
+
+      (void)job_save(pjob, SAVEJOB_QUICK, 0);
       continue;
       }
 
