@@ -79,21 +79,17 @@
 #include <pbs_config.h>   /* the master config generated by configure */
 
 #include <string.h>
-#include <unistd.h>
 #include <stdio.h>
-#include <pwd.h>
-#include "libpbs.h"
-#include <fcntl.h>
 #include <signal.h>
 #include <errno.h>
-#include <sys/types.h>
-#include <sys/stat.h>
+#include <unistd.h>
 #include <sys/wait.h>
 
 #ifndef USEOLDTTY
 #include <pty.h>
 #endif /* USEOLDTTY */
 
+#include "libpbs.h"
 #include "list_link.h"
 #include "log.h"
 #include "server_limits.h"
@@ -113,7 +109,8 @@ extern int  termin_child;
 extern int       LOGLEVEL;
 
 extern char     *AllocParCmd;
-
+extern int      multi_mom;
+extern unsigned short pbs_rm_port;
 
 /* Private variables */
 
@@ -159,22 +156,22 @@ int set_job(
       ((CPtr = get_job_envvar(pjob, "BATCH_ALLOC_COOKIE")) != NULL) &&
       !strcmp(CPtr, "0"))
     {
-    char  tmpLine[1024];
+    char  tmpLine[MAXLINE];
 
     if (AllocParCmd == NULL)
       AllocParCmd = strdup("/opt/moab/default/tools/partition.create.xt4.pl");
 
 #ifdef USEJOBCREATE
 
-    if ((pjob->ji_wattr[(int)JOB_ATR_pagg_id].at_flags & ATR_VFLAG_SET) &&
-      (pjob->ji_wattr[(int)JOB_ATR_pagg_id].at_val.at_ll != (long)JOB_FAIL) &&
-      (pjob->ji_wattr[(int)JOB_ATR_pagg_id].at_val.at_ll > 0x00000000ffffffff))
+    if ((pjob->ji_wattr[JOB_ATR_pagg_id].at_flags & ATR_VFLAG_SET) &&
+      (pjob->ji_wattr[JOB_ATR_pagg_id].at_val.at_ll != (long)JOB_FAIL) &&
+      (pjob->ji_wattr[JOB_ATR_pagg_id].at_val.at_ll > 0x00000000ffffffff))
       {
       snprintf(tmpLine, sizeof(tmpLine), "%s --confirm -p %s -j %s -a %lld",
                AllocParCmd,
                PPtr,
                pjob->ji_qs.ji_jobid,
-               pjob->ji_wattr[(int)JOB_ATR_pagg_id].at_val.at_ll);
+               pjob->ji_wattr[JOB_ATR_pagg_id].at_val.at_ll);
       }
     else
 #endif  /* USEJOBCREATE */
@@ -196,7 +193,7 @@ int set_job(
 
     if (WEXITSTATUS(rc) != 0)
       {
-      snprintf(log_buffer, 1024, "cannot create alloc partition");
+      snprintf(log_buffer, sizeof(log_buffer), "cannot create alloc partition");
 
       sid = -3;
 
@@ -275,8 +272,8 @@ char *set_shell(
 
   shell = pwdp->pw_shell;
 
-  if ((pjob->ji_wattr[(int)JOB_ATR_shell].at_flags & ATR_VFLAG_SET) &&
-      (vstrs = pjob->ji_wattr[(int)JOB_ATR_shell].at_val.at_arst))
+  if ((pjob->ji_wattr[JOB_ATR_shell].at_flags & ATR_VFLAG_SET) &&
+      (vstrs = pjob->ji_wattr[JOB_ATR_shell].at_val.at_arst))
     {
     for (i = 0; i < vstrs->as_usedptr; ++i)
       {
@@ -328,8 +325,10 @@ void scan_for_terminated(void)
   job *pjob;
   task *ptask = NULL;
   int  statloc;
+  unsigned int momport = 0;
+
 #ifdef USESAVEDRESOURCES
-  int update_stats = TRUE;
+  int  update_stats = TRUE;
 #endif /* USESAVEDRESOURCES */
 
   int    tcount;
@@ -369,14 +368,10 @@ void scan_for_terminated(void)
           {
           if (LOGLEVEL >= 7)
             {
-            snprintf(log_buffer, 1024, "found match for recovering job task for sid=%d",
+            snprintf(log_buffer, sizeof(log_buffer), "found match for recovering job task for sid=%d",
               ptask->ti_qs.ti_sid);
 
-            LOG_EVENT(
-              PBSEVENT_DEBUG,
-              PBS_EVENTCLASS_JOB,
-              pjob->ji_qs.ji_jobid,
-              log_buffer);
+            log_event(PBSEVENT_DEBUG, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buffer);
             }
           
           update_stats = FALSE;
@@ -418,29 +413,23 @@ void scan_for_terminated(void)
 
       if (LOGLEVEL >= 7)
         {
-        snprintf(log_buffer, 1024, "checking job w/subtask pid=%d (child pid=%d)",
+        snprintf(log_buffer, sizeof(log_buffer),
+          "checking job w/subtask pid=%d (child pid=%d)",
           pjob->ji_momsubt,
           pid);
 
-        LOG_EVENT(
-          PBSEVENT_DEBUG,
-          PBS_EVENTCLASS_JOB,
-          pjob->ji_qs.ji_jobid,
-          log_buffer);
+        log_event(PBSEVENT_DEBUG, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buffer);
         }
 
       if (pid == pjob->ji_momsubt)
         {
         if (LOGLEVEL >= 7)
           {
-          snprintf(log_buffer, 1024, "found match with job subtask for pid=%d",
+          snprintf(log_buffer, sizeof(log_buffer),
+            "found match with job subtask for pid=%d",
             pid);
 
-          LOG_EVENT(
-            PBSEVENT_DEBUG,
-            PBS_EVENTCLASS_JOB,
-            pjob->ji_qs.ji_jobid,
-            log_buffer);
+          log_event(PBSEVENT_DEBUG, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buffer);
           }
 
         break;
@@ -461,15 +450,12 @@ void scan_for_terminated(void)
           {
           if (LOGLEVEL >= 7)
             {
-            snprintf(log_buffer, 1024, "found match with job task %d for pid=%d",
+            snprintf(log_buffer, sizeof(log_buffer),
+              "found match with job task %d for pid=%d",
               tcount,
               pid);
 
-            LOG_EVENT(
-              PBSEVENT_DEBUG,
-              PBS_EVENTCLASS_JOB,
-              pjob->ji_qs.ji_jobid,
-              log_buffer);
+            log_event(PBSEVENT_DEBUG, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buffer);
             }
 
           break;
@@ -506,11 +492,7 @@ void scan_for_terminated(void)
           statloc,
           exiteval);
 
-        log_record(
-          PBSEVENT_JOB,
-          PBS_EVENTCLASS_JOB,
-          id,
-          log_buffer);
+        log_record(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, id, log_buffer);
         }
 
       continue;
@@ -546,7 +528,12 @@ void scan_for_terminated(void)
 
       pjob->ji_momsubt = 0;
 
-      job_save(pjob, SAVEJOB_QUICK);
+      if (multi_mom)
+        {
+        momport = pbs_rm_port;
+        }
+
+      job_save(pjob, SAVEJOB_QUICK, momport);
 
       continue;
       }  /* END if (pid == pjob->ji_momsubt) */
@@ -561,11 +548,7 @@ void scan_for_terminated(void)
               ptask->ti_qs.ti_task,
               exiteval);
 
-      log_record(
-        PBSEVENT_JOB,
-        PBS_EVENTCLASS_JOB,
-        id,
-        log_buffer);
+      log_record(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, id, log_buffer);
       }
 
     /* where is job purged?  How do we keep job from progressing in state until the obit is sent? */
@@ -584,11 +567,7 @@ void scan_for_terminated(void)
             ptask->ti_qs.ti_task,
             ptask->ti_qs.ti_sid);
 
-    LOG_EVENT(
-      PBSEVENT_DEBUG,
-      PBS_EVENTCLASS_JOB,
-      pjob->ji_qs.ji_jobid,
-      log_buffer);
+    log_event(PBSEVENT_DEBUG, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buffer);
 
     exiting_tasks = 1;
     }  /* END while ((pid = waitpid(-1,&statloc,WNOHANG)) > 0) */
