@@ -77,6 +77,7 @@
 * without reference to its choice of law rules.
 */
 #include <pbs_config.h>   /* the master config generated by configure */
+#include "prolog.h"
 
 #define PBS_MOM 1
 #include <sys/types.h>
@@ -95,6 +96,7 @@
 #include "attribute.h"
 #include "pbs_job.h"
 #include "log.h"
+#include "../lib/Liblog/pbs_log.h"
 #include "mom_mach.h"
 #include "mom_func.h"
 #include "resource.h"
@@ -103,8 +105,6 @@
 #include "utils.h"
 
 
-#define PBS_PROLOG_TIME 300
-
 extern char PBSNodeMsgBuf[];
 extern int  MOMPrologTimeoutCount;
 extern int  MOMPrologFailureCount;
@@ -127,7 +127,7 @@ static int   run_exit;
 
 extern int pe_input(char *);
 extern int TTmpDirName(job *, char *);
-extern void encode_used(job *, tlist_head *);
+extern void encode_used(job *, int, tlist_head *);
 #ifdef ENABLE_CSA
 extern void add_wkm_end(uint64_t, int64_t, char *);
 
@@ -157,8 +157,8 @@ static char *resc_to_string(
 
   job       *pjob,      /* I (optional - if specified, report total job resources) */
   int        aindex,    /* I which attribute to convert */
-  char      *buf, /* O the buffer into which to convert */
-  int      buflen) /* I the length of the above buffer */
+  char      *buf,       /* O the buffer into which to convert */
+  int        buflen)    /* I the length of the above buffer */
 
   {
   int        need;
@@ -178,14 +178,14 @@ static char *resc_to_string(
 
   if (aindex == JOB_ATR_resource)
     {
-    if (encode_resc(pattr, &svlist, "x", NULL, ATR_ENCODE_CLIENT) <= 0)
+    if (encode_resc(pattr, &svlist, "x", NULL, ATR_ENCODE_CLIENT, ATR_DFLAG_ACCESS) <= 0)
       {
       return(buf);
       }
     }
   else if (aindex == JOB_ATR_resc_used)
     {
-    encode_used(pjob, &svlist);
+    encode_used(pjob, ATR_DFLAG_RDACC, &svlist);
     }
   else
     {
@@ -398,10 +398,10 @@ int run_pelog(
   char   buf[MAXPATHLEN + 1024];
   char   pelog[MAXPATHLEN + 1024];
 
-  uid_t  real_uid = 0;
+  uid_t  real_uid;
   gid_t *real_gids = NULL;
-  gid_t  real_gid = 0;
-  int    num_gids = 0;
+  gid_t  real_gid;
+  int    num_gids;
 
   int    jobtypespecified = 0;
 
@@ -421,7 +421,7 @@ int run_pelog(
     return(0);
     }
 
-  ptr = pjob->ji_wattr[(int)JOB_ATR_jobtype].at_val.at_str;
+  ptr = pjob->ji_wattr[JOB_ATR_jobtype].at_val.at_str;
 
   if (ptr != NULL)
     {
@@ -435,6 +435,15 @@ int run_pelog(
     {
     strncpy(pelog,specpelog,sizeof(pelog));
     }
+    
+  real_uid = getuid();
+  real_gid = getgid();
+  if ((num_gids = getgroups(0,real_gids)) < 0)
+    {
+    log_err(errno,id,"getgroups failed\n");
+    
+    return(-1);
+    }
 
   /* to support root squashing, become the user before performing file checks */
   if ((which == PE_PROLOGUSER) || 
@@ -442,47 +451,49 @@ int run_pelog(
       (which == PE_PROLOGUSERJOB) || 
       (which == PE_EPILOGUSERJOB))
     {
-    real_uid = getuid();
-    real_gid = getgid();
 
-    if ((num_gids = getgroups(0,real_gids)) < 0)
+    real_gids = calloc(num_gids, sizeof(gid_t));
+    
+    if (real_gids == NULL)
+      {
+      log_err(ENOMEM,id,"Cannot allocate memory! FAILURE\n");
+      
+      return(-1);
+      }
+    
+    if (getgroups(num_gids,real_gids) < 0)
       {
       log_err(errno,id,"getgroups failed\n");
-
+      
       return(-1);
       }
+    
+    /* pjob->ji_grpcache will not be set if using LDAP and LDAP not set */
+    /* It is possible that ji_grpcache failed to allocate as well. 
+       Make sure ji_grpcache is not NULL */
+    if (pjob->ji_grpcache != NULL)
+      {
+      if (setgroups(
+            pjob->ji_grpcache->gc_ngroup,
+            (gid_t *)pjob->ji_grpcache->gc_groups) != 0)
+        {
+        snprintf(log_buffer,sizeof(log_buffer),
+          "setgroups() for UID = %lu failed: %s\n",
+          (unsigned long)pjob->ji_qs.ji_un.ji_momt.ji_exuid,
+          strerror(errno));
+      
+        log_err(errno, id, log_buffer);
+      
+        undo_set_euid_egid(which,real_uid,real_gid,num_gids,real_gids,id);
+      
+        return(-1);
+        }
+      }
     else
       {
-      real_gids = malloc(sizeof(gid_t) * num_gids);
-      
-      if (real_gids == NULL)
-        {
-        log_err(ENOMEM,id,"Cannot allocate memory! FAILURE\n");
-
-        return(-1);
-        }
-
-      if (getgroups(num_gids,real_gids) < 0)
-        {
-        log_err(errno,id,"getgroups failed\n");
-        
-        return(-1);
-        }
-      }
-
-    if (setgroups(
-          pjob->ji_grpcache->gc_ngroup,
-          (gid_t *)pjob->ji_grpcache->gc_groups) != 0)
-      {
-      snprintf(log_buffer,sizeof(log_buffer),
-        "setgroups() for UID = %lu failed: %s\n",
-        (unsigned long)pjob->ji_qs.ji_un.ji_momt.ji_exuid,
-        strerror(errno));
-      
-      log_err(errno, id, log_buffer);
-      
+      sprintf(log_buffer, "pjob->ji_grpcache is null. check_pwd likely failed.");
+      log_err(-1, id, log_buffer);
       undo_set_euid_egid(which,real_uid,real_gid,num_gids,real_gids,id);
-      
       return(-1);
       }
     
@@ -559,9 +570,8 @@ int run_pelog(
           log_err(-1, id, log_buffer);
           }
 
-        add_wkm_end(pjob->ji_wattr[(int)JOB_ATR_pagg_id].at_val.at_ll,
-
-                    pjob->ji_qs.ji_un.ji_momt.ji_exitstat, pjob->ji_qs.ji_jobid);
+        add_wkm_end(pjob->ji_wattr[JOB_ATR_pagg_id].at_val.at_ll,
+            pjob->ji_qs.ji_un.ji_momt.ji_exitstat, pjob->ji_qs.ji_jobid);
         }
 
 #endif /* ENABLE_CSA */
@@ -730,9 +740,8 @@ int run_pelog(
         log_err(-1, id, log_buffer);
         }
 
-      add_wkm_end(pjob->ji_wattr[(int)JOB_ATR_pagg_id].at_val.at_ll,
-
-                  pjob->ji_qs.ji_un.ji_momt.ji_exitstat, pjob->ji_qs.ji_jobid);
+      add_wkm_end(pjob->ji_wattr[JOB_ATR_pagg_id].at_val.at_ll,
+          pjob->ji_qs.ji_un.ji_momt.ji_exitstat, pjob->ji_qs.ji_jobid);
       }
 
 #endif /* ENABLE_CSA */
@@ -847,7 +856,10 @@ int run_pelog(
         }
       }
 
-    if ((which == PE_PROLOGUSER) || (which == PE_EPILOGUSER) || (which == PE_PROLOGUSERJOB) || (which == PE_EPILOGUSERJOB))
+    if ((which == PE_PROLOGUSER) || 
+        (which == PE_EPILOGUSER) || 
+        (which == PE_PROLOGUSERJOB) || 
+        (which == PE_EPILOGUSERJOB))
       {
       if (chdir(pjob->ji_grpcache->gc_homedir) != 0)
         {
@@ -872,36 +884,38 @@ int run_pelog(
       fprintf(stderr, "PELOGINFO:  script:'%s'  jobid:'%s'  euser:'%s'  egroup:'%s'  jobname:'%s' SSID:'%ld'  RESC:'%s'\n",
               pelog,
               pjob->ji_qs.ji_jobid,
-              pjob->ji_wattr[(int)JOB_ATR_euser].at_val.at_str,
-              pjob->ji_wattr[(int)JOB_ATR_egroup].at_val.at_str,
-              pjob->ji_wattr[(int)JOB_ATR_jobname].at_val.at_str,
-              pjob->ji_wattr[(int)JOB_ATR_session_id].at_val.at_long,
-              resc_to_string(pjob, (int)JOB_ATR_resource, resc_list, sizeof(resc_list)));
+              pjob->ji_wattr[JOB_ATR_euser].at_val.at_str,
+              pjob->ji_wattr[JOB_ATR_egroup].at_val.at_str,
+              pjob->ji_wattr[JOB_ATR_jobname].at_val.at_str,
+              pjob->ji_wattr[JOB_ATR_session_id].at_val.at_long,
+              resc_to_string(pjob, JOB_ATR_resource, resc_list, sizeof(resc_list)));
       }
 
     arg[0] = pelog;
 
     arg[1] = pjob->ji_qs.ji_jobid;
-    arg[2] = pjob->ji_wattr[(int)JOB_ATR_euser].at_val.at_str;
-    arg[3] = pjob->ji_wattr[(int)JOB_ATR_egroup].at_val.at_str;
-    arg[4] = pjob->ji_wattr[(int)JOB_ATR_jobname].at_val.at_str;
+    arg[2] = pjob->ji_wattr[JOB_ATR_euser].at_val.at_str;
+    arg[3] = pjob->ji_wattr[JOB_ATR_egroup].at_val.at_str;
+    arg[4] = pjob->ji_wattr[JOB_ATR_jobname].at_val.at_str;
 
     /* NOTE:  inside child */
 
-    if ( which == PE_EPILOG || which == PE_EPILOGUSER || which == PE_EPILOGUSERJOB )
+    if ((which == PE_EPILOG) || 
+        (which == PE_EPILOGUSER) || 
+        (which == PE_EPILOGUSERJOB))
       {
       /* for epilog only */
 
       sprintf(sid, "%ld",
-              pjob->ji_wattr[(int)JOB_ATR_session_id].at_val.at_long);
+              pjob->ji_wattr[JOB_ATR_session_id].at_val.at_long);
       sprintf(exit_stat,"%d",
-              pjob->ji_qs.ji_un.ji_exect.ji_exitstat);
+              pjob->ji_qs.ji_un.ji_momt.ji_exitstat);
 
       arg[5] = sid;
-      arg[6] = resc_to_string(pjob, (int)JOB_ATR_resource, resc_list, sizeof(resc_list));
-      arg[7] = resc_to_string(pjob, (int)JOB_ATR_resc_used, resc_used, sizeof(resc_used));
-      arg[8] = pjob->ji_wattr[(int)JOB_ATR_in_queue].at_val.at_str;
-      arg[9] = pjob->ji_wattr[(int)JOB_ATR_account].at_val.at_str;
+      arg[6] = resc_to_string(pjob, JOB_ATR_resource, resc_list, sizeof(resc_list));
+      arg[7] = resc_to_string(pjob, JOB_ATR_resc_used, resc_used, sizeof(resc_used));
+      arg[8] = pjob->ji_wattr[JOB_ATR_in_queue].at_val.at_str;
+      arg[9] = pjob->ji_wattr[JOB_ATR_account].at_val.at_str;
       arg[10] = exit_stat;
       arg[11] = NULL;
 
@@ -911,9 +925,9 @@ int run_pelog(
       {
       /* prolog */
 
-      arg[5] = resc_to_string(pjob, (int)JOB_ATR_resource, resc_list, sizeof(resc_list));
-      arg[6] = pjob->ji_wattr[(int)JOB_ATR_in_queue].at_val.at_str;
-      arg[7] = pjob->ji_wattr[(int)JOB_ATR_account].at_val.at_str;
+      arg[5] = resc_to_string(pjob, JOB_ATR_resource, resc_list, sizeof(resc_list));
+      arg[6] = pjob->ji_wattr[JOB_ATR_in_queue].at_val.at_str;
+      arg[7] = pjob->ji_wattr[JOB_ATR_account].at_val.at_str;
       arg[8] = NULL;
 
       LastArg = 8;
@@ -935,7 +949,7 @@ int run_pelog(
      */
 
     r = find_resc_entry(
-          &pjob->ji_wattr[(int)JOB_ATR_resource],
+          &pjob->ji_wattr[JOB_ATR_resource],
           find_resc_def(svr_resc_def, "nodes", svr_resc_size));
 
     if (r != NULL)
@@ -945,8 +959,7 @@ int run_pelog(
       const char *envname = "PBS_RESOURCE_NODES=";
       char *envstr;
 
-      envstr = malloc(
-                 (strlen(envname) + strlen(r->rs_value.at_val.at_str) + 1) * sizeof(char));
+      envstr = calloc((strlen(envname) + strlen(r->rs_value.at_val.at_str) + 1), sizeof(char));
 
       if (envstr != NULL)
         {
@@ -961,7 +974,7 @@ int run_pelog(
       }  /* END if (r != NULL) */
 
     r = find_resc_entry(
-          &pjob->ji_wattr[(int)JOB_ATR_resource],
+          &pjob->ji_wattr[JOB_ATR_resource],
           find_resc_def(svr_resc_def, "gres", svr_resc_size));
 
     if (r != NULL)
@@ -971,8 +984,7 @@ int run_pelog(
       const char *envname = "PBS_RESOURCE_GRES=";
       char *envstr;
 
-      envstr = malloc(
-                 (strlen(envname) + strlen(r->rs_value.at_val.at_str) + 1) * sizeof(char));
+      envstr = calloc((strlen(envname) + strlen(r->rs_value.at_val.at_str) + 1), sizeof(char));
 
       if (envstr != NULL)
         {
@@ -991,8 +1003,7 @@ int run_pelog(
       const char *envname = "TMPDIR=";
       char *envstr;
 
-      envstr = malloc(
-                 (strlen(envname) + strlen(buf) + 1) * sizeof(char));
+      envstr = calloc((strlen(envname) + strlen(buf) + 1), sizeof(char));
 
       if (envstr != NULL)
         {
@@ -1015,7 +1026,7 @@ int run_pelog(
 
       if ((envval = get_job_envvar(pjob, envname)) != NULL)
         {
-        envstr = malloc((strlen(envname) + strlen(envval) + 2) * sizeof(char));
+        envstr = calloc((strlen(envname) + strlen(envval) + 2), sizeof(char));
 
         if (envstr != NULL)
           {
@@ -1036,7 +1047,7 @@ int run_pelog(
       sprintf(buf, "%d",
         pjob->ji_nodeid);
 
-      envstr = malloc((strlen(envname) + strlen(buf) + 2) * sizeof(char));
+      envstr = calloc((strlen(envname) + strlen(buf) + 2), sizeof(char));
 
       if (envstr != NULL)
         {
@@ -1055,7 +1066,7 @@ int run_pelog(
 
       if ((pjob->ji_vnods[0].vn_host != NULL) && (pjob->ji_vnods[0].vn_host->hn_host != NULL))
         {
-        envstr = malloc((strlen(envname) + strlen(pjob->ji_vnods[0].vn_host->hn_host) + 2) * sizeof(char));
+        envstr = calloc((strlen(envname) + strlen(pjob->ji_vnods[0].vn_host->hn_host) + 2), sizeof(char));
 
         if (envstr != NULL)
           {
@@ -1079,7 +1090,7 @@ int run_pelog(
           path_aux,
           pjob->ji_qs.ji_jobid);
 
-        envstr = malloc((strlen(envname) + strlen(buf) + 2) * sizeof(char));
+        envstr = calloc((strlen(envname) + strlen(buf) + 2), sizeof(char));
 
         if (envstr != NULL)
           {
@@ -1101,7 +1112,7 @@ int run_pelog(
       workdir_val = get_job_envvar(pjob,envname);
       if (workdir_val != NULL)
         {
-        envstr = malloc((strlen(workdir_val) + strlen(envname) + 2) * sizeof(char));
+        envstr = calloc((strlen(workdir_val) + strlen(envname) + 2), sizeof(char));
 
         if (envstr != NULL)
           {
@@ -1123,7 +1134,7 @@ int run_pelog(
       int VarIsSet = 0;
       int j;
 
-      vstrs = pjob->ji_wattr[(int)JOB_ATR_variables].at_val.at_arst;
+      vstrs = pjob->ji_wattr[JOB_ATR_variables].at_val.at_arst;
 
       for (j = 0;j < vstrs->as_usedptr;++j)
         {
@@ -1142,7 +1153,7 @@ int run_pelog(
         {
         char *envstr;
 
-        envstr = malloc((strlen(vstrs->as_string[j])) * sizeof(char));
+        envstr = calloc((strlen(vstrs->as_string[j])), sizeof(char));
 
         if (envstr != NULL)
           {
