@@ -83,14 +83,12 @@
 #include <fcntl.h>
 #include <grp.h>
 #include <limits.h>
-#include <pwd.h>
 #include <signal.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 #include <assert.h>
-#include <dirent.h>
 #include <time.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
@@ -104,13 +102,19 @@
 #include "resource.h"
 #include "pbs_job.h"
 #include "batch_request.h"
+#include "mom_comm.h" /* im_compose */
 #include "mom_mach.h"
 #include "mom_func.h"
+#include "mom_hierarchy.h"
 #include "log.h"
+#include "../lib/Liblog/pbs_log.h"
+#include "../lib/Liblog/log_event.h"
+#include "checkpoint.h" /* start_checkpoint */
 #include "rpp.h"
 #include "resmon.h"
 #include "net_connect.h"
 #include "utils.h"
+
 #ifdef _CRAY
 #include <sys/category.h>
 #endif
@@ -121,6 +125,8 @@
 extern struct var_table vtable;      /* see start_exec.c */
 extern char           **environ;
 
+int reply_send_mom(struct batch_request *request);
+
 extern int InitUserEnv(
 
     job            *pjob,   /* I */
@@ -141,7 +147,6 @@ extern int TTmpDirName(job*, char *);
 #ifdef NVIDIA_GPUS
 int setgpumode(char *, int);
 int resetgpuecc(char *, int, int);
-extern void mom_server_all_update_gpustat(void);
 #endif /* NVIDIA_GPUS */
 
 /* External Global Data Items */
@@ -158,7 +163,6 @@ extern attribute_def job_attr_def[];
 extern char            *msg_jobmod;
 extern char            *msg_manager;
 extern time_t  time_now;
-extern int  resc_access_perm; /* see encode_resc() */
 extern int  multi_mom;
 extern int spoolasfinalname;
 /* in attr_fn_resc.c */
@@ -185,8 +189,6 @@ static char   rcperr[MAXPATHLEN]; /* file to contain rcp error */
 extern char PBSNodeMsgBuf[1024];
 extern int  LOGLEVEL;
 
-extern int im_compose(int, char *, char *, int, tm_event_t, tm_task_id);
-extern int mom_open_socket_to_jobs_server(job *, char *, void (*)(int));
 
 /* prototypes */
 
@@ -493,11 +495,11 @@ static pid_t fork_to_user(
 
     char *envstr;
 
-    envstr = malloc((strlen("HOME=") + strlen(hdir) + 1) * sizeof(char));
+    envstr = calloc((strlen("HOME=") + strlen(hdir) + 1), sizeof(char));
 
     if (envstr == NULL)
       {
-      sprintf(log_buffer, "malloc failed, errno=%d (%s)",
+      sprintf(log_buffer, "calloc failed, errno=%d (%s)",
               errno,
               strerror(errno));
 
@@ -515,11 +517,11 @@ static pid_t fork_to_user(
 
     putenv(envstr);
 
-    envstr = malloc((strlen("PBS_JOBID=") + strlen(preq->rq_ind.rq_cpyfile.rq_jobid) + 1) * sizeof(char));
+    envstr = calloc((strlen("PBS_JOBID=") + strlen(preq->rq_ind.rq_cpyfile.rq_jobid) + 1), sizeof(char));
 
     if (envstr == NULL)
       {
-      sprintf(log_buffer, "malloc failed, errno=%d (%s)",
+      sprintf(log_buffer, "calloc failed, errno=%d (%s)",
               errno,
               strerror(errno));
 
@@ -570,7 +572,7 @@ static void add_bad_list(
     {
     needed += strlen(newtext) + nl + 1;
 
-    pnew = malloc(needed);
+    pnew = calloc(1, needed);
 
     if (pnew != NULL)
       *pnew = '\0';
@@ -603,20 +605,20 @@ static void add_bad_list(
 
 static int return_file(
 
-  job         *pjob,
+  job           *pjob,
   enum job_file  which,
-  int          sock,
-  int          remove_file)
+  int            sock,
+  int            remove_file)
 
   {
-  int        amt;
-  char        buf[RT_BLK_SZ];
-  int        fds;
-  char       *filename;
+  int                   amt;
+  char                  buf[RT_BLK_SZ];
+  int                   fds;
+  char                 *filename;
 
   struct batch_request *prq;
-  int        rc = 0;
-  int        seq = 0;
+  int                   rc = 0;
+  int                   seq = 0;
 
   filename = std_file_name(pjob, which, &amt); /* amt is place holder */
 
@@ -632,13 +634,6 @@ static int return_file(
     return(0);
     }
 
-  fds = open(filename, O_RDONLY, 0);
-
-  if (fds < 0)
-    {
-    return(0);
-    }
-
   prq = alloc_br(PBS_BATCH_MvJobFile);
 
   if (prq == NULL)
@@ -648,6 +643,13 @@ static int return_file(
     return(PBSE_SYSTEM);
     }
 
+  fds = open(filename, O_RDONLY, 0);
+
+  if (fds < 0)
+    {
+    return(0);
+    }
+
   strcpy(prq->rq_host, mom_host);
 
   strcpy(prq->rq_ind.rq_jobfile.rq_jobid, pjob->ji_qs.ji_jobid);
@@ -673,8 +675,6 @@ static int return_file(
     if ((DIS_reply_read(sock, &prq->rq_reply) != 0) ||
         (prq->rq_reply.brp_code != 0))
       {
-      close(fds);
-
       rc = -1;
 
       break;
@@ -1149,7 +1149,7 @@ void req_deletejob(
  * req_holdjob - checkpoint and terminate job
  */
 
-void req_holdjob(
+void mom_req_holdjob(
 
   struct batch_request *preq)
 
@@ -1178,7 +1178,8 @@ void req_holdjob(
         &tmph,
         pal->al_name,
         NULL,
-        pal->al_value);
+        pal->al_value,
+        0);
       }
 
     if ((rc = start_checkpoint(pjob,1,preq)) != PBSE_NONE)
@@ -1226,7 +1227,7 @@ void req_checkpointjob(
 
 
 
-void req_gpuctrl(
+void req_gpuctrl_mom(
 
   struct batch_request *preq)  /* I */
 
@@ -1284,7 +1285,7 @@ void req_gpuctrl(
 
     if (gpumode != -1)
       {
-      mom_server_all_update_gpustat();
+      send_update_soon();
       }
     }
   else
@@ -1293,9 +1294,7 @@ void req_gpuctrl(
     }
 #else
 
-    sprintf(
-      log_buffer,
-      "GPU control request not supported: node %s gpuid %s mode %d reset_perm %d reset_vol %d",
+    sprintf(log_buffer, "GPU control request not supported: node %s gpuid %s mode %d reset_perm %d reset_vol %d",
       mom_node,
       gpuid,
       gpumode,
@@ -1304,7 +1303,7 @@ void req_gpuctrl(
 
   if (LOGLEVEL >= 3)
     {
-      log_ext(-1, id, log_buffer, LOG_INFO);
+    log_ext(-1, id, log_buffer, LOG_INFO);
     }
 
   req_reject(PBSE_NOSUP, 0, preq, NULL, NULL);
@@ -1376,8 +1375,10 @@ int message_job(
 
   if (text[len - 1] != '\n')
     {
-    if ((pstr = malloc(len + 2)) == NULL)
+    if ((pstr = calloc(1, len + 2)) == NULL)
       {
+      close(fds);
+
       return(PBSE_INTERNAL);
       }
 
@@ -1529,10 +1530,10 @@ const char *TJobAttr[] =
 #ifdef USEJOBCREATE
   "pagg_id",
 #endif /* USEJOBCREATE */
-  "job_id",
 #ifdef NVIDIA_GPUS
   "gpu_flags",
 #endif  /* NVIDIA_GPUS */
+  "job_id",
   NULL
   };
 
@@ -1751,11 +1752,7 @@ void req_modifyjob(
           preq->rq_user,
           preq->rq_host);
 
-  LOG_EVENT(
-    PBSEVENT_JOB,
-    PBS_EVENTCLASS_JOB,
-    pjob->ji_qs.ji_jobid,
-    log_buffer);
+  log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buffer);
 
   /* SUCCESS */
 
@@ -1892,7 +1889,8 @@ int sigalltasks_sisters(
 #endif
   char     *cookie;
   eventent *ep;
-  int i;
+  int       i;
+  int       stream;
 
   cookie = pjob->ji_wattr[JOB_ATR_Cookie].at_val.at_str;
 
@@ -1906,53 +1904,38 @@ int sigalltasks_sisters(
 
     DBPRT(("%s: sending sig%d to all tasks on sister %s\n", id, signum, np->hn_host));
 
-    if (np->hn_stream == -1)
-      np->hn_stream = rpp_open(np->hn_host, np->hn_port, NULL);
-
     ep = event_alloc(IM_SIGNAL_TASK, np, TM_NULL_EVENT, TM_NULL_TASK);
 
-    if (np->hn_stream == -1)
+    if ((stream = tcp_connect_sockaddr((struct sockaddr *)&np->sock_addr,sizeof(np->sock_addr))) < 0)
+        return(-1);
+
+    DIS_tcp_setup(stream);
+
+    ret = im_compose(stream, pjob->ji_qs.ji_jobid, cookie, IM_SIGNAL_TASK, ep->ee_event, TM_NULL_TASK);
+
+    if (ret == DIS_SUCCESS)
       {
-      np->hn_stream = rpp_open(np->hn_host, np->hn_port, NULL);
+      if ((ret = diswui(stream, pjob->ji_nodeid)) == DIS_SUCCESS)
+        {
+        if ((ret = diswsi(stream, TM_NULL_TASK)) == DIS_SUCCESS)
+          {
+          if ((ret = diswsi(stream, signum)) == DIS_SUCCESS)
+            {
+            ret = DIS_tcp_wflush(stream);
+
+            /* NYI: add code to read reply from sister */
+            }
+          }
+        }
       }
 
-    ret = im_compose(np->hn_stream, pjob->ji_qs.ji_jobid, cookie, IM_SIGNAL_TASK, ep->ee_event, TM_NULL_TASK);
+    close(stream);
 
     if (ret != DIS_SUCCESS)
-      {
       return ret;
-      }
+    } /* END for each node in ji_hosts */
 
-    ret = diswui(np->hn_stream, pjob->ji_nodeid); /* XXX */
-
-    if (ret != DIS_SUCCESS)
-      {
-      return ret;
-      }
-
-    ret = diswsi(np->hn_stream, TM_NULL_TASK);
-
-    if (ret != DIS_SUCCESS)
-      {
-      return ret;
-      }
-
-    ret = diswsi(np->hn_stream, signum);
-
-    if (ret != DIS_SUCCESS)
-      {
-      return ret;
-      }
-
-    ret = rpp_flush(np->hn_stream);
-
-    if (ret != DIS_SUCCESS)
-      {
-      return ret;
-      }
-    }
-
-  return(0);
+  return(PBSE_NONE);
   }  /* END sigalltasks_sisters() */
 
 
@@ -2168,9 +2151,10 @@ static void resume_suspend(
  * Signal may be either a numeric string or a signal name
  * with or without the "SIG" prefix.
  *
- * NOTE:  process_request() set up as request handler via accept_conn()
+ * NOTE:  mom_process_request() set up as request handler via 
+ * accept_conn() 
  *
- * @see process_request->dispatch_request() - parent
+ * @see mom_process_request->dispatch_request() - parent
  * @see req_signaljob() in server/req_signal.c - peer
  */
 
@@ -2243,7 +2227,7 @@ void req_signaljob(
     {
     if (pjob->ji_qs.ji_substate != JOB_SUBSTATE_SUSPEND)
       {
-      LOG_EVENT(
+      log_event(
         PBSEVENT_JOB,
         PBS_EVENTCLASS_JOB,
         pjob->ji_qs.ji_jobid,
@@ -2318,11 +2302,7 @@ void req_signaljob(
     sprintf(log_buffer, "job recycled into exiting on SIGNULL/KILL from substate %d",
       pjob->ji_qs.ji_substate);
 
-    LOG_EVENT(
-      PBSEVENT_ERROR,
-      PBS_EVENTCLASS_JOB,
-      pjob->ji_qs.ji_jobid,
-      log_buffer);
+    log_event(PBSEVENT_ERROR, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buffer);
 
     pjob->ji_qs.ji_substate = JOB_SUBSTATE_EXITING;
 
@@ -2352,6 +2332,7 @@ void req_signaljob(
 void encode_used(
 
   job        *pjob,   /* I */
+  int         perm,   /* I */
   tlist_head *phead)  /* O */
 
   {
@@ -2377,7 +2358,7 @@ void encode_used(
     attribute     val;
     int           rc;
 
-    if ((rd->rs_flags & resc_access_perm) == 0)
+    if ((rd->rs_flags & perm) == 0)
       continue;
 
     val = rs->rs_value; /* copy resource attribute */
@@ -2414,12 +2395,12 @@ void encode_used(
       }
 
     rc = rd->rs_encode(
-
            &val,
            phead,
            ad->at_name,
            rd->rs_name,
-           ATR_ENCODE_CLIENT);
+           ATR_ENCODE_CLIENT,
+           perm);
 
     if (rc < 0)
       break;
@@ -2432,6 +2413,7 @@ void encode_used(
 void encode_flagged_attrs(
 
   job        *pjob,   /* I */
+  int         perm,   /* I */
   tlist_head *phead)  /* O */
 
   {
@@ -2455,20 +2437,16 @@ void encode_flagged_attrs(
         sprintf(log_buffer, "encoding \"send flagged\" attr: %s",
                 ad->at_name);
 
-        LOG_EVENT(
-          PBSEVENT_DEBUG,
-          PBS_EVENTCLASS_JOB,
-          pjob->ji_qs.ji_jobid,
-          log_buffer);
+        log_event(PBSEVENT_DEBUG, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buffer);
         }
 
       ad->at_encode(
-
         at,
         phead,
         ad->at_name,
         NULL,
-        ATR_ENCODE_CLIENT);
+        ATR_ENCODE_CLIENT,
+        perm);
       }
     }
   }
@@ -2494,9 +2472,10 @@ void req_stat_job(
   struct batch_request *preq)  /* I */
 
   {
-  int   all;
+  int     all;
+  int     resc_access_perm = preq->rq_perm & ATR_DFLAG_RDACC;
   char   *name;
-  job   *pjob;
+  job    *pjob;
 
   struct batch_reply *preply = &preq->rq_reply;
 
@@ -2533,10 +2512,6 @@ void req_stat_job(
 
   CLEAR_HEAD(preply->brp_un.brp_status);
 
-  /* pass user-client privilege to encode_resc() */
-
-  resc_access_perm = preq->rq_perm & ATR_DFLAG_RDACC;
-
   for (;pjob;pjob = all ? (job *)GET_NEXT(pjob->ji_alljobs) : NULL)
     {
     if ((pjob->ji_qs.ji_svrflags & JOB_SVFLG_HERE) == 0)
@@ -2547,7 +2522,7 @@ void req_stat_job(
 
     /* allocate reply structure and fill in header portion */
 
-    pstat = (struct brp_status *)malloc(sizeof(struct brp_status));
+    pstat = (struct brp_status *)calloc(1, sizeof(struct brp_status));
 
     assert(pstat != NULL);
 
@@ -2561,12 +2536,12 @@ void req_stat_job(
 
     append_link(&preply->brp_un.brp_status, &pstat->brp_stlink, pstat);
 
-    encode_used(pjob, &pstat->brp_attr); /* adds resources_used attr */
+    encode_used(pjob, resc_access_perm, &pstat->brp_attr); /* adds resources_used attr */
 
-    encode_flagged_attrs(pjob, &pstat->brp_attr); /* adds other flagged attrs */
+    encode_flagged_attrs(pjob, resc_access_perm, &pstat->brp_attr); /* adds other flagged attrs */
     }
 
-  reply_send(preq);
+  reply_send_mom(preq);
 
   return;
   }  /* END req_stat_job() */
@@ -2602,7 +2577,6 @@ static int del_files(
   char  *path;
   char  *pp;
   char  *prmt;
-  int   del_dir = 0;
 
   struct stat  sb;
 #if NO_SPOOL_OUTPUT == 1
@@ -2617,11 +2591,11 @@ static int del_files(
   wordexp_t pathexp;
 #endif
 
-  path = malloc(sizeof(char)*(MAXPATHLEN + 1));
+  path = calloc((MAXPATHLEN + 1), sizeof(char));
 
   if (path==NULL)
     {
-    add_bad_list(pbadfile,"malloc failed",1);
+    add_bad_list(pbadfile,"calloc failed",1);
 
     return(-1);
     }
@@ -2708,7 +2682,7 @@ static int del_files(
         add_bad_list(pbadfile,log_buffer,1);
 
         return(-1);
-      }
+        }
 
       /* run as the user */
       if (setuid(useruid) != 0 && EUID0 == TRUE)
@@ -2722,7 +2696,7 @@ static int del_files(
         add_bad_list(pbadfile,log_buffer,1);
 
         return(-1);
-      }
+        }
 
       EUID0 = FALSE;
       UID0 = FALSE;
@@ -2750,7 +2724,7 @@ static int del_files(
 
     strcat(path, pair->fp_local);
 
-    del_dir = replace_checkpoint_path(path);
+    replace_checkpoint_path(path);
 
     if (local_or_remote(&prmt) == 0)
       {
@@ -2847,11 +2821,7 @@ static int del_files(
       sprintf(log_buffer, "cannot stat %s",
               path);
 
-      LOG_EVENT(
-        PBSEVENT_JOB,
-        PBS_EVENTCLASS_REQUEST,
-        id,
-        log_buffer);
+      log_event(PBSEVENT_JOB, PBS_EVENTCLASS_REQUEST, id, log_buffer);
       }
 
     /*
@@ -2861,7 +2831,6 @@ static int del_files(
      * Do not remove if it is in the remote checkpoint directory list
      */
 
-/*    if (del_dir) */
     if (pair->fp_flag == JOBCKPFILE)
       {
       char *ptr;
@@ -2879,16 +2848,12 @@ static int del_files(
         if (errno != ENOENT)
           {
           sprintf(log_buffer, "Unable to delete file %s for user %s, error = %d %s",
-                  path,
-                  preq->rq_ind.rq_cpyfile.rq_user,
-                  errno,
-                  pbs_strerror(errno));
+            path,
+            preq->rq_ind.rq_cpyfile.rq_user,
+            errno,
+            pbs_strerror(errno));
 
-          LOG_EVENT(
-            PBSEVENT_JOB,
-            PBS_EVENTCLASS_REQUEST,
-            id,
-            log_buffer);
+          log_event(PBSEVENT_JOB, PBS_EVENTCLASS_REQUEST, id, log_buffer);
 
           add_bad_list(pbadfile, log_buffer, 2);
 
@@ -2901,14 +2866,10 @@ static int del_files(
       else
         {
         sprintf(log_buffer, "Deleted file %s for user %s",
-                path,
-                preq->rq_ind.rq_cpyfile.rq_user);
+          path,
+          preq->rq_ind.rq_cpyfile.rq_user);
 
-        LOG_EVENT(
-          PBSEVENT_DEBUG,
-          PBS_EVENTCLASS_FILE,
-          id,
-          log_buffer);
+        log_event(PBSEVENT_DEBUG, PBS_EVENTCLASS_FILE, id, log_buffer);
 
 #endif  /* DEBUG */
         }
@@ -2929,9 +2890,10 @@ void req_rerunjob(
   {
   static char   *id = "req_rerunjob";
 
-  job  *pjob;
-  int        sock;
-  int   rc;
+  job           *pjob;
+  int            sock;
+  int            rc;
+  int            retrycnt = 0;
 
   pjob = find_job(preq->rq_ind.rq_rerun);
 
@@ -2944,11 +2906,7 @@ void req_rerunjob(
 
   if (LOGLEVEL >= 3)
     {
-    log_record(
-      PBSEVENT_JOB,
-      PBS_EVENTCLASS_JOB,
-      pjob->ji_qs.ji_jobid,
-      "rerunning job");
+    log_record(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, "rerunning job");
     }
 
   /* fork to send files back */
@@ -2971,15 +2929,21 @@ void req_rerunjob(
    * This is acceptable because we are a child process, not pbs_mom.
    */
 
-  sock = mom_open_socket_to_jobs_server(pjob, id, NULL);
-
-  if (sock < 0)
+  while ((sock = mom_open_socket_to_jobs_server(pjob, id, NULL)) < 0)
     {
-    /* FAILURE */
-
-    req_reject(PBSE_NOSERVER, 0, preq, NULL, NULL);
-
-    exit(0);
+    retrycnt++;
+    if (retrycnt < 10)
+      {
+      sleep(1);
+      }
+    else
+      {
+      /* FAILURE */
+      
+      req_reject(PBSE_NOSERVER, 0, preq, NULL, NULL);
+      
+      exit(0);
+      }
     }
 
   if (((rc = return_file(pjob, StdOut, sock, TRUE)) != 0) ||
@@ -2988,13 +2952,15 @@ void req_rerunjob(
     {
     /* FAILURE - cannot report file to server */
 
-    LOG_EVENT(
+    log_event(
       PBSEVENT_ERROR,
       PBS_EVENTCLASS_REQUEST,
       id,
       "cannot move output files to server");
 
     req_reject(rc, 0, preq, NULL, NULL);
+    
+    close(sock);
     }
   else
     {
@@ -3013,47 +2979,52 @@ void req_rerunjob(
 
 
 void req_returnfiles(
-  struct batch_request *preq)
-  {
-  int rc;
 
-  struct job *pjob;
-  int sock;
+  struct batch_request *preq)
+
+  {
+  struct job  *pjob;
+  int          sock;
+  int          retry_attempts = 0;
   static char *id = "req_returnfiles";
 
   pjob = find_job(preq->rq_ind.rq_returnfiles.rq_jobid);
 
   if (pjob != NULL)
     {
-retry:
-    sock = mom_open_socket_to_jobs_server(pjob, id, NULL);
-
-    if (sock < 0)
+    while ((sock = mom_open_socket_to_jobs_server(pjob, id, NULL)) < 0)
       {
-      /* XXX TODO */
-      sprintf(log_buffer, "mom_open_socket_to_jobs_server FAILED to get socket: %d for job %s",
-              sock,
-              pjob->ji_qs.ji_jobid);
+      if (retry_attempts++ >= 10)
+        {
+        req_reject(PBSE_NOSERVER, 0, preq, NULL, "Unable to open socket to pbs_server");
 
+        break;
+        }
+
+      sprintf(log_buffer, "mom_open_socket_to_jobs_server FAILED to get socket: %d for job %s",
+        sock,
+        pjob->ji_qs.ji_jobid);
+      
       log_err(-1, id, log_buffer);
       sleep(1);
-      goto retry;
       }
 
-    if (preq->rq_ind.rq_returnfiles.rq_return_stdout)
+    if (sock >= 0)
       {
-      rc = return_file(pjob, StdOut, sock, FALSE);
-
+      if (preq->rq_ind.rq_returnfiles.rq_return_stdout)
+        {
+        return_file(pjob, StdOut, sock, FALSE);
+        }
+      
+      if (preq->rq_ind.rq_returnfiles.rq_return_stderr)
+        {
+        return_file(pjob, StdErr, sock, FALSE);
+        }
+      
+      reply_ack(preq);
+      
+      close(sock);
       }
-
-    if (preq->rq_ind.rq_returnfiles.rq_return_stderr)
-      {
-      rc = return_file(pjob, StdErr, sock, FALSE);
-      }
-
-    reply_ack(preq);
-
-    close(sock);
     }
   else
     {
@@ -3061,7 +3032,7 @@ retry:
     }
 
   return;
-  }
+  } /* END req_returnfiles() */
 
 
 /*
@@ -3094,8 +3065,8 @@ static int sys_copy(
     /* local copy */
 
     ag0 = "/bin/cp";
-      ag1 = "-rp";
-      }
+    ag1 = "-rp";
+    }
   else
     {
     ag0 = rcp_path;
@@ -3151,8 +3122,6 @@ static int sys_copy(
 
       /* child - exec the copy command */
 
-      rpp_terminate();
-
       close(conn);
 
       /* redirect stderr to make error from rcp available to MOM */
@@ -3258,47 +3227,48 @@ void req_cpyfile(
   struct batch_request *preq)  /* I */
 
   {
-  char   id[] = "req_cpyfile";
+  static char     id[] = "req_cpyfile";
 
-  char  *arg2 = NULL;
-  char  *arg3 = NULL;
-  int   bad_files = 0;
-  char  *bad_list = NULL;
-  int   dir = 0;
-  int   from_spool = 0;  /* boolean - set if file must be removed from spool after copy */
-  int   len;
-  char   localname[MAXPATHLEN + 1];  /* used only for in-bound */
+  char           *arg2 = NULL;
+  char           *arg3 = NULL;
+  int             bad_files = 0;
+  char           *bad_list = NULL;
+  int             dir = 0;
+  int             from_spool = 0;  /* boolean - set if file must be removed from spool after copy */
+  int             len;
+  char            localname[MAXPATHLEN + 1];  /* used only for in-bound */
 
   struct rqfpair *pair = NULL;
-  char  *prmt;
-  int   rc;
-  int   rmtflag = 0;
+  char           *prmt;
+  int             rc;
+  int             rmtflag = 0;
 #if NO_SPOOL_OUTPUT == 0
-  char   undelname[MAXPATHLEN + 1];
+  char            undelname[MAXPATHLEN + 1];
 #endif /* !NO_SPOOL_OUTPUT */
 
 #ifdef  _CRAY
-  char   tmpdirname[MAXPATHLEN + 1];
+  char            tmpdirname[MAXPATHLEN + 1];
 #endif  /* _CRAY */
-  char   localname_alt[MAXPATHLEN + 1];
+  char            localname_alt[MAXPATHLEN + 1];
 
-  struct stat  myspooldir;
-  int   rcstat;
-  char           homespool[MAXPATHLEN + 1];
-  int            havehomespool;
+  struct stat     myspooldir;
+  int             rcstat;
+  char            homespool[MAXPATHLEN + 1];
+  int             havehomespool;
 
-  char           EMsg[1024];
-  char           HDir[1024];
+  char            EMsg[1024];
+  char            HDir[1024];
 
-  job           *pjob = NULL;
+  job            *pjob = NULL;
 
 #ifdef HAVE_WORDEXP
-  int   madefaketmpdir = 0;
-  int   usedfaketmpdir = 0;
-  wordexp_t  arg2exp, arg3exp;
-  int            arg2index = -1;
-  char   faketmpdir[1024];
-  int   wordexperr = 0;
+  int             madefaketmpdir = 0;
+  int             usedfaketmpdir = 0;
+  wordexp_t       arg2exp;
+  wordexp_t       arg3exp;
+  int             arg2index = -1;
+  char            faketmpdir[1024];
+  int             wordexperr = 0;
 #endif
 
   /* there is nothing to copy */
@@ -3323,12 +3293,7 @@ void req_cpyfile(
       sprintf(log_buffer, "copy file request is corrupt");
       }
 
-    LOG_EVENT(
-
-      PBSEVENT_JOB,
-      PBS_EVENTCLASS_JOB,
-      preq->rq_ind.rq_cpyfile.rq_jobid,
-      log_buffer);
+    log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, preq->rq_ind.rq_cpyfile.rq_jobid, log_buffer);
     }
 
   rc = (int)fork_to_user(preq, TRUE, HDir, EMsg);
@@ -3477,7 +3442,7 @@ void req_cpyfile(
         {
         char *envstr;
 
-        envstr = malloc((strlen("TMPDIR=") + strlen(faketmpdir) + 1) * sizeof(char));
+        envstr = calloc((strlen("TMPDIR=") + strlen(faketmpdir) + 1), sizeof(char));
 
         if (envstr == NULL)
           {
@@ -3515,8 +3480,8 @@ void req_cpyfile(
 
   /* build up cp/rcp command(s), one per file pair */
 
-  arg2 = malloc(sizeof(char)*(MAXPATHLEN + 1));
-  arg3 = malloc(sizeof(char)*(MAXPATHLEN + 1));
+  arg2 = calloc((MAXPATHLEN + 1), sizeof(char));
+  arg3 = calloc((MAXPATHLEN + 1), sizeof(char));
 
   if ((arg2==NULL) || (arg3==NULL))
     {
@@ -3703,12 +3668,11 @@ void req_cpyfile(
 
       if (pair->fp_flag == JOBCKPFILE)
         {
-        int path_changed = 0;
         char needdir[MAXPATHLEN + 1];
         int saveumask;
         char *ptr;
 
-        path_changed = replace_checkpoint_path(arg3);
+        replace_checkpoint_path(arg3);
 
         /*
          * If the checkpoint directory
