@@ -136,7 +136,7 @@
 **  loadave  current load average
 */
 
-static char ident[] = "@(#) solaris5/$RCSfile$ $Revision: 3230 $";
+static char ident[] = "@(#) solaris5/$RCSfile$ $Revision: 5451 $";
 
 #ifndef TRUE
 #define FALSE 0
@@ -259,7 +259,7 @@ dep_cleanup(void)
  */
 
 static int
-getsize(resource *pres, unsigned long *ret)
+mm_getsize(resource *pres, unsigned long *ret)
   {
   unsigned long value;
 
@@ -295,7 +295,7 @@ getsize(resource *pres, unsigned long *ret)
  */
 
 static int
-gettime(resource *pres, unsigned long *ret)
+mm_gettime(resource *pres, unsigned long *ret)
   {
 
   if (pres->rs_value.at_type != ATR_TYPE_LONG)
@@ -565,7 +565,7 @@ mom_set_limits(
       {
       if (igncput == FALSE)
         {
-        retval = gettime(pres, &value);
+        retval = mm_gettime(pres, &value);
 
         if (retval != PBSE_NONE)
           return (error(pname, retval));
@@ -576,7 +576,7 @@ mom_set_limits(
       if (igncput == FALSE)
         {
         /* process cpu time - set */
-        retval = gettime(pres, &value);
+        retval = mm_gettime(pres, &value);
 
         if (retval != PBSE_NONE)
           return (error(pname, retval));
@@ -592,7 +592,7 @@ mom_set_limits(
       {
       if (set_mode == SET_LIMIT_SET)
         {
-        retval = getsize(pres, &value);
+        retval = mm_getsize(pres, &value);
 
         if (retval != PBSE_NONE)
           return (error(pname, retval));
@@ -610,7 +610,7 @@ mom_set_limits(
       {
       if (ignmem == FALSE)
         {
-        retval = getsize(pres, &value);
+        retval = mm_getsize(pres, &value);
 
         if (retval != PBSE_NONE)
           return (error(pname, retval));
@@ -622,7 +622,7 @@ mom_set_limits(
         {
         if (set_mode == SET_LIMIT_SET)
           {
-          retval = getsize(pres, &value);
+          retval = mm_getsize(pres, &value);
 
           if (retval != PBSE_NONE)
             return (error(pname, retval));
@@ -644,7 +644,7 @@ mom_set_limits(
       {
       if (ignvmem == FALSE)
         {
-        retval = getsize(pres, &value);
+        retval = mm_getsize(pres, &value);
 
         if (retval != PBSE_NONE)
           return (error(pname, retval));
@@ -657,7 +657,7 @@ mom_set_limits(
       {
       if (ignvmem == FALSE) 
         {
-        retval = getsize(pres, &value);
+        retval = mm_getsize(pres, &value);
 
         if (retval != PBSE_NONE)
           return (error(pname, retval));
@@ -671,7 +671,7 @@ mom_set_limits(
       }
     else if (strcmp(pname, "walltime") == 0)   /* Check */
       {
-      retval = gettime(pres, &value);
+      retval = mm_gettime(pres, &value);
 
       if (retval != PBSE_NONE)
         return (error(pname, retval));
@@ -760,12 +760,12 @@ mom_open_poll(void)
   char  *id = "mom_open_poll";
 
   DBPRT(("%s: entered\n", id))
-  proc_status = (prstatus_t *)malloc(sizeof(prstatus_t) * TBL_INC);
-  proc_info = (prpsinfo_t *)malloc(sizeof(prpsinfo_t) * TBL_INC);
+  proc_status = (prstatus_t *)calloc(sizeof(prstatus_t), TBL_INC);
+  proc_info = (prpsinfo_t *)calloc(sizeof(prpsinfo_t), TBL_INC);
 
   if (proc_status == NULL || proc_info == NULL)
     {
-    log_err(errno, id, "malloc");
+    log_err(errno, id, "calloc");
     return (PBSE_SYSTEM);
     }
 
@@ -882,7 +882,7 @@ mom_over_limit(job *pjob)
 
     if ((igncput == FALSE) && (strcmp(pname, "cput") == 0))
       {
-      retval = gettime(pres, &value);
+      retval = mm_gettime(pres, &value);
 
       if (retval != PBSE_NONE)
         continue;
@@ -897,7 +897,7 @@ mom_over_limit(job *pjob)
       }
     else if ((igncput == FALSE) && (strcmp(pname, "pcput") == 0))
       {
-      retval = gettime(pres, &value);
+      retval = mm_gettime(pres, &value);
 
       if (retval != PBSE_NONE)
         continue;
@@ -912,7 +912,7 @@ mom_over_limit(job *pjob)
       }
     else if (strcmp(pname, "vmem") == 0)
       {
-      retval = getsize(pres, &value);
+      retval = mm_getsize(pres, &value);
 
       if (retval != PBSE_NONE)
         continue;
@@ -930,7 +930,7 @@ mom_over_limit(job *pjob)
       if ((pjob->ji_qs.ji_svrflags & JOB_SVFLG_HERE) == 0)
         continue;
 
-      retval = gettime(pres, &value);
+      retval = mm_gettime(pres, &value);
 
       if (retval != PBSE_NONE)
         continue;
