@@ -97,7 +97,7 @@
 #include "mom_mach.h"
 #include "mom_func.h"
 
-static char ident[] = "@(#) $RCSfile$ $Revision: 3384 $";
+static char ident[] = "@(#) $RCSfile$ $Revision: 3876 $";
 
 /* Global Variables */
 
@@ -105,6 +105,7 @@ extern int  exiting_tasks;
 extern char  mom_host[];
 extern tlist_head svr_alljobs;
 extern int  termin_child;
+extern int  multi_mom;
 
 /* Private variables */
 
@@ -202,8 +203,9 @@ scan_for_terminated(void)
   int  exiteval;
   pid_t  pid;
   job  *pjob;
-  task  *ptask = NULL;
+  task  *ptask;
   int  statloc;
+  unsigned int momport = 0;
 
   /* update the latest intelligence about the running jobs;         */
   /* must be done before we reap the zombies, else we lose the info */
@@ -266,7 +268,7 @@ scan_for_terminated(void)
     if (pjob == NULL)
       {
       DBPRT(("%s: pid %d not tracked, exit %d\n",
-             id, (int)pid, (int)exiteval))
+             id, pid, exiteval))
       continue;
       }
 
@@ -280,13 +282,18 @@ scan_for_terminated(void)
 
       pjob->ji_momsubt = 0;
 
-      (void)job_save(pjob, SAVEJOB_QUICK);
+      if(multi_mom)
+        {
+        momport = pbs_rm_port;
+        }
+
+      (void)job_save(pjob, SAVEJOB_QUICK, momport);
       continue;
       }
 
     DBPRT(("%s: task %d pid %d exit value %d\n", id,
-           (int)ptask->ti_qs.ti_task, (int)pid, exiteval))
 
+           ptask->ti_qs.ti_task, pid, exiteval))
     kill_task(ptask, SIGKILL, 0);
     ptask->ti_qs.ti_exitstat = exiteval;
     ptask->ti_qs.ti_status = TI_STATE_EXITED;
@@ -322,8 +329,6 @@ open_master(
 
   (void)strncpy(pty_name, "/dev/ptyXY", PTY_SIZE);
 
-  pc1 = ident; /* this is to remove unused ident warning in compiler */
-
   for (pc1 = ptcchar1; *pc1 != '\0'; ++pc1)
     {
     pty_name[8] = *pc1;
