@@ -135,7 +135,7 @@
 #include "resmon.h"
 #include "../rm_dep.h"
 
-static char ident[] = "@(#) c90/$RCSfile$ $Revision: 3151 $";
+static char ident[] = "@(#) c90/$RCSfile$ $Revision: 5451 $";
 
 #ifndef max
 #define max(a,b) (a>b ? a : b)
@@ -306,9 +306,9 @@ extern char *path_checkpoint;
 ** external functions and data
 */
 
-extern struct config  *search A_((struct config *, char *));
+extern struct config  *search(struct config *, char *);
 
-extern struct rm_attribute *momgetattr A_((char *));
+extern struct rm_attribute *momgetattr(char *);
 extern int   rm_errno;
 extern unsigned int reqnum;
 extern char   *ret_string;
@@ -321,26 +321,26 @@ extern  int     igncput;
 /*
 ** local functions and data
 */
-static char *ncpus   A_((struct rm_attribute *attrib));
-static char *totmem  A_((struct rm_attribute *attrib));
-static char *availmem A_((struct rm_attribute *attrib));
-static char *physmem A_((struct rm_attribute *attrib));
-static char *quota  A_((struct rm_attribute *attrib));
-static char *srfs_reserve A_((struct rm_attribute *attrib));
-static char *swaprate A_((struct rm_attribute *attrib));
-static char *swapinrate A_((struct rm_attribute *attrib));
-static char *swapoutrate A_((struct rm_attribute *attrib));
-static char *swapused A_((struct rm_attribute *attrib));
-static char *swapavail A_((struct rm_attribute *attrib));
-static char *swaptotal A_((struct rm_attribute *attrib));
-static char *cpuidle A_((struct rm_attribute *attrib));
-static char *cpuguest A_((struct rm_attribute *attrib));
-static char *cpuuser A_((struct rm_attribute *attrib));
-static char *cpuunix A_((struct rm_attribute *attrib));
-static char *cpusysw A_((struct rm_attribute *attrib));
+static char *ncpus(struct rm_attribute *attrib);
+static char *totmem(struct rm_attribute *attrib);
+static char *availmem(struct rm_attribute *attrib);
+static char *physmem(struct rm_attribute *attrib);
+static char *quota(struct rm_attribute *attrib);
+static char *srfs_reserve(struct rm_attribute *attrib);
+static char *swaprate(struct rm_attribute *attrib);
+static char *swapinrate(struct rm_attribute *attrib);
+static char *swapoutrate(struct rm_attribute *attrib);
+static char *swapused(struct rm_attribute *attrib);
+static char *swapavail(struct rm_attribute *attrib);
+static char *swaptotal(struct rm_attribute *attrib);
+static char *cpuidle(struct rm_attribute *attrib);
+static char *cpuguest(struct rm_attribute *attrib);
+static char *cpuuser(struct rm_attribute *attrib);
+static char *cpuunix(struct rm_attribute *attrib);
+static char *cpusysw(struct rm_attribute *attrib);
 
-extern char *loadave A_((struct rm_attribute *attrib));
-extern char *nullproc A_((struct rm_attribute *attrib));
+extern char *loadave(struct rm_attribute *attrib);
+extern char *nullproc(struct rm_attribute *attrib);
 
 extern char extra_parm[];
 extern char no_parm[];
@@ -607,7 +607,7 @@ dep_cleanup(void)
  */
 
 static int
-getsize(resource *pres, unsigned long *ret)
+mm_getsize(resource *pres, unsigned long *ret)
   {
   unsigned long value;
 
@@ -643,7 +643,7 @@ getsize(resource *pres, unsigned long *ret)
  */
 
 static int
-gettime(resource *pres, unsigned long *ret)
+mm_gettime(resource *pres, unsigned long *ret)
   {
 
   if (pres->rs_value.at_type != ATR_TYPE_LONG)
@@ -1323,7 +1323,7 @@ mom_set_limits(
       {
       if (igncput == FALSE)
         {
-        retval = gettime(pres, &value);
+        retval = mm_gettime(pres, &value);
         value = (unsigned long)((double)value / cputfactor);
 
         if (retval != PBSE_NONE)
@@ -1343,7 +1343,7 @@ mom_set_limits(
       {
       if (igncput == FALSE) 
         {
-        retval = gettime(pres, &value);
+        retval = mm_gettime(pres, &value);
         value = (unsigned long)((double)value / cputfactor);
 
         if (retval != PBSE_NONE)
@@ -1362,7 +1362,7 @@ mom_set_limits(
       {
       if (ignmem == FALSE) 
         {
-        retval = getsize(pres, &value);
+        retval = mm_getsize(pres, &value);
 
         if (retval != PBSE_NONE)
           return (error(pname, retval));
@@ -1381,7 +1381,7 @@ mom_set_limits(
       {
       if (ignmem == FALSE) 
         {
-        retval = getsize(pres, &value);
+        retval = mm_getsize(pres, &value);
 
         if (retval != PBSE_NONE)
           return (error(pname, retval));
@@ -1427,7 +1427,7 @@ mom_set_limits(
       }
     else if (strcmp(pname, "pf") == 0)
       {
-      retval = getsize(pres, &value);
+      retval = mm_getsize(pres, &value);
 
       if (retval != PBSE_NONE)
         return (error(pname, retval));
@@ -1445,7 +1445,7 @@ mom_set_limits(
       }
     else if (strcmp(pname, "ppf") == 0)
       {
-      retval = getsize(pres, &value);
+      retval = mm_getsize(pres, &value);
 
       if (retval != PBSE_NONE)
         return (error(pname, retval));
@@ -1463,7 +1463,7 @@ mom_set_limits(
       }
     else if (strcmp(pname, "sds") == 0)
       {
-      retval = getsize(pres, &value);
+      retval = mm_getsize(pres, &value);
 
       if (retval != PBSE_NONE)
         return (error(pname, retval));
@@ -1479,7 +1479,7 @@ mom_set_limits(
       }
     else if (strcmp(pname, "psds") == 0)
       {
-      retval = getsize(pres, &value);
+      retval = mm_getsize(pres, &value);
 
       if (retval != PBSE_NONE)
         return (error(pname, retval));
@@ -1527,7 +1527,7 @@ mom_set_limits(
       }
     else if (strcmp(pname, "mppt") == 0)
       {
-      retval = gettime(pres, &value);
+      retval = mm_gettime(pres, &value);
 
       if (retval != PBSE_NONE)
         return (error(pname, retval));
@@ -1543,7 +1543,7 @@ mom_set_limits(
       }
     else if (strcmp(pname, "pmppt") == 0)
       {
-      retval = gettime(pres, &value);
+      retval = mm_gettime(pres, &value);
 
       if (retval != PBSE_NONE)
         return (error(pname, retval));
@@ -1561,7 +1561,7 @@ mom_set_limits(
       }
     else if (strcmp(pname, "srfs_tmp") == 0)
       {
-      retval = getsize(pres, &value);
+      retval = mm_getsize(pres, &value);
 
       if (retval != PBSE_NONE)
         return (error(pname, retval));
@@ -1575,7 +1575,7 @@ mom_set_limits(
       }
     else if (strcmp(pname, "srfs_big") == 0)
       {
-      retval = getsize(pres, &value);
+      retval = mm_getsize(pres, &value);
 
       if (retval != PBSE_NONE)
         return (error(pname, retval));
@@ -1589,7 +1589,7 @@ mom_set_limits(
       }
     else if (strcmp(pname, "srfs_fast") == 0)
       {
-      retval = getsize(pres, &value);
+      retval = mm_getsize(pres, &value);
 
       if (retval != PBSE_NONE)
         return (error(pname, retval));
@@ -1614,7 +1614,7 @@ mom_set_limits(
       }
     else if (strcmp(pname, "walltime") == 0)
       {
-      retval = gettime(pres, &value);
+      retval = mm_gettime(pres, &value);
 
       if (retval != PBSE_NONE)
         return (error(pname, retval));
@@ -1927,7 +1927,7 @@ mom_get_sample(void)
 
   session_table_size = info.ent * info.len;
 
-  session_table = (struct sess*)malloc(session_table_size);
+  session_table = (struct sess*)calloc(1, session_table_size);
 
   if (session_table == NULL)
     return (PBSE_SYSTEM);
@@ -1946,7 +1946,7 @@ mom_get_sample(void)
 
   process_table_size = info.ent * info.len;
 
-  process_table = (struct proc*)malloc(process_table_size);
+  process_table = (struct proc*)calloc(1, process_table_size);
 
   if (process_table == NULL)
     return (PBSE_SYSTEM);
@@ -2026,7 +2026,7 @@ mom_over_limit(job *pjob)
       if ((pjob->ji_qs.ji_svrflags & JOB_SVFLG_HERE) == 0)
         continue;
 
-      retval = gettime(pres, &value);
+      retval = mm_gettime(pres, &value);
 
       if (retval != PBSE_NONE)
         continue;
