@@ -99,7 +99,7 @@
 #include "mom_mach.h"
 #include "mom_func.h"
 
-static char ident[] = "@(#) $RCSfile$ $Revision: 4236 $";
+static char ident[] = "@(#) $RCSfile$ $Revision: 4239 $";
 
 /* Global Variables */
 
@@ -108,6 +108,7 @@ extern char  mom_host[];
 extern tlist_head svr_alljobs;
 extern int  termin_child;
 extern time_t    time_now;
+extern int multi_mom;
 
 #if SRFS
 char *fast_dir;
@@ -512,6 +513,7 @@ scan_for_terminated(void)
   job  *pjob;
   task  *ptask;
   int  statloc;
+  unsigned int momport = 0;
 
   /* update the latest intelligence about the running jobs;         */
   /* must be done before we reap the zombies, else we lose the info */
@@ -558,7 +560,12 @@ scan_for_terminated(void)
 
         pjob->ji_momsubt = 0;
 
-        (void)job_save(pjob, SAVEJOB_QUICK);
+        if(multi_mom)
+          {
+          momport = pbs_rm_port;
+          }
+
+        (void)job_save(pjob, SAVEJOB_QUICK, momport);
         break;
         }
 
