@@ -123,7 +123,7 @@
 #include "../rm_dep.h"
 
 static char ident[] =
-  "@(#) unicosmk2/$RCSfile$ $Revision: 3230 $";
+  "@(#) unicosmk2/$RCSfile$ $Revision: 4975 $";
 
 #ifndef max
 #define max(a,b) (a>b ? a : b)
@@ -272,7 +272,7 @@ dep_cleanup(void)
  */
 
 static int
-getsize(resource *pres, unsigned long *ret)
+mm_getsize(resource *pres, unsigned long *ret)
   {
   unsigned long value;
 
@@ -308,7 +308,7 @@ getsize(resource *pres, unsigned long *ret)
  */
 
 static int
-gettime(resource *pres, unsigned long *ret)
+mm_gettime(resource *pres, unsigned long *ret)
   {
 
   if (pres->rs_value.at_type != ATR_TYPE_LONG)
@@ -728,7 +728,7 @@ mom_set_limits(
       {
       if (igncput == FALSE)
         {
-        retval = gettime(pres, &value);
+        retval = mm_gettime(pres, &value);
         value = (unsigned long)((double)value / cputfactor);
 
         if (retval != PBSE_NONE)
@@ -748,7 +748,7 @@ mom_set_limits(
       {
       if (igncput == FALSE)
         {
-        retval = gettime(pres, &value);
+        retval = mm_gettime(pres, &value);
         value = (unsigned long)((double)value / cputfactor);
 
         if (retval != PBSE_NONE)
@@ -768,7 +768,7 @@ mom_set_limits(
       {
       if (ignmem == FALSE)
         {
-        retval = getsize(pres, &value);
+        retval = mm_getsize(pres, &value);
 
         if (retval != PBSE_NONE)
           return (error(pname, retval));
@@ -787,7 +787,7 @@ mom_set_limits(
       {
       if (ignmem == FALSE)
         {
-        retval = getsize(pres, &value);
+        retval = mm_getsize(pres, &value);
 
         if (retval != PBSE_NONE)
           return (error(pname, retval));
@@ -833,7 +833,7 @@ mom_set_limits(
       }
     else if (strcmp(pname, "pf") == 0)
       {
-      retval = getsize(pres, &value);
+      retval = mm_getsize(pres, &value);
 
       if (retval != PBSE_NONE)
         return (error(pname, retval));
@@ -851,7 +851,7 @@ mom_set_limits(
       }
     else if (strcmp(pname, "ppf") == 0)
       {
-      retval = getsize(pres, &value);
+      retval = mm_getsize(pres, &value);
 
       if (retval != PBSE_NONE)
         return (error(pname, retval));
@@ -869,7 +869,7 @@ mom_set_limits(
       }
     else if (strcmp(pname, "sds") == 0)
       {
-      retval = getsize(pres, &value);
+      retval = mm_getsize(pres, &value);
 
       if (retval != PBSE_NONE)
         return (error(pname, retval));
@@ -885,7 +885,7 @@ mom_set_limits(
       }
     else if (strcmp(pname, "psds") == 0)
       {
-      retval = getsize(pres, &value);
+      retval = mm_getsize(pres, &value);
 
       if (retval != PBSE_NONE)
         return (error(pname, retval));
@@ -933,7 +933,7 @@ mom_set_limits(
       }
     else if (strcmp(pname, "mppt") == 0)
       {
-      retval = gettime(pres, &value);
+      retval = mm_gettime(pres, &value);
 
       if (retval != PBSE_NONE)
         return (error(pname, retval));
@@ -949,7 +949,7 @@ mom_set_limits(
       }
     else if (strcmp(pname, "pmppt") == 0)
       {
-      retval = gettime(pres, &value);
+      retval = mm_gettime(pres, &value);
 
       if (retval != PBSE_NONE)
         return (error(pname, retval));
@@ -965,7 +965,7 @@ mom_set_limits(
       }
     else if (strcmp(pname, "walltime") == 0)
       {
-      retval = gettime(pres, &value);
+      retval = mm_gettime(pres, &value);
 
       if (retval != PBSE_NONE)
         return (error(pname, retval));
@@ -1220,7 +1220,7 @@ mom_over_limit(job *pjob)
       if ((pjob->ji_qs.ji_svrflags & JOB_SVFLG_HERE) == 0)
         continue;
 
-      retval = gettime(pres, &value);
+      retval = mm_gettime(pres, &value);
 
       if (retval != PBSE_NONE)
         continue;
