@@ -88,6 +88,8 @@
 #include "node_info.h"
 #include "misc.h"
 #include "globals.h"
+#include "dis.h" /* DIS_tcp_setup */
+
 
 
 /* Internal functions */
@@ -115,8 +117,9 @@ node_info **query_nodes(int pbs_sd, server_info *sinfo)
   char *err;    /* used with pbs_geterrmsg() */
   int num_nodes = 0;   /* the number of nodes */
   int i;
+  int local_errno;
 
-  if ((nodes = pbs_statnode(pbs_sd, NULL, NULL, NULL)) == NULL)
+  if ((nodes = pbs_statnode_err(pbs_sd, NULL, NULL, NULL, &local_errno)) == NULL)
     {
     err = pbs_geterrmsg(pbs_sd);
     sprintf(errbuf, "Error getting nodes: %s", err);
@@ -420,6 +423,7 @@ int talk_with_mom(
   int testi;   /* used to convert string -> int */
   char errbuf[256];
   int i;
+  int local_errno = 0;
 
   if ((ninfo != NULL) && !ninfo->is_down && !ninfo->is_offline)
     {
@@ -429,10 +433,11 @@ int talk_with_mom(
       return 1;
       }
 
+    DIS_tcp_setup(mom_sd);
     for (i = 0; i < num_resget; i++)
-      addreq(mom_sd, (char *) res_to_get[i]);
+      addreq_err(mom_sd, &local_errno, (char *) res_to_get[i]);
 
-    for (i = 0; i < num_resget && (mom_ans = getreq(mom_sd)) != NULL; i++)
+    for (i = 0; i < num_resget && (mom_ans = getreq_err(&local_errno, (mom_sd))) != NULL; i++)
       {
       if (!strcmp(res_to_get[i], "max_load"))
         {
@@ -492,7 +497,7 @@ int talk_with_mom(
         }
       }
 
-    closerm(mom_sd);
+    closerm_err(&local_errno, mom_sd);
     }
 
   return 0;
@@ -518,6 +523,7 @@ node_info **node_filter(node_info **nodes, int size,
                         int (*filter_func)(node_info*, void*), void *arg)
   {
   node_info **new_nodes = NULL;   /* the new node array */
+  node_info **tmp = NULL;
   int i, j;
 
   if ((new_nodes = (node_info **) malloc((size + 1) * sizeof(node_info *))) == NULL)
@@ -542,10 +548,15 @@ node_info **node_filter(node_info **nodes, int size,
     free(new_nodes);
     new_nodes = NULL;
     }
-  else if ((new_nodes = realloc(new_nodes, (j + 1) * sizeof(node_info *))) == NULL)
+  else if ((tmp = realloc(new_nodes, (j + 1) * sizeof(node_info *))) == NULL)
     {
     perror("Memory Allocation Error");
     free(new_nodes);
+    new_nodes = NULL;
+    }
+  else
+    {
+    new_nodes = tmp;
     }
 
   return new_nodes;
@@ -644,7 +655,7 @@ node_info *find_best_node(job_info *jinfo, node_info **ninfo_arr)
     if (ninfo_arr[ln_i] == NULL)
       {
       strcpy(last_node_name, ninfo_arr[0] -> name);
-      ln_i = 0;
+      /* ln_i = 0; */
       }
 
     /* if the last node is at the end of the list, cycle back to the front */
