@@ -76,7 +76,7 @@
 * This license will be governed by the laws of the Commonwealth of Virginia,
 * without reference to its choice of law rules.
 */
-/* $Id: comment.c 2367 2008-09-03 17:17:34Z josh $ */
+/* $Id: comment.c 5178 2011-10-11 21:46:02Z dbeer $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -207,6 +207,7 @@ schd_alterjob(int sv_conn, Job *job, char *name, char *value, char *rsrc)
   {
   char   *id = "schd_alterjob";
   int     err = 0;
+  int     local_errno = 0;
   AttrList atp;
 
   /* Fill the attribute structure with function parameters */
@@ -215,13 +216,13 @@ schd_alterjob(int sv_conn, Job *job, char *name, char *value, char *rsrc)
   atp.name     = name;
   atp.next     = NULL;
 
-  err = pbs_alterjob(sv_conn, job->jobid, &atp, NULL);
+  err = pbs_alterjob_err(sv_conn, job->jobid, &atp, NULL, &local_errno);
 
   if (err)
     {
     (void)sprintf(log_buffer,
                   "pbs_alterjob(%s, %s, %s) failed: %d",
-                  job->jobid, name, value, pbs_errno);
+                  job->jobid, name, value, local_errno);
     log_record(PBSEVENT_SYSTEM, PBS_EVENTCLASS_SERVER, id, log_buffer);
     }
 
@@ -233,6 +234,7 @@ schd_alterserver(int sv_conn, char *name, char *value)
   {
   char   *id = "schd_alterserver";
   int     err;
+  int     local_errno = 0;
   AttrOpList alist;
 
   /* Fill in the attribute struct with appropriate parameters */
@@ -243,13 +245,13 @@ schd_alterserver(int sv_conn, char *name, char *value)
   alist.next     = NULL;
   alist.op       = SET;
 
-  err = pbs_manager(sv_conn, MGR_CMD_SET, MGR_OBJ_SERVER, "", &alist,
-                    NULL);
+  err = pbs_manager_err(sv_conn, MGR_CMD_SET, MGR_OBJ_SERVER, "", &alist,
+                    NULL, &local_errno);
 
   if (err)
     {
     (void)sprintf(log_buffer,
-                  "pbs_alterserver(%s, %s) failed: %d", name, value, pbs_errno);
+                  "pbs_alterserver(%s, %s) failed: %d", name, value, local_errno);
     log_record(PBSEVENT_SYSTEM, PBS_EVENTCLASS_SERVER, id, log_buffer);
     }
 
