@@ -76,7 +76,7 @@
 * This license will be governed by the laws of the Commonwealth of Virginia,
 * without reference to its choice of law rules.
 */
-/* $Id: getrsrcs.c 2367 2008-09-03 17:17:34Z josh $ */
+/* $Id: getrsrcs.c 5536 2011-12-21 22:42:08Z dbeer $ */
 
 /*
  *  Obtain resource information from the resource monitor and
@@ -135,6 +135,7 @@ schd_get_resources(char *exechost)
   char            *id = "schd_get_resources";
   Resources       *rptr, *new_rsrcs;
   int              rm;
+  int              local_errno = 0;
   char            *response = NULL;
   int              badreply   = 0;
 
@@ -192,7 +193,7 @@ schd_get_resources(char *exechost)
   if ((rm = openrm(exechost, pbs_rm_port)) == -1)
     {
     (void)sprintf(log_buffer,
-                  "Unable to contact resmom@%s (%d)", exechost, pbs_errno);
+                  "Unable to contact resmom@%s ", exechost);
     log_record(PBSEVENT_SYSTEM, PBS_EVENTCLASS_SERVER, id, log_buffer);
 
     badreply = 1;
@@ -220,7 +221,7 @@ schd_get_resources(char *exechost)
   /* Get the values back from the resource monitor, and round up. */
 
   /* Receive LOADAVE response from resource monitor. */
-  response = getreq(rm);
+  response = getreq_err(&local_errno, rm);
 
   if (response != NULL)
     {
@@ -230,14 +231,14 @@ schd_get_resources(char *exechost)
   else
     {
     (void)sprintf(log_buffer, "bad return from getreq(loadave), %d, %d",
-                  pbs_errno, errno);
+                  local_errno, errno);
     log_record(PBSEVENT_SYSTEM, PBS_EVENTCLASS_SERVER, id, log_buffer);
     badreply = 1;
     goto bail;
     }
 
   /* Receive AVAILMEM response from resource monitor. */
-  response = getreq(rm);
+  response = getreq_err(&local_errno, rm);
 
   if (response != NULL)
     {
@@ -247,14 +248,14 @@ schd_get_resources(char *exechost)
   else
     {
     (void)sprintf(log_buffer, "bad return from getreq(freemem), %d, %d",
-                  pbs_errno, errno);
+                  local_errno, errno);
     log_record(PBSEVENT_SYSTEM, PBS_EVENTCLASS_SERVER, id, log_buffer);
     badreply = 1;
     goto bail;
     }
 
   /* Receive PHYSMEM response from resource monitor. */
-  response = getreq(rm);
+  response = getreq_err(&local_errno, rm);
 
   if (response != NULL)
     {
@@ -264,14 +265,14 @@ schd_get_resources(char *exechost)
   else
     {
     (void)sprintf(log_buffer, "bad return from getreq(realmem), %d, %d",
-                  pbs_errno, errno);
+                  local_errno, errno);
     log_record(PBSEVENT_SYSTEM, PBS_EVENTCLASS_SERVER, id, log_buffer);
     badreply = 1;
     goto bail;
     }
 
   /* Receive NCPUS response from resource monitor. */
-  response = getreq(rm);
+  response = getreq_err(&local_errno, rm);
 
   if (response != NULL)
     {
@@ -281,7 +282,7 @@ schd_get_resources(char *exechost)
   else
     {
     (void)sprintf(log_buffer, "bad return from getreq(ncpus), %d, %d",
-                  pbs_errno, errno);
+                  local_errno, errno);
     log_record(PBSEVENT_SYSTEM, PBS_EVENTCLASS_SERVER, id, log_buffer);
     badreply = 1;
     goto bail;
@@ -290,7 +291,7 @@ schd_get_resources(char *exechost)
   /* DEBUG*/  /*new_rsrcs->ncpus_total *= 4; */
 
   /* Receive ARCH response from resource monitor. */
-  response = getreq(rm);
+  response = getreq_err(&local_errno, rm);
 
   if (response != NULL)
     {
@@ -300,7 +301,7 @@ schd_get_resources(char *exechost)
   else
     {
     (void)sprintf(log_buffer, "bad return from getreq(arch), %d, %d",
-                  pbs_errno, errno);
+                  local_errno, errno);
     log_record(PBSEVENT_SYSTEM, PBS_EVENTCLASS_SERVER, id, log_buffer);
     badreply = 1;
     goto bail;
