@@ -76,7 +76,7 @@
 * This license will be governed by the laws of the Commonwealth of Virginia,
 * without reference to its choice of law rules.
 */
-/* $Id: hpm.c 2367 2008-09-03 17:17:34Z josh $ */
+/* $Id: hpm.c 5536 2011-12-21 22:42:08Z dbeer $ */
 
 /*
  * HPM stands for Harware Performance Monitor.  This is an attribute that
@@ -191,6 +191,7 @@ setup_hpm(Resources *rsrcs, int mode)
   char   *response, *value;
   char    hpm_ctl[64];
   int     rm;
+  int     local_errno = 0;
 
   switch (mode)
     {
@@ -246,7 +247,7 @@ setup_hpm(Resources *rsrcs, int mode)
   if ((rm = openrm(rsrcs->exechost, 0)) < 0)
     {
     (void)sprintf(log_buffer,
-                  "Unable to contact resmom@%s (%d)", rsrcs->exechost, pbs_errno);
+                  "Unable to contact resmom@%s", rsrcs->exechost);
     log_record(PBSEVENT_SYSTEM, PBS_EVENTCLASS_SERVER, id, log_buffer);
     return (1);
     }
@@ -254,15 +255,15 @@ setup_hpm(Resources *rsrcs, int mode)
   /* Ask the resource monitor on the remote host to set the mode for us. */
   response = NULL;
 
-  if (addreq(rm, hpm_ctl) == 0)
-    response = getreq(rm);
+  if (addreq_err(rm, &local_errno, hpm_ctl) == 0)
+    response = getreq_err(&local_errno, rm);
 
   closerm(rm);
 
   if (response == NULL)
     {
     (void)sprintf(log_buffer, "bad return from getreq(%s) @%s, %d",
-                  hpm_ctl, rsrcs->exechost, pbs_errno);
+                  hpm_ctl, rsrcs->exechost, local_errno);
     log_record(PBSEVENT_SYSTEM, PBS_EVENTCLASS_SERVER, id, log_buffer);
     DBPRT(("%s: %s\n", id, log_buffer));
     return (1);
