@@ -79,7 +79,6 @@
 #include <pbs_config.h>   /* the master config generated by configure */
 
 #include <errno.h>
-#include <sys/types.h>
 #include <sys/param.h>
 #include "pbs_ifl.h"
 #include <stdio.h>
@@ -95,14 +94,17 @@
 #include "pbs_error.h"
 #include "svrfunc.h"
 #include "log.h"
+#include "../lib/Liblog/pbs_log.h"
 #include "utils.h"
 #ifdef _CRAY
 #include <udb.h>
 #endif /* _CRAY */
+#include "svr_func.h" /* get_svr_attr_* */
 
 /* External Data */
 
 extern char server_host[];
+extern int LOGLEVEL;
 
 /*
  * geteusernam - get effective user name
@@ -127,7 +129,8 @@ static char *geteusernam(
   struct array_strings *parst;
   char *pn;
   char *ptr;
-  static char username[PBS_MAXUSER + 1];
+  char  username[PBS_MAXUSER + 1];
+  char *ret_user;
 
   /* search the user-list attribute */
 
@@ -176,10 +179,13 @@ static char *geteusernam(
     ptr = site_map_user(username, get_variable(pjob, "PBS_O_HOST"));
 
     if (ptr != username)
-      strcpy(username, ptr);
+      snprintf(username, sizeof(username), "%s", ptr);
     }
 
-  return(username);
+  ret_user = calloc(1, strlen(username) + 1);
+  strcpy(ret_user,username);
+
+  return(ret_user);
   }  /* END geteusernam() */
 
 
@@ -209,7 +215,8 @@ static char *getegroup(
   struct array_strings *parst;
   char *pn;
   char *ptr;
-  static char groupname[PBS_MAXUSER + 1];
+  char  groupname[PBS_MAXUSER + 1];
+  char *ret_group;
 
   /* search the group-list attribute */
 
@@ -251,7 +258,10 @@ static char *getegroup(
 
   get_jobowner(hit, groupname);
 
-  return(groupname);
+  ret_group = calloc(1, strlen(groupname) + 1);
+  strcpy(ret_group,groupname);
+
+  return(ret_group);
   }  /* END getegroup() */
 
 
@@ -282,40 +292,43 @@ int set_jobexid(
   char      *EMsg)    /* O (optional,minsize=1024) */
 
   {
-  int   addflags = 0;
-  attribute *pattr;
-  char        **pmem;
+  int             addflags = 0;
+  attribute      *pattr;
+  char          **pmem;
 
-  struct group *gpent;
-  char  *puser = NULL;
-  char  *id = "set_jobexid";
-  char *at;
-  char *usr_at_host = NULL;
-  int len;
+  struct group   *gpent;
+  int             free_puser = FALSE;
+  char           *puser = NULL;
+  char           *id = "set_jobexid";
+  char           *at;
+  char           *usr_at_host = NULL;
+  int             len;
 
 
-  struct passwd *pwent = NULL;
-  char  *pgrpn;
-  char   gname[PBS_MAXGRPN + 1];
+  struct passwd  *pwent = NULL;
+  char           *pgrpn;
+  int             free_pgrpn = TRUE;
+  char            gname[PBS_MAXGRPN + 1];
 #ifdef _CRAY
 
-  struct udb    *pudb;
+  struct udb     *pudb;
 #endif
 
-  char           tmpLine[1024 + 1];
+  char            tmpLine[1024 + 1];
+  char            log_buf[LOCAL_LOG_BUF_SIZE];
 
-  int            CheckID;  /* boolean */
+  long            disable_id_check = 0;
+  int             CheckID;  /* boolean */
 
   if (EMsg != NULL)
     EMsg[0] = '\0';
 
   /* use the passed User_List if set, may be a newly modified one     */
   /* if not set, fall back to the job's actual User_List, may be same */
-
-  if (server.sv_attr[SRV_ATR_DisableServerIdCheck].at_val.at_long)
-    CheckID = 0;
-  else
+  if (get_svr_attr_l(SRV_ATR_DisableServerIdCheck, &disable_id_check) != PBSE_NONE)
     CheckID = 1;
+  else
+    CheckID = !disable_id_check;
 
   if (CheckID == 0)
     {
@@ -342,14 +355,14 @@ int set_jobexid(
         if (at != NULL)
           {
           len += strlen(at);
-          usr_at_host = (char *)malloc(len * sizeof(char));
+          usr_at_host = (char *)calloc(len, sizeof(char));
           snprintf(usr_at_host,len,"%s%s",
             puser,
             at);
           }
         else
           {
-          usr_at_host = (char *)malloc(len * sizeof(char));
+          usr_at_host = (char *)calloc(len, sizeof(char));
 
           snprintf(usr_at_host,len,"%s",
             puser);
@@ -366,6 +379,8 @@ int set_jobexid(
         return(PBSE_BADUSER);
         }
 
+      free_puser = TRUE;
+
       sprintf(tmpLine, "%s",
 
               puser);
@@ -379,13 +394,13 @@ int set_jobexid(
     }  /* END if (CheckID == 0) */
   else
     {
-    int perm;
-
     if ((attrry + JOB_ATR_userlst)->at_flags & ATR_VFLAG_SET)
       pattr = attrry + JOB_ATR_userlst;
     else
       pattr = &pjob->ji_wattr[JOB_ATR_userlst];
 
+    free_puser = TRUE;
+
     if ((puser = geteusernam(pjob, pattr)) == NULL)
       {
       if (EMsg != NULL)
@@ -396,42 +411,47 @@ int set_jobexid(
 
     pwent = getpwnam_ext(puser);
 
-    perm = svr_get_privilege(puser,get_variable(pjob,"PBS_O_HOST"));
-
     if (pwent == NULL)
       {
-      snprintf(log_buffer,sizeof(log_buffer),
+      snprintf(log_buf,sizeof(log_buf),
         "User %s does not exist in server password file\n",
         puser);
 
-      log_err(errno, id, log_buffer);
+      log_err(errno, id, log_buf);
 
       if (EMsg != NULL)
-        snprintf(EMsg,1024,"%s",log_buffer);
+        snprintf(EMsg,1024,"%s",log_buf);
+
+      free(puser);
 
       return(PBSE_BADUSER);
       }
 
-    if ((pwent->pw_uid == 0) ||
-        (perm & ATR_DFLAG_MGWR))
+    if (pwent->pw_uid == 0)
       {
+      struct array_strings *pas = NULL;
+      get_svr_attr_arst(SRV_ATR_AclRoot, &pas);
+
       /* add check here for virtual user */
       if (pjob->ji_wattr[JOB_ATR_proxy_user].at_flags & ATR_VFLAG_SET)
         {
+        free(puser);
+        free_puser = FALSE;
+
         puser = pjob->ji_wattr[JOB_ATR_proxy_user].at_val.at_str;
 
         pwent = getpwnam_ext(puser);
 
         if (pwent == NULL)
           {
-          snprintf(log_buffer,sizeof(log_buffer),
+          snprintf(log_buf,sizeof(log_buf),
             "User %s does not exist in server password file\n",
             puser);
 
-          log_err(errno, id, log_buffer);
+          log_err(errno, id, log_buf);
 
           if (EMsg != NULL)
-            snprintf(EMsg,1024,"%s",log_buffer);
+            snprintf(EMsg,1024,"%s",log_buf);
 
           return(PBSE_BADUSER);
           }
@@ -442,14 +462,14 @@ int set_jobexid(
         if (at != NULL)
           {
           len += strlen(at);
-          usr_at_host = (char *)malloc(len * sizeof(char));
+          usr_at_host = (char *)calloc(len, sizeof(char));
           snprintf(usr_at_host,len,"%s%s",
             puser,
             at);
           }
         else
           {
-          usr_at_host = (char *)malloc(len * sizeof(char));
+          usr_at_host = (char *)calloc(len, sizeof(char));
 
           snprintf(usr_at_host,len,"%s",
             puser);
@@ -458,12 +478,9 @@ int set_jobexid(
         free(pjob->ji_wattr[JOB_ATR_job_owner].at_val.at_str);
         pjob->ji_wattr[JOB_ATR_job_owner].at_val.at_str = usr_at_host;
         }
-      else if (server.sv_attr[SRV_ATR_AclRoot].at_flags & ATR_VFLAG_SET)
+      else if (get_svr_attr_arst(SRV_ATR_AclRoot, &pas) == PBSE_NONE)
         {
-        if (acl_check(
-              &server.sv_attr[SRV_ATR_AclRoot],
-              pjob->ji_wattr[JOB_ATR_job_owner].at_val.at_str,
-              ACL_User) == 0)
+        if (acl_check_my_array_string(pas, pjob->ji_wattr[JOB_ATR_job_owner].at_val.at_str, ACL_User) == 0)
           {
           if (EMsg != NULL)
             snprintf(EMsg, 1024, "root user %s fails ACL check",
@@ -472,7 +489,7 @@ int set_jobexid(
           return(PBSE_BADUSER); /* root not allowed */
           }
         }
-      else if (pwent->pw_uid == 0)
+      else
         {
         if (EMsg != NULL)
           snprintf(EMsg, 1024, "root user %s not allowed",
@@ -481,39 +498,20 @@ int set_jobexid(
         return(PBSE_BADUSER); /* root not allowed */
         }
       }    /* END if (pwent->pw_uid == 0) */
-    else if (pjob->ji_wattr[JOB_ATR_proxy_user].at_flags & ATR_VFLAG_SET)
-      {
-      /* cannot submit a proxy job if not root or a manager */            
-      if (EMsg != NULL)                      
-        {                              
-        snprintf(EMsg, 1024,                                            
-          "User '%s' is attempting to submit a proxy job for user '%s' but is not a manager",
-          puser,
-          pjob->ji_wattr[JOB_ATR_proxy_user].at_val.at_str);                                      
-        }
-      
-      snprintf(log_buffer, 1024,          
-        "User '%s' is attempting to submit a proxy job for user '%s' but is not a manager",
-        puser,        
-        pjob->ji_wattr[JOB_ATR_proxy_user].at_val.at_str);                        
-      log_err(PBSE_BADUSER, id, log_buffer);
-      
-      return(PBSE_BADUSER);      
-      }
-    
-    if (site_check_user_map(pjob, puser, EMsg) == -1)
+
+    if (site_check_user_map(pjob, puser, EMsg, LOGLEVEL) == -1)
       {
       return(PBSE_BADUSER);
       }
 
-    strncpy(tmpLine, puser, sizeof(tmpLine));
+    snprintf(tmpLine, sizeof(tmpLine), "%s", puser);
     }  /* END else (CheckID == 0) */
 
   pattr = attrry + JOB_ATR_euser;
 
   job_attr_def[JOB_ATR_euser].at_free(pattr);
 
-  job_attr_def[JOB_ATR_euser].at_decode(pattr, NULL, NULL, tmpLine);
+  job_attr_def[JOB_ATR_euser].at_decode(pattr, NULL, NULL, tmpLine, 0);
 
 #ifdef _CRAY
 
@@ -531,11 +529,17 @@ int set_jobexid(
         snprintf(EMsg, 1024, "user %s not located in user data base",
                  puser);
 
+      if (free_puser == TRUE)
+        free(puser);
+
       return(PBSE_BADUSER);
       }
 
     if (pudb->ue_permbits & (PERMBITS_NOBATCH | PERMBITS_RESTRICTED))
       {
+      if (free_puser == TRUE)
+        free(puser);
+
       return(PBSE_QACESS);
       }
 
@@ -575,6 +579,8 @@ int set_jobexid(
 
   if (pgrpn == NULL)
     {
+    free_pgrpn = FALSE;
+
     if ((pwent != NULL) || ((pwent = getpwnam_ext(puser)) != NULL))
       {
       /* egroup not specified - use user login group */
@@ -606,6 +612,9 @@ int set_jobexid(
       if (EMsg != NULL)
         snprintf(EMsg, 1024, "user does not exist in server password file");
 
+      if (free_puser == TRUE)
+        free(puser);
+
       return(PBSE_BADUSER);
       }
 
@@ -644,6 +653,11 @@ int set_jobexid(
           snprintf(EMsg, 1024, "cannot locate group %s in server group file",
                    pgrpn);
 
+      if (free_puser == TRUE)
+        free(puser);
+
+      free(pgrpn);
+
       return(PBSE_BADGRP);  /* no such group */
       }
 
@@ -664,15 +678,20 @@ int set_jobexid(
       if (*pmem == NULL)
         {
         /* requested group not allowed */
-        snprintf(log_buffer,sizeof(log_buffer),
+        snprintf(log_buf,sizeof(log_buf),
           "user %s is not a member of group %s in server password file",
           puser,
           pgrpn);
 
-        log_err(-1,id,log_buffer);
+        log_err(-1,id,log_buf);
 
         if (EMsg != NULL)
-          snprintf(EMsg, 1024, "%s",log_buffer);
+          snprintf(EMsg, 1024, "%s",log_buf);
+
+        if (free_puser == TRUE)
+          free(puser);
+
+        free(pgrpn);
 
         return(PBSE_BADGRP); /* user not in group */
         }
@@ -685,11 +704,16 @@ int set_jobexid(
 
   job_attr_def[JOB_ATR_egroup].at_free(pattr);
 
-  job_attr_def[JOB_ATR_egroup].at_decode(pattr, NULL, NULL, pgrpn);
+  job_attr_def[JOB_ATR_egroup].at_decode(pattr, NULL, NULL, pgrpn, 0);
 
   pattr->at_flags |= addflags;
 
   /* SUCCESS */
+  if (free_puser == TRUE)
+    free(puser);
+
+  if (free_pgrpn == TRUE)
+    free(pgrpn);
 
   return(0);
   }  /* END set_jobexid() */
