@@ -86,6 +86,7 @@
  * the API routines in libpbs.a
  */
 #include <pbs_config.h>   /* the master config generated by configure */
+#include "issue_request.h"
 
 #include <stdio.h>
 #include <errno.h>
@@ -99,20 +100,24 @@
 #include "credential.h"
 #include "batch_request.h"
 #include "log.h"
+#include "../lib/Liblog/pbs_log.h"
+#include "../lib/Liblog/log_event.h"
 #include "work_task.h"
 #include "net_connect.h"
 #include "svrfunc.h"
 #include "pbs_nodes.h"
-
-
+#include "pbs_job.h"
+#include "server.h"
+#include <stdint.h>
+#include "../lib/Libutils/u_lock_ctl.h" /* lock_node, unlock_node */
 
 /* Global Data Items: */
 
+extern struct server server;
 extern char *server_name;
 extern struct connect_handle connection[];
 extern char     *msg_daemonname;
-extern tlist_head task_list_event;
-extern time_t time_now;
+extern all_tasks task_list_event;
 extern char *msg_daemonname;
 extern char *msg_issuebad;
 extern char     *msg_norelytomom;
@@ -121,7 +126,7 @@ extern unsigned int pbs_mom_port;
 extern unsigned int pbs_server_port_dis;
 
 extern struct  connection svr_conn[];
-extern struct pbsnode *tfind_addr(const u_long); 
+extern struct pbsnode *tfind_addr(const u_long, uint16_t,job *); 
 
 extern int       LOGLEVEL;
 
@@ -141,60 +146,77 @@ int issue_to_svr(char *, struct batch_request *, void (*f)(struct work_task *));
 
 int relay_to_mom(
 
-  pbs_net_t          momaddr, /* address of mom */
-  struct batch_request  *request, /* the request to send */
-  void (*func)(struct work_task *))
+  job                   **pjob_ptr,
+  struct batch_request   *request, /* the request to send */
+  void                  (*func)(struct work_task *))
 
   {
-  char *id = "relay_to_mom";
-
-  int conn; /* a client style connection handle */
-  int   rc;
+  int             conn; /* a client style connection handle */
+  int             rc;
+  int             local_errno = 0;
+  pbs_net_t       addr;
+  unsigned short  port;
+  job            *pjob = *pjob_ptr;
+  char            jobid[PBS_MAXSVRJOBID + 1];
 
   struct pbsnode *node;
+  char            log_buf[LOCAL_LOG_BUF_SIZE];
 
   /* if MOM is down don't try to connect */
+  addr = pjob->ji_qs.ji_un.ji_exect.ji_momaddr;
+  port = pjob->ji_qs.ji_un.ji_exect.ji_momport;
 
-  if (((node = tfind_addr(momaddr)) != NULL) &&
-       (node->nd_state & (INUSE_DELETED|INUSE_DOWN)))
+  if ((node = tfind_addr(addr, port, pjob)) == NULL)
     {
+    free_br(request);
+    return(PBSE_NORELYMOM);
+    }
+
+  if ((node != NULL) &&
+      (node->nd_state & INUSE_DOWN))
+    {
+    unlock_node(node, __func__, "no rely mom", LOGLEVEL);
+    free_br(request);
     return(PBSE_NORELYMOM);
     }
 
   if (LOGLEVEL >= 7)
     {
-    sprintf(log_buffer, "momaddr=%s",
-            netaddr_pbs_net_t(momaddr));
+    char *tmp = netaddr_pbs_net_t(pjob->ji_qs.ji_un.ji_exect.ji_momaddr);
+    sprintf(log_buf, "momaddr=%s",tmp);
 
-    log_record(
-      PBSEVENT_SCHED,
-      PBS_EVENTCLASS_REQUEST,
-      id,
-      log_buffer);
+    log_record(PBSEVENT_SCHED, PBS_EVENTCLASS_REQUEST, __func__, log_buf);
+
+    free(tmp);
     }
 
   conn = svr_connect(
-
-           momaddr,
-           pbs_mom_port,
+           pjob->ji_qs.ji_un.ji_exect.ji_momaddr,
+           pjob->ji_qs.ji_un.ji_exect.ji_momport,
+           &local_errno,
+           node,
            process_Dreply,
            ToServerDIS);
+    
+  unlock_node(node, __func__, "after svr_connect", LOGLEVEL);
 
   if (conn < 0)
     {
-    LOG_EVENT(
-      PBSEVENT_ERROR,
-      PBS_EVENTCLASS_REQUEST,
-      "",
-      msg_norelytomom);
+    log_event(PBSEVENT_ERROR,PBS_EVENTCLASS_REQUEST,"",msg_norelytomom);
 
+    free_br(request);
     return(PBSE_NORELYMOM);
     }
 
+  strcpy(jobid, pjob->ji_qs.ji_jobid);
+  pthread_mutex_unlock(pjob->ji_mutex);
+
   request->rq_orgconn = request->rq_conn; /* save client socket */
 
   rc = issue_Drequest(conn, request, func, NULL);
 
+  *pjob_ptr = find_job(jobid);
+
   return(rc);
   }  /* END relay_to_mom() */
 
@@ -212,13 +234,13 @@ static void reissue_to_svr(
   struct work_task *pwt)
 
   {
-
+  time_t                time_now = time(NULL);
   struct batch_request *preq = pwt->wt_parm1;
 
   /* if not timed-out, retry send to remote server */
 
   if (((time_now - preq->rq_time) > PBS_NET_RETRY_LIMIT) ||
-      (issue_to_svr(preq->rq_host, preq, pwt->wt_parmfunc) == -1))
+      (issue_to_svr(preq->rq_host, preq, pwt->wt_parmfunc) != PBSE_NONE))
     {
     /* either timed-out or got hard error, tell post-function  */
 
@@ -228,8 +250,9 @@ static void reissue_to_svr(
     ((void (*)())pwt->wt_parmfunc)(pwt);
     }
 
-  return;
-  }  /* END resissue_to_svr() */
+  free(pwt->wt_mutex);
+  free(pwt);
+  }  /* END reissue_to_svr() */
 
 
 
@@ -252,25 +275,30 @@ int issue_to_svr(
   void (*replyfunc)    (struct work_task *))      /* I */
 
   {
-  int   do_retry = 0;
-  int   handle;
-  pbs_net_t svraddr;
-  char  *svrname;
-  unsigned int  port = pbs_server_port_dis;
+  int               rc = PBSE_NONE;
+  int               do_retry = 0;
+  int               handle;
+  int               my_err = 0;
+  pbs_net_t         svraddr;
+  char             *svrname;
+  unsigned int      port = pbs_server_port_dis;
 
   struct work_task *pwt;
+  time_t            time_now = time(NULL);
 
-  strcpy(preq->rq_host, servern);
+  snprintf(preq->rq_host, sizeof(preq->rq_host), "%s", servern);
 
   preq->rq_fromsvr = 1;
   preq->rq_perm = ATR_DFLAG_MGRD | ATR_DFLAG_MGWR | ATR_DFLAG_SvWR;
 
   svrname = parse_servername(servern, &port); 
-  svraddr = get_hostaddr(svrname);
+  svraddr = get_hostaddr(&my_err,svrname);
+
+  free(svrname);
 
   if (svraddr == (pbs_net_t)0)
     {
-    if (pbs_errno == PBS_NET_RC_RETRY)
+    if (my_err == PBS_NET_RC_RETRY)
       {
       /* Non fatal error - retry */
 
@@ -279,11 +307,15 @@ int issue_to_svr(
     }
   else
     {
-    handle = svr_connect(svraddr, port, process_Dreply, ToServerDIS);
+    handle = svr_connect(svraddr, port, &my_err, NULL, process_Dreply, ToServerDIS);
 
     if (handle >= 0)
       {
-      return(issue_Drequest(handle, preq, replyfunc, NULL));
+      if ((rc = issue_Drequest(handle, preq, replyfunc, NULL)) == PBSE_NONE)
+        {
+        rc = preq->rq_reply.brp_code;
+        }
+      return rc;
       }
     else if (handle == PBS_NET_RC_RETRY)
       {
@@ -295,20 +327,18 @@ int issue_to_svr(
 
   if (do_retry)
     {
-    pwt = set_task(
-            WORK_Timed,
-            (long)(time_now + PBS_NET_RETRY_TIME),
-            reissue_to_svr,
-            (void *)preq);
+    pwt = set_task(WORK_Timed,(long)(time_now + PBS_NET_RETRY_TIME),reissue_to_svr,preq,TRUE);
 
     pwt->wt_parmfunc = replyfunc;
 
-    return(0);
+    pthread_mutex_unlock(pwt->wt_mutex);
+
+    return PBSE_NONE;
     }
 
   /* FAILURE */
 
-  return(-1);
+  return PBSE_INTERNAL;
   }  /* END issue_to_svr() */
 
 
@@ -336,8 +366,9 @@ void release_req(
   if (pwt->wt_event != -1)
     svr_disconnect(pwt->wt_event);
 
-  return;
-  }
+  free(pwt->wt_mutex);
+  free(pwt);
+  } /* END release_req() */
 
 
 
@@ -373,22 +404,22 @@ void release_req(
 
 int issue_Drequest(
 
-  int          conn,
-  struct batch_request *request,
-  void (*func) (struct work_task *),
-  struct work_task    **ppwt)
+  int                    conn,
+  struct batch_request  *request,
+  void                 (*func) (struct work_task *),
+  struct work_task     **ppwt)
 
   {
-
   struct attropl   *patrl;
 
   struct work_task *ptask;
 
   struct svrattrl  *psvratl;
-  int    rc;
-  int    sock = 0;
-  enum work_type  wt;
-  char   *id = "issue_Drequest";
+  int               rc;
+  int               sock = 0;
+  enum work_type    wt;
+  char             *id = "issue_Drequest";
+  char              log_buf[LOCAL_LOG_BUF_SIZE];
 
   if (conn == PBS_LOCAL_CONNECTION)
     {
@@ -398,8 +429,12 @@ int issue_Drequest(
     }
   else
     {
+    pthread_mutex_lock(connection[conn].ch_mutex);
+
     sock = connection[conn].ch_socket;
 
+    pthread_mutex_unlock(connection[conn].ch_mutex);
+
     request->rq_conn = sock;
 
     wt = WORK_Deferred_Reply;
@@ -407,15 +442,15 @@ int issue_Drequest(
     DIS_tcp_setup(sock);
     }
 
-  ptask = set_task(wt, (long)conn, func, (void *)request);
+  ptask = set_task(wt, (long)conn, func, (void *)request, FALSE);
+
+  if (ppwt != NULL)
+    *ppwt = ptask;
 
   if (ptask == NULL)
     {
     log_err(errno, id, "could not set_task");
 
-    if (ppwt != NULL)
-      *ppwt = 0;
-
     return(-1);
     }
 
@@ -425,9 +460,6 @@ int issue_Drequest(
 
     dispatch_request(PBS_LOCAL_CONNECTION, request);
 
-    if (ppwt != NULL)
-      *ppwt = ptask;
-
     return(0);
     }
 
@@ -595,6 +627,7 @@ int issue_Drequest(
       break;
 
     case PBS_BATCH_ReturnFiles:
+
       if ((rc = encode_DIS_ReqHdr(sock, PBS_BATCH_ReturnFiles, msg_daemonname)))
         break;
 
@@ -661,21 +694,22 @@ int issue_Drequest(
 
     default:
 
-      sprintf(log_buffer, msg_issuebad,
+      sprintf(log_buf, msg_issuebad,
               request->rq_type);
 
-      log_err(-1, id, log_buffer);
+      log_err(-1, id, log_buf);
 
-      delete_task(ptask);
+      pthread_mutex_lock(ptask->wt_mutex);
+      if (ptask->wt_being_recycled == FALSE)
+        delete_task(ptask);
+      else
+        pthread_mutex_unlock(ptask->wt_mutex);
 
       rc = -1;
 
       break;
     }  /* END switch (request->rq_type) */
 
-  if (ppwt != NULL)
-    *ppwt = ptask;
-
   return(rc);
   }  /* END issue_Drequest() */
 
@@ -686,42 +720,48 @@ int issue_Drequest(
 /*
  * process_reply - process the reply received for a request issued to
  * another server via issue_request()
+ *
+ * should be called when the relevant svr_conn mutex is already held
  */
 
-void process_Dreply(
+void *process_Dreply(
 
-  int sock)
+  void *new_sock)
 
   {
+  static char *id = "process_Dreply";
   int    handle;
 
   struct work_task *ptask;
   int    rc;
+  int    iter = -1;
 
   struct batch_request *request;
+  int sock = *(int *)new_sock;
+
+  pthread_mutex_lock(svr_conn[sock].cn_mutex);
 
   /* find the work task for the socket, it will point us to the request */
-
-  ptask = (struct work_task *)GET_NEXT(task_list_event);
-
   handle = svr_conn[sock].cn_handle;
 
-  while (ptask != NULL)
+  pthread_mutex_unlock(svr_conn[sock].cn_mutex);
+
+  while ((ptask = next_task(&task_list_event,&iter)) != NULL)
     {
     if ((ptask->wt_type == WORK_Deferred_Reply) &&
         (ptask->wt_event == handle))
       break;
 
-    ptask = (struct work_task *)GET_NEXT(ptask->wt_linkall);
+    pthread_mutex_unlock(ptask->wt_mutex);
     }
 
   if (ptask == NULL)
     {
-    log_err(-1, "process_Dreply", msg_err_malloc);
+    log_err(-1, id, "Unable to find work task for connection");
 
-    close_conn(sock);
+    close_conn(sock, FALSE);
 
-    return;
+    return(NULL);
     }
 
   request = ptask->wt_parm1;
@@ -730,17 +770,16 @@ void process_Dreply(
 
   if ((rc = DIS_reply_read(sock, &request->rq_reply)) != 0)
     {
-    close_conn(sock);
+    close_conn(sock, FALSE);
 
     request->rq_reply.brp_code = rc;
     request->rq_reply.brp_choice = BATCH_REPLY_CHOICE_NULL;
     }
 
   /* now dispatch the reply to the routine in the work task */
-
   dispatch_task(ptask);
 
-  return;
+  return(NULL);
   }  /* END process_Dreply() */
 
 /* END issue_request.c */
