@@ -291,7 +291,7 @@ attribute_def job_attr_def[] =
     comp_str,
     free_str,
     NULL_FUNC,
-    READ_ONLY | ATR_DFLAG_MOM | ATR_DFLAG_OPWR | ATR_DFLAG_SvWR,  /* allow operator/scheduler to modify exec_host */
+    READ_ONLY | ATR_DFLAG_MOM | ATR_DFLAG_OPWR | ATR_DFLAG_SvWR,  /* allow operator/scheduler to modify exec_port */
     ATR_TYPE_STR,
     PARENT_TYPE_JOB
   },
@@ -946,6 +946,45 @@ attribute_def job_attr_def[] =
     PARENT_TYPE_JOB
   },
 
+  /* JOB_ATR_job_radix */
+  { ATTR_job_radix,  /* "job_radix" */
+    decode_l,
+    encode_l,
+    set_l,
+    comp_l,
+    free_null,
+    job_radix_action,
+    READ_WRITE | ATR_DFLAG_SSET | ATR_DFLAG_MOM,
+    ATR_TYPE_LONG,
+    PARENT_TYPE_JOB
+  },
+
+  /* JOB_ATR_total_runtime */
+  { ATTR_total_runtime, /* total time from start_time to comp_time in milliseconds */
+    decode_tv,
+    encode_tv,
+    set_tv,
+    comp_tv,
+    free_null,
+    NULL_FUNC,
+    READ_ONLY | ATR_DFLAG_SSET,
+    ATR_TYPE_TV,
+    PARENT_TYPE_JOB
+  },
+
+  /* JOB_ATR_sister_list */
+  { ATTR_sister_list, /* sister_list */
+    decode_str,
+    encode_str,
+    set_str,
+    comp_str,
+    free_str,
+    NULL_FUNC,
+    READ_ONLY | ATR_DFLAG_MOM | ATR_DFLAG_OPWR | ATR_DFLAG_SvWR,  /* allow operator/scheduler to modify exec_host */
+    ATR_TYPE_STR,
+    PARENT_TYPE_JOB
+  },
+
   /* JOB_ATR_proxy_user */
   { ATTR_P, /* "proxy_user" */
     decode_str,
@@ -972,6 +1011,30 @@ attribute_def job_attr_def[] =
     PARENT_TYPE_JOB
   },
 
+  /* JOB_ATR_submit_host */
+  {ATTR_submit_host,		/* "submit_host - undocumented */
+   decode_str,
+   encode_str,
+   set_str,
+   comp_str,
+   free_str,
+   NULL_FUNC,
+   READ_ONLY | ATR_DFLAG_SvRD | ATR_DFLAG_Creat,
+   ATR_TYPE_STR,
+   PARENT_TYPE_JOB},
+
+  /* JOB_ATR_init_work_dir */
+  {ATTR_init_work_dir,		/* "init_work_dir - undocumented */
+   decode_str,
+   encode_str,
+   set_str,
+   comp_str,
+   free_str,
+   NULL_FUNC,
+   READ_ONLY | ATR_DFLAG_SvRD | ATR_DFLAG_Creat,
+   ATR_TYPE_STR,
+   PARENT_TYPE_JOB},
+
 #ifdef USEJOBCREATE
   /* JOB_ATR_pagg_id */
   { ATTR_pagg,  /* "pagg_id" */
@@ -987,6 +1050,44 @@ attribute_def job_attr_def[] =
   },
 #endif /* USEJOBCREATE */
 
+#ifdef NVIDIA_GPUS
+  /* JOB_ATR_gpu_flags */
+  {ATTR_gpu_flags,		/* "gpu_flags" - mode and reset flags */
+   decode_l,
+   encode_l,
+   set_l,
+   comp_l,
+   free_null,
+   NULL_FUNC,
+   READ_ONLY | ATR_DFLAG_MOM,
+   ATR_TYPE_LONG,
+   PARENT_TYPE_JOB},
+#endif  /* NVIDIA_GPUS */
+
+  /* JOB_ATR_job_id */
+  {ATTR_J,      /* job_id */
+   decode_str,
+   encode_str,
+   set_str,
+   comp_str,
+   free_str,
+   NULL_FUNC,
+   ATR_DFLAG_Creat | ATR_DFLAG_MGRD | ATR_DFLAG_USRD | ATR_DFLAG_OPRD,
+   ATR_TYPE_STR,
+   PARENT_TYPE_JOB},
+
+  /* JOB_ATR_arguments*/
+  {ATTR_args,		/* job_arguments */
+   decode_str,
+   encode_str,
+   set_str,
+   comp_str,
+   free_str,
+   NULL_FUNC,
+   READ_WRITE | ATR_DFLAG_SELEQ | ATR_DFLAG_MOM,
+   ATR_TYPE_STR,
+   PARENT_TYPE_JOB},
+
   /* Site defined attributes if any, see site_job_attr_*.h  */
 #include "site_job_attr_def.h"
 
