@@ -86,7 +86,8 @@
 
 #include <pbs_config.h>   /* the master config generated by configure */
 
-#include <sys/types.h>
+#include <pthread.h>
+
 #include <sys/param.h>
 #include "pbs_ifl.h"
 #include <errno.h>
@@ -101,21 +102,23 @@
 #include "work_task.h"
 #include "server.h"
 #include "log.h"
+#include "../lib/Liblog/pbs_log.h"
+#include "../lib/Liblog/log_event.h"
 #include "queue.h"
 #include "pbs_job.h"
 #include "credential.h"
 #include "batch_request.h"
-#include "resource.h"
-#include "sched_cmds.h"
+#include "threadpool.h"
+#include "resource.h" /* struct resource */
 #if __STDC__ != 1
 #include <memory.h>
 #endif
 
+#define ROUTE_RETRY_TIME 30
+
 /* External functions called */
-extern int svr_movejob(job *, char *, struct batch_request *);
-extern long count_proc(char *);
-extern int svr_do_schedule;
-extern int listener_command;
+int svr_movejob(job *, char *, int *, struct batch_request *, int);
+long count_proc(char *spec);
 
 /* Local Functions */
 
@@ -125,8 +128,6 @@ void queue_route(pbs_queue *);
 /* Global Data */
 extern char *msg_routexceed;
 extern char *msg_err_malloc;
-extern char *msg_routexceed;
-extern time_t  time_now;
 
 /*
  * Add an entry to the list of bad destinations for a job.
@@ -139,15 +140,14 @@ void add_dest(
   job *jobp)
 
   {
-  char      *id = "add_dest";
   badplace  *bp;
   char      *baddest = jobp->ji_qs.ji_destin;
 
-  bp = (badplace *)malloc(sizeof(badplace));
+  bp = (badplace *)calloc(1, sizeof(badplace));
 
   if (bp == NULL)
     {
-    log_err(errno, id, msg_err_malloc);
+    log_err(errno, __func__, msg_err_malloc);
 
     return;
     }
@@ -203,9 +203,11 @@ badplace *is_bad_dest(
  *  
  * Returns 0 on success. Non-zero on failure
  */ 
-int initialize_procct(job *pjob)
+int initialize_procct(
+    
+  job *pjob)
+
   { 
-  char id[] = "initialize_procct";
   resource     *pnodesp = NULL;
   resource_def *pnodes_def = NULL;
   resource     *pprocsp = NULL;
@@ -213,41 +215,34 @@ int initialize_procct(job *pjob)
   resource     *procctp = NULL;
   resource_def *procct_def = NULL;
   attribute    *pattr = NULL;
+  char          log_buf[LOCAL_LOG_BUF_SIZE];
 
   pattr = &pjob->ji_wattr[JOB_ATR_resource];
-  if(pattr == NULL)
+  if (pattr == NULL)
     {
     /* Something is really wrong. ji_wattr[JOB_ATR_resource] should always be set
        by the time this function is called */
-    sprintf(log_buffer, "%s: Resource_List is NULL. Cannot proceed", id);
-    log_event(PBSEVENT_JOB,
-              PBS_EVENTCLASS_JOB,
-              pjob->ji_qs.ji_jobid,
-              log_buffer);
-    pbs_errno = PBSE_INTERNAL;
+    sprintf(log_buf, "%s: Resource_List is NULL. Cannot proceed", __func__);
+    log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buf);
     return(ROUTE_PERM_FAILURE);
     }
 
   /* Has nodes been initialzed */
-  if(pattr->at_flags & ATR_VFLAG_SET)
+  if (pattr->at_flags & ATR_VFLAG_SET)
     {
     /* get the node spec from the nodes resource */
     pnodes_def = find_resc_def(svr_resc_def, "nodes", svr_resc_size);
-    if(pnodes_def == NULL)
+    if (pnodes_def == NULL)
       {
-      sprintf(log_buffer, "%s: Could not get nodes resource definition. Cannot proceed", id);
-      log_event(PBSEVENT_JOB,
-                PBS_EVENTCLASS_JOB,
-                pjob->ji_qs.ji_jobid,
-                log_buffer);
-      pbs_errno = PBSE_INTERNAL;
+      sprintf(log_buf, "%s: Could not get nodes resource definition. Cannot proceed", __func__);
+      log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buf);
       return(ROUTE_PERM_FAILURE);
       }
     pnodesp = find_resc_entry(pattr, pnodes_def);
 
     /* Get the procs count if the procs resource attribute is set */
     pprocs_def = find_resc_def(svr_resc_def, "procs", svr_resc_size);
-    if(pprocs_def != NULL)
+    if (pprocs_def != NULL)
       {
       /* if pprocs_def is NULL we just go on. Otherwise we will get its value now */
       pprocsp = find_resc_entry(pattr, pprocs_def);
@@ -255,57 +250,41 @@ int initialize_procct(job *pjob)
       }
 
     /* if neither pnodesp nor pprocsp are set, terminate */
-    if(pnodesp == NULL && pprocsp == NULL)
+    if (pnodesp == NULL && pprocsp == NULL)
       {
       /* nodes and procs were not set. Hopefully req_quejob set procct to 1 for us already */
       procct_def = find_resc_def(svr_resc_def, "procct", svr_resc_size);
-      if(procct_def == NULL)
+      if (procct_def == NULL)
         {
-        sprintf(log_buffer, "%s: Could not get procct resource definition. Cannot proceed", id);
-        log_event(PBSEVENT_JOB,
-                PBS_EVENTCLASS_JOB,
-                pjob->ji_qs.ji_jobid,
-                log_buffer);
-        pbs_errno = PBSE_INTERNAL;
+        sprintf(log_buf, "%s: Could not get procct resource definition. Cannot proceed", __func__);
+        log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buf);
         return(ROUTE_PERM_FAILURE);
         }
       procctp = find_resc_entry(pattr, procct_def);
-      if(procctp == NULL)
+      if (procctp == NULL)
         {
-        sprintf(log_buffer, "%s: Could not get nodes nor procs entry from Resource_List. Cannot proceed", id);
-        log_event(PBSEVENT_JOB,
-                PBS_EVENTCLASS_JOB,
-                pjob->ji_qs.ji_jobid,
-                log_buffer);
-        pbs_errno = PBSE_INTERNAL;
+        sprintf(log_buf, "%s: Could not get nodes nor procs entry from Resource_List. Cannot proceed", __func__);
+        log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buf);
         return(ROUTE_PERM_FAILURE);
         }
       }
 
     /* we now set pjob->procct and we also set the resource attribute procct */
     procct_def = find_resc_def(svr_resc_def, "procct", svr_resc_size);
-    if(procct_def == NULL)
+    if (procct_def == NULL)
       {
-      sprintf(log_buffer, "%s: Could not get procct resource definition. Cannot proceed", id);
-      log_event(PBSEVENT_JOB,
-                PBS_EVENTCLASS_JOB,
-                pjob->ji_qs.ji_jobid,
-                log_buffer);
-      pbs_errno = PBSE_INTERNAL;
+      sprintf(log_buf, "%s: Could not get procct resource definition. Cannot proceed", __func__);
+      log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buf);
       return(ROUTE_PERM_FAILURE);
       }
     procctp = find_resc_entry(pattr, procct_def);
-    if(procctp == NULL)
+    if (procctp == NULL)
       {
       procctp = add_resource_entry(pattr, procct_def);
-      if(procctp == NULL)
+      if (procctp == NULL)
         {
-        sprintf(log_buffer, "%s: Could not add procct resource. Cannot proceed", id);
-        log_event(PBSEVENT_JOB,
-                  PBS_EVENTCLASS_JOB,
-                  pjob->ji_qs.ji_jobid,
-                  log_buffer);
-        pbs_errno = PBSE_INTERNAL;
+        sprintf(log_buf, "%s: Could not add procct resource. Cannot proceed", __func__);
+        log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buf);
         return(ROUTE_PERM_FAILURE);
         }
       }
@@ -313,12 +292,12 @@ int initialize_procct(job *pjob)
     /* Finally the moment of truth. We have the nodes and procs resources. Add them
        to the procct resoruce*/
     procctp->rs_value.at_val.at_long = 0;
-    if(pnodesp != NULL)
+    if (pnodesp != NULL)
       {
       procctp->rs_value.at_val.at_long = count_proc(pnodesp->rs_value.at_val.at_str);
       }
 
-    if(pprocsp != NULL)
+    if (pprocsp != NULL)
       {
       procctp->rs_value.at_val.at_long += pprocsp->rs_value.at_val.at_long;
       }
@@ -328,12 +307,8 @@ int initialize_procct(job *pjob)
     {
     /* Something is really wrong. ji_wattr[JOB_ATR_resource] should always be set
        by the time this function is called */
-    sprintf(log_buffer, "%s: Resource_List not set. Cannot proceed", id);
-    log_event(PBSEVENT_JOB,
-              PBS_EVENTCLASS_JOB,
-              pjob->ji_qs.ji_jobid,
-              log_buffer);
-    pbs_errno = PBSE_INTERNAL;
+    sprintf(log_buf, "%s: Resource_List not set. Cannot proceed", __func__);
+    log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buf);
     return(ROUTE_PERM_FAILURE);
     }
 
@@ -341,24 +316,23 @@ int initialize_procct(job *pjob)
   } /* END initialize_procct */
 
 
-int remove_procct(job *pjob)
+int remove_procct(
+
+  job *pjob)
+
   {
-  char id[] = "remove_procct";
   attribute    *pattr;
   resource_def *pctdef;
   resource     *pctresc;
+  char          log_buf[LOCAL_LOG_BUF_SIZE];
 
   pattr = &pjob->ji_wattr[JOB_ATR_resource];
-  if(pattr == NULL)
+  if (pattr == NULL)
     {
     /* Something is really wrong. ji_wattr[JOB_ATR_resource] should always be set
        by the time this function is called */
-    sprintf(log_buffer, "%s: Resource_List is NULL. Cannot proceed", id);
-    log_event(PBSEVENT_JOB,
-              PBS_EVENTCLASS_JOB,
-              pjob->ji_qs.ji_jobid,
-              log_buffer);
-    pbs_errno = PBSE_INTERNAL;
+    sprintf(log_buf, "%s: Resource_List is NULL. Cannot proceed", __func__);
+    log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buf);
     return(ROUTE_PERM_FAILURE);
     }
 
@@ -371,6 +345,9 @@ int remove_procct(job *pjob)
   return(PBSE_NONE);
   } /* END remove_procct */
 
+
+
+
 /*
  * default_router - basic function for "routing" jobs.
  * Does a round-robin attempt on the destinations as listed,
@@ -387,15 +364,15 @@ int default_router(
   long              retry_time)
 
   {
-
   struct array_strings *dest_attr = NULL;
-  char       *destination;
-  int        last;
-  int        rc;
+  char                 *destination;
+  int                   last;
+  int                   local_errno = 0;
+  int                   rc = 0;
 
-  if (qp->qu_attr[(int)QR_ATR_RouteDestin].at_flags & ATR_VFLAG_SET)
+  if (qp->qu_attr[QR_ATR_RouteDestin].at_flags & ATR_VFLAG_SET)
     {
-    dest_attr = qp->qu_attr[(int)QR_ATR_RouteDestin].at_val.at_arst;
+    dest_attr = qp->qu_attr[QR_ATR_RouteDestin].at_val.at_arst;
 
     last = dest_attr->as_usedptr;
     }
@@ -438,7 +415,7 @@ int default_router(
 
     if (is_bad_dest(jobp, destination))
       continue;
-  
+
     /* We need to manage the procct resource which is 
        part of the Resource_List attribute. At this point 
        we need to remember what the procct value is and also
@@ -451,16 +428,11 @@ int default_router(
       return(rc);
       }
 
-    switch (svr_movejob(jobp, destination, NULL))
+    switch (svr_movejob(jobp, destination, &local_errno, NULL, TRUE))
       {
 
       case ROUTE_PERM_FAILURE: /* permanent failure */
 
-        rc = remove_procct(jobp);
-        if(rc != PBSE_NONE)
-          {
-          return(rc);
-          }
         add_dest(jobp);
 
         break;
@@ -476,11 +448,11 @@ int default_router(
         break;
 
       case ROUTE_RETRY:  /* failed, but try destination again */
-			/* There are no available queues for this job so it is 
-			   going to stay in the routing queue. But we need to remove
+      /* There are no available queues for this job so it is 
+         going to stay in the routing queue. But we need to remove
          procct from the Resource_List so it does not get sent
-				 to the scheduler as part of the job. procct is for 
-				 TORQUE use only. */
+         to the scheduler as part of the job. procct is for 
+         TORQUE use only. */
         rc = remove_procct(jobp);
         if(rc != PBSE_NONE)
           {
@@ -490,6 +462,7 @@ int default_router(
         jobp->ji_retryok = 1;
 
         break;
+
       }
     }
 
@@ -497,9 +470,14 @@ int default_router(
   }  /* END default_router() */
 
 
+
+
+
 /*
  * job_route - route a job to another queue
  *
+ * NOTE: This MUST always called by a thread holding queue's mutex.
+ *
  * This is only called for jobs in a routing queue.
  * Loop over all the possible destinations for the route queue.
  * Check each one to see if it is ok to try it.  It could have been
@@ -514,19 +492,18 @@ int default_router(
 
 int job_route(
 
-  job *jobp) /* job to route */
+  job *jobp)      /* job to route */
 
   {
-  int      bad_state = 0;
-  int      rc;
-  char     *id = "job_route";
-  time_t     life;
+  int               bad_state = 0;
+  time_t            life;
+  time_t            time_now = time(NULL);
+  char              log_buf[LOCAL_LOG_BUF_SIZE];
 
-  struct pbs_queue *qp;
+  struct pbs_queue *qp = jobp->ji_qhdr;
   long              retry_time;
 
   /* see if the job is able to be routed */
-
   switch (jobp->ji_qs.ji_state)
     {
 
@@ -548,7 +525,7 @@ int job_route(
 
       /* job may be acceptable */
 
-      bad_state = !jobp->ji_qhdr->qu_attr[QR_ATR_RouteHeld].at_val.at_long;
+      bad_state = !qp->qu_attr[QR_ATR_RouteHeld].at_val.at_long;
 
       break;
 
@@ -556,24 +533,18 @@ int job_route(
 
       /* job may be acceptable */
 
-      bad_state = !jobp->ji_qhdr->qu_attr[QR_ATR_RouteWaiting].at_val.at_long;
+      bad_state = !qp->qu_attr[QR_ATR_RouteWaiting].at_val.at_long;
 
       break;
 
     default:
 
-      sprintf(log_buffer, "%s %d", pbse_to_txt(PBSE_BADSTATE),
-              jobp->ji_qs.ji_state);
+      snprintf(log_buf, sizeof(log_buf), "%s %d %s", 
+        pbse_to_txt(PBSE_BADSTATE), jobp->ji_qs.ji_state, __func__);
 
-      strcat(log_buffer, id);
-
-      log_event(
-        PBSEVENT_DEBUG,
-        PBS_EVENTCLASS_JOB,
-        jobp->ji_qs.ji_jobid,
-        log_buffer);
-
-      return(0);
+      log_event(PBSEVENT_DEBUG,PBS_EVENTCLASS_JOB,jobp->ji_qs.ji_jobid,log_buf);
+      
+      return(PBSE_NONE);
 
       /*NOTREACHED*/
 
@@ -581,9 +552,6 @@ int job_route(
     }
 
   /* check the queue limits, can we route any (more) */
-
-  qp = jobp->ji_qhdr;
-
   if (qp->qu_attr[QA_ATR_Started].at_val.at_long == 0)
     {
     /* queue not started - no routing */
@@ -625,14 +593,9 @@ int job_route(
 
   if (life && (life < time_now))
     {
-    log_event(
-      PBSEVENT_JOB,
-      PBS_EVENTCLASS_JOB,
-      jobp->ji_qs.ji_jobid,
-      msg_routexceed);
+    log_event(PBSEVENT_JOB,PBS_EVENTCLASS_JOB,jobp->ji_qs.ji_jobid,msg_routexceed);
 
     /* job too long in queue */
-
     return(PBSE_ROUTEEXPD);
     }
 
@@ -640,33 +603,61 @@ int job_route(
     {
     /* not currently routing this job */
 
-    return(0);
+    return(PBSE_NONE);
     }
 
   if (qp->qu_attr[QR_ATR_AltRouter].at_val.at_long == 0)
     {
-    rc = default_router(jobp, qp, retry_time);
-    if(rc == ROUTE_SUCCESS)
-      {
-      svr_do_schedule = SCH_SCHEDULE_NEW;
-      listener_command = SCH_SCHEDULE_NEW;
-      }
-    return(rc);
+    return(default_router(jobp, qp, retry_time));
     }
 
-  rc = site_alt_router(jobp, qp, retry_time);
-  if(rc == ROUTE_SUCCESS)
-    {
-    svr_do_schedule = SCH_SCHEDULE_NEW;
-    listener_command = SCH_SCHEDULE_NEW;
-    }
-
-  return(rc);
+  return(site_alt_router(jobp, qp, retry_time));
   }  /* END job_route() */
 
 
 
 
+void *reroute_job(
+
+  void *vp)
+
+  {
+  pbs_queue *pque;
+  char      *jobid;
+  job       *pjob;
+  int        rc;
+
+  jobid = (char *)vp;
+
+  if ((jobid != NULL) &&
+      ((pjob = find_job(jobid)) != NULL))
+    {
+    pque = get_jobs_queue(&pjob);
+
+    if ((pque != NULL) &&
+        (pque->qu_qs.qu_type == QTYPE_RoutePush))
+      {
+      rc = job_route(pjob);
+      
+      unlock_queue(pque, __func__, NULL, 0);
+
+      if (rc == PBSE_ROUTEREJ)
+        job_abt(&pjob, pbse_to_txt(PBSE_ROUTEREJ));
+      else if (rc == PBSE_ROUTEEXPD)
+        job_abt(&pjob, msg_routexceed);
+      }
+    else if (pque != NULL)
+      unlock_queue(pque, __func__, NULL, 0);
+
+    if (pjob != NULL)
+      pthread_mutex_unlock(pjob->ji_mutex);
+    }
+
+  return(NULL);      
+  } /* END reroute_job() */
+
+
+
 
 /*
  * queue_route - route any "ready" jobs in a specific queue
@@ -684,28 +675,26 @@ void queue_route(
   pbs_queue *pque)
 
   {
-  job *nxjb;
-  job *pjob;
-  int  rc;
+  job    *pjob = NULL;
 
-  pjob = (job *)GET_NEXT(pque->qu_jobs);
+  int     iter = -1;
+  time_t  time_now = time(NULL);
 
-  while (pjob != NULL)
+  while ((pjob = next_job(pque->qu_jobs,&iter)) != NULL)
     {
-    nxjb = (job *)GET_NEXT(pjob->ji_jobque);
-
-    if (pjob->ji_qs.ji_un.ji_routet.ji_rteretry <= time_now)
+    /* the second condition says we only want to try if routing
+     * has been tried once - this is to let req_commit have the 
+     * first crack at routing always */
+    if ((pjob->ji_qs.ji_un.ji_routet.ji_rteretry <= time_now - ROUTE_RETRY_TIME) &&
+        (pjob->ji_qs.ji_un.ji_routet.ji_rteretry != 0))
       {
-      if ((rc = job_route(pjob)) == PBSE_ROUTEREJ)
-        job_abt(&pjob, pbse_to_txt(PBSE_ROUTEREJ));
-      else if (rc == PBSE_ROUTEEXPD)
-        job_abt(&pjob, msg_routexceed);
+      enqueue_threadpool_request(reroute_job, strdup(pjob->ji_qs.ji_jobid));
       }
 
-    pjob = nxjb;
+    pthread_mutex_unlock(pjob->ji_mutex);
     }
 
   return;
-  }
+  } /* END queue_route() */
 
 
