@@ -82,13 +82,9 @@
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <sys/types.h>
 #include <sys/time.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
 #include <memory.h>
 #include <time.h>
-#include <pwd.h>
 #include <sys/param.h>
 #if HAVE_SYS_UCRED_H
 #include <sys/ucred.h>
@@ -96,6 +92,7 @@
 #if HAVE_SYS_UIO_H
 #include <sys/uio.h>
 #endif
+#include <pthread.h>
 
 #include "libpbs.h"
 #include "pbs_error.h"
@@ -109,14 +106,19 @@
 #include "batch_request.h"
 #include "net_connect.h"
 #include "log.h"
+#include "../lib/Liblog/pbs_log.h"
+#include "../lib/Liblog/log_event.h"
 #include "svrfunc.h"
 #include "pbs_proto.h"
 #include "csv.h"
 #include "u_tree.h"
-
-#ifndef PBS_MOM
+#include "threadpool.h"
+#include "dis.h"
 #include "array.h"
-#endif
+#include "../lib/Libutils/u_lock_ctl.h" /* lock_node, unlock_node */
+#include "svr_func.h" /* get_svr_attr_* */
+#include "req_gpuctrl.h" /* req_gpuctrl_svr */
+
 
 /*
  * process_request - this function gets, checks, and invokes the proper
@@ -132,7 +134,7 @@
 
 /* global data items */
 
-tlist_head svr_requests;
+pthread_mutex_t *svr_requests_mutex = NULL;
 
 extern struct    connection svr_conn[];
 
@@ -140,18 +142,16 @@ extern struct    credential conn_credent[PBS_NET_MAX_CONNECTIONS];
 
 extern struct server server;
 extern char      server_host[];
-extern tlist_head svr_newjobs;
-extern time_t    time_now;
-extern char  *msg_err_noqueue;
-extern char  *msg_err_malloc;
-extern char  *msg_request;
-#ifndef PBS_MOM
-extern char            server_name[];
-#endif
+struct all_jobs newjobs;
 
-extern AvlTree okclients;
+extern char     *msg_err_noqueue;
+extern char     *msg_err_malloc;
+extern char     *msg_request;
+extern char      server_name[];
 
-extern int LOGLEVEL;
+extern AvlTree   okclients;
+
+extern int       LOGLEVEL;
 
 /* private functions local to this file */
 
@@ -161,62 +161,54 @@ static const int munge_on = 1;
 static const int munge_on = 0;
 #endif 
 
-static void close_client(int sfds);
 static void freebr_manage(struct rq_manage *);
 static void freebr_cpyfile(struct rq_cpyfile *);
 static void close_quejob(int sfds);
-#ifndef PBS_MOM
 static void free_rescrq(struct rq_rescq *);
-#endif /* PBS_MOM */
 
 /* END private prototypes */
 
-#ifndef PBS_MOM
 
 extern struct pbsnode *PGetNodeFromAddr(pbs_net_t);
-#endif
 
 /* request processing prototypes */
-void req_quejob(struct batch_request *preq);
-void req_jobcredential(struct batch_request *preq);
-void req_jobscript(struct batch_request *preq);
-void req_rdytocommit(struct batch_request *preq);
-void req_commit(struct batch_request *preq);
-void req_deletejob(struct batch_request *preq);
-void req_holdjob(struct batch_request *preq);
-void req_checkpointjob(struct batch_request *preq);
-void req_messagejob(struct batch_request *preq);
-void req_modifyjob(struct batch_request *preq);
-void req_gpuctrl(struct batch_request *preq);
-#ifndef PBS_MOM
-void req_orderjob(struct batch_request *preq);
-void req_rescreserve(struct batch_request *preq);
-void req_rescfree(struct batch_request *preq);
-#endif
-#ifdef PBS_MOM
-void req_rerunjob(struct batch_request *preq);
-#endif
-void req_shutdown(struct batch_request *preq);
-void req_signaljob(struct batch_request *preq);
-void req_mvjobfile(struct batch_request *preq);
-#ifndef PBS_MOM
-void req_stat_node(struct batch_request *preq);
-void req_track(struct batch_request *preq);
-void req_jobobit(struct batch_request *preq);
-void req_stagein(struct batch_request *preq);
+void *req_jobcredential(void *preq);
+void *req_quejob(void *preq);
+void *req_rdytocommit(void *preq);
+void *req_jobscript(void *preq);
+void *req_commit(void *preq);
+void *req_holdjob(void *preq);
+void *req_holdarray(void *preq);
+void *req_stat_node(void *vp);
+void *req_jobobit(void *preq);
+void *req_stagein(void *preq);
+void *req_deletearray(void *preq);
+void *req_deletejob(void *preq);
+void *req_orderjob(void *preq);
+void *req_modifyarray(void *preq);
+void *req_modifyjob(void *preq);
 
-void req_deletearray(struct batch_request *preq);
-void req_holdarray(struct batch_request *preq);
-void req_modifyarray(struct batch_request *preq);
+void *req_track(void *preq);
+void *req_rescreserve(void *preq);
+void *req_rescfree(void *preq);
 
-#endif
+
+void  req_shutdown(struct batch_request *preq);
+void *req_signaljob(void *preq);
+void *req_mvjobfile(void *preq);
+void *req_checkpointjob(void *preq);
+void *req_messagejob(void *preq);
 
 /* END request processing prototypes */
 
 
 #ifdef ENABLE_UNIX_SOCKETS
-#ifndef PBS_MOM
-int get_creds(int sd, char *username, char *hostname)
+int get_creds(
+    
+  int   sd,
+  char *username,
+  char *hostname)
+
   {
   int             nb/*, sync*/;
   char            ctrl[CMSG_SPACE(sizeof(struct ucred))];
@@ -252,7 +244,6 @@ int get_creds(int sd, char *username, char *hostname)
   if (setsockopt(sd, SOL_SOCKET, SO_PASSCRED, &nb, sizeof(nb)) == -1)
     return 0;
 
-#endif
 #endif
 
   dummy = '\0';
@@ -275,18 +266,23 @@ int get_creds(int sd, char *username, char *hostname)
   size = sizeof(ucreds);
 
 #else
-  if (cmptr->cmsg_len < SOCKCREDSIZE(0)) return 0;
+  if (cmptr->cmsg_len < SOCKCREDSIZE(0))
+    return(0);
 
   size = SOCKCREDSIZE(((cred *)CMSG_DATA(cmptr))->sc_ngroups);
 
 #endif
-  if ((unsigned)cmptr->cmsg_len != CMSG_LEN(size)) return 0;
+  if ((unsigned)cmptr->cmsg_len != CMSG_LEN(size))
+    return(0);
 
-  if (cmptr->cmsg_level != SOL_SOCKET) return 0;
+  if (cmptr->cmsg_level != SOL_SOCKET)
+    return 0;
 
-  if (cmptr->cmsg_type != SCM_CREDS) return 0;
+  if (cmptr->cmsg_type != SCM_CREDS)
+    return 0;
 
-  if (!(credentials = (ucreds *)malloc(size))) return 0;
+  if (!(credentials = (ucreds *)calloc(1, size)))
+    return 0;
 
   *credentials = *(ucreds *)CMSG_DATA(cmptr);
 
@@ -313,24 +309,38 @@ int get_creds(int sd, char *username, char *hostname)
  * - Validate requesting host and user.
  * - Call function to process request based on type.
  *    That function MUST free the request by calling free_br()
+ *
+ * NOTE: the caller functions hold the mutex for the connection
  */
 
-void process_request(
+void *process_request(
 
-  int sfds) /* file descriptor (socket) to get request */
+  void *new_sock) /* file descriptor (socket) to get request */
 
   {
-#ifdef PBS_MOM
-  char *id = "process_request";
-#endif
-
   int                   rc;
-
+  
   struct batch_request *request = NULL;
+  char                  log_buf[LOCAL_LOG_BUF_SIZE];
+  long                  acl_enable = FALSE;
+  long                  state = SV_STATE_DOWN;
 
-  time_now = time(NULL);
+  time_t                time_now = time(NULL);
+  int                   free_request = TRUE;
+  int                   sfds = *(int *)new_sock;
+  char                  tmpLine[MAXLINE];
+  int                   unlock_mutex = TRUE;
+  char                 *auth_err = NULL;
 
-  request = alloc_br(0);
+  if ((request = alloc_br(0)) == NULL)
+    {
+    snprintf(tmpLine, sizeof(tmpLine),
+        "cannot allocate memory for request from %lu",
+        get_connectaddr(sfds,FALSE));
+    req_reject(PBSE_BADHOST, 0, request, NULL, tmpLine);
+    free_request = FALSE;
+    goto process_request_cleanup;
+    }
 
   request->rq_conn = sfds;
 
@@ -338,7 +348,7 @@ void process_request(
    * Read in the request and decode it to the internal request structure.
    */
 
-#ifndef PBS_MOM
+  pthread_mutex_lock(svr_conn[sfds].cn_mutex);
 
   if (svr_conn[sfds].cn_active == FromClientDIS)
     {
@@ -355,50 +365,31 @@ void process_request(
     }
   else
     {
-    LOG_EVENT(
-      PBSEVENT_SYSTEM,
-      PBS_EVENTCLASS_REQUEST,
-      "process_req",
-      "request on invalid type of connection");
-
-    close_conn(sfds);
-
-    free_br(request);
-
-    return;
+    log_event(PBSEVENT_SYSTEM, PBS_EVENTCLASS_REQUEST,
+      "process_req", "request on invalid type of connection");
+    snprintf(tmpLine, sizeof(tmpLine),
+        "request on invalid type of connection from%lu",
+        get_connectaddr(sfds,FALSE));
+    pthread_mutex_unlock(svr_conn[sfds].cn_mutex);
+    unlock_mutex = FALSE;
+    req_reject(PBSE_BADHOST, 0, request, NULL, tmpLine);
+    free_request = FALSE;
+    goto process_request_cleanup;
     }
 
-#else /* PBS_MOM */
-  rc = dis_request_read(sfds, request);
-
-#endif /* PBS_MOM */
-
   if (rc == -1)
     {
     /* FAILURE */
-
     /* premature end of file */
-
-    close_client(sfds);
-
-    free_br(request);
-
-    return;
+    goto process_request_cleanup;
     }
 
   if ((rc == PBSE_SYSTEM) || (rc == PBSE_INTERNAL))
     {
     /* FAILURE */
-
     /* read error, likely cannot send reply so just disconnect */
-
     /* ??? not sure about this ??? */
-
-    close_client(sfds);
-
-    free_br(request);
-
-    return;
+    goto process_request_cleanup;
     }
 
   if (rc > 0)
@@ -410,79 +401,78 @@ void process_request(
      * request type, in either case, return reject-reply
      */
 
+    pthread_mutex_unlock(svr_conn[sfds].cn_mutex);
+    unlock_mutex = FALSE;
     req_reject(rc, 0, request, NULL, "cannot decode message");
-
-    close_client(sfds);
-
-    return;
+    free_request = FALSE;
+    goto process_request_cleanup;
     }
 
   if (get_connecthost(sfds, request->rq_host, PBS_MAXHOSTNAME) != 0)
     {
-    char tmpLine[1024];
+    sprintf(log_buf, "%s: %lu",
+      pbse_to_txt(PBSE_BADHOST),
+      get_connectaddr(sfds,FALSE));
 
-    sprintf(log_buffer, "%s: %lu",
-            pbse_to_txt(PBSE_BADHOST),
-            get_connectaddr(sfds));
+    log_event(PBSEVENT_DEBUG, PBS_EVENTCLASS_REQUEST, "", log_buf);
 
-    LOG_EVENT(PBSEVENT_DEBUG, PBS_EVENTCLASS_REQUEST, "", log_buffer);
-
-    snprintf(tmpLine, sizeof(tmpLine), "cannot determine hostname for connection from %lu",
-             get_connectaddr(sfds));
+    snprintf(tmpLine, sizeof(tmpLine),
+        "cannot determine hostname for connection from %lu",
+        get_connectaddr(sfds,FALSE));
 
+    pthread_mutex_unlock(svr_conn[sfds].cn_mutex);
+    unlock_mutex = FALSE;
     req_reject(PBSE_BADHOST, 0, request, NULL, tmpLine);
-
-    return;
+    free_request = FALSE;
+    goto process_request_cleanup;
     }
 
   if (LOGLEVEL >= 1)
     {
-    sprintf(
-      log_buffer,
+    sprintf(log_buf,
       msg_request,
       reqtype_to_txt(request->rq_type),
       request->rq_user,
       request->rq_host,
       sfds);
 
-    LOG_EVENT(PBSEVENT_DEBUG2, PBS_EVENTCLASS_REQUEST, "", log_buffer);
+    log_event(PBSEVENT_DEBUG2, PBS_EVENTCLASS_REQUEST, "", log_buf);
     }
 
   /* is the request from a host acceptable to the server */
 
-#ifndef PBS_MOM
-
   if (svr_conn[sfds].cn_socktype & PBS_SOCK_UNIX)
     {
     strcpy(request->rq_host, server_name);
     }
 
-  if (server.sv_attr[SRV_ATR_acl_host_enable].at_val.at_long)
+  get_svr_attr_l(SRV_ATR_acl_host_enable, &acl_enable);
+  if (acl_enable)
     {
     /* acl enabled, check it; always allow myself and nodes */
+    struct array_strings *pas = NULL;
+    struct pbsnode       *isanode;
 
-    struct pbsnode *isanode;
-
-    isanode = PGetNodeFromAddr(get_connectaddr(sfds));
+    get_svr_attr_arst(SRV_ATR_acl_hosts, &pas);
+    isanode = PGetNodeFromAddr(get_connectaddr(sfds,FALSE));
 
     if ((isanode == NULL) &&
         (strcmp(server_host, request->rq_host) != 0) &&
-        (acl_check(
-           &server.sv_attr[SRV_ATR_acl_hosts],
-           request->rq_host,
-           ACL_Host) == 0))
+        (acl_check_my_array_string(pas, request->rq_host, ACL_Host) == 0))
       {
-      char tmpLine[1024];
-
+      char tmpLine[MAXLINE];
       snprintf(tmpLine, sizeof(tmpLine), "request not authorized from host %s",
                request->rq_host);
 
+      pthread_mutex_unlock(svr_conn[sfds].cn_mutex);
+      unlock_mutex = FALSE;
       req_reject(PBSE_BADHOST, 0, request, NULL, tmpLine);
-
-      close_client(sfds);
-
-      return;
+      free_request = FALSE;
+      goto process_request_cleanup;
       }
+
+    if (isanode != NULL)
+      unlock_node(isanode, "process_request", NULL, LOGLEVEL);
     }
 
   /*
@@ -505,6 +495,7 @@ void process_request(
   else
     {
     /* request not from another server */
+    conn_credent[sfds].timestamp = time_now;
 
     request->rq_fromsvr = 0;
 
@@ -529,17 +520,21 @@ void process_request(
       req_connect(request);
 
       if (svr_conn[sfds].cn_socktype == PBS_SOCK_INET)
-        return;
+        {
+        pthread_mutex_unlock(svr_conn[sfds].cn_mutex);
+        unlock_mutex = FALSE;
+        req_reject(rc, 0, request, NULL, NULL);
+        free_request = FALSE;
+        goto process_request_cleanup;
+        }
 
       }
 
     if (svr_conn[sfds].cn_socktype & PBS_SOCK_UNIX)
       {
-      conn_credent[sfds].timestamp = time_now;
       svr_conn[sfds].cn_authen = PBS_NET_CONN_AUTHENTICATED;
       }
 
-
     if (ENABLE_TRUSTED_AUTH == TRUE )
       rc = 0;  /* bypass the authentication of the user--trust the client completely */
     else if (munge_on)
@@ -547,31 +542,33 @@ void process_request(
       /* If munge_on is true we will validate the connection now */
       if ( request->rq_type == PBS_BATCH_AltAuthenUser)
         {
-        rc = req_altauthenuser(request);
-        if (rc == PBSE_NONE)
-          {
-          conn_credent[sfds].timestamp = time_now;
-          svr_conn[sfds].cn_authen = PBS_NET_CONN_AUTHENTICATED;
-          }
-        return;
+        enqueue_threadpool_request(req_altauthenuser,request); 
+      
+        if (unlock_mutex == TRUE)
+          pthread_mutex_unlock(svr_conn[sfds].cn_mutex);
+
+        return(NULL);
         }
       else
         {
-        rc = authenticate_user(request, &conn_credent[sfds]);
+        rc = authenticate_user(request, &conn_credent[sfds], &auth_err);
         }
       }
     else if (svr_conn[sfds].cn_authen != PBS_NET_CONN_AUTHENTICATED)
+      /* skip checking user if we did not get an authenticated credential */
       rc = PBSE_BADCRED;
     else
-      rc = authenticate_user(request, &conn_credent[sfds]);
+      rc = authenticate_user(request, &conn_credent[sfds], &auth_err);
 
     if (rc != 0)
       {
-      req_reject(rc, 0, request, NULL, NULL);
-
-      close_client(sfds);
-
-      return;
+      pthread_mutex_unlock(svr_conn[sfds].cn_mutex);
+      unlock_mutex = FALSE;
+      req_reject(rc, 0, request, NULL, auth_err);
+      if (auth_err != NULL)
+        free(auth_err);
+      free_request = FALSE;
+      goto process_request_cleanup;
       }
 
     /*
@@ -597,22 +594,25 @@ void process_request(
         *dptr = '\0';
         }
       
-      if (((pjob = find_job(request->rq_ind.rq_modify.rq_objname)) != (job *)0) &&
-          (pjob->ji_qs.ji_state == JOB_STATE_RUNNING))
+      if ((pjob = find_job(request->rq_ind.rq_modify.rq_objname)) != (job *)0)
         {
-
-        if ((pjob->ji_wattr[JOB_ATR_checkpoint].at_flags & ATR_VFLAG_SET) &&
-          ((csv_find_string(pjob->ji_wattr[JOB_ATR_checkpoint].at_val.at_str, "s") != NULL) ||
-          (csv_find_string(pjob->ji_wattr[JOB_ATR_checkpoint].at_val.at_str, "c") != NULL) ||
-          (csv_find_string(pjob->ji_wattr[JOB_ATR_checkpoint].at_val.at_str, "enabled") != NULL)) &&
-          (strstr(pjob->ji_wattr[JOB_ATR_exec_host].at_val.at_str, short_host) != NULL))
+        if (pjob->ji_qs.ji_state == JOB_STATE_RUNNING)
           {
 
-          request->rq_perm = svr_get_privilege(request->rq_user, server_host);
-          skip = TRUE;
+          if ((pjob->ji_wattr[JOB_ATR_checkpoint].at_flags & ATR_VFLAG_SET) &&
+              ((csv_find_string(pjob->ji_wattr[JOB_ATR_checkpoint].at_val.at_str, "s") != NULL) ||
+               (csv_find_string(pjob->ji_wattr[JOB_ATR_checkpoint].at_val.at_str, "c") != NULL) ||
+               (csv_find_string(pjob->ji_wattr[JOB_ATR_checkpoint].at_val.at_str, "enabled") != NULL)) &&
+              (strstr(pjob->ji_wattr[JOB_ATR_exec_host].at_val.at_str, short_host) != NULL))
+            {
+            request->rq_perm = svr_get_privilege(request->rq_user, server_host);
+            skip = TRUE;
+            }
 
           }
+        pthread_mutex_unlock(pjob->ji_mutex);
         }
+      
       if (!skip)
         {
         request->rq_perm = svr_get_privilege(request->rq_user, request->rq_host);
@@ -625,8 +625,9 @@ void process_request(
     }  /* END else (svr_conn[sfds].cn_authen == PBS_NET_CONN_FROM_PRIVIL) */
 
   /* if server shutting down, disallow new jobs and new running */
+  get_svr_attr_l(SRV_ATR_State, &state);
 
-  if (server.sv_attr[SRV_ATR_State].at_val.at_long > SV_STATE_RUN)
+  if (state > SV_STATE_RUN)
     {
     switch (request->rq_type)
       {
@@ -638,83 +639,18 @@ void process_request(
       case PBS_BATCH_StageIn:
       case PBS_BATCH_jobscript:
 
+        pthread_mutex_unlock(svr_conn[sfds].cn_mutex);
+        unlock_mutex = FALSE;
         req_reject(PBSE_SVRDOWN, 0, request, NULL, NULL);
-
-        return;
-
+        free_request = FALSE;
+        goto process_request_cleanup;
         /*NOTREACHED*/
 
         break;
       }
     }
 
-#else /* THIS CODE FOR MOM ONLY */
-
-    {
-    /*extern tree *okclients; */
-
-    extern void mom_server_update_receive_time_by_ip(u_long ipaddr, const char *cmd);
-
-    /* check connecting host against allowed list of ok clients */
-
-    if (LOGLEVEL >= 6)
-      {
-      sprintf(log_buffer, "request type %s from host %s received",
-        reqtype_to_txt(request->rq_type),
-        request->rq_host);
-
-      log_record(
-        PBSEVENT_JOB,
-        PBS_EVENTCLASS_JOB,
-        id,
-        log_buffer);
-      }
-
-/*    if (!tfind(svr_conn[sfds].cn_addr, &okclients)) */
-    if (!AVL_is_in_tree(svr_conn[sfds].cn_addr, 0, okclients))
-      {
-      sprintf(log_buffer, "request type %s from host %s rejected (host not authorized)",
-        reqtype_to_txt(request->rq_type),
-        request->rq_host);
-
-      log_record(
-        PBSEVENT_JOB,
-        PBS_EVENTCLASS_JOB,
-        id,
-        log_buffer);
-
-      req_reject(PBSE_BADHOST, 0, request, NULL, "request not authorized");
-
-      close_client(sfds);
-
-      return;
-      }
-
-    if (LOGLEVEL >= 3)
-      {
-      sprintf(log_buffer, "request type %s from host %s allowed",
-        reqtype_to_txt(request->rq_type),
-        request->rq_host);
-
-      log_record(
-        PBSEVENT_JOB,
-        PBS_EVENTCLASS_JOB,
-        id,
-        log_buffer);
-      }
-
-    mom_server_update_receive_time_by_ip(svr_conn[sfds].cn_addr, reqtype_to_txt(request->rq_type));
-    }    /* END BLOCK */
-
-  request->rq_fromsvr = 1;
-
-  request->rq_perm =
-    ATR_DFLAG_USRD | ATR_DFLAG_USWR |
-    ATR_DFLAG_OPRD | ATR_DFLAG_OPWR |
-    ATR_DFLAG_MGRD | ATR_DFLAG_MGWR |
-    ATR_DFLAG_SvWR | ATR_DFLAG_MOM;
-
-#endif /* END else !PBS_MOM */
+  pthread_mutex_unlock(svr_conn[sfds].cn_mutex);
 
   /*
    * dispatch the request to the correct processing function.
@@ -724,7 +660,18 @@ void process_request(
 
   dispatch_request(sfds, request);
 
-  return;
+  return(NULL);
+
+process_request_cleanup:
+
+  close_conn(sfds, TRUE);
+  if (unlock_mutex == TRUE)
+    pthread_mutex_unlock(svr_conn[sfds].cn_mutex);
+
+  if (free_request == TRUE)
+    free_br(request);
+
+  return(NULL);
   }  /* END process_request() */
 
 
@@ -744,190 +691,151 @@ void dispatch_request(
 
   {
   char *id = "dispatch_request";
+  char  log_buf[LOCAL_LOG_BUF_SIZE];
 
   if (LOGLEVEL >= 5)
     {
-    sprintf(log_buffer,"dispatching request %s on sd=%d",
+    sprintf(log_buf,"dispatching request %s on sd=%d",
       reqtype_to_txt(request->rq_type),
       sfds);
 
-    log_record(
-      PBSEVENT_JOB,
-      PBS_EVENTCLASS_JOB,
-      id,
-      log_buffer);
+    log_record(PBSEVENT_JOB,PBS_EVENTCLASS_JOB,id,log_buf);
     }
 
   switch (request->rq_type)
     {
     case PBS_BATCH_QueueJob:
 
-      net_add_close_func(sfds, close_quejob);
+      net_add_close_func(sfds, close_quejob, FALSE);
 
-      req_quejob(request);
+      enqueue_threadpool_request(req_quejob,request);
 
       break;
 
     case PBS_BATCH_JobCred:
 
-      req_jobcredential(request);
+      enqueue_threadpool_request(req_jobcredential,request);
 
       break;
 
     case PBS_BATCH_jobscript:
 
-      req_jobscript(request);
-
+      enqueue_threadpool_request(req_jobscript,request);
       break;
 
     case PBS_BATCH_RdytoCommit:
 
-      req_rdytocommit(request);
-
+      enqueue_threadpool_request(req_rdytocommit,request);
       break;
 
     case PBS_BATCH_Commit:
 
-      req_commit(request);
+      enqueue_threadpool_request(req_commit,request);
 
-      net_add_close_func(sfds, (void (*)())0);
+      net_add_close_func(sfds, (void (*)())0, FALSE);
 
       break;
 
     case PBS_BATCH_DeleteJob:
 
-#ifdef PBS_MOM
-      req_deletejob(request);
-#else
-      /* if this is a server size job delete request, then the request could also be
-      * for an entire array.  we check to see if the request object name is an array id.
-      * if so we hand off the the req_deletearray() function.  If not we pass along to the
-      * normal req_deltejob() function.
+      /* if this is a server size job delete request, then the request
+       * could also be for an entire array.  we check to see if the
+       * request object name is an array id. If so we hand off the the
+       * req_deletearray() function.  If not we pass along to the normal
+       * req_deletejob() function.
       */
 
       if (is_array(request->rq_ind.rq_delete.rq_objname))
-        {
-        req_deletearray(request);
-        }
+        enqueue_threadpool_request(req_deletearray,request);
       else
-        {
-        req_deletejob(request);
-        }
+        enqueue_threadpool_request(req_deletejob,request);
 
-#endif
       break;
 
     case PBS_BATCH_HoldJob:
-#ifdef PBS_MOM
-      req_holdjob(request);
-
-#else
       if (is_array(request->rq_ind.rq_hold.rq_orig.rq_objname))
-        {
-        req_holdarray(request);
-        }
+        enqueue_threadpool_request(req_holdarray, request);
       else
-        {
-        req_holdjob(request);
-        }
+        enqueue_threadpool_request(req_holdjob, request);
 
-#endif
       break;
 
     case PBS_BATCH_CheckpointJob:
 
-      req_checkpointjob(request);
+      enqueue_threadpool_request(req_checkpointjob, request);
 
       break;
 
-#ifndef PBS_MOM
-
     case PBS_BATCH_LocateJob:
 
-      req_locatejob(request);
+      enqueue_threadpool_request(req_locatejob,request);
 
       break;
 
     case PBS_BATCH_Manager:
 
-      req_manager(request);
+      enqueue_threadpool_request(req_manager, request);
 
       break;
 
-#endif  /* END !PBS_MOM */
-
     case PBS_BATCH_MessJob:
 
-      req_messagejob(request);
+      enqueue_threadpool_request(req_messagejob, request);
 
       break;
 
     case PBS_BATCH_AsyModifyJob:
 
     case PBS_BATCH_ModifyJob:
-#ifndef PBS_MOM
       if (is_array(request->rq_ind.rq_delete.rq_objname))
-        {
-        req_modifyarray(request);
-        }
+        enqueue_threadpool_request(req_modifyarray, request);
       else
-        {
-        req_modifyjob(request);
-        }
-#else /* END ifndef PBS_MOM */
-      req_modifyjob(request);
-#endif /* PBS_MOM */
+        enqueue_threadpool_request(req_modifyjob, request);
 
       break;
 
     case PBS_BATCH_Rerun:
 
-      req_rerunjob(request);
+      enqueue_threadpool_request(req_rerunjob, request);
 
       break;
 
-#ifndef PBS_MOM
-
     case PBS_BATCH_MoveJob:
 
-      req_movejob(request);
+      enqueue_threadpool_request(req_movejob, request);
 
       break;
 
     case PBS_BATCH_OrderJob:
 
-      req_orderjob(request);
+      enqueue_threadpool_request(req_orderjob, request);
 
       break;
 
     case PBS_BATCH_Rescq:
 
-      req_rescq(request);
+      enqueue_threadpool_request(req_rescq, request);
 
       break;
 
     case PBS_BATCH_ReserveResc:
 
-      req_rescreserve(request);
+      enqueue_threadpool_request(req_rescreserve, request);
 
       break;
 
     case PBS_BATCH_ReleaseResc:
 
-      req_rescfree(request);
+      enqueue_threadpool_request(req_rescfree, request);
 
       break;
 
     case PBS_BATCH_ReleaseJob:
 
       if (is_array(request->rq_ind.rq_delete.rq_objname))
-        {
-        req_releasearray(request);
-        }
+        enqueue_threadpool_request(req_releasearray, request);
       else
-        {
-        req_releasejob(request);
-        }
+        enqueue_threadpool_request(req_releasejob, request);
 
       break;
 
@@ -935,7 +843,7 @@ void dispatch_request(
 
     case PBS_BATCH_AsyrunJob:
 
-      req_runjob(request);
+      enqueue_threadpool_request(req_runjob, request);
 
       break;
 
@@ -946,14 +854,12 @@ void dispatch_request(
       /* handle special 'truncated' keyword */
 
       if (!strncasecmp(request->rq_ind.rq_status.rq_id, "truncated", strlen("truncated")))
-        req_stat_job(request);
-      else
-        req_selectjobs(request);
+        enqueue_threadpool_request(req_stat_job, request);
+      else 
+        enqueue_threadpool_request(req_selectjobs, request);
 
       break;
 
-#endif  /* !PBS_MOM */
-
     case PBS_BATCH_Shutdown:
 
       req_shutdown(request);
@@ -964,45 +870,37 @@ void dispatch_request(
 
     case PBS_BATCH_AsySignalJob:
 
-      req_signaljob(request);
+      enqueue_threadpool_request(req_signaljob, request);
 
       break;
 
     case PBS_BATCH_GpuCtrl:
 
-      req_gpuctrl(request);
-
-      break;
-
-    case PBS_BATCH_StatusJob:
-
-      req_stat_job(request);
+      enqueue_threadpool_request(req_gpuctrl_svr, request);
 
       break;
 
     case PBS_BATCH_MvJobFile:
 
-      req_mvjobfile(request);
+      enqueue_threadpool_request(req_mvjobfile, request);
 
       break;
 
-#ifndef PBS_MOM  /* server only functions */
-
     case PBS_BATCH_StatusQue:
 
-      req_stat_que(request);
+      enqueue_threadpool_request(req_stat_que, request);
 
       break;
 
     case PBS_BATCH_StatusNode:
-
-      req_stat_node(request);
+      
+      enqueue_threadpool_request(req_stat_node, request);
 
       break;
 
     case PBS_BATCH_StatusSvr:
 
-      req_stat_svr(request);
+      enqueue_threadpool_request(req_stat_svr, request);
 
       break;
 
@@ -1010,7 +908,7 @@ void dispatch_request(
 
     case PBS_BATCH_TrackJob:
 
-      req_track(request);
+      enqueue_threadpool_request(req_track, request);
 
       break;
 
@@ -1018,11 +916,11 @@ void dispatch_request(
 
       if (is_array(request->rq_ind.rq_register.rq_parent))
         {
-        req_registerarray(request);
+        enqueue_threadpool_request(req_registerarray, request);
         }
       else
         {
-        req_register(request);
+        enqueue_threadpool_request(req_register, request);
         }
 
       break;
@@ -1030,8 +928,7 @@ void dispatch_request(
     case PBS_BATCH_AuthenUser:
 
       /* determine if user is valid */
-
-      req_authenuser(request);
+      enqueue_threadpool_request(req_authenuser, request); 
 
       break;
 
@@ -1041,43 +938,31 @@ void dispatch_request(
 
     case PBS_BATCH_JobObit:
 
-      req_jobobit(request);
+      enqueue_threadpool_request(req_jobobit, request);
 
       break;
 
     case PBS_BATCH_StageIn:
 
-      req_stagein(request);
+      enqueue_threadpool_request(req_stagein,request);
 
       break;
 
-#else /* MOM only functions */
-
-    case PBS_BATCH_ReturnFiles:
-
-      req_returnfiles(request);
+    case PBS_BATCH_StatusJob:
 
+      enqueue_threadpool_request(req_stat_job,request);
+      
       break;
 
-    case PBS_BATCH_CopyFiles:
-
-      req_cpyfile(request);
-
-      break;
-
-    case PBS_BATCH_DelFiles:
-
-      req_delfile(request);
-
-      break;
-
-#endif /* !PBS_MOM */
-
     default:
 
       req_reject(PBSE_UNKREQ, 0, request, NULL, NULL);
 
-      close_client(sfds);
+      pthread_mutex_lock(svr_conn[sfds].cn_mutex);
+
+      close_conn(sfds, TRUE);
+      
+      pthread_mutex_unlock(svr_conn[sfds].cn_mutex);
 
       break;
     }  /* END switch (request->rq_type) */
@@ -1088,80 +973,44 @@ void dispatch_request(
 
 
 
-
-/*
- * close_client - close a connection to a client, also "inactivate"
- *    any outstanding batch requests on that connection.
- */
-
-static void close_client(
-
-  int sfds)  /* connection socket */
-
-  {
-
-  struct batch_request *preq;
-
-  close_conn(sfds); /* close the connection */
-
-  preq = (struct batch_request *)GET_NEXT(svr_requests);
-
-  while (preq != NULL)
-    {
-    /* list of outstanding requests */
-
-    if (preq->rq_conn == sfds)
-      preq->rq_conn = -1;
-
-    if (preq->rq_orgconn == sfds)
-      preq->rq_orgconn = -1;
-
-    preq = (struct batch_request *)GET_NEXT(preq->rq_link);
-    }
-
-  return;
-  }  /* END close_client() */
-
-
-
-
 /*
  * alloc_br - allocate and clear a batch_request structure
  */
 
 struct batch_request *alloc_br(
 
-        int type)
+  int type)
 
   {
 
   struct batch_request *req = NULL;
+  int rc = PBSE_NONE;
 
-  req = (struct batch_request *)malloc(sizeof(struct batch_request));
-
-  if (req == NULL)
+  memmgr *mm = NULL;
+  if ((rc = memmgr_init(&mm, 0)) != PBSE_NONE)
+    {
+    printf("rc = %d\n", rc);
+    log_err(errno, "alloc_br", "memmgr_init in alloc_br failed");
+    }
+  else if ((req = (struct batch_request *)memmgr_calloc(&mm, 1, sizeof(struct batch_request))) == NULL)
     {
     log_err(errno, "alloc_br", msg_err_malloc);
+    }
+  else
+    {
 
-    return(NULL);
+    req->mm = mm;
+    req->rq_type = type;
+
+    req->rq_conn = -1;  /* indicate not connected */
+    req->rq_orgconn = -1;  /* indicate not connected */
+    req->rq_time = time(NULL);
+    req->rq_reply.brp_choice = BATCH_REPLY_CHOICE_NULL;
+    req->rq_noreply = FALSE;  /* indicate reply is needed */
     }
 
-  memset((void *)req, (int)0, sizeof(struct batch_request));
-
-  req->rq_type = type;
-
-  CLEAR_LINK(req->rq_link);
-
-  req->rq_conn = -1;  /* indicate not connected */
-  req->rq_orgconn = -1;  /* indicate not connected */
-  req->rq_time = time_now;
-  req->rq_reply.brp_choice = BATCH_REPLY_CHOICE_NULL;
-  req->rq_noreply = FALSE;  /* indicate reply is needed */
-
-  append_link(&svr_requests, &req->rq_link, req);
-
   return(req);
-  }
+  } /* END alloc_br() */
 
 
 
@@ -1178,21 +1027,15 @@ static void close_quejob(
 
   {
   job *pjob;
-  job *npjob;
 
-  pjob = (job *)GET_NEXT(svr_newjobs);
+  int iter = -1;
 
-  while (pjob != NULL)
+  while ((pjob = next_job(&newjobs,&iter)) != NULL)
     {
-    npjob = GET_NEXT(pjob->ji_alljobs);
-
     if (pjob->ji_qs.ji_un.ji_newt.ji_fromsock == sfds)
       {
       if (pjob->ji_qs.ji_substate == JOB_SUBSTATE_TRANSICM)
         {
-
-#ifndef PBS_MOM
-
         if (pjob->ji_qs.ji_svrflags & JOB_SVFLG_HERE)
           {
           /*
@@ -1202,33 +1045,34 @@ static void close_quejob(
            * server again to commit.
            */
 
-          delete_link(&pjob->ji_alljobs);
+          remove_job(&newjobs,pjob);
 
           pjob->ji_qs.ji_state = JOB_STATE_QUEUED;
           pjob->ji_qs.ji_substate = JOB_SUBSTATE_QUEUED;
 
-          if (svr_enquejob(pjob))
+          if (svr_enquejob(pjob, FALSE, -1))
             job_abt(&pjob, msg_err_noqueue);
           }
-
-#endif /* PBS_MOM */
-
         }
       else
         {
         /* else delete the job */
-
-        delete_link(&pjob->ji_alljobs);
+        remove_job(&newjobs,pjob);
 
         job_purge(pjob);
+
+        pjob = NULL;
         }
 
       break;
       }  /* END if (..) */
 
-    pjob = npjob;
+    pthread_mutex_unlock(pjob->ji_mutex);
     }
 
+  if (pjob != NULL)
+    pthread_mutex_unlock(pjob->ji_mutex);
+
   return;
   }  /* END close_quejob() */
 
@@ -1246,7 +1090,7 @@ void free_br(
   struct batch_request *preq)
 
   {
-  delete_link(&preq->rq_link);
+  memmgr *mm = preq->mm;
 
   reply_free(&preq->rq_reply);
 
@@ -1331,8 +1175,6 @@ void free_br(
 
       break;
 
-#ifndef PBS_MOM /* Server Only */
-
     case PBS_BATCH_Manager:
 
       freebr_manage(&preq->rq_ind.rq_manager);
@@ -1368,8 +1210,6 @@ void free_br(
 
       break;
 
-#endif /* !PBS_MOM */
-
     default:
 
       /* NO-OP */
@@ -1377,7 +1217,7 @@ void free_br(
       break;
     }  /* END switch (preq->rq_type) */
 
-  free(preq);
+  memmgr_destroy(&mm);
 
   return;
   }  /* END free_br() */
@@ -1427,7 +1267,6 @@ static void freebr_cpyfile(
 
 
 
-#ifndef PBS_MOM
 static void free_rescrq(
 
   struct rq_rescq *pq)
@@ -1449,8 +1288,6 @@ static void free_rescrq(
   return;
   }  /* END free_rescrq() */
 
-#endif /* PBS_MOM */
-
 /* END process_requests.c */
 
 
