@@ -92,9 +92,10 @@
 #include <time.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <errno.h>
 #include <sys/types.h>
-#include <sys/stat.h>
 #include <ctype.h>
+#include <pthread.h>
 #include "libpbs.h"
 #include "server_limits.h"
 #include "list_link.h"
@@ -102,12 +103,14 @@
 #include "credential.h"
 #include "net_connect.h"
 #include "batch_request.h"
+#include "log.h"
+#include "../lib/Liblog/pbs_log.h"
+#include "../lib/Libnet/lib_net.h" /* global_sock_add */
 
 #define SPACE 32 /* ASCII space character */
 
 /* External Global Data Items Referenced */
 
-extern time_t time_now;
 
 extern struct connection svr_conn[];
 extern char *path_credentials;
@@ -129,20 +132,33 @@ void req_connect(
   {
   int  sock = preq->rq_conn;
 
+  /* Called from one location inside a lock */
+/*   pthread_mutex_lock(svr_conn[sock].cn_mutex); */
+
   if (svr_conn[sock].cn_authen == 0)
     {
     reply_ack(preq);
     }
   else
     {
-    req_reject(PBSE_BADCRED, 0, preq, NULL, NULL);
+    req_reject(PBSE_BADCRED, 0, preq, NULL, "Connection not authorized");
     }
 
+/*   pthread_mutex_unlock(svr_conn[sock].cn_mutex); */
+
   return;
   }  /* END req_connect() */
 
-int get_encode_host(int s, char *munge_buf, struct batch_request *preq)
-{
+
+
+
+int get_encode_host(
+  
+  int s, 
+  char *munge_buf, 
+  struct batch_request *preq)
+  
+  {
   char *ptr;
   char host_name[PBS_MAXHOSTNAME];
   int i;
@@ -159,25 +175,31 @@ int get_encode_host(int s, char *munge_buf, struct batch_request *preq)
    	 between the : and the first letter of the host name */
   ptr = strchr(ptr, ':');
   ptr++;
-  while(*ptr == SPACE)
+  while (*ptr == SPACE)
     {
     ptr++;
     }
 
   memset(host_name, 0, PBS_MAXHOSTNAME);
   i = 0;
-  while(*ptr != SPACE && !isspace(*ptr))
-	{
-	host_name[i++] = *ptr;
-	ptr++;
-	}
+  while (*ptr != SPACE && !isspace(*ptr))
+    {
+    host_name[i++] = *ptr;
+    ptr++;
+    }
 
   strcpy(conn_credent[s].hostname, host_name);
+
   return(0);
+  } /* END get_encode_host() */
 
-}
 
-int get_UID(int s, char *munge_buf, struct batch_request *preq)
+int get_UID(
+    
+  int s, 
+  char *munge_buf, 
+  struct batch_request *preq)
+  
   {
   char *ptr;
   char user_name[PBS_MAXHOSTNAME];
@@ -193,84 +215,60 @@ int get_UID(int s, char *munge_buf, struct batch_request *preq)
 
 	ptr = strchr(ptr, ':');
 	ptr++;
-	while(*ptr == SPACE)
+	while (*ptr == SPACE)
 	  {
 	  ptr++;
 	  }
 
 	memset(user_name, 0, PBS_MAXHOSTNAME);
 	i = 0;
-	while(*ptr != SPACE && !isspace(*ptr))
+	while (*ptr != SPACE && !isspace(*ptr))
 	  {
 	  user_name[i++] = *ptr;
 	  ptr++;
 	  }
 
 	strcpy(conn_credent[s].username, user_name);
-	return(0);
-
+	
+  return(0);
   }
 
-int unmunge_request(int s, struct batch_request *preq)
-  {
-  time_t myTime;
-  struct timeval tv;
-  suseconds_t millisecs;
-  struct tm *timeinfo;
-  char mungeFileName[MAXPATHLEN + MAXNAMLEN+1];
-  int fd, newfd;
-  char buf[MUNGE_SIZE];
-  char munge_buf[MUNGE_SIZE];
-  int bytes_written;
-  int cred_size;
-  pid_t pid;
 
-  int fd_pipe[2];
-  char execname[20]; /* for execvp. will be "munge" */
-  char *options[3];  /* argument list for execvp */
-  char com1[20];     /* first argument to execvp */
-  char com2[MAXPATHLEN + MAXNAMLEN+5];      /* second argument to execvp */
-  int bytes_read;
-  int total_bytes_read = 0;        
-  char *ptr; /* pointer to the current place to copy data into munge_buf */
+int write_munge_temp_file(
+
+  struct batch_request *preq,          /* I */
+  char                 *mungeFileName) /* I */
+
+  {
+  int fd;
+  int cred_size;
+  int bytes_written;
   int rc;
 
-  /* create a sudo randome file name */
-  gettimeofday(&tv, NULL);
-  myTime = tv.tv_sec;
-  timeinfo = localtime(&myTime);
-  millisecs = tv.tv_usec;
-  sprintf(mungeFileName, "%smunge-%d-%d-%d-%d", 
-  path_credentials, timeinfo->tm_hour, timeinfo->tm_min, 
-  timeinfo->tm_sec, (int)millisecs);
-
-  fd = open(mungeFileName, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR);
-  if(fd == -1)
+  if ((fd = open(mungeFileName, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR)) < 0)
     {
     req_reject(PBSE_SYSTEM, 0, preq, NULL, "could not create temporary munge file");
-	  return(-1);
+    return(-1);
     }
 
-  /* Write the munge credential to the newly created file */
-
-  cred_size = strlen(preq->rq_ind.rq_authen.rq_cred);
-  if(cred_size == 0)
+  if ((cred_size = strlen(preq->rq_ind.rq_authen.rq_cred)) == 0)
     {
-    req_reject(PBSE_BADCRED, 0, preq, NULL, NULL);
-		close(fd);
-		return(-1);
-    }
-
-  bytes_written = write(fd, preq->rq_ind.rq_authen.rq_cred, cred_size);
-  if(bytes_written == -1 || (bytes_written != cred_size))
-	  {
-	  req_reject(PBSE_SYSTEM, 0, preq, NULL, "could not write credential to temporary munge file");
+    req_reject(PBSE_BADCRED, 0, preq, NULL, "munge credential invalid");
     close(fd);
     return(-1);
     }
 
-	rc = fsync(fd);
-	if (rc < 0)
+  bytes_written = write(fd, preq->rq_ind.rq_authen.rq_cred, cred_size);
+
+  if ((bytes_written == -1) || 
+      (bytes_written != cred_size))
+    {
+    req_reject(PBSE_SYSTEM, 0, preq, NULL, "could not write credential to temporary munge file");
+    close(fd);
+    return(-1);
+    }
+
+	if ((rc = fsync(fd)) < 0)
 		{
 		close(fd);
 		return(rc);
@@ -278,101 +276,126 @@ int unmunge_request(int s, struct batch_request *preq)
 
   close(fd);
 
-  /* For the child to run the unmunge utility on the file we just created.
-   	 The parent will read from the child and use the unmunged data to validate
-	 the user */
-  rc = pipe(fd_pipe);
-  if (rc == -1)
+  return(PBSE_NONE);
+  } /* END write_munge_temp_file() */
+
+
+
+
+int pipe_and_read_unmunge(
+
+  char                 *mungeFileName, /* I */
+  struct batch_request *preq,          /* I */
+  int                   sock)          /* I */
+
+  {
+  static char *id = "pipe_and_read_unmunge";
+  char         munge_buf[MUNGE_SIZE << 4];
+  char         log_buf[LOCAL_LOG_BUF_SIZE];
+
+  FILE *munge_pipe;
+  char *ptr; /* pointer to the current place to copy data into munge_buf */
+  char  munge_command[MAXPATHLEN<<1];
+  int   bytes_read;
+  int   total_bytes_read = 0;
+  int   fd;
+  int   rc;
+  
+  snprintf(munge_command,sizeof(munge_command),
+    "unmunge --input=%s",
+    mungeFileName);
+  
+  if ((munge_pipe = popen(munge_command,"r")) == NULL)
     {
+    /* FAILURE */
+    snprintf(log_buf,sizeof(log_buf),
+      "Unable to popen command '%s' for reading",
+      munge_command);
+    log_err(errno,id,log_buf);
+    
     unlink(mungeFileName);
-    req_reject(PBSE_SYSTEM, 0, preq, NULL, "could not create pipe to unmunge");
+    req_reject(PBSE_SYSTEM, 0, preq, NULL, "couldn't create pipe to unmunge");
     return(-1);
     }
+  
+  memset(munge_buf, 0, MUNGE_SIZE);
+  ptr = munge_buf;
+  
+  fd = fileno(munge_pipe);
+  
+  while ((bytes_read = read(fd, ptr, MUNGE_SIZE)) > 0)
+    {
+    total_bytes_read += bytes_read;
+    ptr += bytes_read;
+    }
+  
+  pclose(munge_pipe);
+  
+  if (bytes_read == -1)
+    {
+    /* read failed */
+    req_reject(PBSE_SYSTEM, 0, preq, NULL, "error reading unmunge data");
+    rc = -1;
+    }
+  else if (total_bytes_read == 0)
+    {
+    /* 
+     * unmunge failed. Probably a bad credential. But we do not know for sure.
+     * Bad credential gives us ECHILD error which gets added to log message
+     * and confuses users, so reset it to zero show it does not show up in log
+     */
+    if (errno == ECHILD)
+      errno = 0;
+    req_reject(PBSE_SYSTEM, 0, preq, NULL, "could not unmunge credentials");
+    rc = -1;
+    }
+  else if ((rc = get_encode_host(sock, munge_buf, preq)) == PBSE_NONE)
+    {
+    rc = get_UID(sock, munge_buf, preq);
+    }
 
-  pid = fork();
-  if (pid != 0)
-	  {
-	  /* This is the parent*/
-	  /* set up the pipe to be able to read from the child */
-	  close(fd_pipe[1]);
-	  
-	  memset(buf, 0, MUNGE_SIZE);
-	  memset(munge_buf, 0, MUNGE_SIZE);
-	  ptr = munge_buf; 
-	  
-	  do
-      {
-      bytes_read = read(fd_pipe[0], buf, MUNGE_SIZE);
-      if (bytes_read > 0)
-        {
-        total_bytes_read += bytes_read;
-        memcpy(ptr, buf, bytes_read);
-        ptr += bytes_read;
-        }
-      } while(bytes_read > 0);
+  return(rc);
+  } /* END pipe_and_read_unmunge() */
+
+
+
+
+
+int unmunge_request(
     
-	    if (bytes_read == -1)
-	  	  {
-	  	  /* read failed */
-	  	  unlink(mungeFileName);
-	  	  req_reject(PBSE_SYSTEM, 0, preq, NULL, "error reading unmunge data");
-	  		close(fd_pipe[0]);
-	  	  return(-1);
-	  	  }
-	  
-	  
-	    if (total_bytes_read == 0)
-	  	  {
-	  	  /* unmunge failed. Probably a bad credential. But we do not know */
-	  	  req_reject(PBSE_SYSTEM, 0, preq, NULL, "could not unmunge credentials");
-				unlink(mungeFileName);
-	  		close(fd_pipe[0]);
-	  	  return(-1);
-	  	  }
-	  
-	    rc = get_encode_host(s, munge_buf, preq);
-	    if (rc)
-	  	  {
-				unlink(mungeFileName);
-	  		close(fd_pipe[0]);
-	  	  return(rc);
-	  	  }
-	  
-	    rc = get_UID(s, munge_buf, preq);
-	    if (rc)
-	    	{
-				unlink(mungeFileName);
-	  		close(fd_pipe[0]);
-    		return(rc);
-	  	  }
-	  
-	  	unlink(mungeFileName);
-	  }
-  else
-	  {
-	  close(fd_pipe[0]);
-	  newfd = dup2(fd_pipe[1], 1);
-	  strcpy(execname, "unmunge");
-	  strcpy(com1, "unmunge");
-	  strcpy(com2, "--input=");
-	  strcat(com2, mungeFileName);
-	  options[0] = com1;
-	  options[1] = com2;
-	  options[2] = NULL;
-	  
-	  rc = execvp(execname, options);
-	  
-	  /* Something went wrong. We will have to depend on the parent
-	     to let everyone know */
-	  close(fd_pipe[1]);
-	  exit(0);
-	  
-	  }
+  int                   sock, /* I */
+  struct batch_request *preq) /* M */
+ 
+  {
+  time_t          myTime;
+  struct timeval  tv;
+  suseconds_t     millisecs;
+  struct tm      *timeinfo;
+  char            mungeFileName[MAXPATHLEN + MAXNAMLEN+1];
+  int             rc = PBSE_NONE;
 
-	close(fd_pipe[0]);
-  return(0);
+  /* create a sudo random file name */
+  gettimeofday(&tv, NULL);
+  myTime = tv.tv_sec;
+  timeinfo = localtime(&myTime);
+  millisecs = tv.tv_usec;
+  sprintf(mungeFileName, "%smunge-%d-%d-%d-%d", 
+	  path_credentials, timeinfo->tm_hour, timeinfo->tm_min, 
+	  timeinfo->tm_sec, (int)millisecs);
+
+  /* Write the munge credential to the newly created file */
+  if ((rc = write_munge_temp_file(preq,mungeFileName)) == PBSE_NONE)
+    {
+    /* open the munge command as a pipe and read the result */
+    rc = pipe_and_read_unmunge(mungeFileName,preq,sock);
+    }
+  
+  /* delete the old file */
+  unlink(mungeFileName);
+
+  return(rc);
+  } /* END unmunge_request */
 
-}
 
 
 
@@ -383,51 +406,73 @@ int unmunge_request(int s, struct batch_request *preq)
  * non-privileged connection.  This connection is marked as authenticated.
  */
 
-void req_authenuser(
+void *req_authenuser(
 
-  struct batch_request *preq)  /* I */
+  void *vp)  /* I */
 
   {
-  int s;
+  int                   s;
+  int                   debug = 0;
+  int                   delay_cntr = 0;
+  char                  log_buf[LOCAL_LOG_BUF_SIZE];
+  struct batch_request *preq = (struct batch_request *)vp;
 
   /*
    * find the socket whose client side is bound to the port named
    * in the request
    */
 
-  for (s = 0;s < PBS_NET_MAX_CONNECTIONS;++s)
+  if (getenv("PBSDEBUG"))
     {
-    if (preq->rq_ind.rq_authen.rq_port != svr_conn[s].cn_port)
+    debug = 1;
+    }
+  for (delay_cntr = 0; delay_cntr < 5;delay_cntr++)
+    {
+    for (s = 0;s < PBS_NET_MAX_CONNECTIONS;++s)
       {
-      continue;
-      }
+      pthread_mutex_lock(svr_conn[s].cn_mutex);
+  
+      if (preq->rq_ind.rq_authen.rq_port != svr_conn[s].cn_port)
+        {
+        pthread_mutex_unlock(svr_conn[s].cn_mutex);
+        continue;
+        }
 
 #ifndef NOPRIVPORTS
-    if (svr_conn[s].cn_authen == 0)
+      if (svr_conn[s].cn_authen == 0)
 #endif
-      {
-      strcpy(conn_credent[s].username, preq->rq_user);
-      strcpy(conn_credent[s].hostname, preq->rq_host);
+        {
+        strcpy(conn_credent[s].username, preq->rq_user);
+        strcpy(conn_credent[s].hostname, preq->rq_host);
 
-      /* time stamp just for the record */
+        /* time stamp just for the record */
 
-      conn_credent[s].timestamp = time_now;
+        conn_credent[s].timestamp = time(NULL);
 
-      svr_conn[s].cn_authen = PBS_NET_CONN_AUTHENTICATED;
-      }
+        svr_conn[s].cn_authen = PBS_NET_CONN_AUTHENTICATED;
+        }
 
-    reply_ack(preq);
+      reply_ack(preq);
 
-    /* SUCCESS */
+      /* SUCCESS */
+      pthread_mutex_unlock(svr_conn[s].cn_mutex);
+      if (debug) printf("(FOUND_PROCESSED) unlock %d (port %d)\n", s, svr_conn[s].cn_port);
 
-    return;
-    }  /* END for (s) */
+      return NULL;
+      }  /* END for (s) */
+    if (debug) fprintf(stderr, "sock not found, sleeping (%d)\n", delay_cntr);
+    usleep(10);
+    }
 
-  req_reject(PBSE_BADCRED, 0, preq, NULL, "cannot authenticate user");
+  sprintf(log_buf, "trqauthd fail %d", preq->rq_ind.rq_authen.rq_port);
+  log_err(PBSE_BADCRED, "req_authenuser", log_buf);
+  if (debug) 
+    printf("%s\n", log_buf);
+  req_reject(PBSE_BADCRED, 0, preq, NULL, "cannot authenticate user. Client connection not found");
 
   /* FAILURE */
 
-  return;
+  return NULL;
   }  /* END req_authenuser() */
 
 
@@ -438,11 +483,12 @@ void req_authenuser(
  * utility 
  * 
 */
-int req_altauthenuser(
+void *req_altauthenuser(
 
-  struct batch_request *preq)  /* I */
+  void *vp)  /* I */
 
   {
+  struct batch_request *preq = (struct batch_request *)vp;
   int s;
   int rc;
   
@@ -453,39 +499,46 @@ int req_altauthenuser(
 
   for (s = 0;s < PBS_NET_MAX_CONNECTIONS;++s)
     {
+    pthread_mutex_lock(svr_conn[s].cn_mutex);
+
     if (preq->rq_ind.rq_authen.rq_port != svr_conn[s].cn_port)
       {
+      pthread_mutex_unlock(svr_conn[s].cn_mutex);
+
       continue;
       }
-	break;
+    break;
     }  /* END for (s) */
 
   /* If s is less than PBS_NET_MAX_CONNECTIONS we have our port */
-  if(s >= PBS_NET_MAX_CONNECTIONS)
-	  {
+  if (s >= PBS_NET_MAX_CONNECTIONS)
+    {
+    pthread_mutex_unlock(svr_conn[s].cn_mutex);
 	  req_reject(PBSE_BADCRED, 0, preq, NULL, "cannot authenticate user. Client connection not found");
-	  return (PBSE_BADCRED);
-  	}
-
+    return(NULL);
+    }
 
   rc = unmunge_request(s, preq);
   if (rc)
     {
     /* FAILED */
-    return (PBSE_SYSTEM);
+    pthread_mutex_unlock(svr_conn[s].cn_mutex);
+    return(NULL);
     }
 
   /* SUCCESS */
 
   /* time stamp just for the record */
 
-  conn_credent[s].timestamp = time_now;
+  conn_credent[s].timestamp = time(NULL);
 
   svr_conn[s].cn_authen = PBS_NET_CONN_AUTHENTICATED;
 
-  reply_ack(preq);
-  return (PBSE_NONE);
+  pthread_mutex_unlock(svr_conn[s].cn_mutex);
 
+  reply_ack(preq);
+  
+  return(NULL);
   }  /* END req_altauthenuser() */
 
 
