@@ -88,7 +88,7 @@
 #include <pbs_config.h>   /* the master config generated by configure */
 
 #include <stdio.h>
-#include <sys/types.h>
+#include <pthread.h>
 #include "libpbs.h"
 #include "server_limits.h"
 #include "list_link.h"
@@ -100,6 +100,7 @@
 #include "work_task.h"
 #include "pbs_error.h"
 #include "log.h"
+#include "../lib/Liblog/log_event.h"
 #include "acct.h"
 #include "svrfunc.h"
 #include "csv.h"
@@ -117,14 +118,15 @@ extern char *msg_jobholdset;
 extern char *msg_jobholdrel;
 extern char *msg_mombadhold;
 extern char *msg_postmomnojob;
-extern time_t  time_now;
+extern int LOGLEVEL;
 
 int chk_hold_priv(long val, int perm);
 int get_hold(tlist_head *, char **, attribute *);
 
 /* external functions */
 extern int svr_authorize_jobreq(struct batch_request *,job *);
-
+extern job *chk_job_request(char *,struct batch_request *);
+int copy_batchrequest(struct batch_request **newreq, struct batch_request *preq, int type, int jobid);
 
 /*
  * chk_hold_priv - check that client has privilege to set/clear hold
@@ -158,9 +160,9 @@ int chk_hold_priv(
  * The state of the job may change as a result.
  */
 
-void req_holdjob(
+void *req_holdjob(
 
-  struct batch_request *preq)
+  void *vp) /* I */
 
   {
   long  *hold_val;
@@ -172,12 +174,15 @@ void req_holdjob(
   int     rc;
   attribute temphold;
   attribute *pattr;
+  struct batch_request *preq = (struct batch_request *)vp;
+  char                  log_buf[LOCAL_LOG_BUF_SIZE];
+  struct batch_request *dup_req = NULL;
 
   pjob = chk_job_request(preq->rq_ind.rq_hold.rq_orig.rq_objname, preq);
 
   if (pjob == NULL)
     {
-    return;
+    return(NULL);
     }
 
   /* cannot do anything until we decode the holds to be set */
@@ -186,7 +191,10 @@ void req_holdjob(
                      &temphold)) != 0)
     {
     req_reject(rc, 0, preq, NULL, NULL);
-    return;
+
+    pthread_mutex_unlock(pjob->ji_mutex);
+
+    return(NULL);
     }
 
   /* if other than HOLD_u is being set, must have privil */
@@ -194,18 +202,20 @@ void req_holdjob(
   if ((rc = chk_hold_priv(temphold.at_val.at_long, preq->rq_perm)) != 0)
     {
     req_reject(rc, 0, preq, NULL, NULL);
-    return;
+
+    pthread_mutex_unlock(pjob->ji_mutex);
+
+    return(NULL);
     }
 
-  hold_val = &pjob->ji_wattr[(int)JOB_ATR_hold].at_val.at_long;
+  hold_val = &pjob->ji_wattr[JOB_ATR_hold].at_val.at_long;
 
   old_hold = *hold_val;
   *hold_val |= temphold.at_val.at_long;
-  pjob->ji_wattr[(int)JOB_ATR_hold].at_flags |= ATR_VFLAG_SET;
-  sprintf(log_buffer, msg_jobholdset, pset, preq->rq_user,
-          preq->rq_host);
+  pjob->ji_wattr[JOB_ATR_hold].at_flags |= ATR_VFLAG_SET;
+  sprintf(log_buf, msg_jobholdset, pset, preq->rq_user, preq->rq_host);
 
-  pattr = &pjob->ji_wattr[(int)JOB_ATR_checkpoint];
+  pattr = &pjob->ji_wattr[JOB_ATR_checkpoint];
 
   if ((pjob->ji_qs.ji_state == JOB_STATE_RUNNING) &&
       ((pattr->at_flags & ATR_VFLAG_SET) &&
@@ -216,25 +226,31 @@ void req_holdjob(
 
     /* have MOM attempt checkpointing */
 
-    if ((rc = relay_to_mom(pjob->ji_qs.ji_un.ji_exect.ji_momaddr,
-                           preq, process_hold_reply)) != 0)
+    if ((rc = copy_batchrequest(&dup_req, preq, 0, -1)) != 0)
+      {
+      req_reject(rc, 0, preq, NULL, "memory allocation failure");
+      }
+    /* The dup_req is freed in relay_to_mom (failure)
+     * or in issue_Drequest (success) */
+    else if ((rc = relay_to_mom(&pjob, dup_req, process_hold_reply)) != 0)
       {
       *hold_val = old_hold;  /* reset to the old value */
       req_reject(rc, 0, preq, NULL, NULL);
       }
     else
       {
-      pjob->ji_qs.ji_svrflags |=
-        JOB_SVFLG_HASRUN | JOB_SVFLG_CHECKPOINT_FILE;
-      job_save(pjob, SAVEJOB_QUICK);
-      
-      /* fill in log_buffer again, since relay_to_mom changed it */
-      
-      sprintf(log_buffer, msg_jobholdset, pset, preq->rq_user,
-          preq->rq_host);
-          
-      LOG_EVENT(PBSEVENT_JOB, PBS_EVENTCLASS_JOB,
-        pjob->ji_qs.ji_jobid, log_buffer);
+      if (pjob != NULL)
+        {
+        pjob->ji_qs.ji_svrflags |= JOB_SVFLG_HASRUN | JOB_SVFLG_CHECKPOINT_FILE;
+        
+        job_save(pjob, SAVEJOB_QUICK, 0);
+        
+        /* fill in log_buf again, since relay_to_mom changed it */
+        sprintf(log_buf, msg_jobholdset, pset, preq->rq_user, preq->rq_host);
+        
+        log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buf);
+        req_reject(rc, 0, preq, NULL, "relay to mom failed");
+        }
       }
     }
 #ifdef ENABLE_BLCR
@@ -246,11 +262,7 @@ void req_holdjob(
      * so we reject the request
      */
 
-    LOG_EVENT(
-      PBSEVENT_JOB,
-      PBS_EVENTCLASS_JOB,
-      pjob->ji_qs.ji_jobid,
-      log_buffer);
+    log_event(PBSEVENT_JOB,PBS_EVENTCLASS_JOB,pjob->ji_qs.ji_jobid,log_buf);
 
     req_reject(PBSE_IVALREQ, 0, preq, NULL,
         "job not held since checkpointing is expected but not enabled for job");
@@ -260,11 +272,7 @@ void req_holdjob(
     {
     /* everything went well, may need to update the job state */
 
-    LOG_EVENT(
-      PBSEVENT_JOB,
-      PBS_EVENTCLASS_JOB,
-      pjob->ji_qs.ji_jobid,
-      log_buffer);
+    log_event(PBSEVENT_JOB,PBS_EVENTCLASS_JOB,pjob->ji_qs.ji_jobid,log_buf);
 
     if (old_hold != *hold_val)
       {
@@ -274,11 +282,16 @@ void req_holdjob(
 
       svr_evaljobstate(pjob, &newstate, &newsub, 0);
 
-      svr_setjobstate(pjob, newstate, newsub);
+      svr_setjobstate(pjob, newstate, newsub, FALSE);
       }
 
     reply_ack(preq);
     }
+
+  if (pjob != NULL)
+    pthread_mutex_unlock(pjob->ji_mutex);
+
+  return(NULL);
   }  /* END req_holdjob() */
 
 
@@ -289,21 +302,24 @@ void req_holdjob(
  *
  */
 
-void req_checkpointjob(
+void *req_checkpointjob(
 
-  struct batch_request *preq)
+  void *vp)
 
   {
-  job    *pjob;
-  int     rc;
-  attribute *pattr;
+  struct batch_request *preq = (struct batch_request *)vp;
+  job                  *pjob;
+  int                   rc;
+  attribute            *pattr;
+  char                  log_buf[LOCAL_LOG_BUF_SIZE];
+  struct batch_request *dup_req = NULL;
 
   if ((pjob = chk_job_request(preq->rq_ind.rq_manager.rq_objname, preq)) == NULL)
     {
-    return;
+    return(NULL);
     }
 
-  pattr = &pjob->ji_wattr[(int)JOB_ATR_checkpoint];
+  pattr = &pjob->ji_wattr[JOB_ATR_checkpoint];
 
   if ((pjob->ji_qs.ji_state == JOB_STATE_RUNNING) &&
       ((pattr->at_flags & ATR_VFLAG_SET) &&
@@ -313,34 +329,45 @@ void req_checkpointjob(
     {
     /* have MOM attempt checkpointing */
 
-    if ((rc = relay_to_mom(pjob->ji_qs.ji_un.ji_exect.ji_momaddr,
-                           preq, process_checkpoint_reply)) != 0)
+    if ((rc = copy_batchrequest(&dup_req, preq, 0, -1)) != 0)
+      {
+      req_reject(rc, 0, preq, NULL, "failure to allocate memory");
+      }
+    /* The dup_req is freed in relay_to_mom (failure)
+     * or in issue_Drequest (success) */
+    else if ((rc = relay_to_mom(&pjob, dup_req, process_checkpoint_reply)) != 0)
       {
       req_reject(rc, 0, preq, NULL, NULL);
       }
     else
       {
-      pjob->ji_qs.ji_svrflags |= JOB_SVFLG_CHECKPOINT_FILE;
-      job_save(pjob, SAVEJOB_QUICK);
-      LOG_EVENT(PBSEVENT_JOB, PBS_EVENTCLASS_JOB,
-                pjob->ji_qs.ji_jobid, log_buffer);
+      if (pjob != NULL)
+        {
+        pjob->ji_qs.ji_svrflags |= JOB_SVFLG_CHECKPOINT_FILE;
+        
+        job_save(pjob, SAVEJOB_QUICK, 0);
+        log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buf);
+        }
       }
     }
   else
     {
     /* Job does not have checkpointing enabled, so reject the request */
 
-    LOG_EVENT(
-      PBSEVENT_JOB,
-      PBS_EVENTCLASS_JOB,
-      pjob->ji_qs.ji_jobid,
-      log_buffer);
+    log_event(PBSEVENT_JOB,PBS_EVENTCLASS_JOB,pjob->ji_qs.ji_jobid,log_buf);
 
     req_reject(PBSE_IVALREQ, 0, preq, NULL, "job is not checkpointable");
     }
+
+  if (pjob != NULL)
+    pthread_mutex_unlock(pjob->ji_mutex);
+
+  return(NULL);
   }  /* END req_checkpointjob() */
 
 
+
+
 /*
  * release_job - releases the hold on job j
  * @param j - the job to modify
@@ -358,6 +385,7 @@ int release_job(
   int    newsub;
   char  *pset;
   job   *pjob = (job *)j;
+  char   log_buf[LOCAL_LOG_BUF_SIZE];
 
   attribute      temphold;
 
@@ -377,34 +405,33 @@ int release_job(
 
   /* unset the hold */
 
-  old_hold = pjob->ji_wattr[(int)JOB_ATR_hold].at_val.at_long;
+  old_hold = pjob->ji_wattr[JOB_ATR_hold].at_val.at_long;
 
-  if ((rc = job_attr_def[(int)JOB_ATR_hold].at_set(&pjob->ji_wattr[(int)JOB_ATR_hold], &temphold, DECR)))
+  if ((rc = job_attr_def[JOB_ATR_hold].at_set(&pjob->ji_wattr[JOB_ATR_hold], &temphold, DECR)))
     {
     return(rc);
     }
 
   /* everything went well, if holds changed, update the job state */
 
-  if (old_hold != pjob->ji_wattr[(int)JOB_ATR_hold].at_val.at_long)
+  if (old_hold != pjob->ji_wattr[JOB_ATR_hold].at_val.at_long)
     {
     pjob->ji_modified = 1; /* indicates attributes changed */
 
     svr_evaljobstate(pjob, &newstate, &newsub, 0);
 
-    svr_setjobstate(pjob, newstate, newsub); /* saves job */
+    svr_setjobstate(pjob, newstate, newsub, FALSE); /* saves job */
     }
 
-  sprintf(log_buffer, msg_jobholdrel,
+  sprintf(log_buf, msg_jobholdrel,
     pset,
     preq->rq_user,
     preq->rq_host);
 
-  LOG_EVENT(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid,
-            log_buffer);
+  log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buf);
 
   return(rc);
-  }
+  } /* END release_job() */
 
 
 
@@ -416,19 +443,20 @@ int release_job(
  * As a result, the job might change state.
  */
 
-void req_releasejob(
+void *req_releasejob(
 
-  struct batch_request *preq) /* ptr to the decoded request   */
+  void *vp) /* ptr to the decoded request   */
 
   {
   job  *pjob;
   int   rc;
+  struct batch_request *preq = (struct batch_request *)vp; 
 
   pjob = chk_job_request(preq->rq_ind.rq_release.rq_objname, preq);
 
   if (pjob == NULL)
     {
-    return;
+    return(NULL);
     }
 
   if ((rc = release_job(preq,pjob)) != 0)
@@ -440,7 +468,9 @@ void req_releasejob(
     reply_ack(preq);
     }
 
-  return;
+  pthread_mutex_unlock(pjob->ji_mutex);
+
+  return(NULL);
   }  /* END req_releasejob() */
 
 
@@ -451,47 +481,86 @@ int release_whole_array(
   struct batch_request *preq) /* I */
 
   {
-  int i;
-  int rc;
+  int  i;
+  int  rc;
+  job *pjob;
 
   for (i = 0; i < pa->ai_qs.array_size; i++)
     {
-    if (pa->jobs[i] == NULL)
+    if (pa->job_ids[i] == NULL)
       continue;
 
-    if ((rc = release_job(preq,pa->jobs[i])) != 0)
+    if ((pjob = find_job(pa->job_ids[i])) == NULL)
       {
-      return(rc);
+      free(pa->job_ids[i]);
+      pa->job_ids[i] = NULL;
+      }
+    else
+      {
+      if ((rc = release_job(preq, pjob)) != 0)
+        {
+        pthread_mutex_unlock(pjob->ji_mutex);
+        return(rc);
+        }
+  
+      pthread_mutex_unlock(pjob->ji_mutex);
       }
     }
 
   /* SUCCESS */
-
-  return(0);
+  return(PBSE_NONE);
   } /* END release_whole_array */
 
 
 
-void req_releasearray(
+void *req_releasearray(
 
-  struct batch_request *preq)
+  void *vp) /* I */
 
   {
+  char      id[] = "req_releasearray";
+  char      log_buf[LOCAL_LOG_BUF_SIZE];
   job       *pjob;
   job_array *pa;
   char      *range;
   int        rc;
+  int        index;
+  struct batch_request *preq = (struct batch_request *)vp;
 
   pa = get_array(preq->rq_ind.rq_release.rq_objname);
-  pjob = (job *)pa->jobs[first_job_index(pa)];
+  if (pa == NULL)
+    {
+    req_reject(PBSE_IVALREQ,0,preq,NULL,"Cannot find array");
+    return(NULL);
+    }
+
+  while (TRUE)
+    {
+    index = first_job_index(pa);
+    if (pa->job_ids[index] == NULL)
+      return(NULL);
+
+    if ((pjob = find_job(pa->job_ids[index])) == NULL)
+      {
+      free(pa->job_ids[index]);
+      pa->job_ids[index] = NULL;
+      }
+    else
+      break;
+    }
 
   if (svr_authorize_jobreq(preq, pjob) == -1)
     {
     req_reject(PBSE_PERM,0,preq,NULL,NULL);
 
-    return;
+    pthread_mutex_unlock(pa->ai_mutex);
+    pthread_mutex_unlock(pjob->ji_mutex);
+
+    return(NULL);
     }
 
+  pthread_mutex_unlock(pjob->ji_mutex);
+
   range = preq->rq_extend;
   if ((range != NULL) &&
       (strstr(range,ARRAY_RANGE) != NULL))
@@ -499,20 +568,43 @@ void req_releasearray(
     /* parse the array range */
     if ((rc = release_array_range(pa,preq,range)) != 0)
       {
+      if (LOGLEVEL >= 7)
+        {
+        sprintf(log_buf, "%s: unlocking ai_mutex", id);
+        log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pa->ai_qs.parent_id, log_buf);
+        }
+      pthread_mutex_unlock(pa->ai_mutex);
+
       req_reject(rc,0,preq,NULL,NULL);
 
-      return;
+      return(NULL);
       }
     }
   else if ((rc = release_whole_array(pa,preq)) != 0)
     {
+    pthread_mutex_unlock(pa->ai_mutex);
+    if(LOGLEVEL >= 7)
+      {
+      sprintf(log_buf, "%s: unlocking ai_mutex", id);
+      log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buf);
+      }
+
     req_reject(rc,0,preq,NULL,NULL);
 
-    return;
+    return(NULL);
+    }
+
+  pthread_mutex_unlock(pa->ai_mutex);
+  if(LOGLEVEL >= 7)
+    {
+    sprintf(log_buf, "%s: unlocking ai_mutex", id);
+    log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buf);
     }
 
   reply_ack(preq);
-  }
+
+  return(NULL);
+  } /* END req_releasearray() */
 
 
 
@@ -530,8 +622,8 @@ int get_hold(
 
   tlist_head *phead,
   char      **pset,     /* O - ptr to hold value */
-  attribute *temphold   /* O - ptr to attribute to decode value into  */
-)
+  attribute *temphold)   /* O - ptr to attribute to decode value into  */
+
 
   {
   int   have_one = 0;
@@ -544,7 +636,7 @@ int get_hold(
 
   while (pal != NULL)
     {
-    if (!strcmp(pal->al_name, job_attr_def[(int)JOB_ATR_hold].at_name))
+    if (!strcmp(pal->al_name, job_attr_def[JOB_ATR_hold].at_name))
       {
       holdattr = pal;
 
@@ -567,13 +659,14 @@ int get_hold(
 
   /* decode into temporary attribute structure */
 
-  clear_attr(temphold, &job_attr_def[(int)JOB_ATR_hold]);
+  clear_attr(temphold, &job_attr_def[JOB_ATR_hold]);
 
-  return(job_attr_def[(int)JOB_ATR_hold].at_decode(
+  return(job_attr_def[JOB_ATR_hold].at_decode(
            temphold,
            holdattr->al_name,
            (char *)0,
-           holdattr->al_value));
+           holdattr->al_value,
+           0));
   }
 
 
@@ -590,15 +683,17 @@ int get_hold(
 static void process_hold_reply(
 
   struct work_task *pwt)
+
   {
-  job       *pjob;
+  job                  *pjob;
+  attribute             temphold;
 
   struct batch_request *preq;
-  int   newstate;
-  int   newsub;
-  attribute temphold;
-  char *pset;
-  int rc;
+  int                   newstate;
+  int                   newsub;
+  int                   rc;
+  char                 *pset;
+  char                  log_buf[LOCAL_LOG_BUF_SIZE];
 
   svr_disconnect(pwt->wt_event); /* close connection to MOM */
 
@@ -607,10 +702,15 @@ static void process_hold_reply(
 
   if ((pjob = find_job(preq->rq_ind.rq_hold.rq_orig.rq_objname)) == (job *)0)
     {
-    LOG_EVENT(PBSEVENT_DEBUG, PBS_EVENTCLASS_JOB,
+    log_event(PBSEVENT_DEBUG, PBS_EVENTCLASS_JOB,
               preq->rq_ind.rq_hold.rq_orig.rq_objname,
               msg_postmomnojob);
     req_reject(PBSE_UNKJOBID, 0, preq, NULL, msg_postmomnojob);
+
+    free(pwt->wt_mutex);
+    free(pwt);
+
+    return;
     }
   else if (preq->rq_reply.brp_code != 0)
     {
@@ -619,7 +719,7 @@ static void process_hold_reply(
 
     if (rc == 0)
       {
-      rc = job_attr_def[(int)JOB_ATR_hold].at_set(&pjob->ji_wattr[(int)JOB_ATR_hold],
+      rc = job_attr_def[JOB_ATR_hold].at_set(&pjob->ji_wattr[JOB_ATR_hold],
            &temphold, DECR);
       }
 
@@ -627,14 +727,13 @@ static void process_hold_reply(
 
     pjob->ji_modified = 1;    /* indicate attributes changed */
     svr_evaljobstate(pjob, &newstate, &newsub, 0);
-    svr_setjobstate(pjob, newstate, newsub); /* saves job */
+    svr_setjobstate(pjob, newstate, newsub, FALSE); /* saves job */
 
     if (preq->rq_reply.brp_code != PBSE_NOSUP)
       {
-      sprintf(log_buffer, msg_mombadhold, preq->rq_reply.brp_code);
-      LOG_EVENT(PBSEVENT_DEBUG, PBS_EVENTCLASS_JOB,
-                pjob->ji_qs.ji_jobid, log_buffer);
-      req_reject(preq->rq_reply.brp_code, 0, preq, NULL, log_buffer);
+      sprintf(log_buf, msg_mombadhold, preq->rq_reply.brp_code);
+      log_event(PBSEVENT_DEBUG, PBS_EVENTCLASS_JOB, pjob->ji_qs.ji_jobid, log_buf);
+      req_reject(preq->rq_reply.brp_code, 0, preq, NULL, log_buf);
       }
     else
       {
@@ -661,12 +760,17 @@ static void process_hold_reply(
     pjob->ji_modified = 1;    /* indicate attributes changed     */
 
     svr_evaljobstate(pjob, &newstate, &newsub, 0);
-    svr_setjobstate(pjob, newstate, newsub); /* saves job */
+    svr_setjobstate(pjob, newstate, newsub, FALSE); /* saves job */
 
     account_record(PBS_ACCT_CHKPNT, pjob, "Checkpointed and held"); /* note in accounting file */
     reply_ack(preq);
     }
-  }
+
+  pthread_mutex_unlock(pjob->ji_mutex);
+
+  free(pwt->wt_mutex);
+  free(pwt);
+  } /* END process_hold_reply() */
 
 /*
  * process_checkpoint_reply
@@ -677,6 +781,7 @@ static void process_hold_reply(
 static void process_checkpoint_reply(
 
   struct work_task *pwt)
+
   {
   job       *pjob;
 
@@ -689,7 +794,7 @@ static void process_checkpoint_reply(
 
   if ((pjob = find_job(preq->rq_ind.rq_manager.rq_objname)) == (job *)0)
     {
-    LOG_EVENT(PBSEVENT_DEBUG, PBS_EVENTCLASS_JOB,
+    log_event(PBSEVENT_DEBUG, PBS_EVENTCLASS_JOB,
               preq->rq_ind.rq_manager.rq_objname,
               msg_postmomnojob);
     req_reject(PBSE_UNKJOBID, 0, preq, NULL, msg_postmomnojob);
@@ -700,5 +805,11 @@ static void process_checkpoint_reply(
 
     account_record(PBS_ACCT_CHKPNT, pjob, "Checkpointed"); /* note in accounting file */
     reply_ack(preq);
+
+    pthread_mutex_unlock(pjob->ji_mutex);
     }
-  }
+
+  free(pwt->wt_mutex);
+  free(pwt);
+  } /* END process_checkpoint_reply() */
+
