@@ -100,6 +100,7 @@
  * disallowed_types_chk() - at action for queue disallowed_types attribute
  */
 #include <pbs_config.h>   /* the master config generated by configure */
+#include "req_manager.h"
 
 #include <sys/types.h>
 #include "libpbs.h"
@@ -109,6 +110,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <unistd.h>
+#include <pthread.h>
 #include "server_limits.h"
 #include "list_link.h"
 #include "attribute.h"
@@ -116,15 +118,24 @@
 #include "server.h"
 #include "queue.h"
 #include "credential.h"
+#include "utils.h"
 #include "batch_request.h"
 #include "net_connect.h"
 #include "pbs_error.h"
 #include "log.h"
+#include "../lib/Liblog/pbs_log.h"
+#include "../lib/Liblog/log_event.h"
 #include "svrfunc.h"
 #include "pbs_job.h"
 #include "pbs_nodes.h"
 #include "work_task.h"
 #include "mcom.h"
+#include "../lib/Libattr/attr_node_func.h" /* free_prop_list */
+#include "node_func.h" /* init_prop, find_nodebyname, reinitialize_node_iterator, recompute_ntype_cnts, effective_node_delete, create_pbs_node */
+#include "node_manager.h" /* setup_notification */
+#include "../lib/Libutils/u_lock_ctl.h" /* unlock_node */
+#include "queue_func.h" /* find_queuebyname, que_alloc, que_free */
+#include "queue_recov.h" /* que_save */
 
 
 #define PERM_MANAGER (ATR_DFLAG_MGWR | ATR_DFLAG_MGRD)
@@ -132,8 +143,9 @@
 
 /* Global Data Items: */
 
+extern int LOGLEVEL;
 extern struct server server;
-extern tlist_head     svr_queues;
+extern all_queues     svr_queues;
 extern attribute_def que_attr_def[];
 extern attribute_def svr_attr_def[];
 extern attribute_def  node_attr_def[];   /* node attributes defs */
@@ -148,8 +160,8 @@ extern char *msg_man_uns;
 
 
 extern int que_purge(pbs_queue *);
-extern void save_characteristic(struct pbsnode *);
-extern int chk_characteristic(struct pbsnode *, int *);
+extern void save_characteristic(struct pbsnode *, node_check_info *);
+extern int chk_characteristic(struct pbsnode *, node_check_info *, int *);
 extern int hasprop(struct pbsnode *, struct prop *);
 extern int PNodeStateToString(int, char *, int);
 
@@ -158,7 +170,7 @@ extern int PNodeStateToString(int, char *, int);
 /* private data */
 
 static char *all_quename = "_All_";
-static char *all_nodes = "_All_";
+static char *all_nodes_str = "_All_";
 
 /*
  * check_que_attr - check if attributes in request are consistant with
@@ -203,6 +215,9 @@ static char *check_que_attr(
   return ((char *)0); /* all attributes are ok */
   } /* END check_que_attr() */
 
+
+
+
 /*
  * check_que_enable - check if attempt to enable incompletely defined queue
  * This is the at_action() routine for QA_ATR_Enabled
@@ -299,13 +314,13 @@ int set_queue_type(
     if (spectype != -1)   /* set up the attribute */
       {
       ((pbs_queue *)pque)->qu_qs.qu_type = spectype;
-      (void)free(pattr->at_val.at_str);
-      pattr->at_val.at_str = malloc(strlen(qt[i].name) + 1);
+      free(pattr->at_val.at_str);
+      pattr->at_val.at_str = calloc(1, strlen(qt[i].name) + 1);
 
       if (pattr->at_val.at_str == (char *)0)
         return (PBSE_SYSTEM);
 
-      (void)strcpy(pattr->at_val.at_str, qt[i].name);
+      strcpy(pattr->at_val.at_str, qt[i].name);
 
       return(PBSE_NONE);
       }
@@ -330,16 +345,16 @@ static void mgr_log_attr(
 
   {
   char *pstr;
+  char  log_buf[LOCAL_LOG_BUF_SIZE];
 
   while (plist)
     {
-    (void)strcpy(log_buffer, msg);
-    (void)strcat(log_buffer, plist->al_name);
+    snprintf(log_buf, sizeof(log_buf), "%s%s", msg, plist->al_name);
 
     if (plist->al_rescln)
       {
-      (void)strcat(log_buffer, ".");
-      (void)strcat(log_buffer, plist->al_resc);
+      (void)strcat(log_buf, ".");
+      (void)strcat(log_buf, plist->al_resc);
       }
 
     if (plist->al_op == INCR)
@@ -349,15 +364,14 @@ static void mgr_log_attr(
     else
       pstr = " = ";
 
-    (void)strcat(log_buffer, pstr);
+    (void)strcat(log_buf, pstr);
 
     if (plist->al_valln)
-      (void)strncat(log_buffer, plist->al_value,
-                    LOG_BUF_SIZE - strlen(log_buffer) - 1);
+      safe_strncat(log_buf, plist->al_value, sizeof(log_buf) - strlen(log_buf) - 1);
 
-    log_buffer[LOG_BUF_SIZE-1] = '\0';
+    log_buf[LOCAL_LOG_BUF_SIZE-1] = '\0';
 
-    log_event(PBSEVENT_ADMIN, logclass, objname, log_buffer);
+    log_event(PBSEVENT_ADMIN, logclass, objname, log_buf);
 
     plist = (struct svrattrl *)GET_NEXT(plist->al_link);
     }
@@ -697,6 +711,7 @@ int mgr_set_node_attr(
   struct prop     *pdest;
 
   struct prop    **plink;
+  char             log_buf[LOCAL_LOG_BUF_SIZE];
 
   if (plist == NULL)
     {
@@ -779,7 +794,7 @@ int mgr_set_node_attr(
        * call the specified "at_action" routine for that attribute passing it
        * the address of the temporary attribute and the address of the temporary
        * pbsnode object.  If any of the update actions fail along the way, undo
-       * everything.   Any malloc'd substructures should be hanging off the
+       * everything.   Any calloc'd substructures should be hanging off the
        * appropriate attribute of the attribute array "new".
        */
 
@@ -797,7 +812,7 @@ int mgr_set_node_attr(
 
   /*
    * we have successfully updated tnode; use it to modify pnode
-   * if pnode has any malloc-ed storage that is being replaced
+   * if pnode has any calloc-ed storage that is being replaced
    * be sure to free the old.
    */
 
@@ -825,16 +840,12 @@ int mgr_set_node_attr(
     PNodeStateToString(pnode->nd_state, OrigState, sizeof(OrigState));
     PNodeStateToString(tnode.nd_state, FinalState, sizeof(FinalState));
 
-    sprintf(log_buffer, "node %s state changed from %s to %s",
-            pnode->nd_name,
-            OrigState,
-            FinalState);
-
-    log_event(
-      PBSEVENT_ADMIN,
-      PBS_EVENTCLASS_NODE,
+    sprintf(log_buf, "node %s state changed from %s to %s",
       pnode->nd_name,
-      log_buffer);
+      OrigState,
+      FinalState);
+
+    log_event(PBSEVENT_ADMIN,PBS_EVENTCLASS_NODE,pnode->nd_name,log_buf);
     }
 
   /* NOTE:  nd_status properly freed during attribute alter */
@@ -944,11 +955,12 @@ void mgr_queue_create(
   struct batch_request *preq)
 
   {
-  int   bad;
-  char  *badattr;
-  svrattrl *plist;
+  int        bad;
+  char      *badattr;
+  svrattrl  *plist;
   pbs_queue *pque;
-  int   rc;
+  int        rc;
+  char       log_buf[LOCAL_LOG_BUF_SIZE];
 
   rc = strlen(preq->rq_ind.rq_manager.rq_objname);
 
@@ -959,17 +971,16 @@ void mgr_queue_create(
     return;
     }
 
-  if (find_queuebyname(preq->rq_ind.rq_manager.rq_objname))
+  if ((pque = find_queuebyname(preq->rq_ind.rq_manager.rq_objname)))
     {
+    unlock_queue(pque, "mgr_queue_create", "fail", LOGLEVEL);
     req_reject(PBSE_QUEEXIST, 0, preq, NULL, NULL);
-
     return;
     }
 
-  pque = que_alloc(preq->rq_ind.rq_manager.rq_objname);
+  pque = que_alloc(preq->rq_ind.rq_manager.rq_objname, FALSE);
 
   /* set the queue attributes */
-
   plist = (svrattrl *)GET_NEXT(preq->rq_ind.rq_manager.rq_attr);
 
   rc = mgr_set_attr(
@@ -986,23 +997,19 @@ void mgr_queue_create(
     {
     reply_badattr(rc, bad, plist, preq);
 
-    que_free(pque);
+    que_free(pque, FALSE);
     }
   else
     {
     que_save(pque);
     svr_save(&server, SVR_SAVE_QUICK);
 
-    sprintf(log_buffer, msg_manager,
+    sprintf(log_buf, msg_manager,
             msg_man_cre,
             preq->rq_user,
             preq->rq_host);
 
-    log_event(
-      PBSEVENT_ADMIN,
-      PBS_EVENTCLASS_QUEUE,
-      pque->qu_qs.qu_name,
-      log_buffer);
+    log_event(PBSEVENT_ADMIN,PBS_EVENTCLASS_QUEUE,pque->qu_qs.qu_name,log_buf);
 
     mgr_log_attr(
       msg_man_set,
@@ -1011,21 +1018,19 @@ void mgr_queue_create(
       preq->rq_ind.rq_manager.rq_objname);
 
     /* check the appropriateness of the attributes vs. queue type */
-
     if ((badattr = check_que_attr(pque)) != NULL)
       {
       /* mismatch, issue warning */
+      sprintf(log_buf, msg_attrtype, pque->qu_qs.qu_name, badattr);
 
-      sprintf(log_buffer, msg_attrtype,
-              pque->qu_qs.qu_name,
-              badattr);
-
-      reply_text(preq, PBSE_ATTRTYPE, log_buffer);
+      reply_text(preq, PBSE_ATTRTYPE, log_buf);
       }
     else
       {
       reply_ack(preq);
       }
+
+    unlock_queue(pque, "mgr_queue_create", "success", LOGLEVEL);
     }
 
   return;
@@ -1048,40 +1053,36 @@ void mgr_queue_delete(
 
   {
   pbs_queue *pque;
-  int    rc;
+  int        rc;
+  char       log_buf[LOCAL_LOG_BUF_SIZE];
 
   pque = find_queuebyname(preq->rq_ind.rq_manager.rq_objname);
 
   if (pque == NULL)
     {
     /* FAILURE */
-
     req_reject(PBSE_UNKQUE, 0, preq, NULL, NULL);
 
     return;
     }
 
-  if ((rc = que_purge(pque)) != 0)
+  if ((rc = que_purge(pque)) != PBSE_NONE)
     {
     /* FAILURE */
-
+    unlock_queue(pque, __func__, "", LOGLEVEL);
     req_reject(rc, 0, preq, NULL, NULL);
-
     return;
     }
 
   svr_save(&server, SVR_SAVE_QUICK);
 
-  sprintf(log_buffer, msg_manager,
-          msg_man_del,
-          preq->rq_user,
-          preq->rq_host);
+  sprintf(log_buf, msg_manager, msg_man_del, preq->rq_user, preq->rq_host);
 
   log_event(
     PBSEVENT_ADMIN,
     PBS_EVENTCLASS_QUEUE,
     preq->rq_ind.rq_manager.rq_objname,
-    log_buffer);
+    log_buf);
 
   reply_ack(preq);
 
@@ -1098,9 +1099,9 @@ int hostname_check(
   char *hostname)
 
   {
-  char  myhost[PBS_MAXHOSTNAME];
-  char  extension[PBS_MAXHOSTNAME];
-  char  ret_hostname[PBS_MAXHOSTNAME];
+  char  myhost[PBS_MAXHOSTNAME + 1];
+  char  extension[PBS_MAXHOSTNAME + 1];
+  char  ret_hostname[PBS_MAXHOSTNAME + 1];
   char *open_bracket;
 
   if (hostname == NULL)
@@ -1117,7 +1118,7 @@ int hostname_check(
     {
     /* handle normally */
     if ((get_fullhostname(hostname,ret_hostname,PBS_MAXHOSTNAME,NULL)) ||
-        strncmp(hostname, ret_hostname, PBS_MAXHOSTNAME))
+        strncasecmp(hostname, ret_hostname, PBS_MAXHOSTNAME))
       {
       return(FAILURE);
       }
@@ -1188,11 +1189,12 @@ void mgr_server_set(
   {
   int       bad_attr = 0;
   svrattrl *plist;
-  int     rc;
-
+  int       rc;
+  char      log_buf[LOCAL_LOG_BUF_SIZE];
 
   plist = (svrattrl *)GET_NEXT(preq->rq_ind.rq_manager.rq_attr);
 
+  pthread_mutex_lock(server.sv_attr_mutex);
   rc = mgr_set_attr(
          server.sv_attr,
          svr_attr_def,
@@ -1202,6 +1204,7 @@ void mgr_server_set(
          &bad_attr,
          (void *) & server,
          ATR_ACTION_ALTER);
+  pthread_mutex_unlock(server.sv_attr_mutex);
 
   /* PBSE_BADACLHOST - lets show the user the first bad host in the ACL  */
 
@@ -1217,7 +1220,7 @@ void mgr_server_set(
     int       bhstrlen;
 
     bhstrlen = PBS_MAXHOSTNAME + 17;
-    bad_host = malloc(sizeof(char) * (bhstrlen + 1));
+    bad_host = calloc(sizeof(char), (bhstrlen + 1));
 
     if (bad_host == NULL)
       {
@@ -1242,7 +1245,7 @@ void mgr_server_set(
       index = find_attr(svr_attr_def, plist->al_name, SRV_ATR_LAST);
 
       clear_attr(&temp, &svr_attr_def[index]);
-      svr_attr_def[index].at_decode(&temp, plist->al_name, plist->al_resc, plist->al_value);
+      svr_attr_def[index].at_decode(&temp, plist->al_name, plist->al_resc, plist->al_value, 0);
 
       pstr = temp.at_val.at_arst;
 
@@ -1270,6 +1273,7 @@ void mgr_server_set(
       /* nothing wrong found in the request, let's try again
          with the server's list */
 
+      pthread_mutex_lock(server.sv_attr_mutex);
       pstr = server.sv_attr[index].at_val.at_arst;
 
       for (i = 0; i < pstr->as_usedptr; ++i)
@@ -1284,6 +1288,7 @@ void mgr_server_set(
           break;
           }
         }
+      pthread_mutex_unlock(server.sv_attr_mutex);
 
       if (bad_host[0] != '\0')
         break;
@@ -1314,15 +1319,13 @@ void mgr_server_set(
 
   svr_save(&server, SVR_SAVE_FULL);
 
-  sprintf(log_buffer, msg_manager, msg_man_set,
-          preq->rq_user,
-          preq->rq_host);
+  sprintf(log_buf, msg_manager, msg_man_set, preq->rq_user, preq->rq_host);
 
   log_event(
     PBSEVENT_ADMIN,
     PBS_EVENTCLASS_SERVER,
     msg_daemonname,
-    log_buffer);
+    log_buf);
 
   mgr_log_attr(msg_man_set, plist, PBS_EVENTCLASS_SERVER, msg_daemonname);
 
@@ -1347,12 +1350,14 @@ void mgr_server_unset(
   struct batch_request *preq)
 
   {
-  int   bad_attr = 0;
+  int       bad_attr = 0;
   svrattrl *plist;
-  int   rc;
+  int       rc;
+  char      log_buf[LOCAL_LOG_BUF_SIZE];
 
   plist = (svrattrl *)GET_NEXT(preq->rq_ind.rq_manager.rq_attr);
 
+  pthread_mutex_lock(server.sv_attr_mutex);
   rc = mgr_unset_attr(
          server.sv_attr,
          svr_attr_def,
@@ -1360,6 +1365,7 @@ void mgr_server_unset(
          plist,
          preq->rq_perm,
          &bad_attr);
+  pthread_mutex_unlock(server.sv_attr_mutex);
 
   if (rc != 0)
     {
@@ -1369,16 +1375,9 @@ void mgr_server_unset(
     {
     svr_save(&server, SVR_SAVE_FULL);
 
-    sprintf(log_buffer, msg_manager,
-            msg_man_uns,
-            preq->rq_user,
-            preq->rq_host);
+    sprintf(log_buf, msg_manager, msg_man_uns, preq->rq_user, preq->rq_host);
 
-    log_event(
-      PBSEVENT_ADMIN,
-      PBS_EVENTCLASS_SERVER,
-      msg_daemonname,
-      log_buffer);
+    log_event(PBSEVENT_ADMIN,PBS_EVENTCLASS_SERVER,msg_daemonname,log_buf);
 
     mgr_log_attr(msg_man_uns, plist, PBS_EVENTCLASS_SERVER, msg_daemonname);
 
@@ -1403,21 +1402,22 @@ void mgr_queue_set(
   struct batch_request *preq)
 
   {
-  int    allques;
-  int    bad = 0;
-  char   *badattr;
+  int        allques;
+  int        bad = 0;
+  char      *badattr;
   svrattrl  *plist;
   pbs_queue *pque;
   char      *qname;
-  int    rc;
+  int        rc;
+  int        iter = -1;
+  char       log_buf[LOCAL_LOG_BUF_SIZE];
 
   if ((*preq->rq_ind.rq_manager.rq_objname == '\0') ||
       (*preq->rq_ind.rq_manager.rq_objname == '@'))
     {
     qname   = all_quename;
     allques = TRUE;
-
-    pque = (pbs_queue *)GET_NEXT(svr_queues);
+    pque = next_queue(&svr_queues,&iter);
     }
   else
     {
@@ -1435,16 +1435,9 @@ void mgr_queue_set(
     }
 
   /* set the attributes */
+  sprintf(log_buf, msg_manager, msg_man_set, preq->rq_user, preq->rq_host);
 
-  sprintf(log_buffer, msg_manager,
-          msg_man_set,
-          preq->rq_user,
-          preq->rq_host);
-
-  log_event(PBSEVENT_ADMIN, PBS_EVENTCLASS_QUEUE, qname, log_buffer);
-
-  if (allques == TRUE)
-    pque = (pbs_queue *)GET_NEXT(svr_queues);
+  log_event(PBSEVENT_ADMIN, PBS_EVENTCLASS_QUEUE, qname, log_buf);
 
   plist = (svrattrl *)GET_NEXT(preq->rq_ind.rq_manager.rq_attr);
 
@@ -1460,11 +1453,10 @@ void mgr_queue_set(
            (void *)pque,
            ATR_ACTION_ALTER);
 
-
     if (rc != 0)
       {
+      unlock_queue(pque, "mgr_queue_set", "fail badattr", LOGLEVEL);
       reply_badattr(rc, bad, plist, preq);
-
       return;
       }
 
@@ -1474,32 +1466,34 @@ void mgr_queue_set(
 
     if (allques == FALSE)
       break;
+    
+    unlock_queue(pque, "mgr_queue_set", "before next_queue call", LOGLEVEL);
 
-    pque = (pbs_queue *)GET_NEXT(pque->qu_link);
+    pque = next_queue(&svr_queues,&iter);
     }  /* END while (pque != NULL) */
 
   /* check the appropriateness of the attributes based on queue type */
+  iter = -1;
 
   if (allques == TRUE)
-    pque = (pbs_queue *)GET_NEXT(svr_queues);
+    pque = next_queue(&svr_queues,&iter);
 
   while (pque != NULL)
     {
     if ((badattr = check_que_attr(pque)) != NULL)
       {
-      sprintf(log_buffer, msg_attrtype,
-              pque->qu_qs.qu_name,
-              badattr);
-
-      reply_text(preq, PBSE_ATTRTYPE, log_buffer);
-
+      sprintf(log_buf, msg_attrtype, pque->qu_qs.qu_name, badattr);
+      unlock_queue(pque, "mgr_queue_set", "attrtype", LOGLEVEL);
+      reply_text(preq, PBSE_ATTRTYPE, log_buf);
       return;
       }
 
+    unlock_queue(pque, "mgr_queue_set", "last loop before next_queue", LOGLEVEL);
+
     if (allques == FALSE)
       break;
 
-    pque = (pbs_queue *)GET_NEXT(pque->qu_link);
+    pque = next_queue(&svr_queues,&iter);
     }  /* END while (pque != NULL) */
 
   reply_ack(preq);
@@ -1528,14 +1522,15 @@ void mgr_queue_unset(
   pbs_queue *pque;
   char      *qname;
   int    rc;
+  int    iter = -1;
+  char   log_buf[LOCAL_LOG_BUF_SIZE];
 
   if ((*preq->rq_ind.rq_manager.rq_objname == '\0') ||
       (*preq->rq_ind.rq_manager.rq_objname == '@'))
     {
     qname   = all_quename;
     allques = TRUE;
-
-    pque = (pbs_queue *)GET_NEXT(svr_queues);
+    pque = next_queue(&svr_queues,&iter);
     }
   else
     {
@@ -1552,34 +1547,19 @@ void mgr_queue_unset(
     return;
     }
 
-  sprintf(log_buffer, msg_manager,
-
-          msg_man_uns,
-          preq->rq_user,
-          preq->rq_host);
-
-  log_event(
-    PBSEVENT_ADMIN,
-    PBS_EVENTCLASS_QUEUE,
-    qname,
-    log_buffer);
+  sprintf(log_buf, msg_manager, msg_man_uns, preq->rq_user, preq->rq_host);
+  log_event(PBSEVENT_ADMIN,PBS_EVENTCLASS_QUEUE,qname,log_buf);
 
   plist = (svrattrl *)GET_NEXT(preq->rq_ind.rq_manager.rq_attr);
 
   while (pque != NULL)
     {
-    rc = mgr_unset_attr(
-           pque->qu_attr,
-           que_attr_def,
-           QA_ATR_LAST,
-           plist,
-           preq->rq_perm,
-           &bad_attr);
+    rc = mgr_unset_attr(pque->qu_attr, que_attr_def, QA_ATR_LAST, plist, preq->rq_perm, &bad_attr);
 
     if (rc != 0)
       {
+      unlock_queue(pque, "mgr_queue_unset", "badattr", LOGLEVEL);
       reply_badattr(rc, bad_attr, plist, preq);
-
       return;
       }
 
@@ -1590,16 +1570,17 @@ void mgr_queue_unset(
     if ((pque->qu_attr[QA_ATR_QType].at_flags & ATR_VFLAG_SET) == 0)
       pque->qu_qs.qu_type = QTYPE_Unset;
 
+    unlock_queue(pque, "mgr_queue_unset", "success", LOGLEVEL);
     if (allques == FALSE)
       break;
-
-    pque = GET_NEXT(pque->qu_link);
+    
+    pque = next_queue(&svr_queues,&iter);
     }
 
   reply_ack(preq);
 
   return;
-  }
+  } /* END mgr_queue_unset() */
 
 
 
@@ -1617,25 +1598,26 @@ void mgr_node_set(
   struct batch_request *preq)  /* I */
 
   {
-  static int need_todo = 0;
+  int               need_todo = 0;
 
-  int  allnodes = 0;
-  int  propnodes = 0;
-  int  bad = 0;
-  svrattrl *plist;
+  int               check_all = 0;
+  int               propnodes = 0;
+  int               rc;
+  int               bad = 0;
+  int               i;
+  int               len;
+  int               problem_cnt = 0;
 
-  struct pbsnode  *pnode = NULL;
-  char  *nodename = NULL;
-  int  rc;
+  char             *nodename = NULL;
+  char             *problem_names;
+  char              log_buf[LOCAL_LOG_BUF_SIZE];
 
-  int  i, len;
-  int  problem_cnt = 0;
-  char  *problem_names;
-
-  node_iterator iter;
+  svrattrl         *plist;
+  node_check_info   nci;
+  node_iterator     iter;
 
+  struct pbsnode   *pnode = NULL;
   struct pbsnode  **problem_nodes = NULL;
-
   struct prop props;
 
   if ((*preq->rq_ind.rq_manager.rq_objname == '\0') ||
@@ -1646,9 +1628,9 @@ void mgr_node_set(
     /*In this instance the set node req is to apply to all */
     /*nodes at the local ('\0')  or specified ('@') server */
 
-    if ((pbsndlist != NULL) && svr_totnodes)
+    if (svr_totnodes)
       {
-      pnode = *pbsndlist;
+      pnode = (struct pbsnode *)1;
 
       if ((*preq->rq_ind.rq_manager.rq_objname == ':') &&
           (strcmp(preq->rq_ind.rq_manager.rq_objname + 1, "ALL") != 0))
@@ -1661,16 +1643,10 @@ void mgr_node_set(
         }
       else
         {
-        allnodes = 1;
-        nodename = all_nodes;
+        check_all = 1;
+        nodename = all_nodes_str;
         }
       }
-    else
-      {
-      /* specified server has no nodes in its node table */
-
-      pnode = NULL;
-      }
     }
   else
     {
@@ -1685,33 +1661,33 @@ void mgr_node_set(
     return;
     }
 
-  /*set "state", "properties", or type of node (nodes if allnodes == 1)*/
+  /*set "state", "properties", or type of node (nodes if check_all == 1)*/
 
-  sprintf(log_buffer, msg_manager,
-          msg_man_set,
-          preq->rq_user,
-          preq->rq_host);
+  sprintf(log_buf, msg_manager, msg_man_set, preq->rq_user, preq->rq_host);
 
-  log_event(PBSEVENT_ADMIN, PBS_EVENTCLASS_NODE, nodename, log_buffer);
+  log_event(PBSEVENT_ADMIN, PBS_EVENTCLASS_NODE, nodename, log_buf);
 
   plist = (svrattrl *)GET_NEXT(preq->rq_ind.rq_manager.rq_attr);
 
-  if (allnodes || propnodes)
+  if (check_all || propnodes)
     {
     /* handle scrolling over all nodes */
-
-    problem_nodes = (struct pbsnode **)malloc(svr_totnodes * sizeof(struct pbsnode *));
+    problem_nodes = (struct pbsnode **)calloc(svr_totnodes, sizeof(struct pbsnode *));
 
     problem_cnt = 0;
   
     reinitialize_node_iterator(&iter);
+    pnode = NULL;
 
-    while ((pnode = next_node(&iter)) != NULL)
+    while ((pnode = next_node(&allnodes,pnode,&iter)) != NULL)
       {
-      if (propnodes && !hasprop(pnode, &props))
+      if ((propnodes == TRUE) && 
+          (!hasprop(pnode, &props)))
+        {
         continue;
+        }
 
-      save_characteristic(pnode);
+      save_characteristic(pnode,&nci);
 
       rc = mgr_set_node_attr(
              pnode,
@@ -1725,31 +1701,25 @@ void mgr_node_set(
 
       if (rc != 0)
         {
-        if (problem_nodes)     /*we have an array in which to save*/
-          problem_nodes[ problem_cnt ] = pnode;
+        if (problem_nodes) /* we have an array in which to save*/
+          problem_nodes[problem_cnt] = pnode;
         
         ++problem_cnt;
         }
       else
         {
         /* modifications succeeded for this node */
+        chk_characteristic(pnode, &nci, &need_todo);
 
-        chk_characteristic(pnode, &need_todo);
-
-        mgr_log_attr(
-          msg_man_set,
-          plist,
-          PBS_EVENTCLASS_NODE,
-          pnode->nd_name);
+        mgr_log_attr(msg_man_set, plist, PBS_EVENTCLASS_NODE, pnode->nd_name);
         }
-
       }  /* END for each node */
 
     } /* END multiple node case */
   else
     {
     /* handle single node case */
-    save_characteristic(pnode);
+    save_characteristic(pnode,&nci);
 
     rc = mgr_set_node_attr(
            pnode,
@@ -1763,7 +1733,7 @@ void mgr_node_set(
 
     if (rc != 0)
       {
-      /*In the specific node case, reply w/ error and return*/
+      /* In the specific node case, reply w/ error and return*/
       
       switch (rc)
         {
@@ -1794,26 +1764,24 @@ void mgr_node_set(
           break;
         }
 
+      unlock_node(pnode, "mgr_node_set", "error", LOGLEVEL);
+      
       return;
       } /* END if (rc != 0) */ 
     else
       {
       /* modifications succeeded for this node */
+      chk_characteristic(pnode, &nci, &need_todo);
       
-      chk_characteristic(pnode, &need_todo);
-      
-      mgr_log_attr(
-        msg_man_set,
-        plist,
-        PBS_EVENTCLASS_NODE,
-        pnode->nd_name);
+      mgr_log_attr(msg_man_set, plist, PBS_EVENTCLASS_NODE, pnode->nd_name);
       }
+
+    unlock_node(pnode, "mgr_node_set", "single_node", LOGLEVEL);
     } /* END single node case */
 
   if (need_todo & WRITENODE_STATE)
     {
     /*some nodes set to "offline"*/
-
     write_node_state();
 
     need_todo &= ~(WRITENODE_STATE);
@@ -1822,7 +1790,6 @@ void mgr_node_set(
   if (need_todo & WRITENODE_NOTE)
     {
     /*some nodes have new "note"s*/
-
     write_node_note();
 
     need_todo &= ~(WRITENODE_NOTE);
@@ -1831,16 +1798,13 @@ void mgr_node_set(
   if (need_todo & WRITE_NEW_NODESFILE)
     {
     /*create/delete/prop/ntype change*/
-
-    if (!update_nodes_file())
-
+    if (!update_nodes_file(NULL))
       need_todo &= ~(WRITE_NEW_NODESFILE);  /*successful on update*/
     }
 
-  if (allnodes || propnodes)
+  if (check_all || propnodes)
     {
     /* modification was for all nodes */
-
     if (problem_cnt)
       {
       /* one or more problems encountered */
@@ -1850,7 +1814,7 @@ void mgr_node_set(
 
       len += strlen(pbse_to_txt(PBSE_GMODERR));
 
-      if ((problem_names = malloc(len)))
+      if ((problem_names = calloc(1, len)))
         {
         strcpy(problem_names, pbse_to_txt(PBSE_GMODERR));
 
@@ -1883,7 +1847,7 @@ void mgr_node_set(
 
       return;
       }
-    }    /* END if (allnodes || propnodes) */
+    }    /* END if (check_all || propnodes) */
 
   recompute_ntype_cnts();
 
@@ -1910,22 +1874,16 @@ static void mgr_node_delete(
   struct batch_request *preq)
 
   {
-  static int need_todo = 0;
-
-  int  allnodes = 0;
+  int             check_all = 0;
+  int             iter;
 
   struct pbsnode *pnode;
-  char  *nodename = NULL;
-  int  rc;
+  char           *nodename = NULL;
 
-  int  i, len;
-  int  problem_cnt = 0;
-  char  *problem_names;
 
-  struct pbsnode  **problem_nodes = NULL;
-  svrattrl *plist;
+  svrattrl       *plist;
 
-  node_iterator iter;
+  char            log_buf[LOCAL_LOG_BUF_SIZE];
 
   if ((*preq->rq_ind.rq_manager.rq_objname == '\0') ||
       (*preq->rq_ind.rq_manager.rq_objname == '@'))
@@ -1933,16 +1891,15 @@ static void mgr_node_delete(
     /* In this instance the set node req is to apply to all */
     /* nodes at the local ('\0')  or specified ('@') server */
 
-    if ((pbsndlist != NULL) && svr_totnodes)
+    if (svr_totnodes)
       {
-      nodename = all_nodes;
-      pnode = *pbsndlist;
-      allnodes = 1;
+      nodename = all_nodes_str;
+      pnode = (struct pbsnode *)1;
+      check_all = 1;
       }
     else
       {
       /* specified server has no nodes in its node table */
-
       pnode = NULL;
       }
     }
@@ -1960,170 +1917,48 @@ static void mgr_node_delete(
     return;
     }
 
-  sprintf(log_buffer, msg_manager,
+  sprintf(log_buf, msg_manager, msg_man_del, preq->rq_user, preq->rq_host);
 
-          msg_man_del,
-          preq->rq_user,
-          preq->rq_host);
-
-  log_event(PBSEVENT_ADMIN, PBS_EVENTCLASS_NODE, nodename, log_buffer);
+  log_event(PBSEVENT_ADMIN, PBS_EVENTCLASS_NODE, nodename, log_buf);
 
   plist = (svrattrl *)GET_NEXT(preq->rq_ind.rq_manager.rq_attr);
 
   /*if doing many and problem arises with some, record them for report*/
   /*the array of "problem nodes" sees no use now and may never see use*/
 
-  if (allnodes)
+  if (check_all)
     {
     /* handle all nodes */
-    problem_nodes = (struct pbsnode **)malloc(
-                      svr_totnodes * sizeof(struct pbsnode *));
+    iter = -1;
 
-    problem_cnt = 0;
-
-    reinitialize_node_iterator(&iter);
-
-    while ((pnode = next_node(&iter)) != NULL)
+    while ((pnode = next_host(&allnodes,&iter,NULL)) != NULL)
       {
-      save_characteristic(pnode);
-
-      nodename = strdup(pnode->nd_name);
+      snprintf(log_buf,sizeof(log_buf),"%s",pnode->nd_name);
 
       effective_node_delete(pnode);
 
-      rc = 0;   /*currently, failure not possible so set rc=0   */
-
-      if (rc != 0)
-        {
-        if (problem_nodes)     /*we have an array in which to save*/
-          problem_nodes[problem_cnt] = pnode;
-
-        ++problem_cnt;
-        }
-      else
-        {
-        /*modifications succeed for this node*/
-
-        chk_characteristic(pnode, &need_todo);
-
-        if (nodename)
-          {
-          mgr_log_attr(msg_man_set, plist, PBS_EVENTCLASS_NODE, nodename);
-
-          free(nodename);
-          }
-        }
-
+      mgr_log_attr(msg_man_set, plist, PBS_EVENTCLASS_NODE, log_buf);
       } /* end loop ( all nodes ) */
     }
   else
     {
     /* handle single nodes */
-    save_characteristic(pnode);
-
+    snprintf(log_buf,sizeof(log_buf),"%s",pnode->nd_name);
+    
     nodename = strdup(pnode->nd_name);
 
     effective_node_delete(pnode);
 
-    rc = 0;   /*currently, failure not possible so set rc=0   */
-        
-    if (rc != 0)
-      {
-      /*In the specific node case, reply w/ error and return*/
-      
-      switch (rc)
-        {
-        
-        default:
-          
-          req_reject(rc, 0, preq, NULL, NULL);
-          
-          break;
-        }
-
-      return;
-      }
-    else
-      {
-      /*modifications succeed for this node*/
-
-      chk_characteristic(pnode, &need_todo);
-
-      if (nodename)
-        {
-        mgr_log_attr(msg_man_set, plist, PBS_EVENTCLASS_NODE, nodename);
-
-        free(nodename);
-        }
-      }
+    mgr_log_attr(msg_man_set, plist, PBS_EVENTCLASS_NODE, log_buf);
     }
 
-  /*set "deleted" bit in node's (nodes, allnodes == 1) "inuse" field*/
+  /*set "deleted" bit in node's (nodes, check_all == 1) "inuse" field*/
   /*remove entire prop list, including the node name, from the node */
   /*remove the IP address array hanging from the node               */
 
-
-  if (need_todo & WRITENODE_STATE)
-    {
-    /*some nodes set to "offline"*/
-
-    write_node_state();
-
-    need_todo &= ~(WRITENODE_STATE);
-    }
-
-  if (need_todo & WRITE_NEW_NODESFILE)
-    {
-    /* create/delete/prop/ntype change */
-
-    if (!update_nodes_file())
-      need_todo &= ~(WRITE_NEW_NODESFILE);  /*successful on update*/
-    }
-
-  if (allnodes)
-    {
-    /*modification was for all nodes  */
-
-    if (problem_cnt)      /*one or more problems encountered*/
-      {
-
-      for (len = 0, i = 0; i < problem_cnt; i++)
-        len += strlen(problem_nodes[i]->nd_name) + 3;
-
-      len += strlen(pbse_to_txt(PBSE_GMODERR));
-
-      if ((problem_names = malloc(len)))
-        {
-
-        strcpy(problem_names, pbse_to_txt(PBSE_GMODERR));
-
-        for (i = 0; i < problem_cnt; i++)
-          {
-          if (i)
-            strcat(problem_names, ", ");
-
-          strcat(problem_names, problem_nodes[i]->nd_name);
-          }
-
-        reply_text(preq, PBSE_GMODERR, problem_names);
-
-        free(problem_names);
-        }
-      else
-        {
-        reply_text(preq, PBSE_GMODERR, pbse_to_txt(PBSE_GMODERR));
-        }
-      }
-
-    if (problem_nodes)  /*maybe problem  malloc failed */
-      free(problem_nodes);
-
-    if (problem_cnt)    /*reply has already been sent  */
-      {
-      recompute_ntype_cnts();
-      return ;
-      }
-    }
+  /* update the nodes file and node state since we deleted node(s) */
+  write_node_state();
+  update_nodes_file(NULL);
 
   recompute_ntype_cnts();
 
@@ -2197,7 +2032,6 @@ void mgr_node_create(
     }
 
   mgr_log_attr(
-
     msg_man_set,
     plist,
     PBS_EVENTCLASS_NODE,
@@ -2206,7 +2040,7 @@ void mgr_node_create(
   setup_notification(preq->rq_ind.rq_manager.rq_objname);     /* set mechanism for notifying */
   /* other nodes of new member   */
 
-  if (update_nodes_file())
+  if (update_nodes_file(NULL))
     {
     /* if update fails now (odd) */
 
@@ -2234,11 +2068,13 @@ void mgr_node_create(
  * appropriate function is called to perform the operation.
  */
 
-void req_manager(
+void *req_manager(
 
-  struct batch_request *preq)  /* I */
+  void *vp) /* I */
 
   {
+  struct batch_request *preq = (struct batch_request *)vp;
+
   switch (preq->rq_ind.rq_manager.rq_cmd)
     {
 
@@ -2250,7 +2086,7 @@ void req_manager(
         {
         req_reject(PBSE_PERM, 0, preq, NULL, NULL);
 
-        return;
+        return(NULL);
         }
 
       switch (preq->rq_ind.rq_manager.rq_objtype)
@@ -2278,7 +2114,7 @@ void req_manager(
 
           req_reject(PBSE_IVALREQ, 0, preq, NULL, NULL);
 
-          return;
+          return(NULL);
 
           break;
         }
@@ -2291,7 +2127,7 @@ void req_manager(
         {
         req_reject(PBSE_PERM, 0, preq, NULL, NULL);
 
-        return;
+        return(NULL);
         }
 
       switch (preq->rq_ind.rq_manager.rq_objtype)
@@ -2330,7 +2166,7 @@ void req_manager(
         {
         req_reject(PBSE_PERM, 0, preq, NULL, NULL);
 
-        return;
+        return(NULL);
         }
 
       switch (preq->rq_ind.rq_manager.rq_objtype)
@@ -2376,7 +2212,7 @@ void req_manager(
       break;
     }  /* END switch() */
 
-  return;
+  return(NULL);
   }  /* END req_manager() */
 
 
@@ -2401,6 +2237,7 @@ int manager_oper_chk(
   int      i;
 
   struct array_strings *pstr;
+  char                  log_buf[LOCAL_LOG_BUF_SIZE];
 
   if (actmode == ATR_ACTION_FREE)
     {
@@ -2431,21 +2268,16 @@ int manager_oper_chk(
       {
       if (actmode == ATR_ACTION_RECOV)
         {
-        sprintf(log_buffer, "bad entry in acl: %s",
+        sprintf(log_buf, "bad entry in acl: %s",
           pstr->as_string[i]);
 
-        log_err(PBSE_BADACLHOST,
-          "manager_oper_chk",
-          log_buffer);
+        log_err(PBSE_BADACLHOST, "manager_oper_chk", log_buf);
         }
       else
         {
-        sprintf(log_buffer, "bad entry in acl: %s",
-          pstr->as_string[i]);
+        sprintf(log_buf, "bad entry in acl: %s", pstr->as_string[i]);
 
-        log_err(PBSE_BADACLHOST,
-          "manager_oper_chk",
-          log_buffer);
+        log_err(PBSE_BADACLHOST, "manager_oper_chk", log_buf);
 
         err = PBSE_BADACLHOST;
         }
@@ -2469,10 +2301,11 @@ int servername_chk(
 
   {
   int    err = 0;
-  char    hostname[PBS_MAXHOSTNAME + 1];
+  char    hostname[PBS_MAXSERVERNAME + 1];
   char *pstr;
 
   extern char              server_name[];
+  char         log_buf[LOCAL_LOG_BUF_SIZE];
 
   if (actmode == ATR_ACTION_FREE)
     {
@@ -2501,12 +2334,9 @@ int servername_chk(
     {
     if (actmode == ATR_ACTION_RECOV)
       {
-      sprintf(log_buffer, "bad servername: %s",
-              pstr);
+      sprintf(log_buf, "bad servername: %s", pstr);
 
-      log_err(PBSE_BADHOST,
-              "servername_chk",
-              log_buffer);
+      log_err(PBSE_BADHOST,"servername_chk",log_buf);
       }
     else
       {
@@ -2535,6 +2365,9 @@ void on_extra_resc(
 
   {
   init_resc_defs();
+
+  free(ptask->wt_mutex);
+  free(ptask);
   }
 
 int extra_resc_chk(
@@ -2544,14 +2377,11 @@ int extra_resc_chk(
   int      actmode) /* I */
 
   {
-
-  struct work_task *ptask;
-
   /* Is there anything to validate?  Maybe check for all alphanum? */
   /* the new resource is at pattr->at_val.at_str */
-  ptask = set_task(WORK_Immed, 0, on_extra_resc, NULL);
+  set_task(WORK_Immed, 0, on_extra_resc, NULL, FALSE);
 
-  return (ptask == NULL);
+  return(TRUE);
   }
 
 /*
@@ -2563,7 +2393,7 @@ void free_extraresc(
   struct attribute *attr)
 
   {
-  set_task(WORK_Immed, 0, on_extra_resc, NULL);
+  set_task(WORK_Immed, 0, on_extra_resc, NULL, FALSE);
 
   free_arst(attr);
   }
@@ -2685,6 +2515,8 @@ int schiter_chk(
   return(mgr_long_action_helper(pattr, actmode, 1, PBS_SCHEDULE_CYCLE));
   }  /* END schiter_chk() */
 
+
+
 /* nextjobnum_action - makes sure value is sane (>=0)
    Note that there must be special code in svr_recov
    to prevent the attribute value from clobbering
@@ -2695,6 +2527,7 @@ int nextjobnum_chk(
   attribute *pattr,
   void      *pobject,
   int        actmode)
+
   {
   if (pattr->at_val.at_long > PBS_SEQNUMTOP)
     {
@@ -2726,35 +2559,45 @@ int set_nextjobnum(
    * Otherwise, if the number is in the default range, it sets it to the new range.
    */
 
+  pthread_mutex_lock(server.sv_qs_mutex);
   switch (op)
     {
 
     case SET:
+    
       attr->at_val.at_long = new->at_val.at_long;
+    
       break;
 
       /* Code in Moab depends on this, do not change until the Moab code is fixed. */
 
     case INCR:
+
+
       attr->at_val.at_long = MAX(server.sv_qs.sv_jobidnumber, new->at_val.at_long);
-      /*case INCR:  attr->at_val.at_long += new->at_val.at_long;*/
+      
       break;
 
     case DECR:
+      
       attr->at_val.at_long -= new->at_val.at_long;
+      
       break;
 
     default:
+     
+      pthread_mutex_unlock(server.sv_qs_mutex);
       return(PBSE_SYSTEM);
     }
 
   attr->at_flags |= ATR_VFLAG_SET | ATR_VFLAG_MODIFY;
 
   server.sv_qs.sv_jobidnumber = attr->at_val.at_long;
-  svr_save(&server, SVR_SAVE_QUICK);
 
-  return 0;
-  }
+  pthread_mutex_unlock(server.sv_qs_mutex);
+
+  return(PBSE_NONE);
+  } /* END set_nextjobnum() */
 
 /* END req_manager.c */
 
