@@ -84,9 +84,9 @@
 #include <pbs_config.h>   /* the master config generated by configure */
 
 #include <stdio.h>
-#include <sys/types.h>
-#include "libpbs.h"
 #include <signal.h>
+#include <pthread.h>
+#include "libpbs.h"
 #include "server_limits.h"
 #include "list_link.h"
 #include "attribute.h"
@@ -97,6 +97,7 @@
 #include "work_task.h"
 #include "pbs_error.h"
 #include "log.h"
+#include "../lib/Liblog/log_event.h"
 #include "svrfunc.h"
 
 
@@ -109,6 +110,8 @@ static void post_message_req(struct work_task *);
 extern int   pbs_mom_port;
 extern char *msg_messagejob;
 
+extern job  *chk_job_request(char *, struct batch_request *);
+int copy_batchrequest(struct batch_request **newreq, struct batch_request *preq, int type, int jobid);
 
 /*
  * req_messagejob - service the Message Job Request
@@ -116,52 +119,74 @@ extern char *msg_messagejob;
  * This request sends (via MOM) a message to a running job.
  */
 
-void
-req_messagejob(struct batch_request *preq)
-  {
-  job   *pjob;
-  int    rc;
+void *req_messagejob(
+    
+  void *vp)
 
-  if ((pjob = chk_job_request(preq->rq_ind.rq_message.rq_jid, preq)) == 0)
-    return;
+  {
+  struct batch_request *preq = (struct batch_request *)vp;
+  job                  *pjob;
+  int                   rc;
+  struct batch_request *dup_req = NULL;
+
+  if ((pjob = chk_job_request(preq->rq_ind.rq_message.rq_jid, preq)) == NULL)
+    return(NULL);
 
   /* the job must be running */
 
   if (pjob->ji_qs.ji_state != JOB_STATE_RUNNING)
     {
     req_reject(PBSE_BADSTATE, 0, preq, NULL, NULL);
-    return;
+
+    pthread_mutex_unlock(pjob->ji_mutex);
+    
+    return(NULL);
     }
 
+  if ((rc = copy_batchrequest(&dup_req, preq, 0, -1)) != 0)
+    {
+    req_reject(PBSE_MEM_MALLOC, 0, preq, NULL, NULL);
+    }
   /* pass the request on to MOM */
-
-  if ((rc = relay_to_mom(pjob->ji_qs.ji_un.ji_exect.ji_momaddr,
-                         preq, post_message_req)))
+  /* The dup_req is freed in relay_to_mom (failure)
+   * or in issue_Drequest (success) */
+  else if ((rc = relay_to_mom(&pjob, dup_req, post_message_req)) != 0)
     req_reject(rc, 0, preq, NULL, NULL); /* unable to get to MOM */
+  else
+    free_br(preq);
 
   /* After MOM acts and replies to us, we pick up in post_message_req() */
-  }
+  if (pjob != NULL)
+    pthread_mutex_unlock(pjob->ji_mutex);
+
+  return(NULL);
+  } /* END req_messagejob() */
 
 /*
  * post_message_req - complete a Message Job Request
  */
 
-static void
-post_message_req(struct work_task *pwt)
-  {
+static void post_message_req(
+    
+  struct work_task *pwt)
 
+  {
   struct batch_request *preq;
+  char                  log_buf[LOCAL_LOG_BUF_SIZE];
 
   svr_disconnect(pwt->wt_event); /* close connection to MOM */
   preq = pwt->wt_parm1;
   preq->rq_conn = preq->rq_orgconn;  /* restore socket to client */
 
-  (void)sprintf(log_buffer, msg_messagejob, preq->rq_reply.brp_code);
-  log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB,
-            preq->rq_ind.rq_message.rq_jid, log_buffer);
+  sprintf(log_buf, msg_messagejob, preq->rq_reply.brp_code);
+  log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, preq->rq_ind.rq_message.rq_jid, log_buf);
 
   if (preq->rq_reply.brp_code)
     req_reject(preq->rq_reply.brp_code, 0, preq, NULL, NULL);
   else
     reply_ack(preq);
-  }
+
+  free(pwt->wt_mutex);
+  free(pwt);
+  } /* END post_message_req() */
+
