@@ -85,10 +85,10 @@
 #include <pbs_config.h>   /* the master config generated by configure */
 
 #include <stdio.h>
-#include <sys/types.h>
 #include <sys/param.h>
 #include "libpbs.h"
 #include <errno.h>
+#include <pthread.h>
 
 #include "server_limits.h"
 #include "list_link.h"
@@ -97,40 +97,44 @@
 #include "batch_request.h"
 #include "pbs_job.h"
 #include "log.h"
+#include "../lib/Liblog/log_event.h"
 #include "pbs_error.h"
 #include "queue.h"
 
 /* Global Data Items: */
 
+extern int LOGLEVEL;
 extern char *msg_unkjobid;
 extern char *msg_manager;
 extern char *msg_movejob;
-extern int  pbs_errno;
 extern char *pbs_o_host;
 
-extern int svr_movejob(job *, char *, struct batch_request *);
-extern int svr_chkque(job *, pbs_queue *, char *, int, char *);
-extern job  *chk_job_request(char *, struct batch_request *);
+int   svr_movejob(job *, char *, int *, struct batch_request *, int);
+int   svr_chkque(job *, pbs_queue *, char *, int, char *);
+job  *chk_job_request(char *, struct batch_request *);
 
 /*
  * req_movejob = move a job to a new destination (local or remote)
  */
 
-void req_movejob(
+void *req_movejob(
 
-  struct batch_request *req)
+  void *vp) /* I */
 
   {
 #ifndef NDEBUG
   char *id = "req_movejob";
 #endif
   job *jobp;
+  struct batch_request *req = (struct batch_request *)vp;
+  char                  log_buf[LOCAL_LOG_BUF_SIZE];
+  int                   local_errno = 0;
 
   jobp = chk_job_request(req->rq_ind.rq_move.rq_jid, req);
 
   if (jobp == NULL)
     {
-    return;
+    return(NULL);
     }
 
   if ((jobp->ji_qs.ji_state != JOB_STATE_QUEUED) &&
@@ -138,22 +142,16 @@ void req_movejob(
       (jobp->ji_qs.ji_state != JOB_STATE_WAITING))
     {
 #ifndef NDEBUG
-    sprintf(log_buffer, "%s %d",
-            pbse_to_txt(PBSE_BADSTATE),
-            jobp->ji_qs.ji_state);
+    sprintf(log_buf, "%s %d %s", pbse_to_txt(PBSE_BADSTATE), jobp->ji_qs.ji_state, id);
 
-    strcat(log_buffer, id);
-
-    log_event(
-      PBSEVENT_DEBUG,
-      PBS_EVENTCLASS_JOB,
-      jobp->ji_qs.ji_jobid,
-      log_buffer);
+    log_event(PBSEVENT_DEBUG,PBS_EVENTCLASS_JOB,jobp->ji_qs.ji_jobid,log_buf);
 #endif /* NDEBUG */
 
     req_reject(PBSE_BADSTATE, 0, req, NULL, NULL);
 
-    return;
+    pthread_mutex_unlock(jobp->ji_mutex);
+
+    return(NULL);
     }
 
   /*
@@ -161,25 +159,17 @@ void req_movejob(
    * network moves
    */
 
-  switch (svr_movejob(jobp, req->rq_ind.rq_move.rq_destin, req))
+  switch (svr_movejob(jobp, req->rq_ind.rq_move.rq_destin, &local_errno, req, FALSE))
     {
 
     case 0:
 
       /* success */
+      snprintf(log_buf, sizeof(log_buf), "%s", msg_movejob);
+      snprintf(log_buf + strlen(log_buf), sizeof(log_buf) - strlen(log_buf), msg_manager,
+        req->rq_ind.rq_move.rq_destin, req->rq_user, req->rq_host);
 
-      strcpy(log_buffer, msg_movejob);
-
-      sprintf(log_buffer + strlen(log_buffer), msg_manager,
-              req->rq_ind.rq_move.rq_destin,
-              req->rq_user,
-              req->rq_host);
-
-      log_event(
-        PBSEVENT_JOB,
-        PBS_EVENTCLASS_JOB,
-        jobp->ji_qs.ji_jobid,
-        log_buffer);
+      log_event(PBSEVENT_JOB,PBS_EVENTCLASS_JOB,jobp->ji_qs.ji_jobid,log_buf);
 
       reply_ack(req);
 
@@ -193,7 +183,7 @@ void req_movejob(
 
       /* NOTE:  can pass detailed response to requestor (NYI) */
 
-      req_reject(pbs_errno, 0, req, NULL, NULL);
+      req_reject(local_errno, 0, req, NULL, NULL);
 
       break;
 
@@ -207,7 +197,9 @@ void req_movejob(
       break;
     }  /* END switch (svr_movejob(jobp,req->rq_ind.rq_move.rq_destin,req)) */
 
-  return;
+  pthread_mutex_unlock(jobp->ji_mutex);
+
+  return(NULL);
   }  /* END req_movejob() */
 
 
@@ -219,72 +211,102 @@ void req_movejob(
  * req_orderjob = reorder the jobs in a queue
  */
 
-void req_orderjob(
+void *req_orderjob(
 
-  struct batch_request *req)  /* I */
+  void *vp) /* I */
 
   {
 #ifndef NDEBUG
-  char *id = "req_orderjob";
+  static char          *id = "req_orderjob";
 #endif
-  job *pjob;
-  job *pjob1;
-  job *pjob2;
-  int  rank;
-  int  rc;
-  char  tmpqn[PBS_MAXQUEUENAME+1];
+  job                  *pjob;
+  job                  *pjob1;
+  job                  *pjob2;
+  int                   rank;
+  int                   rc = 0;
+  char                  tmpqn[PBS_MAXQUEUENAME+1];
+  struct batch_request *req = (struct batch_request *)vp;
+  char                  log_buf[LOCAL_LOG_BUF_SIZE];
+  pbs_queue            *pque1;
+  pbs_queue            *pque2;
 
   if ((pjob1 = chk_job_request(req->rq_ind.rq_move.rq_jid, req)) == NULL)
     {
-    return;
+    return(NULL);
     }
 
   if ((pjob2 = chk_job_request(req->rq_ind.rq_move.rq_destin, req)) == NULL)
     {
-    return;
+    return(NULL);
     }
 
   if (((pjob = pjob1)->ji_qs.ji_state == JOB_STATE_RUNNING) ||
       ((pjob = pjob2)->ji_qs.ji_state == JOB_STATE_RUNNING))
     {
 #ifndef NDEBUG
-    sprintf(log_buffer, "%s %d",
+    sprintf(log_buf, "%s %d",
             pbse_to_txt(PBSE_BADSTATE),
             pjob->ji_qs.ji_state);
 
-    strcat(log_buffer, id);
+    strcat(log_buf, id);
 
     log_event(
       PBSEVENT_DEBUG,
       PBS_EVENTCLASS_JOB,
       pjob->ji_qs.ji_jobid,
-      log_buffer);
+      log_buf);
 #endif /* NDEBUG */
 
     req_reject(PBSE_BADSTATE, 0, req, NULL, NULL);
 
-    return;
+    pthread_mutex_unlock(pjob1->ji_mutex);
+    pthread_mutex_unlock(pjob2->ji_mutex);
+
+    return(NULL);
+    }
+  else if ((pjob1->ji_qhdr == NULL) || (pjob2->ji_qhdr == NULL))
+    {
+    req_reject(PBSE_BADSTATE, 0, req, NULL, "One of the jobs does not have a queue");     
     }
   else if (pjob1->ji_qhdr != pjob2->ji_qhdr)
     {
     /* jobs are in different queues */
+    int ok = FALSE;
 
-    if ((rc = svr_chkque(
-                pjob1,
-                pjob2->ji_qhdr,
-                get_variable(pjob1, pbs_o_host),
-                MOVE_TYPE_Order,
-                NULL)) ||
-        (rc = svr_chkque(
-                pjob2,
-                pjob1->ji_qhdr,
-                get_variable(pjob2, pbs_o_host),
-                MOVE_TYPE_Order,
-                NULL)))
+    if ((pque2 = get_jobs_queue(&pjob2)) == NULL)
+      req_reject(PBSE_BADSTATE, 0, req, NULL, "job2 queue is unavailable");
+    else
+      {
+      if ((rc = svr_chkque(pjob1, pque2, get_variable(pjob1, pbs_o_host), MOVE_TYPE_Order, NULL)) == PBSE_NONE)
+        {
+        unlock_queue(pque2, "req_orderjob", "pque2 svr_chkque pass", LOGLEVEL);
+        if ((pque1 = get_jobs_queue(&pjob1)) == NULL)
+          {
+          req_reject(PBSE_BADSTATE, 0, req, NULL, "job1 queue is unavailable");
+          }
+        else if (pjob1 != NULL)
+          {
+          if ((rc = svr_chkque(pjob2, pque1, get_variable(pjob2, pbs_o_host), MOVE_TYPE_Order, NULL)) == PBSE_NONE)
+            {
+            ok = TRUE;
+            }
+          unlock_queue(pque1, "req_orderjob", "pque1", LOGLEVEL);
+          }
+        }
+      else
+        unlock_queue(pque2, "req_orderjob", "pque2 svr_chkque fail", LOGLEVEL);
+      }
+
+    if (ok == FALSE)
       {
       req_reject(rc, 0, req, NULL, NULL);
 
-      return;
+      if (pjob1 != NULL)
+        pthread_mutex_unlock(pjob1->ji_mutex);
+      if (pjob2 != NULL)
+        pthread_mutex_unlock(pjob2->ji_mutex);
+
+      return(NULL);
       }
     }
 
@@ -299,31 +321,45 @@ void req_orderjob(
 
   if (pjob1->ji_qhdr != pjob2->ji_qhdr)
     {
-    (void)strcpy(tmpqn, pjob1->ji_qs.ji_queue);
-    (void)strcpy(pjob1->ji_qs.ji_queue, pjob2->ji_qs.ji_queue);
-    (void)strcpy(pjob2->ji_qs.ji_queue, tmpqn);
-    svr_dequejob(pjob1);
-    svr_dequejob(pjob2);
-    (void)svr_enquejob(pjob1);
-    (void)svr_enquejob(pjob2);
+    strcpy(tmpqn, pjob1->ji_qs.ji_queue);
+    strcpy(pjob1->ji_qs.ji_queue, pjob2->ji_qs.ji_queue);
+    strcpy(pjob2->ji_qs.ji_queue, tmpqn);
+    svr_dequejob(pjob1, FALSE);
+    svr_dequejob(pjob2, FALSE);
+    if (strlen(pjob1->ji_qs.ji_jobid) > 0)
+      svr_enquejob(pjob1, FALSE, -1);
+    else
+      pjob1 = NULL;
 
+    if (strlen(pjob2->ji_qs.ji_jobid) > 0)
+      svr_enquejob(pjob2, FALSE, -1);
+    else
+      pjob2 = NULL;
     }
   else
     {
-    swap_link(&pjob1->ji_jobque,  &pjob2->ji_jobque);
-    swap_link(&pjob1->ji_alljobs, &pjob2->ji_alljobs);
+    if ((pque1 = get_jobs_queue(&pjob1)) != NULL)
+      {
+      swap_jobs(pque1->qu_jobs,pjob1,pjob2);
+      swap_jobs(NULL,pjob1,pjob2);
+      unlock_queue(pque1, "req_orderjob", "pque1 after swap", LOGLEVEL);
+      }
     }
 
   /* need to update disk copy of both jobs to save new order */
-
-  job_save(pjob1, SAVEJOB_FULL, 0);
-
+  if (pjob1 != NULL)
+    job_save(pjob1, SAVEJOB_FULL, 0);
+  if (pjob2 != NULL)
   job_save(pjob2, SAVEJOB_FULL, 0);
 
   reply_ack(req);
 
   /* SUCCESS */
+  if (pjob1 != NULL)
+    pthread_mutex_unlock(pjob1->ji_mutex);
+  if (pjob2 != NULL)
+    pthread_mutex_unlock(pjob2->ji_mutex);
 
-  return;
+  return(NULL);
   }  /* END req_orderjob() */
 
