@@ -79,12 +79,12 @@
 #include <pbs_config.h>   /* the master config generated by configure */
 
 #include <assert.h>
-#include <sys/types.h>
 #include <stdlib.h>
 #include <ctype.h>
 #include <errno.h>
 #include <stdio.h>
 #include <string.h>
+#include <pthread.h>
 #include "libpbs.h"
 #include "list_link.h"
 #include "attribute.h"
@@ -97,8 +97,11 @@
 #include "work_task.h"
 #include "pbs_error.h"
 #include "log.h"
+#include "../lib/Liblog/pbs_log.h"
+#include "../lib/Liblog/log_event.h"
 #include "svrfunc.h"
 #include "array.h"
+#include "svr_func.h" /* get_svr_attr_* */
 
 #define SYNC_SCHED_HINT_NULL 0
 #define SYNC_SCHED_HINT_FIRST 1
@@ -146,7 +149,6 @@ extern char *msg_illregister;
 extern char *msg_registerdel;
 extern char *msg_registerrel;
 extern char *msg_regrej;
-extern char  log_buffer[];
 extern int   LOGLEVEL;
 extern char *PJobState[];
 
@@ -166,22 +168,24 @@ extern int   svr_chk_owner(struct batch_request *, job *);
  * to be graceful.
  */
 
-void req_register(
+void *req_register(
 
-  struct batch_request *preq)  /* I */
+  void *vp)  /* I */
 
   {
-  int     made;
-  attribute   *pattr;
+  struct batch_request *preq = (struct batch_request *)vp;
+  int                   made;
+  attribute            *pattr;
 
-  struct depend   *pdep;
+  struct depend        *pdep;
 
-  struct depend_job *pdj;
-  job    *pjob;
-  char    *ps;
-  int     rc = 0;
-  int     revtype;
-  int     type;
+  struct depend_job    *pdj;
+  job                  *pjob;
+  char                 *ps;
+  int                   rc = 0;
+  int                   revtype;
+  int                   type;
+  char                  log_buf[LOCAL_LOG_BUF_SIZE];
 
   /*  make sure request is from a server */
 
@@ -189,7 +193,7 @@ void req_register(
     {
     req_reject(PBSE_IVALREQ, 0, preq, NULL, NULL);
 
-    return;
+    return(NULL);
     }
 
   /* find the "parent" job specified in the request */
@@ -200,8 +204,11 @@ void req_register(
      * job not found... if server is initializing, it may not
      * yet recovered, that is not an error.
      */
+    long state = SV_STATE_DOWN;
 
-    if (server.sv_attr[SRV_ATR_State].at_val.at_long != SV_STATE_INIT)
+    get_svr_attr_l(SRV_ATR_State, &state);
+
+    if (state != SV_STATE_INIT)
       {
       log_event(
         PBSEVENT_DEBUG,
@@ -216,7 +223,7 @@ void req_register(
       reply_ack(preq);
       }
 
-    return;
+    return(NULL);
     }
 
   type = preq->rq_ind.rq_register.rq_dependtype;
@@ -230,41 +237,35 @@ void req_register(
     {
     if (LOGLEVEL >= 8)
       {
-      sprintf(log_buffer,"Dependency requested for %s job, parent job %s, child job %s",
+      sprintf(log_buf,"Dependency requested for %s job, parent job %s, child job %s",
         PJobState[pjob->ji_qs.ji_state],
         preq->rq_ind.rq_register.rq_parent,
         preq->rq_ind.rq_register.rq_child);
-
-      log_event(
-        PBSEVENT_JOB,
-        PBS_EVENTCLASS_JOB,
-        pjob->ji_qs.ji_jobid,
-        log_buffer);
+      
+      log_event(PBSEVENT_JOB,PBS_EVENTCLASS_JOB,pjob->ji_qs.ji_jobid,log_buf);
       }
+    
+    log_event(
+      PBSEVENT_DEBUG,
+      PBS_EVENTCLASS_JOB,
+      preq->rq_ind.rq_register.rq_parent,
+      pbse_to_txt(PBSE_BADSTATE));
+    
+    req_reject(PBSE_BADSTATE, 0, preq, NULL, NULL);
 
-      log_event(
-        PBSEVENT_DEBUG,
-        PBS_EVENTCLASS_JOB,
-        preq->rq_ind.rq_register.rq_parent,
-        pbse_to_txt(PBSE_BADSTATE));
-
-      req_reject(PBSE_BADSTATE, 0, preq, NULL, NULL);
-
-      return;
+    pthread_mutex_unlock(pjob->ji_mutex);
+    
+    return(NULL);
     }
 
   if (LOGLEVEL >= 8)
     {
-    sprintf(log_buffer,"Dependency requested parent job %s state (%s), child job %s",
+    sprintf(log_buf,"Dependency requested parent job %s state (%s), child job %s",
       preq->rq_ind.rq_register.rq_parent,
       PJobState[pjob->ji_qs.ji_state],
       preq->rq_ind.rq_register.rq_child);
 
-    log_event(
-      PBSEVENT_JOB,
-      PBS_EVENTCLASS_JOB,
-      pjob->ji_qs.ji_jobid,
-      log_buffer);
+    log_event(PBSEVENT_JOB,PBS_EVENTCLASS_JOB,pjob->ji_qs.ji_jobid,log_buf);
     }
 
   pattr = &pjob->ji_wattr[JOB_ATR_depend];
@@ -275,7 +276,7 @@ void req_register(
 
   if ((ps = strchr(preq->rq_ind.rq_register.rq_child, (int)'@')))
     {
-    strcpy(preq->rq_ind.rq_register.rq_svr, ps + 1);
+    snprintf(preq->rq_ind.rq_register.rq_svr, sizeof(preq->rq_ind.rq_register.rq_svr), "%s", ps + 1);
 
     *ps = '\0';
     }
@@ -284,7 +285,7 @@ void req_register(
     /* We ignore what qsub sent us for the host. We need to use the 
      * server_name so we know we can connect to the active server.
      * This is an high availability consideration */
-    if(server_name != NULL)
+    if (server_name != NULL)
       strcpy(preq->rq_ind.rq_register.rq_svr, server_name);
     else
       strcpy(preq->rq_ind.rq_register.rq_svr, preq->rq_host);
@@ -389,10 +390,10 @@ void req_register(
                 {
                 /* has prior register, update it */
 
-                if(server_name != NULL)
-                  strncpy(pdj->dc_svr, server_name, PBS_MAXSERVERNAME);
+                if (server_name != NULL)
+                  snprintf(pdj->dc_svr, sizeof(pdj->dc_svr), "%s", server_name);
                 else
-                  strncpy(pdj->dc_svr, preq->rq_ind.rq_register.rq_svr, PBS_MAXSERVERNAME);
+                  snprintf(pdj->dc_svr, sizeof(pdj->dc_svr), "%s", preq->rq_ind.rq_register.rq_svr);
                 }
               }
             else if ((rc = register_dep(pattr, preq, type, &made)) == 0)
@@ -446,14 +447,9 @@ void req_register(
               {
               del_depend_job(pdj);
 
-              sprintf(log_buffer, msg_registerrel,
-                      preq->rq_ind.rq_register.rq_child);
+              sprintf(log_buf, msg_registerrel, preq->rq_ind.rq_register.rq_child);
 
-              log_event(
-                PBSEVENT_JOB,
-                PBS_EVENTCLASS_JOB,
-                pjob->ji_qs.ji_jobid,
-                log_buffer);
+              log_event(PBSEVENT_JOB,PBS_EVENTCLASS_JOB,pjob->ji_qs.ji_jobid,log_buf);
 
               if (GET_NEXT(pdep->dp_jobs) == 0)
                 {
@@ -541,17 +537,11 @@ void req_register(
         break;
         }
 
-      sprintf(log_buffer, msg_registerdel,
+      sprintf(log_buf, msg_registerdel, preq->rq_ind.rq_register.rq_child);
 
-              preq->rq_ind.rq_register.rq_child);
+      log_event(PBSEVENT_JOB,PBS_EVENTCLASS_JOB,pjob->ji_qs.ji_jobid,log_buf);
 
-      log_event(
-        PBSEVENT_JOB,
-        PBS_EVENTCLASS_JOB,
-        pjob->ji_qs.ji_jobid,
-        log_buffer);
-
-      job_abt(&pjob, log_buffer);
+      job_abt(&pjob, log_buf);
 
       /* pjob freed and set to NULL */
 
@@ -581,14 +571,13 @@ void req_register(
 
     default:
 
-      sprintf(log_buffer, msg_illregister,
-              preq->rq_ind.rq_register.rq_parent);
+      sprintf(log_buf, msg_illregister, preq->rq_ind.rq_register.rq_parent);
 
       log_event(
         PBSEVENT_DEBUG | PBSEVENT_SYSTEM | PBSEVENT_ERROR,
         PBS_EVENTCLASS_REQUEST,
         preq->rq_host,
-        log_buffer);
+        log_buf);
 
       rc = PBSE_IVALREQ;
 
@@ -614,7 +603,10 @@ void req_register(
     reply_ack(preq);
     }
 
-  return;
+  if (pjob != NULL)
+    pthread_mutex_unlock(pjob->ji_mutex);
+
+  return(NULL);
   }  /* END req_register() */
 
 
@@ -625,14 +617,16 @@ void req_register(
  * registers a dependency on an array
  */
 
-void req_registerarray(
+void *req_registerarray(
 
-  struct batch_request *preq)  /* I */
+  void *vp)  /* I */
 
   {
+  struct batch_request *preq = (struct batch_request *)vp;
+  char        log_buf[LOCAL_LOG_BUF_SIZE];
   job_array  *pa;
-  char        array_name[PBS_MAXSVRJOBID];
-  char        range[MAXPATHLEN >> 4];
+  char        array_name[PBS_MAXSVRJOBID + 1];
+  char        range[MAXPATHLEN];
   int         num_jobs = -1;
   char       *dot_server;
   char       *bracket_ptr;
@@ -644,7 +638,7 @@ void req_registerarray(
     {
     req_reject(PBSE_IVALREQ, 0, preq, NULL, NULL);
 
-    return;
+    return(NULL);
     }
 
   strcpy(array_name,preq->rq_ind.rq_register.rq_parent);
@@ -701,8 +695,10 @@ void req_registerarray(
      * array not found... if server is initializing, it may not
      * yet be recovered, that is not an error.
      */
+    long state = SV_STATE_DOWN;
+    get_svr_attr_l(SRV_ATR_State, &state);
 
-    if (server.sv_attr[SRV_ATR_State].at_val.at_long != SV_STATE_INIT)
+    if (state != SV_STATE_INIT)
       {
       log_event(
         PBSEVENT_DEBUG,
@@ -710,24 +706,31 @@ void req_registerarray(
         preq->rq_ind.rq_register.rq_parent,
         pbse_to_txt(PBSE_UNKJOBID));
 
-      req_reject(PBSE_UNKJOBID, 0, preq, NULL, NULL);
+      req_reject(PBSE_UNKARRAYID, 0, preq, NULL, "unable to find array");
       }
     else
       {
       reply_ack(preq);
       }
 
-    return;
+    return(NULL);
     }
 
   type = preq->rq_ind.rq_register.rq_dependtype;
 
   if (type < JOB_DEPEND_TYPE_AFTERSTARTARRAY)
     {
+    pthread_mutex_unlock(pa->ai_mutex);
+    if (LOGLEVEL >= 7)
+      {
+      sprintf(log_buf, "%s: unlocked ai_mutex", __func__);
+      log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, __func__, log_buf);
+      }
+
     req_reject(PBSE_IVALREQ,0,preq,NULL,
       "Arrays may only be given array dependencies");
 
-    return;
+    return(NULL);
     }
 
   /* register the dependency on the array */
@@ -756,6 +759,14 @@ void req_registerarray(
       break;
     } /* END switch (preq->rq_ind.rq_register.rq_op */
 
+  pthread_mutex_unlock(pa->ai_mutex);
+  if (LOGLEVEL >= 7)
+    {
+    sprintf(log_buf, "%s: unlocked ai_mutex", __func__);
+    log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, __func__, log_buf);
+    }
+
+  return(NULL);
   } /* END req_registerarray() */
 
 
@@ -785,7 +796,7 @@ int register_array_depend(
   /* make dependency of none exists */
   if (pdep == NULL)
     {
-    pdep = malloc(sizeof(struct array_depend));
+    pdep = calloc(1, sizeof(struct array_depend));
 
     if (pdep != NULL)
       {
@@ -817,14 +828,14 @@ int register_array_depend(
     return(0);
 
   /* try to create the job */
-  pdj = (struct array_depend_job *)malloc(sizeof(struct array_depend_job));
+  pdj = (struct array_depend_job *)calloc(1, sizeof(struct array_depend_job));
 
   if (pdj != NULL)
     {
     CLEAR_LINK(pdj->dc_link);
 
     snprintf(pdj->dc_child,sizeof(pdj->dc_child),"%s",preq->rq_ind.rq_register.rq_child);
-    if(server_name != NULL)
+    if (server_name != NULL)
       snprintf(pdj->dc_svr,sizeof(pdj->dc_svr),"%s",server_name);
     else
       snprintf(pdj->dc_svr,sizeof(pdj->dc_svr),"%s",preq->rq_ind.rq_register.rq_svr);
@@ -950,7 +961,7 @@ void set_array_depend_holds(
               "Setting HOLD_s due to dependencies\n");
             }
 
-          svr_setjobstate(pjob, JOB_STATE_HELD, JOB_SUBSTATE_DEPNHOLD);
+          svr_setjobstate(pjob, JOB_STATE_HELD, JOB_SUBSTATE_DEPNHOLD, FALSE);
           }
         else 
           {
@@ -959,8 +970,9 @@ void set_array_depend_holds(
            * logged in set_depend_hold */
           set_depend_hold(pjob,&pjob->ji_wattr[JOB_ATR_depend]);
           }
-        }
 
+        pthread_mutex_unlock(pjob->ji_mutex);
+        }
 
       pdj = (struct array_depend_job *)GET_NEXT(pdj->dc_link);
       }
@@ -981,8 +993,8 @@ static void post_doq(
   struct work_task *pwt)
 
   {
-
   struct batch_request *preq = (struct batch_request *)pwt->wt_parm1;
+  char                  log_buf[LOCAL_LOG_BUF_SIZE];
 
   char *jobid = preq->rq_ind.rq_register.rq_child;
 
@@ -997,28 +1009,25 @@ static void post_doq(
   if (preq->rq_reply.brp_code)
     {
     /* request was rejected */
-
-    strcpy(log_buffer, msg_regrej);
-    strcat(log_buffer, preq->rq_ind.rq_register.rq_parent);
-
-    log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, jobid, log_buffer);
+    snprintf(log_buf, sizeof(log_buf), "%s%s", msg_regrej, preq->rq_ind.rq_register.rq_parent);
+    log_event(PBSEVENT_JOB, PBS_EVENTCLASS_JOB, jobid, log_buf);
 
     pjob = find_job(jobid);
 
     if ((msg = pbse_to_txt(preq->rq_reply.brp_code)) != NULL)
       {
-      strcat(log_buffer, "\n");
-      strcat(log_buffer, msg);
+      strcat(log_buf, "\n");
+      strcat(log_buf, msg);
       }
 
     if (pjob != NULL)
       {
-      strcat(log_buffer, "\n");
-      strcat(log_buffer, "Job held for unknown job dep, use 'qrls' to release");
+      strcat(log_buf, "\n");
+      strcat(log_buf, "Job held for unknown job dep, use 'qrls' to release");
 
       if (preq->rq_reply.brp_code != PBSE_BADSTATE)
         {
-        svr_mailowner(pjob, MAIL_ABORT, MAIL_FORCE, log_buffer);
+        svr_mailowner(pjob, MAIL_ABORT, MAIL_FORCE, log_buf);
         }
 
       pattr = &pjob->ji_wattr[JOB_ATR_depend];
@@ -1036,6 +1045,8 @@ static void post_doq(
           }
 
         set_depend_hold(pjob, pattr);
+
+        pthread_mutex_unlock(pjob->ji_mutex);
         }
       }
     }
@@ -1062,12 +1073,10 @@ static void alter_unreg(
   attribute *new)  /* job dependency attribute after alter */
 
   {
-
-  struct depend *poldd;
-
-  struct depend *pnewd;
-
+  struct depend     *poldd;
+  struct depend     *pnewd;
   struct depend_job *oldjd;
+
   int                type;
 
   for (poldd = (struct depend *)GET_NEXT(old->at_val.at_list);
@@ -1116,35 +1125,50 @@ static void alter_unreg(
 int depend_on_que(
 
   attribute *pattr,
-  void       *pjob,
+  void       *pj,
   int        mode)
 
   {
-  long        cost;
+  long               cost;
 
-  struct depend *pdep;
+  struct depend     *pdep;
 
   struct depend_job *pparent;
-  int        rc;
-  int        type;
+  int                rc;
+  int                type;
+  job               *pjob = (job *)pj;
+  pbs_queue         *pque = get_jobs_queue(&pjob);
 
-  if (((mode != ATR_ACTION_ALTER) && (mode != ATR_ACTION_NOOP)) ||
-      (((job *)pjob)->ji_qhdr == 0) ||
-      (((job *)pjob)->ji_qhdr->qu_qs.qu_type != QTYPE_Execution))
+  if (pque == NULL)
     {
-    return(0);
+    if (pjob == NULL)
+      {
+      log_err(PBSE_JOBNOTFOUND, __func__, "Job lost while acquiring queue");
+      return(PBSE_JOBNOTFOUND);
+      }
+    else
+      return(PBSE_NONE);
     }
+  else if (((mode != ATR_ACTION_ALTER) && 
+            (mode != ATR_ACTION_NOOP)) ||
+           (pque->qu_qs.qu_type != QTYPE_Execution))
+    {
+    unlock_queue(pque, __func__, NULL, LOGLEVEL);
+    return(PBSE_NONE);
+    }
+  else
+    unlock_queue(pque, __func__, NULL, LOGLEVEL);
 
   if (mode == ATR_ACTION_ALTER)
     {
     /* if there are dependencies being removed, unregister them */
 
-    alter_unreg((job *)pjob, &((job *)pjob)->ji_wattr[JOB_ATR_depend], pattr);
+    alter_unreg(pjob, &(pjob)->ji_wattr[JOB_ATR_depend], pattr);
     }
 
   /* First set a System hold if required */
 
-  set_depend_hold((job *)pjob, pattr);
+  set_depend_hold(pjob, pattr);
 
   /* Check if there are dependencies that require registering */
 
@@ -1158,12 +1182,12 @@ int depend_on_que(
       {
       /* register myself - this calculates and records the cost */
 
-      cost = calc_job_cost((job *)pjob);
+      cost = calc_job_cost(pjob);
 
-      register_sync(pdep, ((job *)pjob)->ji_qs.ji_jobid, server_name, cost);
+      register_sync(pdep, (pjob)->ji_qs.ji_jobid, server_name, cost);
 
       if (pdep->dp_numreg > pdep->dp_numexp)
-        release_cheapest((job *)pjob, pdep);
+        release_cheapest(pjob, pdep);
       }
     else if (type != JOB_DEPEND_TYPE_ON)
       {
@@ -1171,7 +1195,7 @@ int depend_on_que(
 
       while (pparent)
         {
-        if ((rc = send_depend_req((job *)pjob, pparent, type, JOB_DEPEND_OP_REGISTER, SYNC_SCHED_HINT_NULL, post_doq)))
+        if ((rc = send_depend_req(pjob, pparent, type, JOB_DEPEND_OP_REGISTER, SYNC_SCHED_HINT_NULL, post_doq)))
 
           return(rc);
 
@@ -1182,7 +1206,7 @@ int depend_on_que(
     pdep = (struct depend *)GET_NEXT(pdep->dp_link);
     }
 
-  return(0);
+  return(PBSE_NONE);
   }  /* END depend_on_que() */
 
 
@@ -1197,7 +1221,6 @@ static void post_doe(
   struct work_task *pwt)
 
   {
-
   struct batch_request *preq = pwt->wt_parm1;
   char       *jobid = preq->rq_ind.rq_register.rq_child;
   attribute      *pattr;
@@ -1224,6 +1247,8 @@ static void post_doe(
 
       del_depend(pdep);
       }
+    
+    pthread_mutex_unlock(pjob->ji_mutex);
     }
 
   release_req(pwt);
@@ -1608,6 +1633,9 @@ void set_depend_hold(
             {
             substate = JOB_SUBSTATE_DEPNHOLD;
             }
+
+          if (djp != NULL)
+            pthread_mutex_unlock(djp->ji_mutex);
           }
 
         break;
@@ -1645,7 +1673,7 @@ void set_depend_hold(
 
       svr_evaljobstate(pjob, &newstate, &newsubst, 0);
 
-      svr_setjobstate(pjob, newstate, newsubst);
+      svr_setjobstate(pjob, newstate, newsubst, FALSE);
       }
     }
   else
@@ -1664,7 +1692,7 @@ void set_depend_hold(
         "Setting HOLD_s due to dependencies\n");
       }
 
-    svr_setjobstate(pjob, JOB_STATE_HELD, substate);
+    svr_setjobstate(pjob, JOB_STATE_HELD, substate, FALSE);
     }
 
   return;
@@ -1716,8 +1744,8 @@ void depend_clrrdy(
 
 static struct depend *find_depend(
 
-        int        type,
-        attribute *pattr)
+  int        type,
+  attribute *pattr)
 
   {
 
@@ -1749,14 +1777,14 @@ static struct depend *find_depend(
 
 static struct depend *make_depend(
 
-        int        type,
-        attribute *pattr)
+  int        type,
+  attribute *pattr)
 
   {
 
   struct depend *pdep = (struct depend *)0;
 
-  pdep = (struct depend *)malloc(sizeof(struct depend));
+  pdep = (struct depend *)calloc(1, sizeof(struct depend));
 
   if (pdep != NULL)
     {
@@ -1791,7 +1819,7 @@ static int register_sync(
     {
     /* existing regist., just update the location of the child */
 
-    if(server_name != NULL)
+    if (server_name != NULL)
       strcpy(pdj->dc_svr, server_name);
     else
       strcpy(pdj->dc_svr, host);
@@ -1859,7 +1887,7 @@ static int register_dep(
 
   if ((pdj = find_dependjob(pdep, preq->rq_ind.rq_register.rq_child)))
     {
-    if(server_name != NULL)
+    if (server_name != NULL)
       strcpy(pdj->dc_svr, server_name);
     else
       strcpy(pdj->dc_svr, preq->rq_ind.rq_register.rq_svr);
@@ -2008,7 +2036,7 @@ static struct depend_job *make_dependjob(
 
   struct depend_job *pdj;
 
-  pdj = (struct depend_job *)malloc(sizeof(struct depend_job));
+  pdj = (struct depend_job *)calloc(1, sizeof(struct depend_job));
 
   if (pdj != NULL)
     {
@@ -2018,7 +2046,7 @@ static struct depend_job *make_dependjob(
     pdj->dc_cost  = 0;
 
     strcpy(pdj->dc_child, jobid);
-    if(server_name != NULL)
+    if (server_name != NULL)
       strcpy(pdj->dc_svr, server_name);
     else
       strcpy(pdj->dc_svr, host);
@@ -2039,18 +2067,21 @@ static struct depend_job *make_dependjob(
 
 static int send_depend_req(
 
-  job *pjob,
-  struct depend_job *pparent,
-  int  type,
-  int  op,
-  int  schedhint,
-  void (*postfunc)(struct work_task *))
+  job                *pjob,
+  struct depend_job  *pparent,
+  int                 type,
+  int                 op,
+  int                 schedhint,
+  void               (*postfunc)(struct work_task *))
 
   {
-  int        i;
-  char        *myid = "send_depend_req";
+  int rc = 0;
+  int                   i;
+  char                 *myid = "send_depend_req";
+  char                  job_id[PBS_MAXSVRJOBID + 1];
 
   struct batch_request *preq;
+  char                  log_buf[LOCAL_LOG_BUF_SIZE];
 
   preq = alloc_br(PBS_BATCH_RegistDep);
 
@@ -2058,7 +2089,7 @@ static int send_depend_req(
     {
     log_err(errno, myid, msg_err_malloc);
 
-    return(PBSE_SYSTEM);
+    return PBSE_SYSTEM;
     }
 
   for (i = 0;i < PBS_MAXUSER;++i)
@@ -2107,15 +2138,25 @@ static int send_depend_req(
     preq->rq_ind.rq_register.rq_cost = 0;
     }
 
-  if (issue_to_svr(pparent->dc_svr, preq, postfunc) == -1)
-    {
-    sprintf(log_buffer, "Unable to perform dependency with job %s\n",
-            pparent->dc_child);
+  /* save jobid and unlock mutex */
+  strcpy(job_id, pjob->ji_qs.ji_jobid);
+  pthread_mutex_unlock(pjob->ji_mutex);
 
-    return(PBSE_BADHOST);
+  if ((rc = issue_to_svr(pparent->dc_svr, preq, postfunc)) != PBSE_NONE)
+    {
+    sprintf(log_buf, "Unable to perform dependency with job %s\n", pparent->dc_child);
+
+    find_job(job_id);
+
+    return rc;
     }
 
-  return(0);
+  pjob = find_job(job_id);
+
+  if (pjob == NULL)
+    return PBSE_JOBNOTFOUND;
+
+  return PBSE_NONE;
   }  /* END send_depend_req() */
 
 
@@ -2130,7 +2171,7 @@ static int send_depend_req(
  * Encoding the internal representation of the attribute to external
  * Setting the value by =, + or - operators.
  * Comparing a (decoded) value with the attribute value.
- * Freeing the space malloc-ed to the attribute value.
+ * Freeing the space calloc-ed to the attribute value.
  *
  * The prototypes are declared in "attribute.h"
  *
@@ -2191,14 +2232,16 @@ struct dependnames
 
 int decode_depend(
 
-  struct attribute *patr,
-  char             *name, /* attribute name */
-  char             *rescn, /* resource name, unused here */
-  char             *val) /* attribute value */
+  attribute *patr,
+  char      *name,  /* attribute name */
+  char      *rescn, /* resource name, unused here */
+  char      *val,   /* attribute value */
+  int        perm)  /* only used for resources */
 
   {
   int  rc;
   char *valwd;
+  char *ptr = NULL;
 
   if ((val == NULL) || (*val == '\0'))
     {
@@ -2214,7 +2257,7 @@ int decode_depend(
    * add a depend or depend_child structure.
    */
 
-  valwd = parse_comma_string(val);
+  valwd = parse_comma_string(val,&ptr);
 
   while (valwd != NULL)
     {
@@ -2225,7 +2268,7 @@ int decode_depend(
       return(rc);
       }
 
-    valwd = parse_comma_string(NULL);
+    valwd = parse_comma_string(NULL,&ptr);
     }
 
   patr->at_flags |= ATR_VFLAG_SET | ATR_VFLAG_MODIFY;
@@ -2403,7 +2446,8 @@ int encode_depend(
   tlist_head   *phead,  /* ptr to head of attrlist list */
   char	       *atname, /* attribute name */
   char	       *rsname, /* resource name or null */
-  int           mode)   /* encode mode, unused here */
+  int           mode,   /* encode mode, unused here */
+  int           perm)   /* only used for resources */
 
   {
   int      ct = 0;
@@ -2421,7 +2465,6 @@ int encode_depend(
   struct dependnames *pn;
 
   char *BPtr = 0;
-  int   BSpace = -1;
 
   if (!attr)
     return (-1);
@@ -2441,6 +2484,7 @@ int encode_depend(
     if ((nxdp->dp_type == JOB_DEPEND_TYPE_SYNCCT) ||
         (nxdp->dp_type == JOB_DEPEND_TYPE_ON))
       {
+      /* Doesn't this mean it could be too short? *MUTSU* */
       ct += 30;   /* a guess at a reasonable amt of space */
       }
     else
@@ -2456,7 +2500,7 @@ int encode_depend(
       }
     }
 
-  if ((pal = attrlist_create(atname, rsname, ct)) == (svrattrl *)0)
+  if ((pal = attrlist_create(atname, rsname, ct+1)) == NULL)
     {
     return (-1);
     }
@@ -2464,32 +2508,31 @@ int encode_depend(
   *pal->al_value = '\0';
 
   BPtr = pal->al_value;
-  BSpace = pal->al_tsize;  /* this is actually a little larger than the actual buffer */
 
-	for (nxdp = pdp; nxdp; nxdp = (struct depend *)GET_NEXT(nxdp->dp_link))
+  for (nxdp = pdp; nxdp; nxdp = (struct depend *)GET_NEXT(nxdp->dp_link))
     {
-		if ((nxdp->dp_type != JOB_DEPEND_TYPE_SYNCCT) &&
-		    (nxdp->dp_type != JOB_DEPEND_TYPE_ON)       &&
-		    !(pdjb = (struct depend_job *)GET_NEXT(nxdp->dp_jobs)))
+    if ((nxdp->dp_type != JOB_DEPEND_TYPE_SYNCCT) &&
+        (nxdp->dp_type != JOB_DEPEND_TYPE_ON)       &&
+        !(pdjb = (struct depend_job *)GET_NEXT(nxdp->dp_jobs)))
       {
-			continue;	/* no value, skip this one */
+      continue; /* no value, skip this one */
       }
-
-		if (nxdp != pdp)
+    
+    if (nxdp != pdp)
       fast_strcat(&BPtr,",");  /* comma between */
 
-		pn = &dependnames[nxdp->dp_type];
+    pn = &dependnames[nxdp->dp_type];
     fast_strcat(&BPtr,pn->name);
 
-		if ((pn->type == JOB_DEPEND_TYPE_SYNCCT) ||
-		    (pn->type == JOB_DEPEND_TYPE_ON))
+    if ((pn->type == JOB_DEPEND_TYPE_SYNCCT) ||
+        (pn->type == JOB_DEPEND_TYPE_ON))
       {
-			sprintf(cvtbuf, ":%d", nxdp->dp_numexp);
+      sprintf(cvtbuf, ":%d", nxdp->dp_numexp);
       fast_strcat(&BPtr,cvtbuf);
       }
     else
       {
-			while (pdjb)
+      while (pdjb)
         {
         fast_strcat(&BPtr,":");
         cat_jobsvr(&BPtr,pdjb->dc_child);
@@ -2502,12 +2545,13 @@ int encode_depend(
           cat_jobsvr(&BPtr,pdjb->dc_svr);
           }
 
-				pdjb = (struct depend_job *)GET_NEXT(pdjb->dc_link);
-			  } 
-		  }
+	pdjb = (struct depend_job *)GET_NEXT(pdjb->dc_link);
+	} 
 
-		++numdep;
-	  }
+      }	
+
+    ++numdep;
+  }
 
   if (numdep)
     {
@@ -2615,8 +2659,11 @@ int set_depend(
  *  -1
  */
 
-int
-comp_depend(struct attribute *attr, struct attribute *with)
+int comp_depend(
+    
+  struct attribute *attr, 
+  struct attribute *with)
+
   {
 
   return (-1);
@@ -2836,7 +2883,7 @@ static int build_depend(
 
       /* a set of job_id[\:port][@server[\:port]] */
 
-      pdjb = (struct depend_job *)malloc(sizeof(*pdjb));
+      pdjb = (struct depend_job *)calloc(1, sizeof(*pdjb));
 
       if (pdjb)
         {
@@ -2873,14 +2920,14 @@ static int build_depend(
 
           if (pwhere)
             {
-            if(server_name != NULL)
-              (void)strcpy(pdjb->dc_svr, server_name);
+            if (server_name != NULL)
+              strcpy(pdjb->dc_svr, server_name);
             else
-              (void)strcpy(pdjb->dc_svr, pwhere + 1);
+              strcpy(pdjb->dc_svr, pwhere + 1);
             }
           else
             {
-            (void)free(pdjb);
+            free(pdjb);
             return (PBSE_BADATVAL);
             }
           }
