@@ -83,9 +83,9 @@
 #include <pbs_config.h>   /* the master config generated by configure */
 
 #include <stdio.h>
-#include <sys/types.h>
 #include "libpbs.h"
 #include <signal.h>
+#include <pthread.h>
 #include "server_limits.h"
 #include "list_link.h"
 #include "work_task.h"
@@ -96,6 +96,8 @@
 #include "pbs_job.h"
 #include "pbs_error.h"
 #include "log.h"
+#include "../lib/Liblog/pbs_log.h"
+#include "../lib/Liblog/log_event.h"
 #include "acct.h"
 #include "svrfunc.h"
 
@@ -126,25 +128,23 @@ static void post_rerun(
   job *pjob;
 
   struct batch_request *preq;
+  char                  log_buf[LOCAL_LOG_BUF_SIZE];
 
   preq = (struct batch_request *)pwt->wt_parm1;
 
   if (preq->rq_reply.brp_code != 0)
     {
-    sprintf(log_buffer, "rerun signal reject by mom: %d",
-            preq->rq_reply.brp_code);
+    sprintf(log_buf, "rerun signal reject by mom: %d", preq->rq_reply.brp_code);
 
-    log_event(
-      PBSEVENT_JOB,
-      PBS_EVENTCLASS_JOB,
-      preq->rq_ind.rq_signal.rq_jid,
-      log_buffer);
+    log_event(PBSEVENT_JOB,PBS_EVENTCLASS_JOB,preq->rq_ind.rq_signal.rq_jid,log_buf);
 
     if ((pjob = find_job(preq->rq_ind.rq_signal.rq_jid)))
       {
       svr_evaljobstate(pjob, &newstate, &newsub, 1);
 
-      svr_setjobstate(pjob, newstate, newsub);
+      svr_setjobstate(pjob, newstate, newsub, FALSE);
+
+      pthread_mutex_unlock(pjob->ji_mutex);
       }
     }
 
@@ -167,19 +167,18 @@ static void post_rerun(
  * NOTE:  can be used to requeue active jobs or completed jobs.
  */
 
-void req_rerunjob(
+void *req_rerunjob(
 
-  struct batch_request *preq) /* I */
+  void *vp) /* I */
 
   {
-  job *pjob;
-  struct work_task	*pwt;
+  struct batch_request *preq = (struct batch_request *)vp;
+  job                  *pjob;
 
-  int  Force;
-
-  int  rc;
-
-  int  MgrRequired = TRUE;
+  int                   Force;
+  int                   rc;
+  int                   MgrRequired = TRUE;
+  char                  log_buf[LOCAL_LOG_BUF_SIZE];
 
   /* check if requestor is admin, job owner, etc */
 
@@ -189,7 +188,7 @@ void req_rerunjob(
 
     /* chk_job_request calls req_reject() */
 
-    return;
+    return(NULL);
     }
 
   /* the job must be running or completed */
@@ -215,7 +214,9 @@ void req_rerunjob(
 
     req_reject(PBSE_BADSTATE, 0, preq, NULL, NULL);
 
-    return;
+    pthread_mutex_unlock(pjob->ji_mutex);
+
+    return(NULL);
     }
 
   if ((MgrRequired == TRUE) &&
@@ -225,7 +226,9 @@ void req_rerunjob(
 
     req_reject(PBSE_PERM, 0, preq, NULL, NULL);
 
-    return;
+    pthread_mutex_unlock(pjob->ji_mutex);
+
+    return(NULL);
     }
 
   /* the job must be rerunnable */
@@ -239,7 +242,9 @@ void req_rerunjob(
 
     req_reject(PBSE_NORERUN, 0, preq, NULL, NULL);
 
-    return;
+    pthread_mutex_unlock(pjob->ji_mutex);
+
+    return(NULL);
     }
 
   if (pjob->ji_qs.ji_state == JOB_STATE_RUNNING)
@@ -248,17 +253,17 @@ void req_rerunjob(
     static char *rerun = "rerun";
     char        *extra = strdup(rerun);
 
-    rc = issue_signal(pjob, "SIGKILL", post_rerun, extra);
+    rc = issue_signal(&pjob, "SIGKILL", post_rerun, extra);
     }
   else
     { 
     if (pjob->ji_wattr[JOB_ATR_hold].at_val.at_long == HOLD_n)
       {
-      svr_setjobstate(pjob, JOB_STATE_QUEUED, JOB_SUBSTATE_QUEUED);
+      svr_setjobstate(pjob, JOB_STATE_QUEUED, JOB_SUBSTATE_QUEUED, FALSE);
       }
     else
       {
-      svr_setjobstate(pjob, JOB_STATE_HELD, JOB_SUBSTATE_HELD);
+      svr_setjobstate(pjob, JOB_STATE_HELD, JOB_SUBSTATE_HELD, FALSE);
       }
 
     /* reset some job attributes */
@@ -266,16 +271,6 @@ void req_rerunjob(
     pjob->ji_wattr[JOB_ATR_comp_time].at_flags &= ~ATR_VFLAG_SET;
     pjob->ji_wattr[JOB_ATR_reported].at_flags &= ~ATR_VFLAG_SET;
 
-    /*
-     * delete any work task entries associated with the job
-     * there may be tasks for keep_completed proccessing
-     */
-
-    while ((pwt = (struct work_task *)GET_NEXT(pjob->ji_svrtask)) != NULL) 
-      {
-      delete_task(pwt);
-      }
-
     set_statechar(pjob);
 
     rc = -1;
@@ -300,7 +295,8 @@ void req_rerunjob(
 
       /* requeue request successful */
 
-      pjob->ji_qs.ji_substate = JOB_SUBSTATE_RERUN;
+      if (pjob != NULL)
+        pjob->ji_qs.ji_substate = JOB_SUBSTATE_RERUN;
 
       break;
 
@@ -308,7 +304,7 @@ void req_rerunjob(
 
       req_reject(PBSE_SYSTEM, 0, preq, NULL, "cannot allocate memory");
 
-      return;
+      return(NULL);
 
       /*NOTREACHED*/
 
@@ -320,87 +316,86 @@ void req_rerunjob(
         {
         req_reject(PBSE_MOMREJECT, 0, preq, NULL, NULL);
 
-        return;
+        if (pjob != NULL)
+          pthread_mutex_unlock(pjob->ji_mutex);
+
+        return(NULL);
         }
       else
         {
         int newstate, newsubst;
         unsigned int dummy;
-
-        /* Cannot communicate with MOM, forcibly requeue job.
-           This is a relatively disgusting thing to do */
-
-        sprintf(log_buffer, "rerun req to %s failed (rc=%d), forcibly requeueing job",
-                parse_servername(pjob->ji_wattr[JOB_ATR_exec_host].at_val.at_str, &dummy),
-                rc);
-
-        log_event(
-          PBSEVENT_ERROR | PBSEVENT_ADMIN | PBSEVENT_JOB,
-          PBS_EVENTCLASS_JOB,
-          pjob->ji_qs.ji_jobid,
-          log_buffer);
-
-        log_err(-1, "req_rerunjob", log_buffer);
-
-        strcat(log_buffer, ", previous output files may be lost");
-
-        svr_mailowner(pjob, MAIL_OTHER, MAIL_FORCE, log_buffer);
-
-        svr_setjobstate(pjob, JOB_STATE_EXITING, JOB_SUBSTATE_RERUN3);
-
-        rel_resc(pjob); /* free resc assigned to job */
-
-        if ((pjob->ji_qs.ji_svrflags & JOB_SVFLG_HOTSTART) == 0)
+        char *tmp;
+       
+        if (pjob != NULL)
           {
-          /* in case of server shutdown, don't clear exec_host */
-          /* will use it on hotstart when next comes up        */
-
-          job_attr_def[JOB_ATR_exec_host].at_free(
-            &pjob->ji_wattr[JOB_ATR_exec_host]);
-
-          job_attr_def[JOB_ATR_session_id].at_free(
-            &pjob->ji_wattr[JOB_ATR_session_id]);
-
-          job_attr_def[JOB_ATR_exec_gpus].at_free(
-            &pjob->ji_wattr[JOB_ATR_exec_gpus]);
-
+          tmp = parse_servername(pjob->ji_wattr[JOB_ATR_exec_host].at_val.at_str, &dummy);
+          
+          /* Cannot communicate with MOM, forcibly requeue job.
+             This is a relatively disgusting thing to do */
+          
+          sprintf(log_buf, "rerun req to %s failed (rc=%d), forcibly requeueing job",
+            tmp, rc);
+  
+          log_event(
+            PBSEVENT_ERROR | PBSEVENT_ADMIN | PBSEVENT_JOB,
+            PBS_EVENTCLASS_JOB,
+            pjob->ji_qs.ji_jobid,
+            log_buf);
+          
+          log_err(-1, "req_rerunjob", log_buf);
+          
+          strcat(log_buf, ", previous output files may be lost");
+  
+          svr_mailowner(pjob, MAIL_OTHER, MAIL_FORCE, log_buf);
+  
+          svr_setjobstate(pjob, JOB_STATE_EXITING, JOB_SUBSTATE_RERUN3, FALSE);
+  
+          rel_resc(pjob); /* free resc assigned to job */
+          
+          if ((pjob->ji_qs.ji_svrflags & JOB_SVFLG_HOTSTART) == 0)
+            {
+            /* in case of server shutdown, don't clear exec_host */
+            /* will use it on hotstart when next comes up        */
+            
+            job_attr_def[JOB_ATR_exec_host].at_free(&pjob->ji_wattr[JOB_ATR_exec_host]);
+  
+            job_attr_def[JOB_ATR_session_id].at_free(&pjob->ji_wattr[JOB_ATR_session_id]);
+            
+            job_attr_def[JOB_ATR_exec_gpus].at_free(&pjob->ji_wattr[JOB_ATR_exec_gpus]);          
+            }
+          
+          pjob->ji_modified = 1;    /* force full job save */
+          
+          pjob->ji_momhandle = -1;
+          pjob->ji_qs.ji_svrflags &= ~JOB_SVFLG_StagedIn;
+          
+          svr_evaljobstate(pjob, &newstate, &newsubst, 0);
+          svr_setjobstate(pjob, newstate, newsubst, FALSE);
           }
-
-        pjob->ji_modified = 1;    /* force full job save */
-
-        pjob->ji_momhandle = -1;
-        pjob->ji_qs.ji_svrflags &= ~JOB_SVFLG_StagedIn;
-
-        svr_evaljobstate(pjob, &newstate, &newsubst, 0);
-
-        svr_setjobstate(pjob, newstate, newsubst);
         }
 
       break;
     }  /* END switch (rc) */
 
   /* So job has run and is to be rerun (not restarted) */
+  if (pjob != NULL)
+    {
+    pjob->ji_qs.ji_svrflags = (pjob->ji_qs.ji_svrflags &
+        ~(JOB_SVFLG_CHECKPOINT_FILE |JOB_SVFLG_CHECKPOINT_MIGRATEABLE |
+          JOB_SVFLG_CHECKPOINT_COPIED)) | JOB_SVFLG_HASRUN;
+    
+    sprintf(log_buf, msg_manager, msg_jobrerun, preq->rq_user, preq->rq_host);
+    log_event(PBSEVENT_JOB,PBS_EVENTCLASS_JOB,pjob->ji_qs.ji_jobid,log_buf);
 
-  pjob->ji_qs.ji_svrflags = (pjob->ji_qs.ji_svrflags &
-      ~(JOB_SVFLG_CHECKPOINT_FILE |JOB_SVFLG_CHECKPOINT_MIGRATEABLE |
-       JOB_SVFLG_CHECKPOINT_COPIED)) | JOB_SVFLG_HASRUN;
+    reply_ack(preq);
+  
+    /* note in accounting file */
+    account_record(PBS_ACCT_RERUN, pjob, NULL);
+    pthread_mutex_unlock(pjob->ji_mutex);
+    }
 
-  sprintf(log_buffer, msg_manager,
-          msg_jobrerun,
-          preq->rq_user,
-          preq->rq_host);
-
-  log_event(
-    PBSEVENT_JOB,
-    PBS_EVENTCLASS_JOB,
-    pjob->ji_qs.ji_jobid,
-    log_buffer);
-
-  reply_ack(preq);
-
-  /* note in accounting file */
-
-  account_record(PBS_ACCT_RERUN, pjob, NULL);
+  return(NULL);
   }  /* END req_rerunjob() */
 
 
