@@ -87,7 +87,6 @@
  */
 #include <pbs_config.h>   /* the master config generated by configure */
 
-#include <sys/types.h>
 #include "libpbs.h"
 #include <ctype.h>
 #include <errno.h>
@@ -102,6 +101,7 @@
 #include "pbs_error.h"
 #include "svrfunc.h"
 #include "log.h"
+#include "reply_send.h" /* reply_send_svr */
 
 /* Global Data Items: */
 
@@ -116,22 +116,25 @@ resource_t next_resource_tag;
  * Currently only recognizes "nodes"
  */
 
-void
-req_rescq(struct batch_request *preq)
+void *req_rescq(
+    
+  void *vp)
+
   {
-  int i;
-  char *pn;
-  char *pv;
-  int rc;
+  struct batch_request *preq = (struct batch_request *)vp;
+  int                   i;
+  char                 *pn;
+  char                 *pv;
+  int                   rc;
 
-  struct brp_rescq *preply;
+  struct brp_rescq     *preply;
 
-  struct rq_rescq *prescq = &preq->rq_ind.rq_rescq;
+  struct rq_rescq      *prescq = &preq->rq_ind.rq_rescq;
 
   if (prescq->rq_num < 1)
     {
     req_reject(RM_ERR_BADPARAM, 0, preq, NULL, NULL);
-    return;
+    return(NULL);
     }
 
   preply = &preq->rq_reply.brp_un.brp_rescq;
@@ -139,16 +142,16 @@ req_rescq(struct batch_request *preq)
   preq->rq_reply.brp_choice = BATCH_REPLY_CHOICE_RescQuery;
   preply->brq_number = prescq->rq_num;
 
-  preply->brq_avail = (int *)malloc(prescq->rq_num * sizeof(int));
-  preply->brq_alloc = (int *)malloc(prescq->rq_num * sizeof(int));
-  preply->brq_resvd = (int *)malloc(prescq->rq_num * sizeof(int));
-  preply->brq_down  = (int *)malloc(prescq->rq_num * sizeof(int));
+  preply->brq_avail = (int *)calloc(prescq->rq_num, sizeof(int));
+  preply->brq_alloc = (int *)calloc(prescq->rq_num, sizeof(int));
+  preply->brq_resvd = (int *)calloc(prescq->rq_num, sizeof(int));
+  preply->brq_down  = (int *)calloc(prescq->rq_num, sizeof(int));
 
   if ((preply->brq_avail == 0) || (preply->brq_alloc == 0) ||
       (preply->brq_resvd == 0) || (preply->brq_down  == 0))
     {
     req_reject(PBSE_SYSTEM, errno, preq, NULL, NULL);
-    return;
+    return(NULL);
     }
 
 
@@ -181,19 +184,24 @@ req_rescq(struct batch_request *preq)
                            preply->brq_down + i)) != 0)
         {
         req_reject(rc, 0, preq, NULL, NULL);
-        return;
+        return(NULL);
         }
       }
     else
       {
       req_reject(RM_ERR_BADPARAM, 0, preq, NULL, NULL);
-      return;
+      return(NULL);
       }
 
     }
 
-  (void)reply_send(preq);
-  }
+  reply_send_svr(preq);
+
+  return(NULL);
+  } /* END req_rescq() */
+
+
+
 
 /*
  * req_rescreserve - Resource reserve
@@ -201,29 +209,32 @@ req_rescq(struct batch_request *preq)
  * Reserve a set of resources (only "nodes" for now)
  */
 
-void
-req_rescreserve(struct batch_request *preq)
-  {
-  int i;
-  int freeold = 0;
+void *req_rescreserve(
+    
+  void *vp)
 
-  struct rq_rescq *prescq = &preq->rq_ind.rq_rescq;
-  int     part = 0;
-  char   *pn;
-  char   *pv;
-  int     rc;
-  resource_t tag;
+  {
+  struct batch_request *preq = (struct batch_request *)vp;
+  int                   i;
+  int                   freeold = 0;
+
+  struct rq_rescq      *prescq = &preq->rq_ind.rq_rescq;
+  int                   part = 0;
+  char                 *pn;
+  char                 *pv;
+  int                   rc;
+  resource_t            tag;
 
   if ((preq->rq_perm & (ATR_DFLAG_MGWR | ATR_DFLAG_OPWR)) == 0)
     {
     req_reject(PBSE_PERM, 0, preq, NULL, NULL);
-    return;
+    return(NULL);
     }
 
   if (prescq->rq_num < 1)
     {
     req_reject(RM_ERR_BADPARAM, 0, preq, NULL, NULL);
-    return;
+    return(NULL);
     }
 
   tag = prescq->rq_rhandle;
@@ -268,7 +279,7 @@ req_rescreserve(struct batch_request *preq)
       if (rc < 0)
         {
         req_reject(rc, 0, preq, NULL, NULL);
-        return;
+        return(NULL);
 
         }
       else if (rc == 0)
@@ -279,7 +290,7 @@ req_rescreserve(struct batch_request *preq)
     else
       {
       req_reject(PBSE_RMUNKNOWN, 0, preq, NULL, NULL);
-      return;
+      return(NULL);
       }
     }
 
@@ -290,23 +301,34 @@ req_rescreserve(struct batch_request *preq)
 
   preq->rq_reply.brp_auxcode = tag;
 
-  reply_send(preq);
-  }
+  reply_send_svr(preq);
+
+  return(NULL);
+  } /* END req_rescreserve() */
+
+
+
 
 /*
  * req_rescfree - Free a reserved set of resources
  */
 
-void
-req_rescfree(struct batch_request *preq)
+void *req_rescfree(
+    
+  void *vp)
+
   {
+  struct batch_request *preq = (struct batch_request *)vp;
   if ((preq->rq_perm & (ATR_DFLAG_MGWR | ATR_DFLAG_OPWR)) == 0)
     {
     req_reject(PBSE_PERM, 0, preq, NULL, NULL);
-    return;
+    return(NULL);
     }
 
   node_unreserve(preq->rq_ind.rq_rescq.rq_rhandle);
 
   reply_ack(preq);
-  }
+
+  return(NULL);
+  } /* END req_rescfree() */
+
