@@ -85,10 +85,10 @@
 
 #define STAT_CNTL 1
 
-#include <sys/types.h>
 #include <stdlib.h>
 #include "libpbs.h"
 #include <string.h>
+#include <pthread.h>
 #include "server_limits.h"
 #include "list_link.h"
 #include "attribute.h"
@@ -100,7 +100,13 @@
 #include "pbs_job.h"
 #include "pbs_error.h"
 #include "log.h"
+#include "../lib/Liblog/pbs_log.h"
+#include "../lib/Liblog/log_event.h"
 #include "svrfunc.h"
+#include "queue_func.h" /* find_queuebyname */
+#include "reply_send.h" /* reply_send_svr */
+#include "svr_func.h" /* get_svr_attr_* */
+
 
 /* Private Data */
 
@@ -112,11 +118,10 @@ extern int   svr_authorize_jobreq(struct batch_request *, job *);
 
 /* Global Data Items  */
 
+extern int LOGLEVEL;
 extern struct server server;
-extern int  resc_access_perm;
-extern tlist_head svr_alljobs;
-extern tlist_head      svr_jobs_array_sum;
-extern time_t  time_now;
+extern struct all_jobs alljobs;
+extern struct all_jobs array_summary;
 
 /* Private Functions  */
 
@@ -282,19 +287,21 @@ static attribute_def state_sel =
  * @see req_statjob() - peer - process qstat request
  */
 
-void req_selectjobs(
+void *req_selectjobs(
 
-  struct batch_request *preq)
+  void *vp)
 
   {
-  int      bad = 0;
+  struct batch_request *preq = (struct batch_request *)vp;
+  int                   bad = 0;
 
-  struct stat_cntl *cntl;
-  svrattrl    *plist;
-  pbs_queue    *pque;
-  int      rc;
+  struct stat_cntl     *cntl;
+  svrattrl             *plist;
+  pbs_queue            *pque = NULL;
+  int                   rc;
+  long                  poll_jobs = 0;
 
-  struct select_list *selistp;
+  struct select_list   *selistp;
 
   plist = (svrattrl *)GET_NEXT(preq->rq_ind.rq_select);
 
@@ -308,10 +315,10 @@ void req_selectjobs(
 
     free_sellist(selistp);
 
-    return;
+    return(NULL);
     }
 
-  if ((cntl = (struct stat_cntl *)malloc(sizeof(struct stat_cntl))) == NULL)
+  if ((cntl = (struct stat_cntl *)calloc(1, sizeof(struct stat_cntl))) == NULL)
     {
     /* FAILURE */
 
@@ -319,7 +326,7 @@ void req_selectjobs(
 
     req_reject(PBSE_SYSTEM, 0, preq, NULL, NULL);
 
-    return;
+    return(NULL);
     }
 
   if (pque != NULL)
@@ -337,11 +344,13 @@ void req_selectjobs(
 
   cntl->sc_select = selistp;  /* the select list */
 
+  get_svr_attr_l(SRV_ATR_PollJobs, &poll_jobs);
+
   if (preq->rq_type == PBS_BATCH_SelectJobs)
     {
     sel_step3(cntl);
     }
-  else if (server.sv_attr[(int)SRV_ATR_PollJobs].at_val.at_long)
+  else if (poll_jobs)
     {
     sel_step3(cntl);
     }
@@ -352,7 +361,10 @@ void req_selectjobs(
     sel_step2(cntl);
     }
 
-  return;
+  if (pque != NULL)
+    unlock_queue(pque, "req_selectjobs", NULL, LOGLEVEL);
+
+  return(NULL);
   }  /* END req_selectjobs() */
 
 
@@ -368,13 +380,17 @@ static void sel_step2(
   struct stat_cntl *cntl)
 
   {
-  job       *pjob;
-  int        rc;
-  int        exec_only = 0;
-  int        summarize_arrays = 0;
-  pbs_queue *pque = NULL;
+  job          *pjob;
+  int           rc;
+  int           exec_only = 0;
+  int           summarize_arrays = 0;
+  pbs_queue    *pque = NULL;
+  int           iter;
+  time_t        time_now = time(NULL);
+  long          query_others = 0;
 
   /* do first pass of finding jobs that match the selection criteria */
+  get_svr_attr_l(SRV_ATR_query_others, &query_others);
 
   if (cntl->sc_jobid[0] == '\0')
     pjob = NULL;
@@ -389,6 +405,8 @@ static void sel_step2(
       summarize_arrays = 1;
     }
 
+  iter = -1;
+
   while (1)
     {
     /*
@@ -402,17 +420,21 @@ static void sel_step2(
       if (pjob == NULL)
         {
         if (cntl->sc_pque)
-          pjob = (job *)GET_NEXT(cntl->sc_pque->qu_jobs_array_sum);
+          pjob = next_job(cntl->sc_pque->qu_jobs_array_sum,&iter);
         else
-          pjob = (job *)GET_NEXT(svr_jobs_array_sum);
+          next_job(&array_summary,&iter);
         }
       else if (cntl->sc_pque)
         {
-        pjob = (job *)GET_NEXT(pjob->ji_jobque_array_sum);
+        pthread_mutex_unlock(pjob->ji_mutex);
+
+        pjob = next_job(cntl->sc_pque->qu_jobs_array_sum,&iter);
         }
       else
         {
-        pjob = (job *)GET_NEXT(pjob->ji_jobs_array_sum);
+        pthread_mutex_unlock(pjob->ji_mutex);
+
+        pjob = next_job(&array_summary,&iter);;
         }
       }
     else
@@ -420,17 +442,21 @@ static void sel_step2(
       if (pjob == NULL)
         {
         if (cntl->sc_pque)
-          pjob = (job *)GET_NEXT(cntl->sc_pque->qu_jobs);
+          pjob = next_job(cntl->sc_pque->qu_jobs,&iter);
         else
-          pjob = (job *)GET_NEXT(svr_alljobs);
+          pjob = next_job(&alljobs,&iter);
         }
       else if (cntl->sc_pque)
         {
-        pjob = (job *)GET_NEXT(pjob->ji_jobque);
+        pthread_mutex_unlock(pjob->ji_mutex);
+
+        pjob = next_job(cntl->sc_pque->qu_jobs_array_sum,&iter);
         }
       else
         {
-        pjob = (job *)GET_NEXT(pjob->ji_alljobs);
+        pthread_mutex_unlock(pjob->ji_mutex);
+
+        pjob = next_job(&alljobs,&iter);
         }
       }
 
@@ -439,13 +465,27 @@ static void sel_step2(
 
     if (exec_only)
       {
-      pque = find_queuebyname(pjob->ji_qs.ji_queue);
-
-      if (pque->qu_qs.qu_type != QTYPE_Execution)
-        continue;
+      if (cntl->sc_pque != NULL)
+        {
+        pque = cntl->sc_pque;
+        
+        if (pque->qu_qs.qu_type != QTYPE_Execution)
+          continue;
+        }
+      else
+        {
+        pque = find_queuebyname(pjob->ji_qs.ji_queue);
+        
+        if (pque->qu_qs.qu_type != QTYPE_Execution)
+          {
+          unlock_queue(pque, "sel_step2", "not exec queue", LOGLEVEL);
+          continue;
+          }
+        unlock_queue(pque, "sel_step2", "exec queue", LOGLEVEL);
+        }
       }
 
-    if (server.sv_attr[(int)SRV_ATR_query_others].at_val.at_long ||
+    if (query_others ||
         (svr_authorize_jobreq(cntl->sc_origrq, pjob) == 0))
       {
       /* have permission to look at job */
@@ -455,17 +495,19 @@ static void sel_step2(
         if ((pjob->ji_qs.ji_substate == JOB_SUBSTATE_RUNNING) &&
             ((time_now - pjob->ji_momstat) > JobStatRate))
           {
-          /* need to ask MOM for new status */
-
           strcpy(cntl->sc_jobid, pjob->ji_qs.ji_jobid);
 
           if ((rc = stat_to_mom(pjob, cntl)) == PBSE_SYSTEM)
             {
+            pthread_mutex_unlock(pjob->ji_mutex);
+
             break;
             }
 
           if (rc == 0)
             {
+            pthread_mutex_unlock(pjob->ji_mutex);
+
             return;
             }
 
@@ -496,6 +538,7 @@ static void sel_step3(
   int        bad = 0;
   int        summarize_arrays = 0;
   job       *pjob;
+  job       *next;
 
   struct batch_request *preq;
 
@@ -508,6 +551,10 @@ static void sel_step3(
   int        exec_only = 0;
   pbs_queue           *pque = NULL;
 
+  int         iter = -1;
+  long        query_others = 0;
+  
+  get_svr_attr_l(SRV_ATR_query_others, &query_others);
   if (cntl->sc_origrq->rq_extend != NULL)
     {
     if (!strncmp(cntl->sc_origrq->rq_extend, "summarize_arrays", strlen("summarize_arrays")))
@@ -537,37 +584,48 @@ static void sel_step3(
       exec_only = 1;
 
   /* now start checking for jobs that match the selection criteria */
-
-
   if (summarize_arrays)
     {
     if (cntl->sc_pque)
-      pjob = (job *)GET_NEXT(cntl->sc_pque->qu_jobs_array_sum);
+      pjob = next_job(cntl->sc_pque->qu_jobs_array_sum,&iter);
     else
-      pjob = (job *)GET_NEXT(svr_jobs_array_sum);
+      {
+      pjob = next_job(&array_summary,&iter);
+      }
     }
   else
     {
     if (cntl->sc_pque)
-      pjob = (job *)GET_NEXT(cntl->sc_pque->qu_jobs);
+      pjob = next_job(cntl->sc_pque->qu_jobs,&iter);
     else
-      pjob = (job *)GET_NEXT(svr_alljobs);
+      pjob = next_job(&alljobs,&iter);
     }
 
-
   while (pjob != NULL)
     {
-    if (server.sv_attr[(int)SRV_ATR_query_others].at_val.at_long ||
+    if (query_others ||
         (svr_authorize_jobreq(preq, pjob) == 0))
       {
       /* either job owner or has special permission to look at job */
 
       if (exec_only)
         {
-        pque = find_queuebyname(pjob->ji_qs.ji_queue);
-
-        if (pque->qu_qs.qu_type != QTYPE_Execution)
-          goto nextjob;
+        if (cntl->sc_pque != NULL)
+          {
+          if (cntl->sc_pque->qu_qs.qu_type != QTYPE_Execution)
+            goto nextjob;
+          }
+        else
+          {
+          pque = find_queuebyname(pjob->ji_qs.ji_queue);
+          
+          if (pque->qu_qs.qu_type != QTYPE_Execution)
+            {
+            unlock_queue(pque, "sel_step3", "not exec queue", LOGLEVEL);
+            goto nextjob;
+            }
+          unlock_queue(pque, "sel_step3", "exec queue", LOGLEVEL);
+          }
         }
 
       if (select_job(pjob, cntl->sc_select))
@@ -578,11 +636,14 @@ static void sel_step3(
           {
           /* Select Jobs */
 
-          pselect = malloc(sizeof(struct brp_select));
+          pselect = calloc(1, sizeof(struct brp_select));
 
           if (pselect == NULL)
             {
             rc = PBSE_SYSTEM;
+
+            pthread_mutex_unlock(pjob->ji_mutex);
+            
             break;
             }
 
@@ -601,27 +662,35 @@ static void sel_step3(
           rc = status_job(pjob, preq, NULL, &preply->brp_un.brp_status, &bad);
 
           if (rc && (rc != PBSE_PERM))
+            {
+            pthread_mutex_unlock(pjob->ji_mutex);
+            
             break;
+            }
           }
         }
       }
 
 nextjob:
+    
+    pthread_mutex_unlock(pjob->ji_mutex);
 
-    if(summarize_arrays)
+    if (summarize_arrays)
       {
       if (cntl->sc_pque)
-        pjob = (job *)GET_NEXT(pjob->ji_jobque_array_sum);
+        next = next_job(cntl->sc_pque->qu_jobs_array_sum,&iter);
       else
-        pjob = (job *)GET_NEXT(pjob->ji_jobs_array_sum);
+        next = next_job(&array_summary,&iter);
       }
     else
       {
       if (cntl->sc_pque)
-        pjob = (job *)GET_NEXT(pjob->ji_jobque);
+        next = next_job(cntl->sc_pque->qu_jobs,&iter);
       else
-        pjob = (job *)GET_NEXT(pjob->ji_alljobs);
+        next = next_job(&alljobs,&iter);
       }
+
+    pjob = next;
     }
 
   free_sellist(cntl->sc_select);
@@ -631,7 +700,7 @@ nextjob:
   if (rc)
     req_reject(rc, 0, preq, NULL, NULL);
   else
-    reply_send(preq);
+    reply_send_svr(preq);
 
   return;
   }  /* END sel_step3() */
@@ -654,11 +723,11 @@ static int select_job(
   {
   while (psel != NULL)
     {
-    if (psel->sl_atindx == (int)JOB_ATR_userlst)
+    if (psel->sl_atindx == JOB_ATR_userlst)
       {
       if (!acl_check(
             &psel->sl_attr,
-            pjob->ji_wattr[(int)JOB_ATR_job_owner].at_val.at_str,
+            pjob->ji_wattr[JOB_ATR_job_owner].at_val.at_str,
             ACL_User))
         {
         /* no match */
@@ -692,8 +761,11 @@ static int select_job(
  * Returns 1 if attribute meets criteria, 0 if not
  */
 
-static int
-sel_attr(attribute *jobat, struct select_list *pselst)
+static int sel_attr(
+    
+  attribute *jobat,
+  struct select_list *pselst)
+
   {
   int    rc;
   resource  *rescjb;
@@ -753,8 +825,10 @@ sel_attr(attribute *jobat, struct select_list *pselst)
  * free_sellist - free a select_list list
  */
 
-static void
-free_sellist(struct select_list *pslist)
+static void free_sellist(
+    
+  struct select_list *pslist)
+
   {
 
   struct select_list *next;
@@ -783,7 +857,7 @@ static int build_selentry(
 
   svrattrl            *plist,
   attribute_def       *pdef,
-  int         perm,
+  int                  perm,
   struct select_list **rtnentry) /* O */
 
   {
@@ -793,7 +867,7 @@ static int build_selentry(
 
   /* create a select list entry for this attribute */
 
-  entry = (struct select_list *)malloc(sizeof(struct select_list));
+  entry = (struct select_list *)calloc(1, sizeof(struct select_list));
 
   if (entry == NULL)
     {
@@ -830,7 +904,8 @@ static int build_selentry(
               &entry->sl_attr,
               plist->al_name,
               plist->al_resc,
-              plist->al_value)))
+              plist->al_value,
+              perm)))
     {
     free(entry);
 
@@ -849,7 +924,7 @@ static int build_selentry(
    * if a resource, use the resource specific one
    */
 
-  if (pdef == &job_attr_def[(int)JOB_ATR_resource])
+  if (pdef == &job_attr_def[JOB_ATR_resource])
     {
     entry->sl_def = (attribute_def *)find_resc_def(
                       svr_resc_def,
@@ -889,25 +964,21 @@ static int build_selentry(
 
 static int build_selist(
 
-  svrattrl     *plist,
-  int       perm,
+  svrattrl            *plist,
+  int                  perm,
   struct select_list **pselist, /* RETURN - select list */
-  pbs_queue    **pque, /* RETURN - queue ptr if limit to que */
-  int      *bad) /* RETURN - index of bad attr */
+  pbs_queue          **pque,    /* RETURN - queue ptr if limit to que */
+  int                 *bad)     /* RETURN - index of bad attr */
 
   {
 
+  char               *pc;
+  int                 i;
+  int                 rc;
+
   struct select_list *entry;
-  int      i;
-  char     *pc;
-  attribute_def    *pdef;
-
   struct select_list *prior = (struct select_list *)0;
-  int      rc;
-
-  /* set permission for decode_resc() */
-
-  resc_access_perm = perm;
+  attribute_def      *pdef;
 
   *pque = (pbs_queue *)0;
   *bad = 0;
@@ -929,8 +1000,7 @@ static int build_selist(
           {
 
           /* does specified destination exist? */
-
-          *pque = find_queuebyname(plist->al_value);
+          *pque = find_queuebyname(plist->al_value); /* mutex freed later */
 
           if (*pque == (pbs_queue *)0)
             return (PBSE_UNKQUE);
