@@ -84,7 +84,6 @@
 #include <pbs_config.h>   /* the master config generated by configure */
 
 #include <errno.h>
-#include <sys/types.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include "libpbs.h"
@@ -102,17 +101,18 @@
 #include "tracking.h"
 #include "svrfunc.h"
 #include "log.h"
+#include "../lib/Liblog/pbs_log.h"
+#include "../lib/Liblog/log_event.h"
 
 /* External functions */
 
-extern int issue_to_svr A_((char *svr, struct batch_request *, void (*func)(struct work_task *)));
+extern int issue_to_svr(char *svr, struct batch_request *, void (*func)(struct work_task *));
 
 /* Global Data Items: */
 
 extern char         *path_track;
 
 extern struct server server;
-extern time_t        time_now;
 extern char          server_name[];
 extern int           LOGLEVEL;
 
@@ -120,21 +120,20 @@ extern int           LOGLEVEL;
  * req_track - record job tracking information
  */
 
-void req_track(
+void *req_track(
 
-  struct batch_request *preq)
+  void *vp)
 
   {
+  struct batch_request *preq = (struct batch_request *)vp;
+  struct tracking      *empty = (struct tracking *)0;
+  int                   i;
+  int                   need;
+  time_t                time_now = time(NULL);
 
-  struct tracking *empty = (struct tracking *)0;
-  int   i;
-  int   need;
-
-  struct tracking *new;
-
-  struct tracking *ptk;
-
-  struct rq_track *prqt;
+  struct tracking      *new;
+  struct tracking      *ptk;
+  struct rq_track      *prqt;
 
   /*  make sure request is from a server */
 
@@ -142,7 +141,7 @@ void req_track(
     {
     req_reject(PBSE_IVALREQ, 0, preq, NULL, NULL);
 
-    return;
+    return(NULL);
     }
 
   /* attempt to locate tracking record for this job    */
@@ -180,7 +179,7 @@ void req_track(
 
         reply_ack(preq);
 
-        return;
+        return(NULL);
         }
       }
     else if (empty == NULL)
@@ -201,21 +200,21 @@ void req_track(
 
       need = server.sv_tracksize * 3 / 2;
 
-      new  = (struct tracking *)realloc(
-               server.sv_track,
-               need * sizeof(struct tracking));
+      new = calloc(need, sizeof(struct tracking));
 
       if (new == NULL)
         {
         /* FAILURE */
 
-        log_err(errno, "req_track", "malloc failed");
+        log_err(errno, "req_track", "calloc failed");
 
         req_reject(PBSE_SYSTEM, 0, preq, NULL, NULL);
 
-        return;
+        return(NULL);
         }
 
+      memcpy(new, server.sv_track, server.sv_tracksize);
+
       empty = new + server.sv_tracksize; /* first new slot */
 
       for (i = server.sv_tracksize;i < need;i++)
@@ -246,8 +245,8 @@ void req_track(
 
   reply_ack(preq);
 
-  return;
-  }
+  return(NULL);
+  } /* END req_track() */
 
 
 
@@ -267,15 +266,21 @@ void track_save(
   struct work_task *pwt)  /* unused */
 
   {
-  int fd;
-  char *myid = "save_track";
+  int        fd;
+  char      *myid = "save_track";
+  time_t     time_now = time(NULL);
+  work_task *wt;
 
   /* set task for next round trip */
 
   if (pwt)    /* set up another work task for next time period */
     {
-    if (!set_task(WORK_Timed, (long)time_now + PBS_SAVE_TRACK_TM,
-                  track_save, 0))
+    free(pwt->wt_mutex);
+    free(pwt);
+
+    wt = set_task(WORK_Timed, (long)time_now + PBS_SAVE_TRACK_TM, track_save, NULL, FALSE);
+
+    if (wt == NULL)
       log_err(errno, myid, "Unable to set task for save");
     }
 
@@ -315,8 +320,10 @@ void track_save(
  * issue_track - issue a Track Job Request to another server
  */
 
-void
-issue_track(job *pjob)
+void issue_track(
+    
+  job *pjob)
+
   {
 
   struct batch_request   *preq;
@@ -327,13 +334,12 @@ issue_track(job *pjob)
   if (preq == (struct batch_request *)0)
     return;
 
-  preq->rq_ind.rq_track.rq_hopcount = pjob->ji_wattr[(int)JOB_ATR_hopcount].at_val.at_long;
+  preq->rq_ind.rq_track.rq_hopcount = pjob->ji_wattr[JOB_ATR_hopcount].at_val.at_long;
 
-  (void)strcpy(preq->rq_ind.rq_track.rq_jid, pjob->ji_qs.ji_jobid);
+  strcpy(preq->rq_ind.rq_track.rq_jid, pjob->ji_qs.ji_jobid);
+  strcpy(preq->rq_ind.rq_track.rq_location, server_name);
 
-  (void)strcpy(preq->rq_ind.rq_track.rq_location, server_name);
-
-  preq->rq_ind.rq_track.rq_state[0] = pjob->ji_wattr[(int)JOB_ATR_state].at_val.at_char;
+  preq->rq_ind.rq_track.rq_state[0] = pjob->ji_wattr[JOB_ATR_state].at_val.at_char;
 
   pc = pjob->ji_qs.ji_jobid;
 
