@@ -88,7 +88,6 @@
 #include <stdio.h>
 #include <stdio.h>
 #include <string.h>
-#include <sys/types.h>
 #include "pbs_ifl.h"
 #include "list_link.h"
 #include "attribute.h"
@@ -97,6 +96,8 @@
 #include "server_limits.h"
 #include "server.h"
 #include "array.h"
+#include "utils.h"
+#include "svr_func.h" /* get_svr_attr_* */
 
 extern struct server server;
 
@@ -114,11 +115,11 @@ extern struct server server;
 
 /* sync w/job_attr_def.c */
 
-static int decode_nodes(struct attribute *, char *, char *, char *);
-static int set_node_ct(resource *, attribute *, int actmode);
-static int set_proc_ct(resource *, attribute *, int actmode);
-static int set_tokens(struct attribute *attr, struct attribute *new, enum batch_op actmode);
-static int set_mppnodect(resource *, attribute *, int actmode);
+int decode_nodes(struct attribute *, char *, char *, char *, int);
+int set_node_ct(resource *, attribute *, int actmode);
+int set_proc_ct(resource *, attribute *, int actmode);
+int set_tokens_nodect(struct attribute *attr, struct attribute *new, enum batch_op actmode);
+int set_mppnodect(resource *, attribute *, int actmode);
 resource_def *svr_resc_def;
 
 resource_def svr_resc_def_const[] =
@@ -472,7 +473,7 @@ resource_def svr_resc_def_const[] =
     set_size,
     comp_size,
     free_null,
-    NULL_FUNC,
+    set_proc_ct,
     READ_WRITE | ATR_DFLAG_MOM | ATR_DFLAG_ALTRUN,
     ATR_TYPE_SIZE
   },
@@ -492,7 +493,7 @@ resource_def svr_resc_def_const[] =
     set_l,
     comp_l,
     free_null,
-    set_proc_ct,
+    NULL_FUNC,
     READ_WRITE | ATR_DFLAG_MOM | ATR_DFLAG_RMOMIG,
     ATR_TYPE_LONG
   },
@@ -591,7 +592,7 @@ resource_def svr_resc_def_const[] =
   { ATTR_tokens,                               /* tokens required to run */
     decode_tokens,
     encode_str,
-    set_tokens,
+    set_tokens_nodect,
     comp_str,
     free_str,
     NULL_FUNC,
@@ -782,9 +783,8 @@ int init_resc_defs(void)
 #ifndef PBS_MOM
   /* build up a temporary list of string resources */
 
-  if (server.sv_attr[SRV_ATR_ExtraResc].at_flags & ATR_VFLAG_SET)
+  if (get_svr_attr_arst(SRV_ATR_ExtraResc, &resc_arst) == PBSE_NONE)
     {
-    resc_arst = server.sv_attr[SRV_ATR_ExtraResc].at_val.at_arst;
 
     tmpresc = calloc(resc_arst->as_usedptr + 1, sizeof(resource_def));
 
@@ -873,12 +873,13 @@ int init_resc_defs(void)
  * where node_spec is: number | property | number:property
  */
 
-static int decode_nodes(
+int decode_nodes(
 
-  struct attribute *patr,
-  char             *name,   /* attribute name */
-  char             *rescn,  /* resource name - unused here */
-  char             *val)    /* attribute value */
+  attribute *patr,
+  char      *name,   /* attribute name */
+  char      *rescn,  /* resource name - unused here */
+  char      *val,    /* attribute value */
+  int        perm)   /* replaces resc_access_perm */
 
   {
   char *pc;
@@ -934,7 +935,7 @@ static int decode_nodes(
     ++pc;
     }  /* END while(1) */
 
-  return(decode_str(patr, name, rescn, val));
+  return(decode_str(patr, name, rescn, val, perm));
   }  /* END decode_nodes() */
 
 
@@ -980,19 +981,24 @@ int ctnodes(
 
 long count_proc(
 
-  char *spec)
+  char *param_spec)
 
   {
-  long num_nodes = 0, num_procs = 0, total_procs = 0;
-  char *substr, *ppnloc;
+  long  num_nodes = 0;
+  long  num_procs = 0;
+  long  total_procs = 0;
+  char *substr;
+  char *ppnloc;
+  char *spec;
+  char *spec_ptr;
 
-  spec = strdup(spec);
-  if (spec == NULL) 
+  if ((spec = strdup(param_spec)) == NULL)
     return(-1);
 
+  spec_ptr = spec;
+
   /* split on + */
-  substr = strtok(spec, "+");
-  do
+  while ((substr = threadsafe_tokenizer(&spec_ptr, "+")) != NULL)
     {
     num_nodes = atoi(substr);
     if (num_nodes == 0)
@@ -1006,12 +1012,11 @@ long count_proc(
       num_procs = atoi(ppnloc + 5);
       
     total_procs += num_procs * num_nodes;
-    substr = strtok(NULL, "+");
-    } while(substr != NULL);
+    }
 
   free(spec);
-  return(total_procs);
 
+  return(total_procs);
   }  /* END count_proc */
 
 
@@ -1088,7 +1093,7 @@ int set_node_ct(
     pndef->rs_free(&pnct->rs_value);
     }
 
-  pndef->rs_decode(&pnct->rs_value, NULL, NULL, pnodesp->rs_value.at_val.at_str);
+  pndef->rs_decode(&pnct->rs_value, NULL, NULL, pnodesp->rs_value.at_val.at_str, ATR_DFLAG_ACCESS);
 
   pnct->rs_value.at_flags |= ATR_VFLAG_SET;
 
@@ -1205,11 +1210,11 @@ int set_proc_ct(
 
 
 /*
- * set_tokens = set node count
+ * set_tokens_nodect = set node count
  *
  */
 
-static int set_tokens(
+int set_tokens_nodect(
 
   struct attribute *attr,
   struct attribute *new,
@@ -1247,14 +1252,14 @@ static int set_tokens(
     }
 
   return(ret);
-  }  /* END set_tokens() */
+  }  /* END set_tokens_nodect() */
 
 /*
  * set_mppnodect
  *
  */
 
-static int set_mppnodect(
+int set_mppnodect(
 
   resource *res,
   attribute *attr,
