@@ -80,16 +80,20 @@
 #include <pbs_config.h>   /* the master config generated by configure */
 
 #include <stdio.h>
-#include <sys/types.h>
 #include "portability.h"
 #include <errno.h>
-#include <netinet/in.h>
+#include <pthread.h>
+#include <arpa/inet.h>
 #include "log.h"
+#include "../lib/Liblog/pbs_log.h"
+#include "../lib/Liblog/log_event.h"
 #include "pbs_ifl.h"
 #include "net_connect.h"
 #include "svrfunc.h"
 #include "sched_cmds.h"
 #include "pbs_error.h"
+#include "../lib/Libdis/lib_dis.h" /* DIS_tcp_setup */
+#include "pbsd_main.h" /* process_pbs_server_port */
 
 /* Global Data */
 
@@ -98,7 +102,9 @@ extern unsigned int pbs_scheduler_port;
 extern char      server_name[];
 
 extern struct connection svr_conn[];
-extern int  svr_do_schedule;
+
+extern pthread_mutex_t *svr_do_schedule_mutex;
+extern int       svr_do_schedule;
 extern char     *msg_sched_called;
 extern char     *msg_sched_nocall;
 extern char     *msg_listnr_called;
@@ -106,9 +112,11 @@ extern char     *msg_listnr_nocall;
 
 extern struct listener_connection listener_conns[];
 
-int scheduler_sock = -1;
-int scheduler_jobct = 0;
+int scheduler_sock;
+int scheduler_jobct;
 int listener_command = SCH_SCHEDULE_NULL;
+extern pthread_mutex_t *listener_command_mutex;
+extern pthread_mutex_t *scheduler_sock_jobct_mutex;
 
 
 /* Functions private to this file */
@@ -118,7 +126,7 @@ static void scheduler_close(int);
 static void listener_close(int);
 
 
-extern void bad_node_warning(pbs_net_t);
+extern void bad_node_warning(pbs_net_t, struct pbsnode *);
 extern ssize_t write_nonblocking_socket(int, const void *, ssize_t);
 
 
@@ -161,27 +169,15 @@ static int contact_sched(
   char  EMsg[1024];
 
   char *id = "contact_sched";
+  char  log_buf[LOCAL_LOG_BUF_SIZE];
 
   /* connect to the Scheduler */
-
-#if 0   /* don't check if scheduler runs on same node as server */
-
-  if (!addr_ok(pbs_scheduler_addr))
-    {
-    pbs_errno = EHOSTDOWN;
-    return -1;
-    }
-
-#endif
-
   sock = client_to_svr(pbs_scheduler_addr, pbs_scheduler_port, 1, EMsg);
 
   if (sock < 0)
     {
     /* FAILURE */
 
-    bad_node_warning(pbs_scheduler_addr);
-
 #if 0
     sprintf(tmpLine, "%s - port %d %s",
             msg_sched_nocall,
@@ -193,19 +189,19 @@ static int contact_sched(
 
     return(-1);
     }
-
   add_conn(
-
     sock,
     FromClientDIS,
     pbs_scheduler_addr,
     pbs_scheduler_port,
     PBS_SOCK_INET,
-    process_request);
+    process_pbs_server_port);
+
+  pthread_mutex_lock(svr_conn[sock].cn_mutex);
 
   svr_conn[sock].cn_authen = PBS_NET_CONN_FROM_PRIVIL;
 
-  net_add_close_func(sock, scheduler_close);
+  net_add_close_func(sock, scheduler_close, TRUE);
 
   /* send command to Scheduler */
 
@@ -217,20 +213,20 @@ static int contact_sched(
 
     log_ext(errno,id,tmpLine,LOG_ALERT);
 
-    close_conn(sock);
+    close_conn(sock, TRUE);
+
+    pthread_mutex_unlock(svr_conn[sock].cn_mutex);
 
     return(-1);
     }
+    
+  pthread_mutex_unlock(svr_conn[sock].cn_mutex);
 
-  sprintf(log_buffer, msg_sched_called,
+  DIS_tcp_setup(sock);
 
-          (cmd != SCH_ERROR) ? PSchedCmdType[cmd] : "ERROR");
+  sprintf(log_buf, msg_sched_called, (cmd != SCH_ERROR) ? PSchedCmdType[cmd] : "ERROR");
 
-  log_event(
-    PBSEVENT_SCHED,
-    PBS_EVENTCLASS_SERVER,
-    server_name,
-    log_buffer);
+  log_event(PBSEVENT_SCHED,PBS_EVENTCLASS_SERVER,server_name,log_buf);
 
   return (sock);
   }  /* END contact_sched() */
@@ -248,14 +244,15 @@ static int contact_sched(
  *   +1 = scheduler busy
  */
 
-int
-schedule_jobs(void)
+int schedule_jobs(void)
 
   {
   int cmd;
 
   static int first_time = 1;
 
+  pthread_mutex_lock(svr_do_schedule_mutex);
+
   if (first_time)
     cmd = SCH_SCHEDULE_FIRST;
   else
@@ -264,6 +261,9 @@ schedule_jobs(void)
   /*listener_command = cmd;*/
 
   svr_do_schedule = SCH_SCHEDULE_NULL;
+  pthread_mutex_unlock(svr_do_schedule_mutex);
+
+  pthread_mutex_lock(scheduler_sock_jobct_mutex);
 
   if (scheduler_sock == -1)
     {
@@ -271,13 +271,20 @@ schedule_jobs(void)
 
     if ((scheduler_sock = contact_sched(cmd)) < 0)
       {
+      pthread_mutex_unlock(scheduler_sock_jobct_mutex);
       return(-1);
       }
+      
+    pthread_mutex_unlock(scheduler_sock_jobct_mutex);
 
     first_time = 0;
 
     return(0);
     }
+  else
+    {
+    pthread_mutex_unlock(scheduler_sock_jobct_mutex);
+    }
 
   return(1);
   }  /* END schedule_jobs() */
@@ -298,11 +305,16 @@ static int contact_listener(
   char  EMsg[1024];
 
   char *id = "contact_listener";
+  char  log_buf[LOCAL_LOG_BUF_SIZE];
 
   /* If this is the first time contacting the scheduler for
    * this listener set the cmd */
-  if(listener_conns[l_idx].first_time)
+  if (listener_conns[l_idx].first_time)
+    {
+    pthread_mutex_lock(listener_command_mutex);
     listener_command = SCH_SCHEDULE_FIRST;
+    pthread_mutex_unlock(listener_command_mutex);
+    }
  
   /* connect to the Listener */
   sock = client_to_svr(listener_conns[l_idx].address,
@@ -312,7 +324,7 @@ static int contact_listener(
     {
     /* FAILURE */
 
-    bad_node_warning(listener_conns[l_idx].address);
+    bad_node_warning(listener_conns[l_idx].address, NULL);
 
     sprintf(tmpLine, "%s %d - port %d %s",
             msg_listnr_nocall,
@@ -330,7 +342,6 @@ static int contact_listener(
   listener_conns[l_idx].first_time = 0;
 
   add_conn(
-
     sock,
     FromClientDIS,
     listener_conns[l_idx].address,
@@ -338,14 +349,20 @@ static int contact_listener(
     PBS_SOCK_INET,
     process_request);
 
+  pthread_mutex_lock(svr_conn[sock].cn_mutex);
+
   svr_conn[sock].cn_authen = PBS_NET_CONN_FROM_PRIVIL;
 
-  net_add_close_func(sock, listener_close);
+
+  net_add_close_func(sock, listener_close, TRUE);
 
   /* send command to Listener */
+  pthread_mutex_lock(listener_command_mutex);
 
   if (put_4byte(sock, listener_command) < 0)
     {
+    pthread_mutex_unlock(listener_command_mutex);
+    
     sprintf(tmpLine, "%s %d - port %d",
             msg_listnr_nocall,
             l_idx + 1,
@@ -353,20 +370,21 @@ static int contact_listener(
 
     log_err(errno, id, tmpLine);
 
-    close_conn(sock);
+    close_conn(sock, TRUE);
+  
+    pthread_mutex_unlock(svr_conn[sock].cn_mutex);
 
     return(-1);
     }
 
-  sprintf(log_buffer, msg_listnr_called, l_idx + 1,
+  pthread_mutex_unlock(listener_command_mutex);
 
-          (listener_command != SCH_ERROR) ? PSchedCmdType[listener_command] : "ERROR");
+  pthread_mutex_unlock(svr_conn[sock].cn_mutex);
 
-  log_event(
-    PBSEVENT_SCHED,
-    PBS_EVENTCLASS_SERVER,
-    server_name,
-    log_buffer);
+  sprintf(log_buf, msg_listnr_called, l_idx + 1,
+    (listener_command != SCH_ERROR) ? PSchedCmdType[listener_command] : "ERROR");
+
+  log_event(PBSEVENT_SCHED,PBS_EVENTCLASS_SERVER,server_name,log_buf);
 
   return (sock);
   }  /* END contact_listener() */
@@ -377,8 +395,7 @@ static int contact_listener(
  *
  */
 
-void
-notify_listeners(void)
+void notify_listeners(void)
 
   {
   int  i;
@@ -416,6 +433,7 @@ static void scheduler_close(
   int sock)
 
   {
+  pthread_mutex_lock(scheduler_sock_jobct_mutex);
   scheduler_sock = -1;
 
   /*
@@ -432,9 +450,16 @@ static void scheduler_close(
     {
     /* recycle the scheduler */
 
+    pthread_mutex_lock(svr_do_schedule_mutex);
     svr_do_schedule = SCH_SCHEDULE_RECYC;
+    pthread_mutex_unlock(svr_do_schedule_mutex);
+
+    pthread_mutex_lock(listener_command_mutex);
     listener_command = SCH_SCHEDULE_RECYC;
+    pthread_mutex_unlock(listener_command_mutex);
     }
+    
+  pthread_mutex_unlock(scheduler_sock_jobct_mutex);
 
   return;
   }  /* END scheduler_close() */
