@@ -87,7 +87,6 @@
  * status_job()
  * status_attrib()
  */
-#include <sys/types.h>
 #include <stdlib.h>
 #include "libpbs.h"
 #include <ctype.h>
@@ -104,7 +103,7 @@
 #include "pbs_error.h"
 #include "svrfunc.h"
 #include "resource.h"
-
+#include "svr_func.h" /* get_svr_attr_* */
 
 extern int     svr_authorize_jobreq(struct batch_request *, job *);
 int status_attrib(svrattrl *, attribute_def *, attribute *, int, int, tlist_head *, int *, int);
@@ -112,10 +111,8 @@ int status_attrib(svrattrl *, attribute_def *, attribute *, int, int, tlist_head
 /* Global Data Items: */
 
 extern attribute_def job_attr_def[];
-extern int      resc_access_perm; /* see encode_resc() in attr_fn_resc.c */
 
 extern struct server server;
-extern time_t time_now;
 
 
 
@@ -137,17 +134,16 @@ int status_job(
   int        *bad) /* RETURN: index of first bad attribute */
 
   {
-
   struct brp_status *pstat;
-
-  int IsOwner = 0;
+  int                IsOwner = 0;
+  long               query_others = 0;
 
   /* see if the client is authorized to status this job */
-
   if (svr_authorize_jobreq(preq, pjob) == 0)
     IsOwner = 1;
 
-  if (!server.sv_attr[SRV_ATR_query_others].at_val.at_long)
+  get_svr_attr_l(SRV_ATR_query_others, &query_others);
+  if (!query_others)
     {
     if (IsOwner == 0)
       {
@@ -156,16 +152,11 @@ int status_job(
     }
 
   /* allocate reply structure and fill in header portion */
-
-  pstat = (struct brp_status *)malloc(sizeof(struct brp_status));
-
-  if (pstat == NULL)
+  if ((pstat = calloc(1, sizeof(struct brp_status))) == NULL)
     {
     return(PBSE_SYSTEM);
     }
 
-  memset(pstat, 0, sizeof(struct brp_status));
-
   CLEAR_LINK(pstat->brp_stlink);
 
   pstat->brp_objtype = MGR_OBJ_JOB;
@@ -198,6 +189,79 @@ int status_job(
 
 
 
+/* Is this dead code? It isn't called anywhere. */
+int add_walltime_remaining(
+   
+  int         index,
+  attribute  *pattr,
+  tlist_head *phead)
+
+  {
+  int            len = 0;
+  char           buf[MAXPATHLEN];
+  char          *pname;
+  svrattrl      *pal;
+  resource      *pres;
+  
+  int            found = 0;
+  unsigned long  remaining = 0;
+  unsigned long  upperBound = 0;
+  time_t         time_now   = time(NULL);
+
+  /* encode walltime remaining, this is custom because walltime 
+   * remaining isn't an attribute */
+  if ((pattr + JOB_ATR_state)->at_val.at_char != 'R')
+    {
+    /* only for running jobs, do nothing */
+    return(PBSE_NONE);
+    }
+  
+  if (((pattr + JOB_ATR_resource)->at_val.at_list.ll_next != NULL) &&
+      ((pattr + JOB_ATR_resource)->at_flags & ATR_VFLAG_SET))
+    {
+    pres = (resource *)GET_NEXT((pattr + JOB_ATR_resource)->at_val.at_list);
+    
+    if ((pattr + JOB_ATR_comp_time)->at_flags & ATR_VFLAG_SET)
+      upperBound = (pattr + JOB_ATR_comp_time)->at_val.at_long;
+    else
+      upperBound = (unsigned long)time_now;
+    
+    /* find the walltime resource */
+    for (;pres != NULL;pres = (resource *)GET_NEXT(pres->rs_link))
+      {
+      pname = pres->rs_defin->rs_name;
+      
+      if (strcmp(pname, "walltime") == 0)
+        {
+        /* found walltime */
+        unsigned long value = (unsigned long)pres->rs_value.at_val.at_long;
+        remaining = value - (time_now - (pattr + index)->at_val.at_long);
+        found = upperBound * 12;
+        found = TRUE;
+        break;
+        }
+      }
+    }
+  
+  if (found == TRUE)
+    {
+    snprintf(buf,sizeof(buf),"%lu",remaining);
+    
+    len = strlen(buf);
+    pal = attrlist_create("Walltime","Remaining",len+1);
+    
+    if (pal != NULL)
+      {
+      memcpy(pal->al_value,buf,len);
+      pal->al_flags = ATR_VFLAG_SET;
+      append_link(phead,&pal->al_link,pal);
+      }
+    }
+
+  return(PBSE_NONE);
+  } /* END add_walltime_remaining() */
+
+
 
 
 
@@ -214,22 +278,22 @@ int status_job(
 
 int status_attrib(
 
-  svrattrl *pal,      /* I */
+  svrattrl      *pal,      /* I */
   attribute_def *padef,
-  attribute *pattr,
-  int   limit,
-  int   priv,
-  tlist_head *phead,
-  int  *bad,
+  attribute     *pattr,
+  int            limit,
+  int            priv,
+  tlist_head    *phead,
+  int           *bad,
   int            IsOwner)  /* 0 == FALSE, 1 == TRUE */
 
   {
-  int   index;
-  int   nth = 0;
+  int    index;
+  int    nth = 0;
+  int    resc_access_perm;
 
   priv &= ATR_DFLAG_RDACC;  /* user-client privilege  */
-
-  resc_access_perm = priv;  /* pass privilege to encode_resc() */
+  resc_access_perm = priv; 
 
   /* for each attribute asked for or for all attributes, add to reply */
 
@@ -248,7 +312,6 @@ int status_attrib(
         *bad = nth;
 
         /* FAILURE */
-
         return(-1);
         }
 
@@ -261,7 +324,8 @@ int status_attrib(
             phead,
             (padef + index)->at_name,
             NULL,
-            ATR_ENCODE_CLIENT);
+            ATR_ENCODE_CLIENT,
+            resc_access_perm);
           }
         }
 
@@ -269,8 +333,7 @@ int status_attrib(
       }
 
     /* SUCCESS */
-
-    return(0);
+    return(PBSE_NONE);
     }    /* END if (pal != NULL) */
 
   /* attrlist not specified, return all readable attributes */
@@ -287,76 +350,19 @@ int status_attrib(
           phead,
           (padef + index)->at_name,
           NULL,
-          ATR_ENCODE_CLIENT);
+          ATR_ENCODE_CLIENT,
+          resc_access_perm);
 
         /* add walltime remaining if started */
         if ((index == JOB_ATR_start_time) &&
             ((pattr + index)->at_flags & ATR_VFLAG_SET))
-          {
-          /* encode walltime remaining, this is custom because walltime 
-           * remaining isn't an attribute */
-          int       len;
-          char      buf[MAXPATHLEN];
-          char     *pname;
-          svrattrl *pal;
-          resource *pres;
-
-          int found = 0;
-          unsigned long remaining = 0;
-          unsigned long upperBound = 0;
-
-          if (((pattr + JOB_ATR_resource)->at_val.at_list.ll_next != NULL) &&
-              ((pattr + JOB_ATR_resource)->at_flags & ATR_VFLAG_SET))
-            {
-            pres = (resource *)GET_NEXT((pattr + JOB_ATR_resource)->at_val.at_list);
-
-            if ((pattr + JOB_ATR_comp_time)->at_flags & ATR_VFLAG_SET)
-              {
-              upperBound = (pattr + JOB_ATR_comp_time)->at_val.at_long;
-              }
-            else
-              {
-              upperBound = (unsigned long)time_now;
-              }
-
-            /* find the walltime resource */
-            for (;pres != NULL;pres = (resource *)GET_NEXT(pres->rs_link))
-              {
-              pname = pres->rs_defin->rs_name;
-
-              if (strcmp(pname, "walltime") == 0)
-                {
-                /* found walltime */
-                unsigned long value = (unsigned long)pres->rs_value.at_val.at_long;
-                remaining = value - (time_now - (pattr + index)->at_val.at_long);
-                found = TRUE;
-                break;
-                }
-              }
-            }
-
-          if (found == TRUE)
-            {
-            snprintf(buf,sizeof(buf),"%lu",remaining);
-
-            len = strlen(buf) + 1;
-            pal = attrlist_create("Walltime","Remaining",len);
-
-            if (pal != NULL)
-              {
-              memcpy(pal->al_value,buf,len);
-              pal->al_flags = ATR_VFLAG_SET;
-              append_link(phead,&pal->al_link,pal);
-              }
-            }
-          } /* END if (index == JOB_ATR_start_time) */
+          add_walltime_remaining(index, pattr, phead);
         }
       }
     }    /* END for (index) */
 
   /* SUCCESS */
-
-  return(0);
+  return(PBSE_NONE);
   }  /* END status_attrib() */
 
 /* END stat_job.c */
