@@ -100,14 +100,13 @@ extern int set_nextjobnum (attribute * attr, attribute * new,
 extern int poke_scheduler (attribute * pattr, void *pobject, int actmode);
 
 extern int encode_svrstate (attribute * pattr, tlist_head * phead,
-			    char *aname, char *rsname, int mode);
+			    char *aname, char *rsname, int mode, int perm);
 
-extern int decode_rcost (attribute * patr, char *name, char *rn, char *val);
+extern int decode_rcost (attribute * patr, char *name, char *rn, char *val, int perm);
 extern int encode_rcost (attribute * attr, tlist_head * phead, char *atname,
-			 char *rsname, int mode);
+			 char *rsname, int mode, int perm);
 extern int set_rcost (attribute * attr, attribute * new, enum batch_op);
 extern void free_rcost (attribute * attr);
-extern int decode_null (attribute * patr, char *name, char *rn, char *val);
 extern int set_null (attribute * patr, attribute * new, enum batch_op op);
 
 extern int token_chk (attribute * pattr, void *pobject, int actmode);
@@ -117,7 +116,6 @@ extern int set_tokens (struct attribute *attr, struct attribute *new,
 extern int extra_resc_chk (attribute * pattr, void *pobject, int actmode);
 extern void free_extraresc (attribute * attr);
 extern void restore_attr_default (struct attribute *);
-extern int update_gpus (attribute * pattr, void *pobject, int actmode);
 
 /* DIAGTODO: write diag_attr_def.c */
 
@@ -585,7 +583,7 @@ attribute_def svr_attr_def[] =
   },
 
   /* SRV_ATR_LogLevel */
-  {   ATTR_loglevel,  /* "loglevel" */
+  {   ATTR_loglevel,  /* "log_level" */
       decode_l,
       encode_l,
       set_l,
@@ -1233,6 +1231,45 @@ attribute_def svr_attr_def[] =
    PARENT_TYPE_SERVER},
 #endif
 
+  /* SRV_ATR_minthreads */
+  {ATTR_minthreads,               /* "min_threads" */
+   decode_l,
+   encode_l,
+   set_l,
+   comp_l,
+   free_null,
+   NULL_FUNC,
+   MGR_ONLY_SET,
+   ATR_TYPE_LONG,
+   PARENT_TYPE_SERVER
+  },
+
+  /* SRV_ATR_maxthreads */
+  {ATTR_maxthreads,               /* "max_threads" */
+   decode_l,
+   encode_l,
+   set_l,
+   comp_l,
+   free_null,
+   NULL_FUNC,
+   MGR_ONLY_SET,
+   ATR_TYPE_LONG,
+   PARENT_TYPE_SERVER
+  },
+
+  /* SRV_ATR_threadidleseconds */
+  {ATTR_threadidleseconds,        /* "thread_idle_seconds" */
+   decode_l,
+   encode_l,
+   set_l,
+   comp_l,
+   free_null,
+   NULL_FUNC,
+   MGR_ONLY_SET,
+   ATR_TYPE_LONG,
+   PARENT_TYPE_SERVER
+  },
+
   /* SRV_ATR_MoabArrayCompatible */
   {ATTR_moabarraycompatible,
    decode_b,
@@ -1246,6 +1283,18 @@ attribute_def svr_attr_def[] =
    PARENT_TYPE_SERVER
   },
 
+  /* SRV_ATR_NoMailForce */
+  {ATTR_nomailforce,		/* "no_mail_force" */
+   decode_b,
+   encode_b,
+   set_b,
+   comp_b,
+   free_null,
+   NULL_FUNC,
+   MGR_ONLY_SET,
+   ATR_TYPE_LONG,
+   PARENT_TYPE_SERVER},
+
   /* site supplied server attribute definitions if any, see site_svr_attr_*.h  */
 #include "site_svr_attr_def.h"
 
