@@ -78,7 +78,8 @@
 */
 #include <pbs_config.h>   /* the master config generated by configure */
 
-#include <sys/types.h>
+#include <pthread.h>
+
 #include <stdio.h>
 #include "libpbs.h"
 #include "string.h"
@@ -91,8 +92,12 @@
 #include "batch_request.h"
 #include "pbs_error.h"
 #include "log.h"
+#include "../lib/Liblog/pbs_log.h"
+#include "../lib/Liblog/log_event.h"
 #include "svrfunc.h"
 #include "mcom.h"
+#include "utils.h"
+#include "svr_func.h" /* get_svr_attr_* */
 
 /* Global Data */
 
@@ -100,7 +105,6 @@ extern struct credential conn_credent[PBS_NET_MAX_CONNECTIONS];
 extern char *pbs_o_host;
 extern char  server_host[];
 extern char *msg_permlog;
-extern time_t time_now;
 
 extern char *PJobState[];
 
@@ -155,10 +159,7 @@ int svr_chk_owner_generic(
     return(-1);
     }
 
-  strncpy(rmtuser, pu, PBS_MAXUSER);
-
-  rmtuser[PBS_MAXUSER] = '\0';
-
+  snprintf(rmtuser, sizeof(rmtuser), "%s", pu);
 
   pu = site_map_user(owner, submit_host);
 
@@ -258,18 +259,13 @@ int svr_get_privilege(
   char  uh[PBS_MAXUSER + PBS_MAXHOSTNAME + 2];
   char  host_no_port[PBS_MAXHOSTNAME+1];
   char *colon_loc = NULL;
+  char  log_buf[LOCAL_LOG_BUF_SIZE];
 
   if (!user)
     {
+    sprintf(log_buf, "Invalid user: %s", "null");
 
-    sprintf(log_buffer, "Invalid user: %s", "null");
-
-    log_record(
-      PBSEVENT_SECURITY,
-      PBS_EVENTCLASS_SERVER,
-      id,
-      log_buffer);
-
+    log_record(PBSEVENT_SECURITY,PBS_EVENTCLASS_SERVER,id,log_buf);
 
     return(0);
     }
@@ -277,13 +273,9 @@ int svr_get_privilege(
   /* user name cannot be longer than PBS_MAXUSER*/
   if (strlen(user) > PBS_MAXUSER)
     {
-    sprintf(log_buffer, "Invalid user: %s", user);
+    sprintf(log_buf, "Invalid user: %s", user);
 
-    log_record(
-      PBSEVENT_SECURITY,
-      PBS_EVENTCLASS_SERVER,
-      id,
-      log_buffer);
+    log_record(PBSEVENT_SECURITY,PBS_EVENTCLASS_SERVER,id,log_buf);
      
     return(0);
     }
@@ -315,17 +307,9 @@ int svr_get_privilege(
   /* num_host_chars cannot be more than PBS_MAXHOSTNAME */
   if (num_host_chars > PBS_MAXHOSTNAME)
     {
-    snprintf(log_buffer, LOG_BUF_SIZE, "Invalid host: %s", host);
-    /* log_buffer is big but just in case host was really long we need to
-       null terminate the last byte. strncpy will copy LOG_BUF_SIZE but
-       not null terminate the array if host is larger. */
-    log_buffer[LOG_BUF_SIZE - 1] = 0;
+    snprintf(log_buf, sizeof(log_buf), "Invalid host: %s", host);
 
-    log_record(
-      PBSEVENT_SECURITY,
-      PBS_EVENTCLASS_SERVER,
-      id,
-      log_buffer);
+    log_record(PBSEVENT_SECURITY,PBS_EVENTCLASS_SERVER,id,log_buf);
 
     return(0);
     }
@@ -342,8 +326,10 @@ int svr_get_privilege(
     }
 #else /* __CYGWIN__ */
 
+/*  snprintf(log_buf, sizeof(log_buf), "%s - %s\n", user, server_host);
+  log_record(PBSEVENT_SECURITY, PBS_EVENTCLASS_SERVER, "svr_get_privilege", log_buf); */
   if ((strcmp(user, PBS_DEFAULT_ADMIN) == 0) &&
-      !strcasecmp(host_no_port, server_host))
+       !strcasecmp(host_no_port, server_host))
     {
     is_root = 1;
 
@@ -353,6 +339,7 @@ int svr_get_privilege(
     }
 #endif /* __CYGWIN__ */
 
+  pthread_mutex_lock(server.sv_attr_mutex);
   if (!(server.sv_attr[SRV_ATR_managers].at_flags & ATR_VFLAG_SET))
     {
     if (is_root)
@@ -372,6 +359,7 @@ int svr_get_privilege(
     {
     priv |= (ATR_DFLAG_OPRD | ATR_DFLAG_OPWR);
     }
+  pthread_mutex_unlock(server.sv_attr_mutex);
 
   return(priv);
   }  /* END svr_get_privilege() */
@@ -395,19 +383,24 @@ int authenticate_user(
 
   {
   char id[] = "authenticate_user";
-  int  rc;
-  char uath[PBS_MAXUSER + PBS_MAXHOSTNAME + 1];
+  int    rc;
+  char   uath[PBS_MAXUSER + PBS_MAXHOSTNAME + 1];
+  time_t time_now = time(NULL);
   char error_msg[1024];
+  long   acl_enabled = FALSE;
 
 #ifdef MUNGE_AUTH
+ 
   if (strncmp(preq->rq_user, pcred->username, PBS_MAXUSER))
     {
     /* extra check for munge */
+    struct array_strings *my_acl = NULL;
     char uh[PBS_MAXUSER + PBS_MAXHOSTNAME + 2];
 
     sprintf(uh, "%s@%s", preq->rq_user, pcred->hostname);
     
-    if ((acl_check(&server.sv_attr[SRV_ATR_authusers], uh, ACL_User_Host)) == 0)
+    get_svr_attr_arst(SRV_ATR_authusers, &my_acl); 
+    if ((acl_check_my_array_string(my_acl, uh, ACL_User_Host)) == 0)
       {
       *autherr = strdup("User not in authorized user list.");
       sprintf(error_msg, "%s Requested user %s: requested from host %s",
@@ -450,11 +443,10 @@ int authenticate_user(
 
   if (pcred->timestamp)
     {
-    long lifetime;
-    if ((server.sv_attr[SRV_ATR_CredentialLifetime].at_flags & ATR_VFLAG_SET) != 0)
+    long lifetime = 0;
+    if (get_svr_attr_l(SRV_ATR_CredentialLifetime, &lifetime) == PBSE_NONE)
       {
       /* use configured value if set */
-      lifetime = server.sv_attr[SRV_ATR_CredentialLifetime].at_val.at_long;
       }
     else 
       {
@@ -475,14 +467,14 @@ int authenticate_user(
     }
 
   /* If Server's Acl_User enabled, check if user in list */
-
-  if (server.sv_attr[SRV_ATR_AclUserEnabled].at_val.at_long)
+  get_svr_attr_l(SRV_ATR_AclUserEnabled, &acl_enabled);
+  if (acl_enabled)
     {
-    strcpy(uath, preq->rq_user);
-    strcat(uath, "@");
-    strcat(uath, preq->rq_host);
-
-    if (acl_check(&server.sv_attr[SRV_ATR_AclUsers], uath, ACL_User) == 0)
+    struct array_strings *acl_users = NULL;
+    snprintf(uath, sizeof(uath), "%s@%s", preq->rq_user, preq->rq_host);
+    
+    get_svr_attr_arst(SRV_ATR_AclUsers, &acl_users);
+    if (acl_check_my_array_string(acl_users, uath, ACL_User) == 0)
       {
 
 #ifdef __CYGWIN__
@@ -519,56 +511,34 @@ int authenticate_user(
 
 
 
-/*
- * chk_job_request - check legality of a request against a job
- *
- * this checks the most conditions common to most job batch requests.
- * It also returns a pointer to the job if found and the tests pass.
- */
+void chk_job_req_permissions(
 
-
-job *chk_job_request(
-
-  char                 *jobid,  /* I */
-  struct batch_request *preq)   /* I */
+  job                  **pjob_ptr, /* M */
+  struct batch_request  *preq) /* I */
 
   {
-  job *pjob;
-
-  if ((pjob = find_job(jobid)) == NULL)
-    {
-    log_event(
-      PBSEVENT_DEBUG,
-      PBS_EVENTCLASS_JOB,
-      jobid,
-      pbse_to_txt(PBSE_UNKJOBID));
-
-    req_reject(PBSE_UNKJOBID, 0, preq, NULL, "cannot locate job");
-
-    return(NULL);
-    }
+  job  *pjob = *pjob_ptr;
+  char  tmpLine[MAXLINE];
+  char  log_buf[LOCAL_LOG_BUF_SIZE];
 
   if (svr_authorize_jobreq(preq, pjob) == -1)
     {
-    sprintf(log_buffer, msg_permlog,
-            preq->rq_type,
-            "Job",
-            pjob->ji_qs.ji_jobid,
-            preq->rq_user,
-            preq->rq_host);
-
-    log_event(
-      PBSEVENT_SECURITY,
-      PBS_EVENTCLASS_JOB,
+    sprintf(log_buf, msg_permlog,
+      preq->rq_type,
+      "Job",
       pjob->ji_qs.ji_jobid,
-      log_buffer);
+      preq->rq_user,
+      preq->rq_host);
+
+    log_event(PBSEVENT_SECURITY,PBS_EVENTCLASS_JOB,pjob->ji_qs.ji_jobid,log_buf);
 
     req_reject(PBSE_PERM, 0, preq, NULL, "operation not permitted");
 
-    return(NULL);
-    }
+    pthread_mutex_unlock(pjob->ji_mutex);
 
-  if (pjob->ji_qs.ji_state >= JOB_STATE_EXITING)
+    *pjob_ptr = NULL;
+    }
+  else if (pjob->ji_qs.ji_state >= JOB_STATE_EXITING)
     {
     /* job has completed */
 
@@ -585,34 +555,64 @@ job *chk_job_request(
 
       default:
 
-        {
-        char tmpLine[1024];
+        sprintf(log_buf, "%s %s",
+          pbse_to_txt(PBSE_BADSTATE),
+          PJobState[pjob->ji_qs.ji_state]);
 
-        sprintf(log_buffer, "%s %s",
-                pbse_to_txt(PBSE_BADSTATE),
-                PJobState[pjob->ji_qs.ji_state]);
+        log_event(PBSEVENT_DEBUG,PBS_EVENTCLASS_JOB,pjob->ji_qs.ji_jobid,log_buf);
 
-        log_event(
-          PBSEVENT_DEBUG,
-          PBS_EVENTCLASS_JOB,
-          pjob->ji_qs.ji_jobid,
-          log_buffer);
-
-        sprintf(tmpLine, "invalid state for job - %s",
-                PJobState[pjob->ji_qs.ji_state]);
+        snprintf(tmpLine, sizeof(tmpLine), 
+          "invalid state for job - %s",
+          PJobState[pjob->ji_qs.ji_state]);
 
         req_reject(PBSE_BADSTATE, 0, preq, NULL, tmpLine);
 
-        return(NULL);
-        }
+        pthread_mutex_unlock(pjob->ji_mutex);
 
-      /*NOTREACHED*/
+        *pjob_ptr = NULL;
 
-      break;
+        break;
       }  /* END switch (preq->rq_type) */
     }    /* END if (pjob->ji_qs.ji_state >= JOB_STATE_EXITING) */
 
   /* SUCCESS - request is valid */
+  } /* END chk_job_req_permissions() */
+
+
+
+
+
+/*
+ * chk_job_request - check legality of a request against a job
+ *
+ * this checks the most conditions common to most job batch requests.
+ * It also returns a pointer to the job if found and the tests pass.
+ */
+
+
+job *chk_job_request(
+
+  char                 *jobid,  /* I */
+  struct batch_request *preq)   /* I */
+
+  {
+  job *pjob = NULL;
+
+  if ((pjob = find_job(jobid)) == NULL)
+    {
+    log_event(
+      PBSEVENT_DEBUG,
+      PBS_EVENTCLASS_JOB,
+      jobid,
+      pbse_to_txt(PBSE_UNKJOBID));
+
+    req_reject(PBSE_UNKJOBID, 0, preq, NULL, "cannot locate job");
+
+    return(NULL);
+    }
+
+  /* if we aren't authorized, pjob will be set to NULL in chk_job_req_permissions */
+  chk_job_req_permissions(&pjob,preq);
 
   return(pjob);
   }  /* END chk_job_request() */
