@@ -102,18 +102,20 @@
 #include <pbs_config.h>   /* the master config generated by configure */
 
 #include <errno.h>
-#include <sys/types.h>
-#include <sys/socket.h>
 #include <time.h>
 #include <stdio.h>
 #include <signal.h>
 #include <unistd.h>
 #include "libpbs.h"
 #include "log.h"
+#include "../lib/Liblog/pbs_log.h"
+#include "../lib/Liblog/log_event.h"
 #include "server_limits.h"
 #include "net_connect.h"
 #include "svrfunc.h"
 #include "dis.h"
+#include "../lib/Libnet/lib_net.h" /* get_connection_entry */
+#include "../lib/Liblog/pbs_log.h" /* print_trace */
 
 
 /* global data */
@@ -121,55 +123,55 @@
 
 extern int   errno;
 
-extern int   pbs_errno;
 extern unsigned int  pbs_server_port_dis;
 
 extern struct connection svr_conn[];
 extern pbs_net_t  pbs_server_addr;
 extern int               LOGLEVEL;
 
-extern int     addr_ok(pbs_net_t);
-extern void    bad_node_warning(pbs_net_t);
+extern void    bad_node_warning(pbs_net_t, struct pbsnode *);
 extern ssize_t read_blocking_socket(int, void *, ssize_t);
 extern int get_num_connections();
+int            addr_ok(pbs_net_t,struct pbsnode *);
 
 
 
 int svr_connect(
 
-  pbs_net_t      hostaddr,  /* host order */
-  unsigned int   port,   /* I */
-  void (*func)(int),
-  enum conn_type cntype)
+  pbs_net_t        hostaddr,  /* host order */
+  unsigned int     port,   /* I */
+  int             *my_err,
+  struct pbsnode  *pnode,
+  void           *(*func)(void *),
+  enum conn_type   cntype)
 
   {
-  char         EMsg[1024];
-
   static char *id = "svr_connect";
 
-  int handle;
-  int sock;
-
-  time_t STime;
-  time_t ETime;
+  char         EMsg[1024];
+  int          handle;
+  int          sock;
+  time_t       STime;
+  time_t       ETime;
+  char         log_buf[LOCAL_LOG_BUF_SIZE];
+  int          rc = PBSE_NONE;
 
   if (LOGLEVEL >= 4)
     {
-    sprintf(log_buffer, "attempting connect to %s %s port %d",
-            (hostaddr == pbs_server_addr) ? "server" : "host",
-            ((int)hostaddr != 0) ? netaddr_pbs_net_t(hostaddr) : "localhost",
-            port);
+    char *tmp = netaddr_pbs_net_t(hostaddr);
+    sprintf(log_buf, "attempting connect to %s %s port %d",
+      (hostaddr == pbs_server_addr) ? "server" : "host",
+      ((int)hostaddr != 0) ? tmp : "localhost",
+      port);
 
-    log_event(
-      PBSEVENT_ADMIN,
-      PBS_EVENTCLASS_SERVER,
-      id,
-      log_buffer);
+    log_event(PBSEVENT_ADMIN, PBS_EVENTCLASS_SERVER, id, log_buf);
+
+    free(tmp);
     }
 
   /* First, determine if the request is to another server or ourselves */
-
-  if ((hostaddr == pbs_server_addr) && (port == pbs_server_port_dis))
+  if ((hostaddr == pbs_server_addr) && 
+      (port == pbs_server_port_dis))
     {
     return(PBS_LOCAL_CONNECTION); /* special value for local */
     }
@@ -177,23 +179,18 @@ int svr_connect(
   time(&STime);
 
   /* obtain the connection to the other server */
-
-  if (!addr_ok(hostaddr))
+  if (!addr_ok(hostaddr,pnode))
     {
     if (LOGLEVEL >= 4)
       {
-      sprintf(log_buffer, "cannot connect to %s port %d - target is down",
-              (hostaddr == pbs_server_addr) ? "server" : "host",
-              port);
+      sprintf(log_buf, "cannot connect to %s port %d - target is down",
+        (hostaddr == pbs_server_addr) ? "server" : "host",
+        port);
 
-      log_event(
-        PBSEVENT_ADMIN,
-        PBS_EVENTCLASS_SERVER,
-        id,
-        log_buffer);
+      log_event(PBSEVENT_ADMIN, PBS_EVENTCLASS_SERVER, id, log_buf);
       }
 
-    pbs_errno = EHOSTDOWN;
+    *my_err = EHOSTDOWN;
 
     return(PBS_NET_RC_RETRY);
     }
@@ -216,38 +213,33 @@ int svr_connect(
     {
     if (LOGLEVEL >= 4)
       {
-      sprintf(log_buffer, "cannot connect to %s port %d - cannot establish connection (%s) - time=%ld seconds",
-              (hostaddr == pbs_server_addr) ? "server" : "host",
-              port,
-              EMsg,
-              (long)(ETime - STime));
+      sprintf(log_buf,
+        "cannot connect to %s port %d - cannot establish connection (%s) - time=%ld seconds",
+        (hostaddr == pbs_server_addr) ? "server" : "host",
+        port,
+        EMsg,
+        (long)(ETime - STime));
 
-      log_event(
-        PBSEVENT_ADMIN,
-        PBS_EVENTCLASS_SERVER,
-        id,
-        log_buffer);
+      log_event(PBSEVENT_ADMIN, PBS_EVENTCLASS_SERVER, id, log_buf);
       }
 
-    bad_node_warning(hostaddr);
+    bad_node_warning(hostaddr,pnode);
 
-    pbs_errno = errno;
+    *my_err = errno;
 
     return(sock);  /* PBS_NET_RC_RETRY or PBS_NET_RC_FATAL */
     }  /* END if (sock < 0) */
 
-  if ((LOGLEVEL >= 2) && (ETime > STime))
+  if ((LOGLEVEL >= 2) && 
+      (ETime > STime))
     {
-    sprintf(log_buffer, "successful connect to %s port %d - time=%ld seconds",
-            (hostaddr == pbs_server_addr) ? "server" : "host",
-            port,
-            (long)(ETime - STime));
+    sprintf(log_buf,
+      "successful connect to %s port %d - time=%ld seconds",
+      (hostaddr == pbs_server_addr) ? "server" : "host",
+      port,
+      (long)(ETime - STime));
 
-    log_event(
-      PBSEVENT_ADMIN,
-      PBS_EVENTCLASS_SERVER,
-      id,
-      log_buffer);
+    log_event(PBSEVENT_ADMIN,PBS_EVENTCLASS_SERVER,id,log_buf);
     }
 
   /* add the connection to the server connection table and select list */
@@ -255,32 +247,35 @@ int svr_connect(
   if (func != NULL)
     {
     /* connect attempt to XXX? */
-
-    add_conn(sock, ToServerDIS, hostaddr, port, PBS_SOCK_INET, func);
+    if ((rc = add_conn(sock, ToServerDIS, hostaddr, port, PBS_SOCK_INET, func)) != PBSE_NONE)
+        {
+        /* Return invalid handle */
+        return -1;
+        }
     }
 
+  pthread_mutex_lock(svr_conn[sock].cn_mutex);
+
   svr_conn[sock].cn_authen = PBS_NET_CONN_AUTHENTICATED;
 
-  /* find a connect_handle entry we can use and pass to the PBS_*() */
+  pthread_mutex_unlock(svr_conn[sock].cn_mutex);
 
-  handle = socket_to_handle(sock);
+  /* find a connect_handle entry we can use and pass to the PBS_*() */
+  handle = socket_to_handle(sock, my_err);
 
   if (handle == -1)
     {
     if (LOGLEVEL >= 4)
       {
-      sprintf(log_buffer, "cannot connect to %s port %d - cannot get handle",
-              (hostaddr == pbs_server_addr) ? "server" : "host",
-              port);
+      sprintf(log_buf,
+        "cannot connect to %s port %d - cannot get handle",
+        (hostaddr == pbs_server_addr) ? "server" : "host",
+        port);
 
-      log_event(
-        PBSEVENT_ADMIN,
-        PBS_EVENTCLASS_SERVER,
-        id,
-        log_buffer);
+      log_event(PBSEVENT_ADMIN,PBS_EVENTCLASS_SERVER,id,log_buf);
       }
 
-    close_conn(sock);
+    close_conn(sock, FALSE);
 
     return(PBS_NET_RC_RETRY);
     }
@@ -316,7 +311,6 @@ static void localalm(
  * server's connection table and the handle table used by
  * the API routines must be cleaned-up.
  */
-
 void svr_disconnect(
 
   int handle)  /* I */
@@ -327,6 +321,8 @@ void svr_disconnect(
 
   if ((handle >= 0) && (handle < PBS_LOCAL_CONNECTION))
     {
+    pthread_mutex_lock(connection[handle].ch_mutex);
+
     sock = connection[handle].ch_socket;
 
     DIS_tcp_setup(sock);
@@ -364,7 +360,7 @@ void svr_disconnect(
 
     shutdown(connection[handle].ch_socket, 2);
 
-    close_conn(connection[handle].ch_socket);
+    close_conn(connection[handle].ch_socket, FALSE);
 
     if (connection[handle].ch_errtxt != NULL)
       {
@@ -374,7 +370,9 @@ void svr_disconnect(
 
     connection[handle].ch_errno = 0;
 
-    connection[handle].ch_inuse = 0;
+    connection[handle].ch_inuse = FALSE;
+
+    pthread_mutex_unlock(connection[handle].ch_mutex);
     }
 
   return;
@@ -388,60 +386,63 @@ void svr_disconnect(
  * as used by the libpbs.a routines.
  *
  * Returns: >=0 connection handle if successful, or
- *    -1 if error, error number set in pbs_errno.
+ *    -1 if error, error number set in local_errno.
  */
 
 int socket_to_handle(
 
-  int sock)  /* opened socket */
+  int  sock,        /* opened socket */
+  int *local_errno) /* O */
 
   {
   char *id = "socket_to_handle";
+  char  log_buf[LOCAL_LOG_BUF_SIZE];
 
-  int i;
+  int   conn_pos = 0;
+  int   rc = PBSE_NONE;
 
-  for (i = 0;i < PBS_NET_MAX_CONNECTIONS;i++)
+  if ((rc = get_connection_entry(&conn_pos)) != PBSE_NONE)
     {
-    if (connection[i].ch_inuse != 0)
-      continue;
+    sprintf(log_buf,"internal socket table full (%d) - num_connections is %d",
+      conn_pos,
+      get_num_connections());
 
-    connection[i].ch_stream = 0;
-    connection[i].ch_inuse  = 1;
-    connection[i].ch_errno  = 0;
-    connection[i].ch_socket = sock;
-    connection[i].ch_errtxt = 0;
+    log_ext(-1,id,log_buf,LOG_ALERT);
+
+    *local_errno = PBSE_NOCONNECTS;
+    }
+  else
+    {
+    /* NOTE: get_connection_entry() locks connection[conn_pos] */
+    connection[conn_pos].ch_stream = 0;
+    connection[conn_pos].ch_inuse  = TRUE;
+    connection[conn_pos].ch_errno  = 0;
+    connection[conn_pos].ch_socket = sock;
+    connection[conn_pos].ch_errtxt = 0;
 
     /* SUCCESS - save handle for later close */
-
-    svr_conn[sock].cn_handle = i;
-
-    if (i >= (PBS_NET_MAX_CONNECTIONS/2))
+    pthread_mutex_unlock(connection[conn_pos].ch_mutex);
+      
+    pthread_mutex_lock(svr_conn[sock].cn_mutex);
+    svr_conn[sock].cn_handle = conn_pos;
+    pthread_mutex_unlock(svr_conn[sock].cn_mutex);
+      
+    if (conn_pos >= (PBS_NET_MAX_CONNECTIONS/2))
       {
-      sprintf(log_buffer,"internal socket table is half-full at %d (expected num_connections is %d)!",
-        i,
+      sprintf(log_buf,"internal socket table is half-full at %d (expected num_connections is %d)!",
+        conn_pos,
         get_num_connections());
-
-      log_ext(-1,id,log_buffer,LOG_CRIT);
+        
+      log_ext(-1,id,log_buf,LOG_CRIT);
       }
 
-    return(i);
-    }  /* END for (i) */
+    rc = conn_pos;
+    }
 
-  sprintf(log_buffer,"internal socket table full (%d) - num_connections is %d",
-    i,
-    get_num_connections());
-
-  log_ext(-1,id,log_buffer,LOG_ALERT);
-
-  pbs_errno = PBSE_NOCONNECTS;
-
-  return(-1);
+  return(rc);
   }  /* END socket_to_handle() */
 
 
-
-
-
 /*
  * parse_servername - parse a server/mom name in the form:
  * hostname[:service_port][/#][+hostname...]
@@ -452,17 +453,18 @@ int socket_to_handle(
  *
  * Warning: as written, Not reentrient/thread safe
  */
-
 char *parse_servername(
 
   char         *name,   /* server name in form name[:port] */
   unsigned int *service)  /* RETURN: service_port if :port */
 
   {
-  static char  buf[PBS_MAXSERVERNAME + PBS_MAXPORTNUM + 2];
+  char  buf[PBS_MAXSERVERNAME + PBS_MAXPORTNUM + 2];
+  char *val;
 
   int   i = 0;
   char *pc;
+  char *tmp_val = NULL;
 
   buf[0] = '\0';
 
@@ -473,8 +475,10 @@ char *parse_servername(
   if (name == NULL)
     {
     /* invalid name specified */
+    tmp_val = (char *)calloc(1,1);
+    tmp_val[0] = '\0';
 
-    return(buf);
+    return tmp_val;
     }
 
   while (*pc && (i < PBS_MAXSERVERNAME + PBS_MAXPORTNUM + 2))
@@ -496,9 +500,11 @@ char *parse_servername(
 
   buf[i] = '\0';
 
-  return(buf);
-  }  /* END parse_servername() */
+  val = calloc(1, strlen(buf) + 1);
+  strcpy(val,buf);
 
+  return(val);
+  }  /* END parse_servername() */
 
 /* END svr_connect.c */
 
